unit AliasDialog;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Grids, DBGrids, Db;

type
  TAlias = class(TForm)
    DBGrid1: TDBGrid;
    PlayerAlias: TEdit;
    OKButton: TButton;
    CancelButton: TButton;
    ButtonID: TEdit;
    procedure FormShow(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure PlayerAliasChange(Sender: TObject);
    procedure PlayerAliasKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure AliasQueryAfterScroll(DataSet: TDataSet);
    procedure DBGrid1DblClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    reentered: Boolean;
    lastsearch: string;

    function Query(like: string = ''): string;
  public
    { Public declarations }
  end;

var
  Alias: TAlias;

implementation

uses TornMain;

{$R *.DFM}

procedure TAlias.FormShow(Sender: TObject);
begin
    with TornForm.ZQuery do begin
        Close;
        SQL.Clear;
        SQL.Add(Query);
        Open;
    end;
    DBGrid1.DataSource := TornForm.DataSource;
    TornForm.ZQuery.AfterScroll := AliasQueryAfterScroll;

    lastsearch := '%'
end;

procedure TAlias.FormResize(Sender: TObject);
begin
    if DBGrid1.Columns.Count > 1 then DBGrid1.Columns[0].Width := Trunc((DBGrid1.Width - 25) * 0.4);
    if DBGrid1.Columns.Count > 2 then DBGrid1.Columns[1].Width := Trunc((DBGrid1.Width - 25) * 0.3);
    if DBGrid1.Columns.Count > 3 then DBGrid1.Columns[2].Width := Trunc((DBGrid1.Width - 25) * 0.3);
end;

procedure TAlias.PlayerAliasChange(Sender: TObject);
var s: string;
begin
    if reentered then Exit;
    reentered := True;

    Application.ProcessMessages;  // if the user has typed several keystrokes into the edit box, we don't want to do several SQL queries.  Ensure that PlayerAlias.Text contains up-to-date info.
    s := UpperCase(Copy(PlayerAlias.Text, 1, PlayerAlias.SelStart));
    if lastsearch = s then Exit;  // if we've already done this search (because the user has typed several keystrokes into the edit box), skip it.
    lastsearch := s;

    with TornForm.ZQuery do begin
        Close;
        SQL.Clear;
        SQL.Add(Query(s));
        TornForm.ZQuery.ParamByName('Name').AsString := s;
        Open;
    end;

    reentered := False;
end;

procedure TAlias.PlayerAliasKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
    if Key = VK_UP then begin
        TornForm.ZQuery.Prior;
        Key := 0;
    end;
    if Key = VK_DOWN then begin
        TornForm.ZQuery.Next;
        Key := 0;
    end;

    if Key in [VK_BACK, VK_LEFT] then begin
        PlayerAlias.SelStart := PlayerAlias.SelStart - 1;
        PlayerAlias.SelLength := 999;
        Key := 0;
    end;
    if Key = VK_RIGHT then begin
        PlayerAlias.SelStart := PlayerAlias.SelStart + 1;
        PlayerAlias.SelLength := 999;
        Key := 0;
    end;
end;

procedure TAlias.AliasQueryAfterScroll(DataSet: TDataSet);
var i: Integer;
begin
    if TornForm.ZQuery.Eof then Exit;

    reentered := True;

    i := PlayerAlias.SelStart;
    with TornForm.ZQuery do
        if UpperCase(Copy(PlayerAlias.Text, 1, i)) = UpperCase(Copy(FieldValues['Player_Alias'], 1, i)) then
            PlayerAlias.Text := FieldValues['Player_Alias']
        else
            PlayerAlias.Text := FieldValues['First_Name'] + ' ' + FieldValues['Last_Name'];

    PlayerAlias.SelStart := i;
    PlayerAlias.SelLength := 999;

    ButtonID.Text := TornForm.ZQuery.FieldValues['User_ID'];

    reentered := False;
end;

procedure TAlias.DBGrid1DblClick(Sender: TObject);
begin
    ModalResult := mrOK;
end;

procedure TAlias.FormClose(Sender: TObject; var Action: TCloseAction);
begin
    TornForm.ZQuery.AfterScroll := nil;
    TornForm.ZQuery.Close;
end;

function TAlias.Query(like: string): string;
begin
    if TornForm.IsNexus then begin
        Result := 'SELECT Alias AS Player_Alias, '''' AS First_Name, '''' AS Last_Name, Button_ID AS User_ID FROM members';
//        if like <> '' then
            Result := Result + ' WHERE UPPER(Alias) LIKE ":Name%" ';
    end else begin
        Result := 'SELECT Player_Alias, First_Name, Last_Name, User_ID FROM MEMBERS';
//        if like <> '' then
            Result := Result +
                ' WHERE UPPER(Player_Alias) LIKE ":Name%" ' +
                'OR UPPER(First_Name) || '' '' || UPPER(Last_Name) LIKE ":Name%"';
    end;
end;

end.
