unit DataModule;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Db, DBTables, IBDatabase, IBCustomDataSet;

type
    THitArray = array [1..6] of Integer;

  TParadoxDataModule = class(TDataModule)
    LeagueDatabase: TDatabase;
    LeagueTable: TTable;
    MClass: TTable;
    Member: TTable;
    MClassLinkID: TIntegerField;
    MClassAlias: TStringField;
    MemberLinkID: TAutoIncField;
    MemberButtonID: TStringField;
    LeagueDataSource: TDataSource;
    MemberDatabase: TDatabase;
    ScoresSetTable: TTable;
    StatsTable: TTable;
    HitTable: TTable;
    ScoreDatabase: TDatabase;
    AliasDataSource: TDataSource;
    AliasQuery: TQuery;
  public
    function  CleanStats(GameN: Integer): Boolean;
    procedure WriteStatsLine(fieldarr: Variant);
    procedure WriteHitLine(GameNum, hitonid, hitbyid: Integer; hit: THitArray);
    procedure WriteHitNil(GameNum, hitonid, hitbyid: Integer);
    function FindID(buttonid: string): string;
  end;

var
  ParadoxDataModule: TParadoxDataModule;

implementation

{$R *.DFM}

// Open StatsTable and HitTable, delete any records which refer to games higher than
// or equal to GameNum, and _leave StatsTable open_.  The caller must close StatsTable.
function TParadoxDataModule.CleanStats(GameN: Integer): Boolean;
begin
    try
        StatsTable.Cancel;
        StatsTable.Close;
        StatsTable.ReadOnly := False;  // open StatsTable for editing
        StatsTable.Open;
        StatsTable.Edit;

        StatsTable.Last;
        while not StatsTable.BOF and (StatsTable.FieldByName('Game').AsInteger >= GameN) do begin
            StatsTable.Delete;
            StatsTable.Last;
        end;

        HitTable.Cancel;
        HitTable.Close;
        HitTable.ReadOnly := False;  // open HitTable for editing
        HitTable.Open;
        HitTable.Edit;

        HitTable.Last;
        while not HitTable.BOF and (HitTable.FieldByName('Game').AsInteger >= GameN) do begin
            HitTable.Delete;
            HitTable.Last;
        end;
        HitTable.Close;

        Result := True;
    except
        on EDBEngineError do begin
            HitTable.Close;
            ShowMessage('Hey!  I can''t find LGScoreDB!');
            Result := False;
        end;
    end;
end;

// Write a single line of data into StatsTable.
// Game, PackID, Team, MemberID, MemberAlias, PlayerClass, ScoresSet, Handicap%,
// Score, Rank, Accuracy, GameTimeStamp, GameTitle, Duration, Played, ...
procedure TParadoxDataModule.WriteStatsLine(fieldarr: Variant);
var i: Integer;
begin
    with StatsTable do begin
        Append;
        for i := VarArrayLowBound(fieldarr, 1) to VarArrayHighBound(fieldarr, 1) do
            case Fields[i].DataType of
              ftInteger, ftSmallint, ftWord, ftLargeint:Fields[i].AsInteger := fieldarr[i];
              ftFloat:                                  Fields[i].AsFloat   := fieldarr[i];
              ftString,ftFixedChar, ftWideString:       Fields[i].AsString  := fieldarr[i];
              ftDate, ftTime, ftDateTime:               Fields[i].AsDateTime:= fieldarr[i];
              else                                      Fields[i].AsVariant := fieldarr[i];
            end;
        Post;
    end;
end;

// Write a single line of data into HitTable.
// Game, HitOnID, HitByID, Zone1 .. Zone6, HitOnTeam, HitByTeam
procedure TParadoxDataModule.WriteHitLine(GameNum, hitonid, hitbyid: Integer; hit: THitArray);
var zone: Integer;
    hitonteam, hitbyteam: Integer;
begin
    if hitonid >=60 then hitonteam := 4
                    else hitonteam := (hitonid mod 3) + 1;

    if hitbyid >=60 then hitbyteam := 4
                    else hitbyteam := (hitbyid mod 3) + 1;

    with HitTable do begin
        Append;
        Fields[0].AsInteger := GameNum;
        Fields[1].AsInteger := hitonid;
        Fields[2].AsInteger := hitbyid;
        Fields[9].AsInteger := hitonteam;
        Fields[10].AsInteger:= hitbyteam;
        for zone := 1 to 6 do
            Fields[zone + 2].AsInteger := hit[zone];
        Post;
    end;
end;

// Write a single line of data into HitTable.
// Game, HitOnID, HitByID, Zone1 .. Zone6, HitOnTeam, HitByTeam
procedure TParadoxDataModule.WriteHitNil(GameNum, hitonid, hitbyid: Integer);
var zone: Integer;
    hitonteam, hitbyteam: Integer;
begin
    if hitonid >=60 then hitonteam := 4
                    else hitonteam := (hitonid mod 3) + 1;

    if hitbyid >=60 then hitbyteam := 4
                    else hitbyteam := (hitbyid mod 3) + 1;

    with HitTable do begin
        Append;
        Fields[0].AsInteger := GameNum;
        Fields[1].AsInteger := hitonid;
        Fields[2].AsInteger := hitbyid;
        Fields[9].AsInteger := hitonteam;
        Fields[10].AsInteger:= hitbyteam;
        for zone := 1 to 6 do
            Fields[zone + 2].Clear;
        Post;
    end;
end;

function TParadoxDataModule.FindID(buttonid: string): string;
begin
    Member.Open;
    MClass.Open;
    if Member.Locate('ButtonID', buttonid, []) and
       MClass.Locate('LinkID', Member.FieldByName('LinkID').AsInteger, []) then
        Result := MClass.FieldByName('Alias').AsString
    else
        Result := buttonid;
    Member.Close;
    MClass.Close;
end;

end.
