unit DramaDialog;

// "Drama Mode" is where we display a report one row (future expansion: maybe one cell)
// at a time, from the bottom up, on the screen and scorebaord.

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, ZoomReport, LadderDialog,
  StdCtrls;

type
  TDramaForm = class(TForm)
    RowButton: TButton;
    ColumnsLabel: TLabel;
    Label1: TLabel;
    ColumnButton: TButton;
    procedure FormShow(Sender: TObject);
    procedure RowButtonClick(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure ColumnButtonClick(Sender: TObject);
  private
    FReport: TZoomReport;  // Original of report to be displayed in Drama Mode.
    FRowCount: Integer;
    FColumnCount: Integer;
    FOwnerForm: TForm;  // Needed by ShowOnScreen, to position LadderForm.
    FLadderForm: TLadderForm;

    procedure DramaMode;
  public
    property Report: TZoomReport read FReport write FReport;
    property OwnerForm: TForm read FOwnerForm write FOwnerForm;
    property LadderForm: TLadderForm read FLadderForm write FLadderForm;
  end;

var
  DramaForm: TDramaForm;

implementation

uses Reports;

{$R *.DFM}

procedure TDramaForm.FormShow(Sender: TObject);
begin
    FRowCount := 0;
    FColumnCount := 0;
    DramaMode;
end;

// Show a single report one row at a time from the bottom up, on both screen and scoreboard.
procedure TDramaForm.DramaMode;
var reports2: TZoomReports;
    outreport: TZoomReport;
    j, k: Integer;
    startrow: Integer;
    startcol: Integer;
    outrow : Integer;
begin
    // Make a copy of the source TZoomReports.
    reports2 := TZoomReports.Create;
    reports2.TitleFontColor := FReport.Owner.TitleFontColor;
    reports2.TitleBackColor := FReport.Owner.TitleBackColor;
    reports2.TextColor := FReport.Owner.TextColor;
    reports2.BackgroundColor := FReport.Owner.BackgroundColor;
    reports2.OddColor := FReport.Owner.OddColor;
    reports2.BarColor := FReport.Owner.BarColor;

    // Make a copy of the source TZoomReport and add it to reports2.
    outreport := reports2.Add(report.Title, '', '');
    for k := 0 to report.Columns.Count - 1 do
        outreport.Columns.Add(FReport.Columns[k].Heading, FReport.Columns[k].Alignment);
    outreport.OnCalcBar := FReport.OnCalcBar;
    outreport.OnPaintBar := FReport.OnPaintBar;
    outreport.OnCellDestroy := FReport.OnCellDestroy;
    outreport.MaxBarByColumn := FReport.MaxBarByColumn;
    outreport.MaxBar := FReport.MaxBar;
    //outreport.Description := FReport.Description;

    if FColumnCount > report.Columns.Count then begin
        Dec(FColumnCount, report.Columns.Count);
        Inc(FRowCount);
    end;

    startrow := FReport.Rows.Count - FRowCount;
    if startrow < 0 then startrow := 0;

    if startrow = 0 then
        outreport.Description := FReport.Description;

    // Create empty rows for the start of the report.
    for j := 0 to startrow - 1 do
        outreport.Rows.Add(TZRow.Create);

    // Create a partly-filled row, if required.
    if FColumnCount <> 0 then begin
        outrow := outreport.Rows.Add(TZRow.Create);

        startcol := FReport.Rows[startrow - 1].Count - FColumnCount;
        if startcol < 0 then startcol := 0;

        for k := 0 to startcol - 1 do
            outreport.Rows[outrow].Add('');

        for k := startcol to FReport.Rows[startrow - 1].Count - 1 do
            outreport.Rows[outrow].Add(FReport.Rows[startrow - 1][k].Clone);
    end;

    // Create full rows for the remainder of the report, by cloning the source report.
    for j := startrow to FReport.Rows.Count - 1 do begin  // Roll through the last NumRows rows of the source report
        outrow := outreport.Rows.Add(TZRow.Create);
        for k := 0 to FReport.Rows[j].Count - 1 do
            outreport.Rows[outrow].Add(FReport.Rows[j][k].Clone);
    end;

//    reports2[0].PurgeBlankColumns;
    ColumnsLabel.Caption := IntToStr(report.Columns.Count) + ' columns; ' + IntToStr(report.Rows.Count) + ' rows.';

    ShowOnScreen(reports2, FOwnerForm, FLadderForm);
    ShowOnScoreboard(reports2[0]);
    Self.SetFocus;

    outreport.Free;
end;

procedure TDramaForm.RowButtonClick(Sender: TObject);
begin
    Inc(FRowCount, TComponent(Sender).Tag);
    FColumnCount := 0;
    DramaMode;
end;

procedure TDramaForm.ColumnButtonClick(Sender: TObject);
begin
    Inc(FColumnCount, TComponent(Sender).Tag);
    DramaMode;
end;

procedure TDramaForm.FormKeyPress(Sender: TObject; var Key: Char);
begin
    if Pos(Key, '1234567890-=') > 0 then begin  // Number key: show that many more rows.
        Inc(FRowCount, Pos(Key, '1234567890-='));
        FColumnCount := 0;
        DramaMode;
    end
    else if Pos(Key, '!@#$%^&*()_+') > 0 then begin  // Shifted number key: show that many more columns within the current row.
        Inc(FColumnCount, Pos(Key, '!@#$%^&*()_+'));
        DramaMode;
    end
    else if Key in ['`', '~'] then begin  // Finish showing this row.
        Inc(FRowCount);
        FColumnCount := 0;
        DramaMode;
    end;
end;

end.
