unit GamesDialog;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Grids, ComCtrls, Spin, Menus, Buttons, Games, ExtCtrls, League;

type
  TGamesForm = class(TForm)
    PageControl1: TPageControl;
    AddTeamTab: TTabSheet;
    AddPlayerTab: TTabSheet;
    TeamCombo: TComboBox;
    AddTeamButton: TButton;
    AliasLabel: TLabel;
    LookupButton: TButton;
    AddPlayerButton: TButton;
    PlayerScore: TSpinEdit;
    PlayerRank: TSpinEdit;
    PlayerScoreLabel: TLabel;
    RankLabel: TLabel;
    TeamLabel: TLabel;
    PackEdit: TEdit;
    PackLabel: TLabel;
    PlayerLabel: TLabel;
    ColourLabel: TLabel;
    TeamScoreLabel: TLabel;
    TeamScore: TSpinEdit;
    TeamPoints: TSpinEdit;
    PointsLabel: TLabel;
    ColourCombo: TComboBox;
    DeleteGameTab: TTabSheet;
    Memo1: TMemo;
    PopupMenu1: TPopupMenu;
    DeleteItem: TMenuItem;
    ButtonID: TEdit;
    DeleteGameButton: TBitBtn;
    TeamGrid: TStringGrid;
    PlayerGrid: TStringGrid;
    Panel1: TPanel;
    GamesCombo: TComboBox;
    Splitter1: TSplitter;
    Panel2: TPanel;
    CloseButton: TButton;
    HelpButton: TButton;
    AddGameTab: TTabSheet;
    GameDate: TDateTimePicker;
    GameTime: TDateTimePicker;
    AddGameButton: TButton;
    procedure FormShow(Sender: TObject);
    procedure GamesComboChange(Sender: TObject);
    procedure GameDateTimeChange(Sender: TObject);
    procedure LookupButtonClick(Sender: TObject);
    procedure PageControl1Change(Sender: TObject);
    procedure TeamComboChange(Sender: TObject);
    procedure AddGameButtonClick(Sender: TObject);
    procedure AddTeamButtonClick(Sender: TObject);
    procedure AddPlayerButtonClick(Sender: TObject);
    procedure DeleteGameButtonClick(Sender: TObject);
    procedure DeleteItemClick(Sender: TObject);
    procedure HelpButtonClick(Sender: TObject);
    procedure CloseButtonClick(Sender: TObject);
//    procedure DBGrid1DrawColumnCell(Sender: TObject; const Rect: TRect;
//      DataCol: Integer; Column: TColumn; State: TGridDrawState);
    procedure GridDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure TeamGridClick(Sender: TObject);
    procedure PlayerGridClick(Sender: TObject);
  private
    FGameList: TGameList;
    FLeagueList: TLeagueList;

    procedure Refresh;
    function FindTeam: TTeam;
  public
    property GameList: TGameList read FGameList write FGameList;
    property LeagueList: TLeagueList read FLeagueList write FLeagueList;
  end;

var
  GamesForm: TGamesForm;

implementation

uses AliasDialog, AliasLaserforceDialog, AliasDBDialog, TornMain;

{$R *.DFM}

procedure TGamesForm.FormShow(Sender: TObject);
var i: Integer;
begin
  for i := 0 to GameList.Count - 1 do
    GamesCombo.Items.Insert(0, DateTimeToStr(GameList[i].GameTime));

  if GamesCombo.ItemHeight > 0 then GamesCombo.DropDownCount := (ClientHeight - 35) div GamesCombo.ItemHeight;

  TeamGrid.RowCount := 2;
  TeamGrid.Cells[0, 0] := 'Team';
  TeamGrid.Cells[1, 0] := 'Colour';
  TeamGrid.Cells[2, 0] := 'Score';
  TeamGrid.Cells[3, 0] := 'Points';
  TeamGrid.Cells[4, 0] := 'Adjustment';

  PlayerGrid.RowCount := 2;
  PlayerGrid.Cells[0, 0] := 'Team';
  PlayerGrid.Cells[1, 0] := 'Player';
  PlayerGrid.Cells[2, 0] := 'Pack';
  PlayerGrid.Cells[3, 0] := 'Score';
  PlayerGrid.Cells[4, 0] := 'Rank';

  GameDate.Date := Date;
  GameDateTimeChange(Sender);
end;

procedure TGamesForm.GamesComboChange(Sender: TObject);
begin
  if GamesCombo.ItemIndex <> -1 then begin
    GameDate.Date     :=  Int(StrToDateTime(GamesCombo.Items[GamesCombo.ItemIndex]));
    GameTime.DateTime := Frac(StrToDateTime(GamesCombo.Items[GamesCombo.ItemIndex]));
    if GameTime.Time <> 0 then
      GameTime.Date := 0;

    Refresh;
  end;
end;

procedure TGamesForm.GameDateTimeChange(Sender: TObject);
begin
    Refresh;
end;

procedure TGamesForm.Refresh;
var i, j: Integer;
    Team: TLeagueTeam;
    s: string;
begin
  i := GameList.Find(Int(GameDate.Date) + Frac(GameTime.Time));
  if i > -1 then
    with GameList[i] do begin
      TeamGrid.RowCount := Teams.Count + 2;
      for j := 0 to Teams.Count - 1 do begin
        Team := FLeagueList.FindTeamID(Teams[j].TeamID);
        if Assigned(Team) then
          TeamGrid.Cells[0, j + 1] := IntToStr(Teams[j].TeamID) + ': ' + Team.TeamName
        else
          TeamGrid.Cells[0, j + 1] := IntToStr(Teams[j].TeamID);
        TeamGrid.Cells[1, j + 1] := Teams[j].Colour;
        TeamGrid.Cells[2, j + 1] := IntToStr(Teams[j].Score);
        TeamGrid.Cells[3, j + 1] := FloatToStr(Teams[j].Points);
        TeamGrid.Cells[4, j + 1] := IntToStr(Teams[j].Adjustment);
      end;
      TeamGrid.Cells[0, Teams.Count + 1] := '';
      TeamGrid.Cells[1, Teams.Count + 1] := '';
      TeamGrid.Cells[2, Teams.Count + 1] := '';
      TeamGrid.Cells[3, Teams.Count + 1] := '';
      TeamGrid.Cells[4, Teams.Count + 1] := '';

      PlayerGrid.RowCount := Players.Count + 2;
      for j := 0 to Players.Count - 1 do begin
        Team := FLeagueList.FindTeamID(Players[j].TeamID);
        if Assigned(Team) then
          PlayerGrid.Cells[0, j + 1] := IntToStr(Players[j].TeamID) + ': ' + Team.TeamName
        else
          PlayerGrid.Cells[0, j + 1] := IntToStr(Players[j].TeamID);

        s := FLeagueList.FindID(Players[j].PlayerID);
        if s = '' then
          PlayerGrid.Cells[1, j + 1] := Players[j].PlayerID
        else
          PlayerGrid.Cells[1, j + 1] := Players[j].PlayerID + ': ' + s;
        PlayerGrid.Cells[2, j + 1] := Players[j].Pack;
        PlayerGrid.Cells[3, j + 1] := IntToStr(Players[j].Score);
        PlayerGrid.Cells[4, j + 1] := IntToStr(Players[j].Rank);
      end;
      PlayerGrid.Cells[0, Players.Count + 1] := '';
      PlayerGrid.Cells[1, Players.Count + 1] := '';
      PlayerGrid.Cells[2, Players.Count + 1] := '';
      PlayerGrid.Cells[3, Players.Count + 1] := '';
      PlayerGrid.Cells[4, Players.Count + 1] := '';
    end;

  DeleteGameButton.Caption := 'Delete game'#13#10 + DateToStr(GameDate.Date) + ' ' + TimeToStr(Frac(GameTime.Time));
end;

procedure TGamesForm.LookupButtonClick(Sender: TObject);
var Alias: TAlias;
begin
  if TornForm.SystemType = stLaserforce then
    Alias := TAliasLaserforce.Create(Application)
  else
    Alias := TAliasDB.Create(Application);
  try
    Alias.PlayerAlias.Text := AliasLabel.Caption;
    if Alias.ShowModal = mrOK then begin
      AliasLabel.Caption := Alias.PlayerAlias.Text;
      ButtonID.Text := Alias.ButtonID.Text;
    end;
  finally
    Alias.Release;
  end;

  TeamComboChange(Sender);
end;

procedure TGamesForm.PageControl1Change(Sender: TObject);
begin
    if (PageControl1.ActivePage = AddTeamTab) or (PageControl1.ActivePage = AddPlayerTab) then begin
        TeamLabel.Parent := PageControl1.ActivePage;
        TeamCombo.Parent := PageControl1.ActivePage;
    end;
end;

procedure TGamesForm.TeamComboChange(Sender: TObject);
begin
  AddTeamButton.Enabled := TeamCombo.Text <> '';

  if FindTeam = nil then
    AddTeamButton.Caption := 'Add this team to this game'
  else
    AddTeamButton.Caption := 'Update this team';

  AddPlayerButton.Enabled := (TeamCombo.Text <> '') and (ButtonID.Text <> '');
end;

procedure TGamesForm.AddGameButtonClick(Sender: TObject);
var i: Integer;
begin
  GameList.Force(Int(GameDate.Date) + Frac(GameTime.Time));

  GamesCombo.Items.Clear;
  for i := 0 to GameList.Count - 1 do
    GamesCombo.Items.Insert(0, DateTimeToStr(GameList[i].GameTime));

  GamesCombo.ItemIndex := GameList.Find(Int(GameDate.Date) + Frac(GameTime.Time));
end;

function TGamesForm.FindTeam: TTeam;
var thisgame: Integer;
    s: string;
begin
  thisgame := GameList.Force(Int(GameDate.Date) + Frac(GameTime.Time));

  s := TeamCombo.Text;
  if Pos(':', s) > 1 then
    s := Copy(s, 1, Pos(':', s) - 1)
  else
    s := Copy(s, 1, 4);

  Result := GameList[thisgame].Teams[GameList[thisgame].Teams.Force(StrToIntDef(s, 0))];
end;

procedure TGamesForm.AddTeamButtonClick(Sender: TObject);
var //thisgame,
    //thisteam: Integer;
    //s: string;
    team: TTeam;//Integer;
begin
//    thisgame := GameList.Force(Int(GameDate.Date) + Frac(GameTime.Time));
//
//    s := TeamCombo.Text;
//    if Pos(':', s) > 1 then
//        s := Copy(s, 1, Pos(':', s) - 1)
//    else
//        s := Copy(s, 1, 4);

//    thisteam := GameList[thisgame].Teams.Force(StrToIntDef(s, 0));
//    with GameList[thisgame].Teams[thisteam] do begin

  team := FindTeam;
  if Assigned(team) then
  with team do begin
      Colour := ColourCombo.Text;
      Score  := TeamScore.Value;
      Points := TeamPoints.Value;
  end;

  Refresh;
end;

procedure TGamesForm.AddPlayerButtonClick(Sender: TObject);
var thisgame,
    thisplayer: Integer;
begin
  thisgame := GameList.Force(Int(GameDate.Date) + Frac(GameTime.Time));
  thisplayer := GameList[thisgame].Players.Force(ButtonID.Text);

  with GameList[thisgame].Players[thisplayer] do begin
    TeamID := StrToInt(Copy(TeamCombo.Text, 1, 4));
    Pack   := PackEdit.Text;
    Score  := PlayerScore.Value;
    Rank   := PlayerRank.Value;
  end;

  Refresh;
end;

procedure TGamesForm.DeleteGameButtonClick(Sender: TObject);
var thisgame: Integer;
    i: Integer;
begin
  thisgame := GameList.Find(Int(GameDate.Date) + Frac(GameTime.Time));
  if thisgame > -1 then begin
    for i := 0 to GameList[thisgame].Teams.Count - 1 do
      GameList[thisgame].Teams[i].Free;

    for i := 0 to GameList[thisgame].Players.Count - 1 do
      GameList[thisgame].Players[i].Free;

    GameList.Delete(thisgame);
  end;

  Refresh;
end;

procedure TGamesForm.DeleteItemClick(Sender: TObject);
begin
//  if not DM.LeagueTable.EOF then DM.LeagueTable.Delete;
end;

procedure TGamesForm.HelpButtonClick(Sender: TObject);
begin
  Application.HelpCommand(HELP_CONTEXT, HelpContext);
end;

procedure TGamesForm.CloseButtonClick(Sender: TObject);
begin
  Close;
end;

// Show fields in the appropriate colour: red, green, blue, yellow.  Nice subtle pastel colours, mmm.
procedure TGamesForm.GridDrawCell(Sender: TObject; ACol, ARow: Integer;
  Rect: TRect; State: TGridDrawState);
var s: string;
begin
  with Sender as TStringGrid do begin
    if (State = []) then begin
      if Sender = TeamGrid then
        s := UpperCase(Copy(Cells[1, ARow], 1, 2))
      else if Sender = PlayerGrid then
        s := UpperCase(Copy(Cells[2, ARow], 1, 2))
      else
        s := '';

      if      s = 'RE' then Canvas.Brush.Color := $02E0E0FF
      else if s = 'GR' then Canvas.Brush.Color := $02E0FFD8
      else if s = 'BL' then Canvas.Brush.Color := $02FFE8D0
      else if s = 'YE' then Canvas.Brush.Color := $02D0FFFF;

      Canvas.FillRect(Rect);
    end;

    Canvas.TextRect(Rect, Rect.Left + 3, Rect.Top + 2, Cells[ACol, ARow]);
//        if gdFocused in State then Canvas.DrawFocusRect(Rect);
  end;
end;

procedure TGamesForm.TeamGridClick(Sender: TObject);
var row: Integer;
begin
  PageControl1.ActivePageIndex := 1;
  PageControl1Change(PageControl1);

  row := TeamGrid.Selection.Top;

  TeamCombo.Text := TeamGrid.Cells[0, row];
  ColourCombo.Text := TeamGrid.Cells[1, row];
  TeamScore.Value := StrToIntDef(TeamGrid.Cells[2, row], 0);
  TeamPoints.Value := StrToIntDef(TeamGrid.Cells[3, row], 0);
  TeamComboChange(Sender);
end;

procedure TGamesForm.PlayerGridClick(Sender: TObject);
var row: Integer;
    s: string;
begin
  PageControl1.ActivePageIndex := 2;
  PageControl1Change(PageControl1);

  row := PlayerGrid.Selection.Top;

  TeamCombo.Text := PlayerGrid.Cells[0, row];

  s := PlayerGrid.Cells[1, row];
  if Pos(':', s) > 1 then
    s := Copy(s, 1, Pos(':', s) - 1);
  ButtonID.Text := s;

  PackEdit.Text := PlayerGrid.Cells[2, row];
  PlayerScore.Value := StrToIntDef(PlayerGrid.Cells[3, row], 0);
  PlayerRank.Value := StrToIntDef(PlayerGrid.Cells[4, row], 0);
end;

end.
