unit HillClimb;

interface

type
  TGames = array of array of Integer;  // One entry for each game, one column for each team in a game.  Integers are the indexes of the three teams that play in this game.
  TReferee = array of Integer;  // One entry for each game.  Integers are the indexes of the three teams that play in this game.
  TPlays = array of array of Integer;  // square: one row for each team, one column for each team.  Integer is a count of how many times these two teams play each other.
  TDifficulties = array of real;  // one entry for each team
  TColoursCount = array of array of Integer;
  TShuffleType = (stGame, stTeam);

  THillClimber = class;

  TCalcScore = function(hill: THillClimber; games: TGames; referee: TReferee): Int64 of object;

  THillClimber = class
  private
    FNumTeams: Integer;
    FGamesPerTeamPerWeek: Integer;
    FTeamsPerGame: Integer;
    FWeeks: Integer;
    FNumGames: Integer;
    FReferees: Boolean;
    FReferee: TReferee;
    FReferee2: TReferee;
    FGames: TGames;
    FGames2: TGames;
    FPlays: TPlays;  // square array showing how many times each team plays each other team
    FColourCount: TColoursCount;
    FLastScore: Integer;
    //    FBackToBacks : string;  // list of teams that play two games in a row
    FDifficulties: TDifficulties;  // current difficulty factor for each team; the average of the ranks of the teams this team plays
//    FTargetDifficulties: TDifficulties;
//    FTimesPlayed,
    //FPlaySelf,
    //FTimeOnSite, FDifficulty: Integer;  // components of overall score

    FCalcScore: TCalcScore;

    procedure CopyGames(const gamesfrom: TGames; var gamesto: TGames; const refereefrom: TReferee; var refereeto: TReferee);
    function Dehisto(s: string): Integer;
    procedure SetupRulers(s: string);
    procedure Ruler(start, numteams, distance1, distance3: Integer);
    procedure Mark(group, numteams: Integer; const m: array of const);
    procedure Marks(n: Integer; const a: array of const; const b: array of const); overload;
    procedure Marks(n: Integer; const a: array of const; const b: array of const; const c: array of const); overload;
    procedure Marks(n: Integer; const a: array of const; const b: array of const; const c: array of const; const d: array of const); overload;
    procedure Marks(n: Integer; const a: array of const; const b: array of const; const c: array of const; const d: array of const; const e: array of const); overload;
    procedure Marks(n: Integer; const a: array of const; const b: array of const; const c: array of const; const d: array of const; const e: array of const; const f: array of const); overload;
    procedure Marks(n: Integer; const a: array of const; const b: array of const; const c: array of const; const d: array of const; const e: array of const; const f: array of const; const g: array of const); overload;
    procedure MixGames(grid: TGames);
    procedure MixTeams(grid: TGames);
    procedure MixReferee(referee: TReferee);
    procedure SetGamesPerTeamPerWeek(const Value: Integer);
    procedure SetNumTeams(const Value: Integer);
    procedure SetTeamsPerGame(const Value: Integer);
    procedure CalcNumGames;
  public
    constructor Create;

    procedure Game(gamenum, mark1, mark2, mark3: Integer);
    procedure Setup;
    procedure Go(shuffletype: TShuffleType; iterations: Integer);
    procedure CalcPlays(games: TGames);
    function ShowGames: string;
    function GamesPerWeek: Integer;

    property NumTeams: Integer read FNumTeams write SetNumTeams;
    property GamesPerTeamPerWeek: Integer read FGamesPerTeamPerWeek write SetGamesPerTeamPerWeek;
    property TeamsPerGame: Integer read FTeamsPerGame write SetTeamsPerGame;
    property Weeks: Integer read FWeeks write FWeeks;
    property NumGames: Integer read FNumGames write FNumGames;
    property Referees: Boolean read FReferees write FReferees;
    property Referee: TReferee read FReferee;
    property Games: TGames read FGames;
    property Plays: TPlays read FPlays;
    property ColourCount: TColoursCount read FColourCount;
    property LastScore: Integer read FLastScore;

    property CalcScore: TCalcScore read FCalcScore write FCalcScore;
  end;

implementation

uses SysUtils, Classes, Math;

constructor THillClimber.Create;
begin
  FNumTeams := 7;
  FGamesPerTeamPerWeek := 3;
  FTeamsPerGame := 3;
  FWeeks := 1;
  FNumGames := 7;
  FReferees := True;
end;

procedure THillClimber.CalcPlays(games: TGames);
var
  game, colour, colour2, team1, team2: Integer;
begin
  SetLength(FPlays, FNumTeams);  // set the dynamic array to have the correct number of teams
  for team1 := Low(FPlays) to High(FPlays) do begin
    SetLength(FPlays[team1], FNumTeams);  // set each row to have the correct number of teams
    for team2 := Low(FPlays[team1]) to High(FPlays[team1]) do
      FPlays[team1, team2] := 0;
  end;

  for colour := Low(FColourCount) to High(FColourCount) do
    for team1 := Low(FPlays) to High(FPlays) do
      FColourCount[colour][team1] := 0;

  for game := 0 to High(games) do begin  // for each game,
    for colour := Low(FColourCount) to High(FColourCount) do begin
      for colour2 := Low(FColourCount) to High(FColourCount) do
        if colour <> colour2 then
          Inc(FPlays[games[game, colour] - 1, games[game, colour2] - 1]);  // mark them as playing each other.

      Inc(FColourCount[colour][games[game, colour] - 1]);
    end;
  end;
end;

function THillClimber.ShowGames: string;
var
  game, team, team2: Integer;
  s: string;
  strings: TStringList;
begin
  strings := TStringList.Create;
  s := '';
  for game := 0 to High(games) do
    s := s + Format('%.02d %.02d %.02d;  ', [FGames[game, 0], FGames[game, 1], FGames[game, 2]]);

  strings.Add(s);

  for team := 1 to FNumTeams do begin
    s := '';
    for game := 0 to High(games) do  // Display fixture.
      if (FGames[game, 0] = team) and (FGames[game, 1] = team) and (FGames[game, 2] = team) then
        s := s + '3'  // This team plays on three differnt colours in this game.  (This is bad.)
      else if ((FGames[game, 0] = team) and (FGames[game, 1] = team)) or
              ((FGames[game, 1] = team) and (FGames[game, 2] = team)) or
              ((FGames[game, 0] = team) and (FGames[game, 2] = team)) then
        s := s + '2'  // This team plays on two different colours in this game.  (This is also bad.)
      else if FGames[game, 0] = team then
        s := s + 'R'  // This team plays on red in this game.
      else if FGames[game, 1] = team then
        s := s + 'B'
      else if FGames[game, 2] = team then
        s := s + 'Y'
      else
        s := s + '.';

    s := Format('%s  %4.1f %3d  %1d%1d%1d  ', [s, FDifficulties[team-1], team,
                FColourCount[0][team-1], FColourCount[1][team-1], FColourCount[0][team-1]]);  // Show difficulty, team id, and how many times this team plays each colour.

    for team2 := Low(FPlays[team-1]) to High(FPlays[team-1]) do  // Show who-plays-who.
      if (team-1 = team2) and (FPlays[team-1, team2] = 0) then
        s := s + '\'
      else
        s := s + Chr(Ord('0') + FPlays[team-1, team2]);

    strings.Add(s);
  end;

  Result := strings.CommaText;
end;

procedure THillClimber.CopyGames(const gamesfrom: TGames; var gamesto: TGames; const refereefrom: TReferee; var refereeto: TReferee);
var
  game, team: Integer;
begin
  SetLength(gamesto, Length(gamesfrom));  // set the dynamic array to have the correct number of games
  for game := Low(gamesfrom) to High(gamesfrom) do begin
    SetLength(gamesto[game], Length(gamesfrom[game]));

    for team := Low(gamesto[game]) to High(gamesto[game]) do
      gamesto[game, team] := gamesfrom[game, team];
  end;

  SetLength(refereeto, High(refereefrom) + 1);  // set the dynamic array to have the correct number of teams
  for team := Low(refereeto) to High(refereeto) do
    refereeto[team] := refereefrom[team];
end;

{$R+}
function THillClimber.Dehisto(s: string): Integer;
begin
  case s[1] of
    '0'..'9':   Result := Ord(s[1]) - Ord('0');
    'a'..'z':   Result := Ord(s[1]) - Ord('a') + 10;
    'A'..'Z':   Result := Ord(s[1]) - Ord('A') + 37;
  else          Result := -1;
  end;
end;

procedure THillClimber.SetupRulers(s: string);
var i, j: Integer;
  rulers: TStringList;
  currentgame: Integer;
  x, y{, z}: Integer;
begin
  rulers := TStringList.Create;
  try
    rulers.CommaText := s;
    currentgame := 0;
    for i := 0 to rulers.Count - 1 do begin
      x := Dehisto(rulers[i][1]);
      y := Dehisto(rulers[i][2]);
  //    z := Dehisto(rulers[i][3]);
      for j := currentgame to Min(Length(FGames), currentgame + FNumTeams) - 1 do begin
        FGames[j, 0] := j mod FNumTeams + 1;
        FGames[j, 1] := (j + x) mod FNumTeams + 1;
        FGames[j, 2] := (j + x + y) mod FNumTeams + 1;
      end;
      Inc(currentgame, FNumTeams);
    end;
  finally
    rulers.Free;
  end;
end;

procedure THillClimber.Game(gamenum, mark1, mark2, mark3: Integer);
begin
  FGames[gamenum, 0] := mark1;
  FGames[gamenum, 1] := mark2;
  FGames[gamenum, 2] := mark3;
end;

procedure THillClimber.Ruler(start, numteams, distance1, distance3: Integer);
var i: Integer;
begin
  for i := start to Min(High(FGames), (start + numteams - 1)) do
    Game(i, i mod numteams + 1, (i + distance1) mod numteams + 1, (i + distance3) mod numteams + 1);
end;

// To convert 3 distances to 3 marks:
// d1 <= d2 <= d3
// m1 = a free variable ranging from 1 to numteams; m2 = m1 + d1; m3 = m1 + d3.

// To convert n marks to n(n-1)/2 distances:
// d(i,j) = abs(m(i) - m(j)).

// In the array passed in, the first mark is missing as it is implicitly 0.
procedure THillClimber.Mark(group, numteams: Integer; const m: array of const);
var i, j: Integer;
begin
  for i := 0 to Min(High(FGames) - group * numteams, numteams - 1) do begin
    FGames[group * numteams + i, 0] := i mod numteams + 1;
    for j := 0 to High(m) do
      FGames[group * numteams + i, j + 1] := (i + m[j].VInteger + numteams) mod numteams + 1;
  end;
end;

procedure THillClimber.Marks(n: Integer; const a: array of const; const b: array of const);
begin
  Mark(0, n, a);
  Mark(1, n, b);
end;

procedure THillClimber.Marks(n: Integer; const a: array of const; const b: array of const; const c: array of const);
begin
  Mark(0, n, a);
  Mark(1, n, b);
  Mark(2, n, c);
end;

procedure THillClimber.Marks(n: Integer; const a: array of const; const b: array of const; const c: array of const; const d: array of const);
begin
  Mark(0, n, a);
  Mark(1, n, b);
  Mark(2, n, c);
  Mark(3, n, d);
end;
procedure THillClimber.Marks(n: Integer; const a: array of const; const b: array of const; const c: array of const; const d: array of const; const e: array of const);
begin
  Mark(0, n, a);
  Mark(1, n, b);
  Mark(2, n, c);
  Mark(3, n, d);
  Mark(4, n, e);
end;

procedure THillClimber.Marks(n: Integer; const a: array of const; const b: array of const; const c: array of const; const d: array of const; const e: array of const; const f: array of const);
begin
  Mark(0, n, a);
  Mark(1, n, b);
  Mark(2, n, c);
  Mark(3, n, d);
  Mark(4, n, e);
  Mark(5, n, f);
end;

procedure THillClimber.Marks(n: Integer; const a: array of const; const b: array of const; const c: array of const; const d: array of const; const e: array of const; const f: array of const; const g: array of const);
begin
  Mark(0, n, a);
  Mark(1, n, b);
  Mark(2, n, c);
  Mark(3, n, d);
  Mark(4, n, e);
  Mark(5, n, f);
  Mark(6, n, g);
end;

{$R-}
{$O-}
procedure THillClimber.MixGames(grid: TGames);
var
  game1, game2, team, temp: Integer;
begin
  game1 := Random(High(grid) + 1);
  game2 := Random(High(grid) + 1);
  while (FNumTeams div FNumTeams > 1) and (game1 = game2) do
    game2 := Random(High(grid)) + 1;

  for team := 0 to FTeamsPerGame - 1 do begin
    temp := grid[game1, team];
    grid[game1, team] := grid[game2, team];
    grid[game2, team] := temp;
  end;
end;

procedure THillClimber.MixTeams(grid: TGames);
var
  game1, game2, team1, team2, temp: Integer;
begin
  game1 := Random(High(grid) + 1);
  game2 := Random(High(grid) + 1);
  while (Length(grid) > 1) and (game1 = game2) do
    game2 := Random(High(grid) + 1);

  team1 := Random(FTeamsPerGame);
  team2 := Random(FTeamsPerGame);

  temp := grid[game1, team1];  // swap two selected teams
  grid[game1, team1] := grid[game2, team2];
  grid[game2, team2] := temp;
end;

procedure THillClimber.MixReferee(referee: TReferee);
var
  game1, game2, temp: Integer;
begin
  game1 := Random(High(referee) + 1);
  game2 := Random(High(referee) + 1);
  while (Length(referee) > 1) and (game1 = game2) do
    game2 := Random(High(referee) + 1);

  temp := referee[game1];  // swap two selected teams
  referee[game1] := referee[game2];
  referee[game2] := temp;
end;

procedure THillClimber.Setup;
var
  i: Integer;
  week, gamenum, gamespot, team, occurrence: Integer;
begin
  SetLength(FGames, FNumGames);  // set the dynamic array to have the correct number of games
  for i := Low(FGames) to High(FGames) do
    SetLength(FGames[i], FTeamsPerGame);

  if FReferees then
    SetLength(FReferee, FNumGames);

  SetLength(FColourCount, FTeamsPerGame);
  for i := Low(FColourCount) to High(FColourCount) do
    SetLength(FColourCount[i], FNumTeams);

  SetLength(FPlays, FNumTeams);
  for i := Low(FPlays) to High(FPlays) do
    SetLength(FPlays[i], FNumTeams);  // set each row to have the correct number of teams

//  if (GridMemo.Lines.Count > 0) and
//     (Length(GridMemo.Lines[GridMemo.Lines.Count - 1]) > 0) and
//     (GridMemo.Lines[GridMemo.Lines.Count - 1][1] = 'x') then
//    SetupRulers(Copy(GridMemo.Lines[GridMemo.Lines.Count - 1], 2, MaxInt))
//  else
  if FTeamsPerGame = 3 then begin
    if (FNumTeams = 3) and (Length(FGames) <= FNumTeams) then  // 5-team grid: 112, 122
      Mark(0, 3, [1, 2])
    else if (FNumTeams = 4) and (Length(FGames) <= FNumTeams) then  // 5-team grid: 112, 122
      Mark(0, 4, [1, 2])
    else if (FNumTeams = 5) and (Length(FGames) <= FNumTeams * 2) then  // 5-team grid: 112, 122
      Marks(5, [1, 2], [1, 3])
    else if (FNumTeams = 6) and (Length(FGames) <= 10) then begin // 6-team grid: 123, half 112, one sixth 222
      Marks(6, [1, 3], [1, 2]);
      if Length(FGames) > 7 then
        Game(7, 5, 6, 1);
      if Length(FGames) > 9 then
        Game(9, 2, 4, 6);
    end else if (FNumTeams = 7) and (Length(FGames) <= FNumTeams) then  // 7-team grid: 123
      Mark(0, 7, [1, 3])
    else if (FNumTeams = 7) and (Length(FGames) <= FNumTeams * 2) then  // 7-team grid: 123, 123
      Marks(7, [1, 3], [-3, -1])
    else if (FNumTeams = 8) and (Length(FGames) <= FNumTeams * 6) then  // 7-team grid: 112, 123, 123, 134, 134, 224, 233
      Marks(8, [1, 2], [1, 3], [1, 4], [2, 4], [2, 5], [1, 3], [1, 4])
  //  else if (FNumTeams <= 10) and (Length(FGames) <= FNumTeams * 3) then  // 10-team grid: 123, half 134, half 145, half 235, half 244
      //
    else if (FNumTeams <= 11) and (FNumTeams > 5) and (Length(FGames) <= FNumTeams * 5) then  // 11-team grid: 123, 134, 145, 235, 245
      Marks(FNumTeams, [1, 3], [-1, 3], [2, 5], [1, 5], [2, -5])
    else if (FNumTeams = 12) and (Length(FGames) <= 28) then   // imperfect 12-team grid: 123, 156, 4
      Marks(12, [1, 3], [1, 6], [4, 8])
    else if (FNumTeams = 12) and (Length(FGames) <= 44) then  // 12-team grid: 123, 156, 235, 44
      Marks(12, [1, 3], [1, 6], [2, 5], [4, 8])
    else if (FNumTeams <= 13) and (FNumTeams > 6) and (Length(FGames) <= FNumTeams * 2) then  // 13-team grid: 134, 256
      Marks(FNumTeams, [1, 4], [2, -6])
    else if (FNumTeams <= 13) and (Length(FGames) <= FNumTeams * 4) then  // 13-team grid: 134, 156, 235, 246
      Marks(FNumTeams, [1, 4], [1, 6], [2, 5], [2, 6])
    else if (FNumTeams <= 19) and (Length(FGames) <= FNumTeams * 3) then  // 19-team grid: 134, 279, 568
      Marks(FNumTeams, [1, 4], [2, 9], [5, 11])
    else if (FNumTeams <= 19) and (Length(FGames) <= FNumTeams * 5) then  // 19-team grid: 112, 235, 347, 469, 568,
      Marks(FNumTeams, [1, 2], [2, 5], [3, 7], [4, 10], [5, 11])
    else if (FNumTeams <= 20) and (Length(FGames) <= FNumTeams * 4) then  // 20-team grid: 112, 257, 389, 46a
      Marks(FNumTeams, [1, 2], [2, 7], [3, 11], [4, 10])
    else if (FNumTeams <= 20) and (Length(FGames) <= FNumTeams * 5) then  // 20-team grid: 112, 268, 347, 369, 55a
      Marks(FNumTeams, [1, 2], [2, 8], [3, 7], [3, 9], [5, 10])
    else if (FNumTeams = 21) and (Length(FGames) = 70) then  // 21-team grid: 145, 28a, 369, 7.
      Marks(FNumTeams, [1, 5], [2, 10], [3, 9], [7, 14])
    else

    if (FNumTeams = 24) and (Length(FGames) = 88) then begin  // 24-team grid: 1ab, 369, 57c, 48.  Note that we're missing a '2' and the 'c' is effectively doubled because this is an even number.  Only solution is to add more rulers.
      Marks(FNumTeams, [1, 11], [3, 9], [5, 12]);
      for i := 72 to 75 do  // 448
        Game(i, i mod 24 + 1, (i + 4) mod 24 + 1, (i + 8) mod 24 + 1);
      for i := 76 to 79 do  // 448
        Game(i, (i + 4) mod 24 + 1, (i + 8) mod 24 + 1, (i + 12) mod 24 + 1);
      for i := 80 to 83 do  // 448
        Game(i, (i + 8) mod 24 + 1, (i + 12) mod 24 + 1, (i + 16) mod 24 + 1);
      for i := 84 to 87 do  // 8
        Game(i, i mod 24 + 1, (i + 8) mod 24 + 1, (i + 16) mod 24 + 1);
    end

    else if (FNumTeams <= 24) and (Length(FGames) <= FNumTeams * 4) then  // 24-team grid: 123, 48c, 59a, 67b.
      Marks(FNumTeams, [1, 3], [4, 12], [5, 14], [6, 13])
    else if (FNumTeams = 25) and (Length(FGames) <= FNumTeams * 4) then  // 25-team grid: 123, 47b, 58c, 69a.
      Marks(FNumTeams, [1, 3], [4, 11], [5, 13], [6, 15])
    else if (FNumTeams = 26) and (Length(FGames) <= 130) then  // 26-team grid: 167, 2bd, 39c, 448, 55a.
      Marks(FNumTeams, [1, 7], [2, 13], [3, 12], [4, 8], [5, 10])
    else if (FNumTeams = 27) and (Length(FGames) <= 117) then  // 27-team grid: 123, 47b, 5ac, 68d, 9.
      Marks(FNumTeams, [1, 3], [4, 11], [5, 15], [6, 14], [9, 18])
    else if (FNumTeams = 28) and (Length(FGames) <= 140) then  // 28-team grid: 112, 369, 4ae, 5bc, 78d.
      Marks(FNumTeams, [1, 2], [3, 9], [4, 14], [5, 16], [7, 15])
    else if (FNumTeams = 29) and (Length(FGames) <= 145) then  // 29-team grid: 1ee, 235, 47b, 6ad, 89c.
      Marks(FNumTeams, [1, 15], [2, 5], [4, 11], [6, 16], [8, 17])
    else if (FNumTeams = 30) and (Length(FGames) <= 150) then  // 30-team grid: 123, 48c, 5be, 69f, 7ad.
      Marks(FNumTeams, [1, 3], [4, 12], [5, 16], [6, 15], [7, 17])
    else if (FNumTeams <= 31) and (FNumTeams > 13) and (Length(FGames) <= FNumTeams * 5) then  // 31-team grid: 123, 47b, 5af, 6cd, 89e.
      Marks(FNumTeams, [1, 3], [4, 11], [5, 15], [6, -13], [8, 17])
    else if (FNumTeams <= 32) and (FNumTeams > 15) and (Length(FGames) <= FNumTeams * 6) then  // 32-team grid: 123, 459, 6bf, 7cd, 8ae, 88g.
      Marks(FNumTeams, [1, 3], [4, 9], [6, -15], [7, -13], [8, -14], [8, 16])
    else if (FNumTeams <= 33) and (FNumTeams > 15) and (Length(FGames) <= FNumTeams * 5.334) then  // 33-team grid: 123, 46a, 5df, 7ce, 89g, b.
      Marks(FNumTeams, [1, 3], [6, 10], [5, -15], [7, -14], [8, 17], [11, 22])

    else
      for i := Low(FGames) to High(FGames) do  // list every team "gamesperteam" times
        Game(i, (i * 3) mod FNumTeams + 1, (i * 3 + 1) mod FNumTeams + 1, (i * 3 + 2) mod FNumTeams + 1)
  end else
  begin
    gamenum := 0;
    gamespot := 0;
    for week := 1 to FWeeks do
      for occurrence := 0 to FGamesPerTeamPerWeek - 1 do
        for team := 1 to FNumTeams do begin
          if gamenum < Length(FGames) then
            FGames[gamenum, gamespot] := team;
          Inc(gamespot);
          if gamespot = FTeamsPerGame then begin
            gamespot := 0;
            Inc(gamenum);
          end;
        end;

    team := 1;
    while gamenum < FNumGames do begin
      FGames[gamenum, gamespot] := team;
      Inc(team);
      if team > FNumTeams then team := 1;

      Inc(gamespot);
      if gamespot = FTeamsPerGame then begin
        gamespot := 0;
        Inc(gamenum);
      end;
    end;
  end;

//      for i := Low(FGames) to High(FGames) do  // list every team "gamesperteam" times
//        for j := 0 to FTeamsPerGame - 1 do
//          FGames[i, j] := (i * FTeamsPerGame + j) mod FNumTeams + 1;

  if FReferees then
    for i := Low(FReferee) to High(FReferee) do
      FReferee[i] := (i mod FNumTeams) + 1;

  CalcPlays(FGames);
end;

// Shuffle teams or games, to reduce overall score.
procedure THillClimber.Go(shuffletype: TShuffleType; iterations: Integer);
var
  i, j: Integer;
  score, bestscore: Int64;
  sincematch: Int64;  // number of shuffles we've made since last successful match
  shuffle: Integer;
begin
  CalcPlays(FGames);
  bestscore := FCalcScore(Self, FGames, FReferee);
  sincematch := 0;

  for j := 1 to iterations do begin
    CopyGames(FGames, FGames2, FReferee, FReferee2);
    shuffle := Max(100 - sincematch, sincematch mod 2 + 1);
    for i := 1 to shuffle do begin
      if shuffletype = stGame then
        MixGames(FGames2)
      else
        MixTeams(FGames2);
      if FReferees then
        MixReferee(FReferee2);
    end;

    CalcPlays(FGames2);
    score := FCalcScore(Self, FGames2, FReferee2);
    Inc(sincematch);
    if bestscore >= score then begin
      CopyGames(FGames2, FGames, FReferee2, FReferee);  // it's at least as good as the one we had, so save it
      if bestscore > score then begin  // it's better than the one we had, so show it
        bestscore := score;
        sincematch := 0;
        FLastScore := score;
        if score = 0 then
          Break;
      end;
    end;
  end;  // for iterations
end;

procedure THillClimber.SetGamesPerTeamPerWeek(const Value: Integer);
begin
  FGamesPerTeamPerWeek := Value;
  CalcNumGames;
end;

procedure THillClimber.SetNumTeams(const Value: Integer);
begin
  FNumTeams := Value;
  CalcNumGames;
end;

procedure THillClimber.SetTeamsPerGame(const Value: Integer);
begin
  FTeamsPerGame := Value;
  CalcNumGames;
end;

procedure THillClimber.CalcNumGames;
begin
  FNumGames := (FNumTeams * FGamesPerTeamPerWeek * FWeeks + (FTeamsPerGame - 1)) div FTeamsPerGame;
end;

function THillClimber.GamesPerWeek: Integer;
begin
  Result := FNumGames div FWeeks;
end;

end.

