unit LeagueDialog;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, IniFiles, ImgList, Grids,
  Spin, Math, Menus, ElTree, Buttons, League, Games, Utility;

const
// ColumnText subitems:
    ctHandicap = 0;
    ctButtonID = 1;
    ctComment  = 2;

type
  TLeagueForm = class(TForm)
    ImageList1: TImageList;
    Splitter1: TSplitter;
    LeagueGroupBox: TGroupBox;
    CopyTeamsFrom: TBitBtn;
    TeamGroupBox: TGroupBox;
    AddTeam: TButton;
    DeleteTeam: TButton;
    RenameTeam: TButton;
    PlayerGroupBox: TGroupBox;
    AddPlayer: TButton;
    DeletePlayer: TButton;
    LeagueStuff: TGroupBox;
    Panel1: TPanel;
    Panel2: TPanel;
    TeamStats: TGroupBox;
    Label11: TLabel;
    TeamHandicap: TEdit;
    Label12: TLabel;
    TeamComment: TEdit;
    TeamTotalScore: TLabel;
    TeamTotalPoints: TLabel;
    TeamGameCount: TLabel;
    PlayerStats: TGroupBox;
    LabelHandicap: TLabel;
    PlayerHandicap: TEdit;
    Label10: TLabel;
    PlayerComment: TEdit;
    PlayerAverageScore: TLabel;
    PlayerAverageRank: TLabel;
    PlayerGameCount: TLabel;
    OKButton: TButton;
    CancelButton: TButton;
    PopupMenu1: TPopupMenu;
    DeleteItem: TMenuItem;
    OpenDialog1: TOpenDialog;
    PointsPanel: TPanel;
    Label00: TLabel;
    RankOnScore: TRadioButton;
    RankOnPoints: TRadioButton;
    HighScorePanel: TPanel;
    LabelLow: TLabel;
    LabelHigh: TLabel;
    PointsHigh: TSpinEdit;
    LabelProportional: TLabel;
    PointsProportional: TSpinEdit;
    ScrollBox1: TScrollBox;
    TeamStatsGrid: TStringGrid;
    PlayerStatsGrid: TStringGrid;
    RenamePlayer: TButton;
    TreeView1: TElTree;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure TreeView1ItemCollapse(Sender: TObject; Item: TElTreeItem);
    procedure TreeView1ItemExpand(Sender: TObject; Item: TElTreeItem);
    procedure TreeView1AfterSelectionChange(Sender: TObject);
    procedure TreeView1TryEdit(Sender: TObject; Item: TElTreeItem;
      Section: TElHeaderSection; var CellType: TFieldTypes; var CanEdit: Boolean);
    procedure DoTeamStats;
    procedure DoPlayerStats;
    procedure AddTeamClick(Sender: TObject);
    procedure DeleteTeamClick(Sender: TObject);
    procedure RenameTeamClick(Sender: TObject);
    procedure HandicapExit(Sender: TObject);
    procedure EditKeyPress(Sender: TObject; var Key: Char);
    procedure AddPlayerClick(Sender: TObject);
    procedure DeletePlayerClick(Sender: TObject);
    procedure OKButtonClick(Sender: TObject);
    procedure RankOnClick(Sender: TObject);
    procedure DeleteItemClick(Sender: TObject);
    procedure CommentExit(Sender: TObject);
    procedure CopyTeamsFromClick(Sender: TObject);
    procedure PointsNNChange(Sender: TObject);
    procedure StatsGridDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure LabelHandicapDblClick(Sender: TObject);
    procedure RenamePlayerClick(Sender: TObject);
  private
    function  CreateNewPointsEdit(Index, ATop: Integer): TSpinEdit;
    function  NthSuffix(i: Integer): string;
  public
    constructor CreateLeague(LL: TLeagueList; GL: TGameList; var Prefs: TPrefs; Name, HD: string; Owner: TComponent);
  end;

var LeagueForm:     TLeagueForm;
    LeagueList:     TLeagueList;
    GameList:       TGameList;
    HandiDefault:   string;
    MyPrefs:        ^TPrefs;

implementation

uses TornMain, AliasDialog, AliasDBDialog, AliasLaserforceDialog;

{$R *.DFM}

function StrDef(sl: TStringList; i: Integer): string;
begin
    if sl.Count > i then
        Result := sl[i]
    else
        Result := '';
end;

// Call the inherited Create, and set up league data in the treeview.
constructor TLeagueForm.CreateLeague(LL: TLeagueList; GL: TGameList; var Prefs: TPrefs; Name, HD: string; Owner: TComponent);
var i, j: integer;
    TopNode, TeamNode, PlayerNode: TElTreeItem;
    thisedit: TSpinEdit;
begin
    LeagueForm := Create(Owner);
    LeagueList := LL;
    GameList := GL;
    HandiDefault := HD;
    MyPrefs := @Prefs;  /// TODO XXX: this is wrong. need a clone, not an assign.

    Caption := 'TORN -- ' + ChangeFileExt(ExtractFileName(Name), '');
    TreeView1.Items.Clear;
    TopNode := TreeView1.Items.Add(nil, ChangeFileExt(ExtractFileName(Name), ''));
    TopNode.ImageIndex := 0;

    for i := 0 to LeagueList.Count - 1 do begin
        if LeagueList[i].TeamName = '' then begin
            if LeagueList[i].Players.Count = 0 then
                TeamNode := TreeView1.Items.AddChild(TopNode, 'Team ' + IntToStr(LeagueList[i].TeamID))
            else
                TeamNode := TreeView1.Items.AddChild(TopNode, LeagueList[i].Players[0].Name + '''s team');
        end else
            TeamNode := TreeView1.Items.AddChild(TopNode, LeagueList[i].TeamName);
        TeamNode.ImageIndex := 1;
        TeamNode.AllowEdit := True;
        TeamNode.Data := LeagueList[i];
        TeamNode.ColumnText.Add(LeagueList[i].Handicap);
        TeamNode.ColumnText.Add('');  // ButtonID
        TeamNode.ColumnText.Add(LeagueList[i].Comment);

        for j := 0 to LeagueList[i].Players.Count - 1 do begin
            PlayerNode := TreeView1.Items.AddChild(TeamNode, LeagueList[i].Players[j].Name);
            PlayerNode.ImageIndex := 3;
            PlayerNode.Data := LeagueList[i];
            PlayerNode.ColumnText.Add(LeagueList[i].Players[j].Handicap);
            PlayerNode.ColumnText.Add(LeagueList[i].Players[j].ButtonID);
            PlayerNode.ColumnText.Add(LeagueList[i].Players[j].Comment);
        end;
    end;

    TopNode.Expanded := True;

    thisedit := nil;
    for i := 0 to Max(Prefs.LeaguePoints.Count - 1, 2) do begin
        thisedit := CreateNewPointsEdit(i + 1, i * 24 + 54);
        if i < Prefs.LeaguePoints.Count then
            thisedit.Value := Round(Prefs.LeaguePoints[i]);
    end;
    thisedit.OnChange := PointsNNChange;

    RankOnPoints.Checked := IsPoints(Prefs);

    PointsHigh.Value := Round(Prefs.LeaguePointsHigh);
    PointsProportional.Value := Round(Prefs.LeaguePointsProportional);

    if (PointsHigh.Value <> 0) then RankOnPoints.Checked := True;
    RankOnClick(Nil);
end;

procedure TLeagueForm.FormCreate(Sender: TObject);
begin
    Top  := 0;//Round(Screen.Height * 0.10);
    Left := (Screen.Width - Width - 247) div 2;
    if Left < 0 then Left := 0;
end;

procedure TLeagueForm.FormShow(Sender: TObject);
begin
    if TreeView1.Items.Count > 0 then begin
        TreeView1.Selected := TreeView1.Items.Item[0];
        TreeView1.Items.Item[0].Selected := True;
    end;
end;

// If it's a folder icon, close it.
procedure TLeagueForm.TreeView1ItemCollapse(Sender: TObject;
  Item: TElTreeItem);
begin
    if Item.ImageIndex = 2 then Item.ImageIndex := 1;
end;

// If it's a folder icon, open it.
procedure TLeagueForm.TreeView1ItemExpand(Sender: TObject; Item: TElTreeItem);
begin
    if Item.ImageIndex = 1 then Item.ImageIndex := 2;
end;

// A new node has been selected.  Enable and disable buttons accordingly.
procedure TLeagueForm.TreeView1AfterSelectionChange(Sender: TObject);
begin
    if TreeView1.Selected = nil then Exit;

    // States:
    //   0:   League header icon
    //   1,2: Team folder icons (closed, open)
    //   3:   Player laser icon

    with TreeView1.Selected do begin
        AddTeam.Enabled     := ImageIndex in [0,1,2];
        DeleteTeam.Enabled  := ImageIndex in [1,2];
        RenameTeam.Enabled  := ImageIndex in [1,2];

        AddPlayer.Enabled   := ImageIndex in [1,2,3];
        DeletePlayer.Enabled:= ImageIndex in [3];
        RenamePlayer.Enabled:= ImageIndex in [3];

        LeagueStuff.Visible := ImageIndex in [0];
        TeamStats.Visible   := ImageIndex in [1,2];
        PlayerStats.Visible := ImageIndex in [3];

        if ImageIndex in [1,2] then DoTeamStats;
        if ImageIndex in [3]   then DoPlayerStats;
    end;
end;

// Disallow editing of Players by just single-clicking twice on the field.
procedure TLeagueForm.TreeView1TryEdit(Sender: TObject; Item: TElTreeItem;
  Section: TElHeaderSection; var CellType: TFieldTypes; var CanEdit: Boolean);
begin
    CanEdit := Item.ImageIndex in [1,2];
end;

// Show statistics for this team in StatsGrid.
procedure TLeagueForm.DoTeamStats;
var i, j: Integer;
    dateformat: string;
    Count, Score: Int64;
    Points: Double;
begin
    if TreeView1.Selected.Data = nil then Exit;

    TeamHandicap.Text := TreeView1.Selected.ColumnText[ctHandicap];
    TeamComment.Text  := TreeView1.Selected.ColumnText[ctComment];

    TeamStats.Caption := 'Stats for ' + TLeagueTeam(TreeView1.Selected.Data).TeamName;

    dateformat := StringReplace(ShortDateFormat, DateSeparator + 'yy', '', []);
    dateformat := StringReplace(dateformat, 'yy' + DateSeparator, '', []);
    dateformat := StringReplace(dateformat, 'yy', '', []);

    Count := 0;
    Score := 0;
    Points := 0;

    TeamStatsGrid.RowCount := 2;
    TeamStatsGrid.Cells[0, 0] := 'Game';
    TeamStatsGrid.Cells[1, 0] := 'Score';
    TeamStatsGrid.Cells[2, 0] := 'Points';

    for i := 0 to GameList.Count - 1 do
        for j := 0 to GameList[i].Teams.Count - 1 do
            if GameList[i].Teams[j].TeamID = TLeagueTeam(TreeView1.Selected.Data).TeamID then begin
                Inc(Count);
                Inc(Score,  GameList[i].Teams[j].Score);
                Points := Points + GameList[i].Teams[j].Points;

                TeamStatsGrid.Cells[0, TeamStatsGrid.RowCount - 1] := FormatDateTime(dateformat + ' ' + ShortTimeFormat, GameList[i].GameTime);
                TeamStatsGrid.Cells[1, TeamStatsGrid.RowCount - 1] := IntToStr(GameList[i].Teams[j].Score);
                TeamStatsGrid.Cells[2, TeamStatsGrid.RowCount - 1] := FloatToStr(GameList[i].Teams[j].Points);
                TeamStatsGrid.Cells[3, TeamStatsGrid.RowCount - 1] := GameList[i].Teams[j].Colour;
                TeamStatsGrid.RowCount := TeamStatsGrid.RowCount + 1;
            end;

    TeamGameCount.Caption   := 'Total ' + IntToStr(Count) + ' games';
    TeamTotalScore.Caption  := IntToStr(Score);
    TeamTotalPoints.Caption := FloatToStr(Points);

    TeamStatsGrid.RowCount := TeamStatsGrid.RowCount - 1;
end;

// Show statistics for this player in StatsGrid.
procedure TLeagueForm.DoPlayerStats;

    function Ratio(numerator, denominator: Integer): string;
    begin
        if denominator = 0 then
            if numerator = 0 then
                Result := 'undefined'
            else
                Result := 'infinite'
        else
            Result := Format('%3.1f%%', [100.0 * numerator / denominator]);
    end;

var i, j: Integer;
    dateformat: string;
    Count, Score, Rank: Integer;
    Team: TLeagueTeam;
begin
    PlayerStats.Caption := 'Stats for ' + TreeView1.Selected.Text;

    PlayerHandicap.Text := TreeView1.Selected.ColumnText[ctHandicap];
    PlayerComment.Text  := TreeView1.Selected.ColumnText[ctComment];

    dateformat := StringReplace(ShortDateFormat, DateSeparator + 'yy', '', []);
    dateformat := StringReplace(dateformat, 'yy' + DateSeparator, '', []);
    dateformat := StringReplace(dateformat, 'yy', '', []);

    Count := 0;
    Score := 0;
    Rank := 0;

    PlayerStatsGrid.RowCount := 2;
    PlayerStatsGrid.Cells[0, 0] := 'Game';
    PlayerStatsGrid.Cells[1, 0] := 'Team';
    PlayerStatsGrid.Cells[2, 0] := 'Score';
    PlayerStatsGrid.Cells[3, 0] := 'Rank';
    PlayerStatsGrid.Cells[4, 0] := 'Tag Ratio';
    PlayerStatsGrid.Cells[5, 0] := 'Effectiveness';
    PlayerStatsGrid.Cells[6, 0] := 'Colour';

    for i := 0 to GameList.Count - 1 do
        for j := 0 to GameList[i].Players.Count - 1 do
            if GameList[i].Players[j].PlayerID = TreeView1.Selected.ColumnText[ctButtonID] then begin
                Inc(Count);
                Inc(Score, GameList[i].Players[j].Score);
                Inc(Rank,  GameList[i].Players[j].Rank);

                PlayerStatsGrid.Cells[0, PlayerStatsGrid.RowCount - 1] := FormatDateTime(dateformat + ' ' + ShortTimeFormat, GameList[i].GameTime);
                Team := LeagueList.FindTeamID(GameList[i].Players[j].TeamID);
                if Assigned(Team) then
                    PlayerStatsGrid.Cells[1, PlayerStatsGrid.RowCount - 1] := Team.TeamName
                else
                    PlayerStatsGrid.Cells[1, PlayerStatsGrid.RowCount - 1] := 'Team ' + IntToStr(GameList[i].Players[j].TeamID);
                PlayerStatsGrid.Cells[2, PlayerStatsGrid.RowCount - 1] := IntToStr(GameList[i].Players[j].Score);
                PlayerStatsGrid.Cells[3, PlayerStatsGrid.RowCount - 1] := IntToStr(GameList[i].Players[j].Rank);
                PlayerStatsGrid.Cells[4, PlayerStatsGrid.RowCount - 1] := Ratio(GameList[i].Players[j].HitsBy, GameList[i].Players[j].HitsOn);
                PlayerStatsGrid.Cells[5, PlayerStatsGrid.RowCount - 1] := Ratio(GameList[i].Players[j].HitsBy, GameList[i].Hits);
                PlayerStatsGrid.Cells[6, PlayerStatsGrid.RowCount - 1] := GameList[i].Players[j].Pack;
                PlayerStatsGrid.RowCount := PlayerStatsGrid.RowCount + 1;
            end;

    PlayerStatsGrid.RowCount := PlayerStatsGrid.RowCount - 1;

    PlayerGameCount.Caption := 'Total ' + IntToStr(Count) + ' games';
    if Count = 0 then begin
        PlayerAverageScore.Caption := '';
        PlayerAverageRank.Caption  := '';
    end else begin
        PlayerAverageScore.Caption := IntToStr(Score div Count);
        PlayerAverageRank.Caption  := Format('%f', [Rank / Count]);
    end;
end;

procedure TLeagueForm.AddTeamClick(Sender: TObject);
var TeamNode: TElTreeItem;
begin
    TreeView1.Items[0].Expanded := True;

    TeamNode := TreeView1.Items.AddChild(TreeView1.Items[0], 'New team');
    TeamNode.ImageIndex := 1;
    TeamNode.AllowEdit := True;
    TeamNode.ColumnText.Add('');  // Handicap
    TeamNode.ColumnText.Add('');  // ButtonID
    TeamNode.ColumnText.Add('');  // Comment

    TreeView1.Selected := TeamNode;
    TreeView1.SetFocus;
    TreeView1AfterSelectionChange(Sender);
    TreeView1.Selected.EditText;

    TeamHandicap.Text := '';
    TeamComment.Text  := '';
end;

procedure TLeagueForm.DeleteTeamClick(Sender: TObject);
begin
    if TreeView1.Selected = nil then Exit;

    TreeView1.Selected.Delete;
end;

procedure TLeagueForm.RenameTeamClick(Sender: TObject);
begin
    if TreeView1.Selected = nil then Exit;

    TreeView1.SetFocus;
    TreeView1.Selected.EditText;
end;

// Check as focus leaves a Handicap field.  Make sure there is a '+', '-' or '%',
// and remove leading zeroes.
procedure TLeagueForm.HandicapExit(Sender: TObject);
var s: String;
    val:     Integer;
    plus:    Boolean;
    minus:   Boolean;
    percent: Boolean;
begin
    with Sender as TEdit do begin
        s := Text;
        plus    := (Length(s) > 0) and (s[1] = '+');
        minus   := (Length(s) > 0) and (s[1] = '-');
        percent := (Length(s) > 0) and (s[Length(s)] = '%');

        if plus or minus then s:= Copy(s, 2, 7);

        if Length(s) = 0 then val := 0
        else if percent then  val := StrToIntDef(Copy(s, 1, Length(s)-1), 100)
        else                  val := StrToIntDef(s, 0);

        if percent then                 Text := IntToStr(val) + '%'
        else if val = 0 then            Text := '0'
        else if minus then              Text := '-' + IntToStr(val)
        else if plus then               Text := '+' + IntToStr(val)
        else if HandiDefault = '+' then Text := '+' + IntToStr(val)
        else if HandiDefault = '-' then Text := '-' + IntToStr(val)
        else { must be '%' }            Text := IntToStr(val) + '%';

        TreeView1.Selected.ColumnText[ctHandicap] := Text;
    end;  // with Sender
end;

procedure TLeagueForm.CommentExit(Sender: TObject);
begin
    TreeView1.Selected.ColumnText[ctComment] := (Sender as TEdit).Text;
end;

// Cause a call to HandicapExit when the user presses Enter.
procedure TLeagueForm.EditKeyPress(Sender: TObject; var Key: Char);
begin
    if Key = #13 then begin
        TreeView1.SetFocus;
        Key := #0;
    end;
end;

procedure TLeagueForm.AddPlayerClick(Sender: TObject);
var TeamNode, PlayerNode: TElTreeItem;
    Alias: TAlias;
begin
    TeamNode := TreeView1.Selected;
    while not (TeamNode = nil) and not (TeamNode.ImageIndex in [1,2]) do
        TeamNode := TeamNode.Parent;  // make sure we're at a Team node
    if TeamNode = nil then Exit;

    if TornForm.SystemType = stLaserforce then
        Alias := TAliasLaserforce.Create(Application)
    else
        Alias := TAliasDB.Create(Application);

    try
        Alias.Alias := '';
        Alias.Left := Self.Left + Self.Width - Alias.Width - 11;
        Alias.Top  := Self.Top;
        Alias.Height := Self.Height;
        if Alias.ShowModal = mrOK then begin
            PlayerNode := TreeView1.Items.AddChild(TeamNode, Alias.PlayerAlias.Text);
            PlayerNode.ImageIndex := 3;
            PlayerNode.ColumnText.Add('');  // Handicap
            PlayerNode.ColumnText.Add(Alias.ID);  // ButtonID
            PlayerNode.ColumnText.Add('');  // Comment

            TeamNode.Expanded := True;
            TreeView1.Selected := PlayerNode;

            PlayerHandicap.Text := '';
            PlayerComment.Text  := '';
        end;
    finally
        Alias.Release;
    end;
end;

procedure TLeagueForm.DeletePlayerClick(Sender: TObject);
begin
    if TreeView1.Selected = nil then Exit;

    TreeView1.Selected.Delete;
    TreeView1.SetFocus;
end;

procedure TLeagueForm.RenamePlayerClick(Sender: TObject);
var TeamNode, PlayerNode: TElTreeItem;
    Alias: TAlias;
    oldid: string;
    i, j: Integer;
begin
    PlayerNode := TreeView1.Selected;
    TeamNode := PlayerNode.Parent;
    while not (TeamNode = nil) and not (TeamNode.ImageIndex in [1,2]) do
        TeamNode := TeamNode.Parent;  // make sure we're at a Team node
    if TeamNode = nil then Exit;

    oldid := PlayerNode.ColumnText[1];

    if TornForm.SystemType = stLaserforce then
        Alias := TAliasLaserforce.Create(Application)
    else
        Alias := TAliasDB.Create(Application);

    try
        Alias.Caption := 'Re-ID ' + PlayerNode.Text + ' as...';
        Alias.ID := PlayerNode.ColumnText[1];
        Alias.Alias := PlayerNode.Text;
        Alias.Left := Self.Left + Self.Width - Alias.Width - 11;
        Alias.Top  := Self.Top;
        Alias.Height := Self.Height;
        if Alias.ShowModal = mrOK then begin
            PlayerNode.Text := Alias.Alias;
            PlayerNode.ColumnText[1] := Alias.ID;
        end;
    finally
        Alias.Release;
    end;

    for i := 0 to GameList.Count - 1 do
        for j := 0 to GameList[i].Players.Count - 1 do
            if GameList[i].Players[j].PlayerID = oldid then
                GameList[i].Players[j].PlayerID := PlayerNode.ColumnText[1];
end;

// The user has clicked the OK button, to save his changes and close the form.
// Recreate the LeagueList based on the information in the TreeView.
// (Form closing is handled automagically because OKButton's ModalResult is set.)
procedure TLeagueForm.OKButtonClick(Sender: TObject);
var i: Integer;
    LeagueTeam: TLeagueTeam;
    League2: TLeagueList;
    highteamID: integer;
begin
    highteamID := 1;
    for i := 0 to LeagueForm.TreeView1.Items.Count - 1 do  // Find the highest teamID, and set highteamID one higher.
        if (TreeView1.Items[i].ImageIndex in [1,2]) and  // this is a team title
           (TreeView1.Items[i].Data <> nil) and          // this team has an allocated teamID
           (highteamID <= TLeagueTeam(TreeView1.Items[i].Data).TeamID) then
            highteamID := TLeagueTeam(TreeView1.Items[i].Data).TeamID + 1;

    League2 := TLeagueList.Create;
    LeagueTeam := nil;

    for i := 0 to TreeView1.Items.Count - 1 do
        with TreeView1.Items[i] do begin
            if ImageIndex in [1,2] then begin  // this is a team title,
                LeagueTeam := TLeagueTeam.Create;  // so set up an entry for this team in the List.
                League2.Add(LeagueTeam);
                LeagueTeam.TeamName := Text;
                if Data = nil then begin
                    LeagueTeam.TeamID := highteamID;
                    Inc(highteamID);
                end else
                    LeagueTeam.TeamID := TLeagueTeam(Data).TeamID;
                LeagueTeam.Handicap := ColumnText[ctHandicap];
                LeagueTeam.Comment  := ColumnText[ctComment];
            end else if ImageIndex = 3 then  // this is a player
                with LeagueTeam.Players.Add(TLeaguePlayer.Create) do begin  // Note that we are inheriting LeagueTeam from a previous iteration of the loop.
                    Name     := Text;
                    ButtonID := ColumnText[ctButtonID];
                    Handicap := ColumnText[ctHandicap];
                    Comment  := ColumnText[ctComment];
                end;
        end;  // with TreeView1, for i

    for i := LeagueList.Count - 1 downto 0 do begin  // Now throw away the old list,
        LeagueList[i].Free;
        LeagueList.Delete(i);
    end;

    for i := 0 to League2.Count - 1 do      // copy in records fom the new one,
        LeagueList.Add(League2[i]);

    for i := League2.Count - 1 downto 0 do  // and throw away originals.
        League2.Delete(i);

    MyPrefs^.LeaguePoints.Clear;
    for i := 0 to ComponentCount - 1 do
        if (Components[i].Tag <> 0) and (Components[i] is TSpinEdit) then begin
            MyPrefs^.LeaguePoints.Add((Components[i] as TSpinEdit).Value);  // This code assumes that the PointsNN spinedits are in the Components array in increasing order.
            if (Components[i] as TSpinEdit).Value = 0 then Break;
        end;
    MyPrefs^.LeaguePointsHigh := PointsHigh.Value;
    MyPrefs^.LeaguePointsProportional := PointsProportional.Value;
end;

// Enable / disable controls
procedure TLeagueForm.RankOnClick(Sender: TObject);
var i: Integer;
begin
    for i := 0 to ComponentCount - 1 do
        if Components[i].Tag <> 0 then
            (Components[i] as TControl).Enabled := RankOnPoints.Checked;

    LabelLow.Enabled           := RankOnPoints.Checked;
    LabelHigh.Enabled          := RankOnPoints.Checked;
    PointsHigh.Enabled         := RankOnPoints.Checked;
    PointsProportional.Enabled := RankOnPoints.Checked;

    if RankOnScore.Checked then begin
    for i := 0 to ComponentCount - 1 do
        if (Components[i].Tag <> 0) and (Components[i] is TSpinEdit) then
            (Components[i] as TSpinEdit).Value := 0;
        PointsHigh.Value := 0;
        PointsProportional.Value := 0;
    end;
end;

procedure TLeagueForm.DeleteItemClick(Sender: TObject);
begin
//    if not DM.LeagueTable.EOF then DM.LeagueTable.Delete;
end;

procedure TLeagueForm.CopyTeamsFromClick(Sender: TObject);
var i, j: Integer;
    F: TextFile;
    LeagueList: TLeagueList;
    TopNode, TeamNode, PlayerNode: TElTreeItem;
begin
    if not OpenDialog1.Execute then Exit;
    AssignFile(F, OpenDialog1.FileName);
    Reset(F);
    try
        LeagueList := TLeagueList.Create;
        ReadLeagueListAsXml(F, LeagueList);

        TopNode := TreeView1.Items.GetFirstNode;

        for i := 0 to LeagueList.Count - 1 do begin
            if LeagueList[i].TeamName = '' then begin
                if LeagueList[i].Players.Count = 0 then
                    TeamNode := TreeView1.Items.AddChild(TopNode, 'Team ' + IntToStr(LeagueList[i].TeamID))
                else
                    TeamNode := TreeView1.Items.AddChild(TopNode, LeagueList[i].Players[0].Name + '''s team');
            end else
                TeamNode := TreeView1.Items.AddChild(TopNode, LeagueList[i].TeamName);
            TeamNode.ImageIndex := 1;
            TeamNode.AllowEdit := True;
            TeamNode.Data := LeagueList[i];
            TeamNode.ColumnText.Add(LeagueList[i].Handicap);
            TeamNode.ColumnText.Add('');  // ButtonID
            TeamNode.ColumnText.Add(LeagueList[i].Comment);

            for j := 0 to LeagueList[i].Players.Count - 1 do begin
                PlayerNode := TreeView1.Items.AddChild(TeamNode, LeagueList[i].Players[j].Name);
                PlayerNode.ImageIndex := 3;
                PlayerNode.Data := LeagueList[i];
                PlayerNode.ColumnText.Add(LeagueList[i].Players[j].Handicap);
                PlayerNode.ColumnText.Add(LeagueList[i].Players[j].ButtonID);
                PlayerNode.ColumnText.Add(LeagueList[i].Players[j].Comment);
            end;
        end;

//    MyIni := TIniFile.Create(OpenDialog1.FileName);
//    TreeView1.Items[0].Expanded := True;
//    Players   := TStringList.Create;
//    ButtonIDs := TStringList.Create;
//    Handicaps := TStringList.Create;
//    Comments  := TStringList.Create;
//
//    i := 0;
//    while True do begin
//        s := 'T' + Format('%2.2d', [i+1]);  // set up the ini section name
//        if MyIni.ReadString(s, 'TeamName', 'no-match') = 'no-match' then break;
//
//        TeamNode := TreeView1.Items.AddChild(TreeView1.Items[0], 'New team');
//        TeamNode.Text := MyIni.ReadString(s, 'TeamName', '');
//        TeamNode.ImageIndex := 1;
//        TeamNode.AllowEdit := True;
//        TeamNode.ColumnText.Add(MyIni.ReadString(s, 'Handicap', '0'));
//        TeamNode.ColumnText.Add('');  // ButtonID
//        TeamNode.ColumnText.Add(MyIni.ReadString(s, 'Comment', ''));
//
//        p := MyIni.ReadString(s, 'Players', '');
//        if Pos(' ', p) < Pos('"', p) then p := '"' + p + '"';  // Note: ReadString is evil.  See TornMain.pas for details.
//        Players.CommaText := p;
//
//        ButtonIDs.CommaText := MyIni.ReadString(s, 'ButtonIDs', '');
//        Handicaps.CommaText := MyIni.ReadString(s, 'Handicaps', '');
//
//        p := MyIni.ReadString(s, 'Comments', '');
//        if Pos(' ', p) < Pos('"', p) then p := '"' + p + '"';  // Note: ReadString is evil.  See TornMain.pas for details.
//        Comments.CommaText  := p;
//
//        for j := 0 to Min(Players.Count, ButtonIDs.Count) - 1 do begin
//            PlayerNode := TreeView1.Items.AddChild(TeamNode, Players[j]);
//            PlayerNode.ImageIndex := 3;
//            PlayerNode.ColumnText.Add(StrDef(Handicaps, j));
//            PlayerNode.ColumnText.Add(StrDef(ButtonIDs, j));
//            PlayerNode.ColumnText.Add(StrDef(Comments,  j));
//        end;
//
//        Inc(i);
//    end;  // for i
//
//    MyIni.Free;
//    Players.Free;
//    ButtonIDs.Free;
//    Handicaps.Free;
//    Comments.Free;


    finally
        CloseFile(F);
    end;
end;

// When the value of this SpinEdit first becomes non-zero,
// add a new SpinEdit below it, and move this code to the new SpinEdit.
procedure TLeagueForm.PointsNNChange(Sender: TObject);
begin
    if (Sender as TSpinEdit).Value <> 0 then begin

        LabelLow.Caption := 'Points for ' + IntToStr((Sender as TSpinEdit).Tag + 1) + 'th and lower: 0';

        CreateNewPointsEdit((Sender as TSpinEdit).Tag + 1, (Sender as TSpinEdit).Top + 24).OnChange := PointsNNChange;

        (Sender as TSpinEdit).OnChange := nil;
    end;
end;

// Add a new SpinEdit and accompanying label
function TLeagueForm.CreateNewPointsEdit(Index, ATop: Integer): TSpinEdit;
begin
    with TLabel.Create(Self) do begin
        Parent  := Label00.Parent;
        Left    := 40;
        Top     := ATop + 3;
        Height  := 13;
        Tag     := Index;
        Caption := 'Points for ' + NthSuffix(Index);
    end;  // with

    Result := TSpinEdit.Create(Self);
    with Result do begin
        Parent  := Label00.Parent;
        Left    := 112;
        Top     := ATop;
        Width   := 49;
        Height  := 22;
        MinValue:= 0;
        MaxValue:= 9999;
        Tag     := Index;
    end;  // with
end;

function TLeagueForm.NthSuffix(i: Integer): string;
begin
    case i mod 10 of
        1:   Result := 'st';
        2:   Result := 'nd';
        3:   Result := 'rd';
        else Result := 'th';
    end;
    if i in [11, 12, 13] then Result := 'th';
    Result := IntToStr(i) + Result;
end;

procedure TLeagueForm.StatsGridDrawCell(Sender: TObject; ACol, ARow: Integer;
  Rect: TRect; State: TGridDrawState);
var s: string;
begin
    with Sender as TStringGrid do begin
        if (State = []) then begin
            if Sender = TeamStatsGrid then
                s := UpperCase(Copy(Cells[3, ARow], 1, 2))
            else if Sender = PlayerStatsGrid then
                s := UpperCase(Copy(Cells[6, ARow], 1, 2))
            else
                s := '';

            if      s = 'RE' then Canvas.Brush.Color := $02E0E0FF
            else if s = 'GR' then Canvas.Brush.Color := $02E0FFD8
            else if s = 'BL' then Canvas.Brush.Color := $02FFE8D0
            else if s = 'YE' then Canvas.Brush.Color := $02D0FFFF;

            Canvas.FillRect(Rect);
        end;

        Canvas.TextRect(Rect, Rect.Left + 3, Rect.Top + 2, Cells[ACol, ARow]);
//        if gdFocused in State then Canvas.DrawFocusRect(Rect);
    end;
end;

procedure TLeagueForm.LabelHandicapDblClick(Sender: TObject);
begin
    PlayerStatsGrid.ColCount := 10 - PlayerStatsGrid.ColCount;
    if PlayerStatsGrid.ColCount = 6 then begin
        PlayerStatsGrid.ColWidths[4] := 52;
        PlayerStatsGrid.ColWidths[5] := 69;
    end;
end;

end.
