unit Lists;

interface

uses Classes;

type
  TInt = class
  public
    Value : Integer;

    constructor Create;
  end;

  TIntList = class(TList)
  protected
    procedure Put(Index: Integer; Value: Integer);
    function Get(Index: Integer): Integer;
  public
    function Add(Value: Integer): Integer;
    procedure Sort; virtual;

    property Items[Index: Integer]: Integer read Get write Put; default;
  end;

  TDouble = class
  public
    Value : Double;

    constructor Create; overload;
    constructor Create(v: Double); overload;
  end;

  TDoubleList = class(TList)
  protected
    procedure Put(Index: Integer; Value: Double);
    function Get(Index: Integer): Double;
  public
    function Add(Value: Double): Integer;
    procedure Sort; virtual;

    property Items[Index: Integer]: Double read Get write Put; default;
  end;

  TDoubleDouble = class  // This used to be used to manage score ratios.
  public
    Key: Double;
    Value: Double;

    constructor Create;
  end;

  TDoubleDoubleList = class(TList)  // This used to be used to manage score ratios.
  protected
    procedure Put(Index: Integer; Value: TDoubleDouble);
    function Get(Index: Integer): TDoubleDouble;
  public
    function Add(Value: TDoubleDouble): Integer; overload;
    function Add(Key, Value: Double): Integer; overload;

    procedure Sort; virtual;

    property Items[Index: Integer]: TDoubleDouble read Get write Put; default;
  end;

function Sign(d: Double): Integer;

implementation

uses Math;

function Sign(d: Double): Integer;
begin
  if Abs(d) <= 0.0000001 then Result := 0
  else if d > 0 then Result := +1
  else               Result := -1;
end;

function TIntCompare(Item1, Item2: Pointer): Integer;
begin
  Result := TInt(Item1).Value - TInt(Item2).value;
end;

function TDoubleCompare(Item1, Item2: Pointer): Integer;
begin
  Result := Sign(TDouble(Item1).Value - TDouble(Item2).Value);
end;

function TDoubleDoubleCompare(Item1, Item2: Pointer): Integer;

  function Ratio(numerator, denominator: Double): Double;
  begin
    if numerator = 0 then Result := 0
    else if denominator = 0 then Result := MaxDouble
    else Result := numerator / denominator;
  end;

begin
  Result := Sign(Ratio(TDoubleDouble(Item1).Key, TDoubleDouble(Item1).Value) - Ratio(TDoubleDouble(Item2).Key, TDoubleDouble(Item2).Value));

  if Result = 0 then
    Result := Sign(TDoubleDouble(Item1).Key - TDoubleDouble(Item2).Key);
end;

{ TInt }

constructor TInt.Create;
begin
  Value := 0;
end;

{ TIntList }

function TIntList.Add(Value: Integer): Integer;
var
  oInt: TInt;
begin
  oInt := TInt.Create;
  oInt.Value := Value;
  Result := inherited Add(oInt);
end;

function TIntList.Get(Index: Integer): Integer;
begin
  Result := TInt(inherited Get(Index)).Value;
end;

procedure TIntList.Put(Index, Value: Integer);
begin
  TInt(inherited Items[Index]).Value := Value;
end;

procedure TIntList.Sort;
begin
  inherited Sort(TIntCompare);
end;

{ TDouble }

constructor TDouble.Create;
begin
  Value := 0;
end;

constructor TDouble.Create(v: Double);
begin
  Value := v;
end;

{ TDoubleList }

function TDoubleList.Add(Value: Double): Integer;
var
  oInt: TDouble;
begin
  oInt := TDouble.Create;
  oInt.Value := Value;
  Result := inherited Add(oInt);
end;

function TDoubleList.Get(Index: Integer): Double;
begin
  Result := TDouble(inherited Get(Index)).Value;
end;

procedure TDoubleList.Put(Index: Integer; Value: Double);
begin
  TDouble(inherited Items[Index]).Value := Value;
end;

procedure TDoubleList.Sort;
begin
  inherited Sort(TDoubleCompare);
end;

{ TDoubleDouble }

constructor TDoubleDouble.Create;
begin
  Key := 0;
  Value := 0;
end;

{ TDoubleDoubleList }

function TDoubleDoubleList.Add(Value: TDoubleDouble): Integer;
begin
  Result := inherited Add(Value);
end;

function TDoubleDoubleList.Add(Key, Value: Double): Integer;
var
  oDoubleDouble: TDoubleDouble;
begin
  oDoubleDouble := TDoubleDouble.Create;
  oDoubleDouble.Key := Key;
  oDoubleDouble.Value := Value;
  Result := inherited Add(oDoubleDouble);
end;

function TDoubleDoubleList.Get(Index: Integer): TDoubleDouble;
begin
  Result := inherited Get(Index);
end;

procedure TDoubleDoubleList.Put(Index: Integer; Value: TDoubleDouble);
begin
  inherited Put(Index, Value);
end;

procedure TDoubleDoubleList.Sort;
begin
  inherited Sort(TDoubleDoubleCompare);
end;

end.
