unit ReportDialog;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, Spin, RegistryDef, TornDatePicker;

type
  TReportForm = class(TForm)
    Reports: TPanel;
    SendTo: TPanel;
    OKButton: TButton;
    CancelButton: TButton;
    Label2: TLabel;
    TeamLadder: TCheckBox;
    SoloLadder: TCheckBox;
    Games: TCheckBox;
    ToScreen: TCheckBox;
    ToScoreboard: TCheckBox;
    ToPrinter: TCheckBox;
    Portrait: TRadioButton;
    Landscape: TRadioButton;
    ToHtml: TCheckBox;
    GameGrid: TCheckBox;
    ScaleGames: TCheckBox;
    TeamWithinRange: TCheckBox;
    SoloWithinRange: TCheckBox;
    SoloHandicaps: TCheckBox;
    ShowComments: TCheckBox;
    Ascension: TCheckBox;
    Bevel1: TBevel;
    Bevel2: TBevel;
    SoloDropGames: TCheckBox;
    TeamDropGames: TCheckBox;
    DateRange: TPanel;
    FromLabel: TLabel;
    DateFrom: TDateTimePicker;
    TimeFrom: TDateTimePicker;
    TimeTo: TDateTimePicker;
    DateTo: TDateTimePicker;
    ToLabel: TLabel;
    DropGames: TPanel;
    BestLabel: TLabel;
    DropBest: TSpinEdit;
    WorstLabel: TLabel;
    DropWorst: TSpinEdit;
    GamesCount: TRadioButton;
    GamesPercent: TRadioButton;
    Bevel3: TBevel;
    PackReport: TCheckBox;
    PackDropGames: TCheckBox;
    PackWithinRange: TCheckBox;
    ToTsv: TCheckBox;
    SoloTopN: TCheckBox;
    TopNSpin: TSpinEdit;
    TopNLabel: TLabel;
    SoloAtLeast: TCheckBox;
    AtLeastSpin: TSpinEdit;
    AtLeastLabel: TLabel;
    SoloOrderByLabel: TLabel;
    SoloOrderBy: TComboBox;
    GridDropGames: TCheckBox;
    TeamsVersus: TCheckBox;
    ShowColours: TCheckBox;
    DescPanel: TPanel;
    Descriptions: TCheckBox;
    Pyramid: TCheckBox;
    HtmlBars: TCheckBox;
    Scattergram: TCheckBox;
    ScoreList: TCheckBox;
    ColorDialog1: TColorDialog;
    BarColor: TButton;
    BarColorPanel: TPanel;
    OddColor: TButton;
    OddColorPanel: TPanel;
    ShowPoints: TCheckBox;
    DramaMode: TCheckBox;
    TeamOrderByLabel: TLabel;
    TeamOrderBy: TComboBox;
    TeamAndPlayerReport: TCheckBox;
    Label3: TLabel;
    GameFrom: TComboBox;
    GameTo: TComboBox;
    Panel1: TPanel;
    GamePicker: TTornDatePicker;
    SanityCheckReport: TCheckBox;
    HtmlFtp: TCheckBox;
    DetailedGames: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure TeamLadderClick(Sender: TObject);
    procedure SoloClick(Sender: TObject);
    procedure PackReportClick(Sender: TObject);
    procedure DatableClick(Sender: TObject);
    procedure ToPrinterClick(Sender: TObject);
    procedure CheckClick(Sender: TObject);
    procedure DateFromCloseUp(Sender: TObject);
    procedure DateToCloseUp(Sender: TObject);
    procedure DropGamesClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure SoloTopNClick(Sender: TObject);
    procedure SoloAtLeastClick(Sender: TObject);
    procedure OKButtonClick(Sender: TObject);
    procedure GameGridClick(Sender: TObject);
    procedure ToHtmlClick(Sender: TObject);
    procedure BarColorClick(Sender: TObject);
    procedure OddColorClick(Sender: TObject);
    procedure TeamsVersusClick(Sender: TObject);
    procedure ToScreenClick(Sender: TObject);
    procedure DramaModeClick(Sender: TObject);
    procedure GameComboDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure GameFromChange(Sender: TObject);
    procedure GameToChange(Sender: TObject);
    procedure GamePickerSelect(StartDate, EndDate: TDateTime);
  private
    function GetSekritsVisible: Boolean;
    procedure SetSekritsVisible(const Value: Boolean);

    function GetGameList: TStrings;
  public
    property SekritsVisible: Boolean read GetSekritsVisible write SetSekritsVisible;

    property GameList: TStrings read GetGameList;
  end;

var

  ReportForm: TReportForm;

implementation

{$R *.DFM}

// Set up starting values for date/time pickers.
procedure TReportForm.FormCreate(Sender: TObject);
var MyRegistry: TRegistryDef;
    i: Integer;
begin
  TimeFrom.DateTime := 0;     // midnight
  TimeTo.DateTime   := 0.999999;  // 11:59:59 PM

  CheckClick(Sender);

  SoloOrderBy.ItemIndex := 3;
  TeamOrderBy.ItemIndex := 1;

  MyRegistry := TRegistryDef.Create;  // open the Registry
  MyRegistry.RootKey := HKEY_LOCAL_MACHINE;
  if MyRegistry.OpenKey('Software\DougWare\Torn', True) then begin
      for i := 0 to ComponentCount - 1 do
          if Components[i].Tag = 42 then
              MyRegistry.ReadPref(Components[i]);
  end;
  DramaMode.Enabled := not ToScreen.Checked and not ToScoreboard.Checked;
end;

// Set position of form.
procedure TReportForm.FormShow(Sender: TObject);
begin
//    if Left < 0 then Left := 0;
//    if Left + Width > Screen.DesktopWidth then
//        Left := Screen.DesktopWidth - Width;
  GameTo.Items.Assign(GameFrom.Items);
end;

// Enable / disable ScaleUp, the team ladder option
procedure TReportForm.TeamLadderClick(Sender: TObject);
begin
  ScaleGames.Enabled      := TeamLadder.Checked;
  TeamWithinRange.Enabled := TeamLadder.Checked;
  TeamDropGames.Enabled   := TeamLadder.Checked;
  ShowColours.Enabled     := TeamLadder.Checked;
  TeamOrderBy.Enabled     := TeamLadder.Checked;
  TeamOrderByLabel.Enabled:= TeamLadder.Checked;

  DropGamesClick(Sender);
  CheckClick(Sender);
end;

procedure TReportForm.TeamsVersusClick(Sender: TObject);
begin
  ShowPoints.Enabled := TeamsVersus.Checked;
  CheckClick(Sender);
end;

// Enable / disable solo ladder options
procedure TReportForm.SoloClick(Sender: TObject);
begin
  SoloHandicaps.Enabled   := SoloLadder.Checked;
  SoloWithinRange.Enabled := SoloLadder.Checked;
  ShowComments.Enabled    := SoloLadder.Checked;
  SoloDropGames.Enabled   := SoloLadder.Checked;
  SoloTopN.Enabled        := SoloLadder.Checked;
  SoloAtLeast.Enabled     := SoloLadder.Checked;
  AtLeastSpin.Enabled     := SoloLadder.Checked and SoloAtLeast.Checked;
  AtLeastLabel.Enabled    := AtLeastSpin.Enabled;
  SoloOrderBy.Enabled     := SoloLadder.Checked;
  SoloOrderByLabel.Enabled:= SoloLadder.Checked;
  Scattergram.Enabled     := SoloLadder.Checked;

  DropGamesClick(Sender);
  SoloTopNClick(Sender);
  CheckClick(Sender);
end;

procedure TReportForm.GameGridClick(Sender: TObject);
begin
  GridDropGames.Enabled   := GameGrid.Checked;

  DropGamesClick(Sender);
  DatableClick(Sender);
end;

procedure TReportForm.PackReportClick(Sender: TObject);
begin
  PackWithinRange.Enabled := PackReport.Checked;
  PackDropGames.Enabled   := PackReport.Checked;

  DropGamesClick(Sender);
  DatableClick(Sender);
end;

procedure TReportForm.SoloTopNClick(Sender: TObject);
begin
  TopNSpin.Enabled := SoloLadder.Checked and SoloTopN.Checked;
  TopNLabel.Enabled := TopNSpin.Enabled;
end;

procedure TReportForm.SoloAtLeastClick(Sender: TObject);
begin
  AtLeastSpin.Enabled := SoloLadder.Checked and SoloAtLeast.Checked;
  AtLeastLabel.Enabled := AtLeastSpin.Enabled;
end;

// Enable/disable date/time pickers
procedure TReportForm.DatableClick(Sender: TObject);
var DatesEnabled: Boolean;
begin
  DatesEnabled := (TeamLadder.Checked and TeamWithinRange.Checked) or
                  (SoloLadder.Checked and SoloWithinRange.Checked) or
                  (PackReport.Checked and PackWithinRange.Checked) or
                  Games.Checked or GameGrid.Checked or Ascension.Checked or Pyramid.Checked or
                  ScoreList.Checked or TeamAndPlayerReport.Checked or DetailedGames.Checked;

  FromLabel.Enabled:=DatesEnabled;
  ToLabel.Enabled := DatesEnabled;
  DateFrom.Enabled:= DatesEnabled;
  TimeFrom.Enabled:= DatesEnabled;
  DateTo.Enabled  := DatesEnabled;
  TimeTo.Enabled  := DatesEnabled;
  GameFrom.Enabled:= DatesEnabled;
  GameTo.Enabled  := DatesEnabled;

  CheckClick(Sender);
end;

procedure TReportForm.CheckClick(Sender: TObject);
begin
  OKButton.Enabled := (TeamLadder.Checked or SoloLadder.Checked or PackReport.Checked or Games.Checked or
                       GameGrid.Checked or Ascension.Checked or Pyramid.Checked or TeamsVersus.Checked or
                       ScoreList.Checked or SanityCheckReport.Checked or TeamAndPlayerReport.Checked or
                       DetailedGames.Checked) and
                      (ToScreen.Checked or ToScoreboard.Checked or DramaMode.Checked or
                       ToHtml.Checked or ToPrinter.Checked or ToTsv.Checked);
end;

procedure TReportForm.ToPrinterClick(Sender: TObject);
begin
  Portrait.Enabled  := ToPrinter.Checked;
  Landscape.Enabled := ToPrinter.Checked;

  CheckClick(Sender);
end;

procedure TReportForm.ToHtmlClick(Sender: TObject);
begin
  HtmlBars.Enabled := ToHtml.Checked;
  HtmlFtp.Enabled := ToHtml.Checked;
  CheckClick(Sender);
end;

procedure TReportForm.ToScreenClick(Sender: TObject);
begin
  if ToScreen.Checked or ToScoreboard.Checked then
     DramaMode.Checked := False;

  DramaMode.Enabled := not ToScreen.Checked and not ToScoreboard.Checked;
  CheckClick(Sender);
end;

procedure TReportForm.DramaModeClick(Sender: TObject);
begin
  if DramaMode.Checked then begin
    ToScreen.Checked := False;
    ToScoreboard.Checked := False;
  end;

  ToScreen.Enabled := not DramaMode.Checked;
  ToScoreboard.Enabled := not DramaMode.Checked;

  CheckClick(Sender);
end;

procedure TReportForm.DateFromCloseUp(Sender: TObject);
begin
  if DateTo.DateTime < DateFrom.DateTime then
    DateTo.DateTime := DateFrom.DateTime;
end;

procedure TReportForm.DateToCloseUp(Sender: TObject);
begin
  if DateFrom.DateTime > DateTo.DateTime then
    DateFrom.DateTime := DateTo.DateTime;
end;

procedure TReportForm.DropGamesClick(Sender: TObject);
var b: Boolean;
begin
  b := TeamLadder.Checked and TeamDropGames.Checked or SoloLadder.Checked and SoloDropGames.Checked or
       GameGrid.Checked and GridDropGames.Checked or PackReport.Checked and PackDropGames.Checked;

  BestLabel.Enabled    := b;
  WorstLabel.Enabled   := b;
  DropBest.Enabled     := b;
  DropWorst.Enabled    := b;
  GamesCount.Enabled   := b;
  GamesPercent.Enabled := b;
end;

procedure TReportForm.BarColorClick(Sender: TObject);
begin
  ColorDialog1.CustomColors.CommaText := 'ColorA=A0C0FF,ColorB=D0D0D0';
  ColorDialog1.Color := BarColorPanel.Color;
  if ColorDialog1.Execute then
    BarColorPanel.Color := ColorDialog1.Color;
end;

procedure TReportForm.OddColorClick(Sender: TObject);
begin
  ColorDialog1.CustomColors.CommaText := 'ColorA=FFFFEE';
  ColorDialog1.Color := OddColorPanel.Color;
  if ColorDialog1.Execute then
    OddColorPanel.Color := ColorDialog1.Color;
end;

procedure TReportForm.OKButtonClick(Sender: TObject);
var MyRegistry: TRegistryDef;
    i: Integer;
begin
    MyRegistry := TRegistryDef.Create;  // open the Registry
    MyRegistry.RootKey := HKEY_LOCAL_MACHINE;
    MyRegistry.OpenKey('Software\DougWare\Torn', False);

    // Write registry values for 31 assorted controls
    for i := 0 to ComponentCount - 1 do
        if Components[i].Tag = 42 then
            MyRegistry.WritePref(Components[i]);
end;

function TReportForm.GetSekritsVisible: Boolean;
begin
  Result := PackReport.Visible;
end;

// Show or hide a group of report features that are either experimental or confidential.
procedure TReportForm.SetSekritsVisible(const Value: Boolean);
var offset : Integer;
begin
  PackReport.Visible := Value;

  if Value then
    offset := 230
  else begin
    offset := 0;
    PackReport.Checked := False;
    ScoreList.Checked := False;
  end;

  ClientWidth := 773 + offset;
  Reports.Width := 755 + offset;
end;

function TReportForm.GetGameList: TStrings;
begin
  Result := GameFrom.Items;
end;

procedure TReportForm.GameComboDrawItem(Control: TWinControl; Index: Integer; Rect: TRect; State: TOwnerDrawState);
var availablewidth: Integer;
    text: string;
    today: string;
begin
  text := TComboBox(Control).Items[Index];
  today := DateToStr(Date);
  if Copy(text, 1, 10) = today then
    text := 'Today' + Copy(text, 11, MaxInt);

	with TComboBox(Control) do begin // draw on control canvas, not on the form
    Canvas.FillRect(Rect);         // clear the rectangle

    if Items.Count <= DropDownCount then
        availablewidth := Width - 2
    else
        availablewidth := Width - 19;

    Canvas.Font.Height := ItemHeight;
    while (Canvas.Font.Height > ItemHeight div 3 + 1) and
          (Canvas.TextWidth(text) > availablewidth) do
        Canvas.Font.Height := Canvas.Font.Height - 1;

    Canvas.TextOut(Rect.Left + 1, (Rect.Top + Rect.Bottom - Canvas.Font.Height - 1) div 2, text)  // display the text
	end;
end;

function LastPos(Substr: char; const S: string): Integer;
begin
  Result := Length(S);
  while (Result > 0) and (S[Result] <> Substr) do
    Dec(Result);
end;

procedure TReportForm.GameFromChange(Sender: TObject);
var
  s: string;
  i: Integer;
begin
   s := Trim(TComboBox(Sender).Text);
   i := LastPos(' ', s);
   if i > 0 then begin
     TimeFrom.DateTime := StrToTime(Copy(s, 1, i - 1));
     DateFrom.DateTime := StrToDate(Copy(s, i + 1, MaxInt));
   end;
end;

procedure TReportForm.GameToChange(Sender: TObject);
var
  s: string;
  i: Integer;
begin
   s := Trim(TComboBox(Sender).Text);
   i := LastPos(' ', s);
   if i > 0 then begin
     TimeTo.DateTime := StrToTime(Copy(s, 1, i - 1)) + 1/1440;  // Set "To" time to one minute after start of selected game.
     DateTo.DateTime := StrToDate(Copy(s, i + 1, MaxInt));
   end;
end;

procedure TReportForm.GamePickerSelect(StartDate, EndDate: TDateTime);
begin
  DateFrom.DateTime := Trunc(StartDate);
  TimeFrom.DateTime := Frac(StartDate);
  DateTo.DateTime := Trunc(EndDate);
  TimeTo.DateTime := Frac(EndDate);
end;

end.
