unit Reports;

interface

uses
  Classes, Dialogs, Forms, Graphics, Printers, SysUtils,
  ZoomReport, Utility, DramaDialog, LadderDialog;

procedure ShowOnScreen(reports: TZoomReports; OwnerForm: TForm; var LadderForm: TLadderForm);
procedure ShowOnScoreboard(report: TZoomReport);
procedure ShowOnPrinter(reports: TZoomReports; PrintDialog: TPrintDialog; portrait: Boolean);
procedure SaveAsHtml(reports: TZoomReports; SaveDialog: TSaveDialog; bars: Boolean);
procedure SaveAsTsv(reports: TZoomReports; SaveDialog: TSaveDialog);
procedure DramaMode(report: TZoomReport; DramaForm: TDramaForm; OwnerForm: TForm; var LadderForm: TLadderForm);

implementation

const
    TBOARD_PORT = 21570;

// Take a list of items to show on this screen, and show 'em, by drawing on the canvas.
// Called by ShowLeagueLadderClick and ShowSoloLadderClick.
procedure ShowOnScreen(reports: TZoomReports; OwnerForm: TForm; var LadderForm: TLadderForm);
begin
    if LadderForm = nil then begin
        LadderForm := TLadderForm.Create(Application);
        LadderForm.Top := 0;
        LadderForm.Height := Screen.Height - 28;
        if OwnerForm.Left > Screen.Width - (OwnerForm.Left + OwnerForm.Width) then begin  // put the report to the left of Torn's main window
            LadderForm.Left := 0;
            LadderForm.Width := OwnerForm.Left;
        end else begin  // put the report to the right of Torn's main window
            LadderForm.Left := OwnerForm.Left + OwnerForm.Width;
            LadderForm.Width := Screen.Width - (OwnerForm.Left + OwnerForm.Width);
        end;
    end;
    LadderForm.Image1.Picture.Bitmap.Height := LadderForm.Image1.Height;
    LadderForm.Image1.Picture.Bitmap.Width  := LadderForm.Image1.Width;

    reports.Show(LadderForm.Image1.Canvas, 0, 0, LadderForm.Image1.Width, LadderForm.Image1.Height);
    LadderForm.Show;
end;

// Take lists of items to show on the scoreboard, and show 'em.
procedure ShowOnScoreboard(report: TZoomReport);
var j, k: Integer;
    ReportData: TStringList;
    headings, alignments: string;
begin
    ReportData := TStringList.Create;
    with ReportData do begin
        Add('DISPLAYREPORTS');  // command

        Add(ColorToString(report.Owner.TitleFontColor));
        Add(ColorToString(report.Owner.TitleBackColor));
        Add(ColorToString(report.Owner.BackgroundColor));
        Add(ColorToString(report.Owner.OddColor));

        Add(report.Title);

        headings := '"';
        alignments := '';
        for j := 0 to report.Columns.Count - 1 do begin
            headings := headings + report.Columns[j].Heading + '","';
            alignments := alignments + report.Columns[j].Alignment + ',';
        end;
        Add(Copy(headings, 1, Length(headings) - 2));
        Add(Copy(alignments, 1, Length(alignments) - 1));

        for j := 0 to report.Rows.Count - 1 do begin // for each row in the table
            for k := 0 to report.Rows[j].Count - 1 do begin  // for each cell in the row
                Add(report.Rows[j][k].Text);
                Add(ColorToString(report.Rows[j][k].Color));
                Add(IntToStr(report.Rows[j][k].BarCell));
            end;
            Add('EOREOR');  // end of report marker
        end;  // for j
        Add('EOTEOT');
        Add('EOSEOS');

        if Count > 1 then
            SendUDPs(TBOARD_PORT, CommaText);
        Free;
    end;  // with ReportData
end;

// Take a list of items to show on this screen, and show 'em, by drawing on the canvas.
// Called by ShowLeagueLadderClick and ShowSoloLadderClick.
procedure ShowOnPrinter(reports: TZoomReports; PrintDialog: TPrintDialog; portrait: Boolean);
var Saved8087CW: Word;
begin
    if portrait then
        Printer.Orientation := poPortrait
    else
        Printer.Orientation := poLandscape;

    if not PrintDialog.Execute then Exit;

    with Printer do begin
        BeginDoc;
        Title := 'Torn report';
        reports.Show(Canvas, PageWidth div 10, PageHeight div 10, PageWidth * 9 div 10, PageHeight * 9 div 10);
//ShowMessage('Show() call done.');

        Saved8087CW := Default8087CW;
        try
            Set8087CW($133f);  // Disable all fpu exceptions
            EndDoc;
        finally
            Set8087CW(Saved8087CW);  // Re-enable whatever was enabled in the first place.
        end;
//ShowMessage('EndDoc done.');
    end;
end;

// Take a list of items to show, and show 'em.
procedure SaveAsHtml(reports: TZoomReports; SaveDialog: TSaveDialog; bars: Boolean);
var F: TextFile;
begin
    SaveDialog.Filter := 'HTML files|*.html;*.htm|All files|*.*';
    SaveDialog.DefaultExt := 'html';
    if not SaveDialog.Execute then Exit;

    AssignFile(F, SaveDialog.FileName);
    reports.SaveAsHtml(F, bars);
end;

// Take a list of items to show, and show 'em.
procedure SaveAsTsv(reports: TZoomReports; SaveDialog: TSaveDialog);
var F: TextFile;
begin
    SaveDialog.Filter := 'TSV files|*.tsv|All files|*.*';
    SaveDialog.DefaultExt := 'tsv';
    if not SaveDialog.Execute then Exit;

    AssignFile(F, SaveDialog.FileName);
    reports.SaveAsCsv(F, #9);
end;

procedure DramaMode(report: TZoomReport; DramaForm: TDramaForm; OwnerForm: TForm; var LadderForm: TLadderForm);
begin
    if DramaForm = nil then
        DramaForm := TDramaForm.Create(Application);

    DramaForm.Report := report;
    DramaForm.OwnerForm := OwnerForm;
    DramaForm.LadderForm := LadderForm;
    DramaForm.ShowModal;
end;

end.
