unit Scorecards;

interface

uses SysUtils, Classes, Math, Scoreboard, League, TornTeamBox1, DataModule;

function  Build3TeamScorecards(Team: TTeamBoxList; LeagueList: TLeagueList; ColourTeam: TColourTeamArray; packid: string; GameNum: Integer; GameTime: TDateTime; GameDuration: Integer; GameTitle: string; ViewGcc: Boolean; DM: TParadoxDataModule): string;
procedure BuildNTeamTranslate(Team: TTeamBoxList; translatefrom, translateto: TStringList);
function  BuildNTeamScorecards(Team: TTeamBoxList; LeagueList: TLeagueList; packid: string; GameNum: Integer; GameTime: TDateTime; GameDuration: Integer; GameTitle: string; DM: TParadoxDataModule): string;
procedure PrintScorecards(Team: TTeamBoxList; LeagueList: TLeagueList; GameNum: Integer; GameTime: TDateTime; GameDuration: Integer; GameTitle: string; ViewGcc: Boolean; Blue2, Blue3: Boolean; NumPrinters: Integer; DM: TParadoxDataModule);
procedure PrintOneScorecard(Team: TTeamBoxList; LeagueList: TLeagueList; GameNum: Integer; GameTime: TDateTime; GameDuration: Integer; GameTitle: string; ViewGcc: Boolean; Blue2, Blue3: Boolean; TeamBox: TTeamBox; DM: TParadoxDataModule);
procedure PrintSummaryScorecard(Team: TTeamBoxList; LeagueList: TLeagueList; GameNum: Integer; GameTime: TDateTime; GameDuration: Integer; GameTitle: string; ViewGcc: Boolean; Blue2, Blue3: Boolean; DM: TParadoxDataModule);

implementation

uses Utility;

// Print scorecards for every player in the three teamboxes listed in ColourTeam[], for three-team games.
// Return the list of pack ID's to be printed; which is either every player pack in the game, or just the one pack that was passed in.
function Build3TeamScorecards(Team: TTeamBoxList; LeagueList: TLeagueList; ColourTeam: TColourTeamArray; packid: string; GameNum: Integer; GameTime: TDateTime; GameDuration: Integer; GameTitle: string; ViewGcc: Boolean; DM: TParadoxDataModule): string;
var TeamColour: array [1..60] of Integer;  // array showing what colour each team is.  This is the inverse of ColourTeam[].
    i, teamon, playeron, teamby, playerby, zone: Integer;  // loop indeces
    IdsUsed:    set of 0..59;   // pack IDs used already
    translatefrom, translateto: TStringList;  // list of pack IDs as listed in the team box, and as we're going to list them in our DM.StatsTable and DM.HitTable entries.  Note that if an ID translates to the same value, it's not listed here.
    idon, idby: Integer;        // IDs that we're going to use for these pack(s)
    baseretaliation: Boolean;   // did base retaliation hit anybody during this game?
    teamname:   string;
    stats:      Variant;        // variant array to hold entries that will be used to build a DM.StatsTable line
    hit:        THitArray;
    newpackids: TStringList;    // list of pack IDs to print

    function FindFreeID(colour: Integer): Integer;
    begin
        Result := 56 + colour;  // start at the top end of the ID list
        while (Result > 2) and (Result in IdsUsed) do  // and search down, 'til we find one free
            Result := Result - 3;
        Include(IdsUsed, Result);
    end;

begin  //---Build3TeamScorecards---
    for i := 1 to Team.Count - 1 do
        TeamColour[i] := (i - 1) mod 3 + 1;  // initialise the TeamColour array: 1, 2, 3, 1, 2, 3, ...

    for i := 1 to 3 do                       // set up to three values in the teamcolour array,
        if ColourTeam[i] <> -1 then          // showing what colour has been assigned
            TeamColour[ColourTeam[i]] := i;  // to each of the top three teams

    newpackids := TStringList.Create;
    translatefrom := TStringList.Create;
    translateto   := TStringList.Create;
    try
        DM.CleanStats(2000);
        DM.HitTable.Close;
        DM.HitTable.ReadOnly := False;
        DM.HitTable.Open;

        IdsUsed := [];
        baseretaliation := False;

        for teamon := 1 to 3 do  // for each of the top 3 Teams,
            if ColourTeam[teamon] <> -1 then begin
                for playeron := 0 to Team[ColourTeam[teamon]].Items.Count - 1 do  // for each player
                    with Team[ColourTeam[teamon]].Items[playeron] do begin
                        if SubItems.Count > tbAccuracy then
                            Include(IdsUsed, StrToIntDef(Team[ColourTeam[teamon]].Items[playeron].SubItems[tbPackID], 59));  // set their element in IdsUsed

                        if SubItems.Count > tbYelBase2 then
                            for i := tbRedBase1 to tbYelBase2 do
                                baseretaliation := baseretaliation or (StrToIntDef(Team[ColourTeam[teamon]].Items[playeron].SubItems[i - 8], 0) > 0);  // check to see if any of RedBase1HitYou .. YellowBase2HitYou are non-zero

                        if SubItems.Count > tbAccuracy then begin
                            idon := StrToIntDef(SubItems[tbPackID], teamon + 1);
                            if (idon mod 3) + 1 <> teamon then begin  // this pack is a different colour to the team
                                translatefrom.Add(IntToStr(idon));  // so add it to the translate list,
                                idon := FindFreeID(teamon);  // and find a pack ID on the correct team
                                translateto.Add(IntToStr(idon));
                            end;
                        end;
                    end;  // with

                teamname := LeagueList.BuildTeamName(Team[ColourTeam[teamon]]);
                if (Team[ColourTeam[teamon]].Handicap <> '0') and (Team[ColourTeam[teamon]].Handicap <> '100%') then
                    teamname := teamname + ' (' + Team[ColourTeam[teamon]].Handicap + ')';

                DM.WriteStatsLine(VarArrayOf([GameNum + 2000, teamon + 59, 4, '', '', 0, 0, BuildTeamHandicap(Team[ColourTeam[teamon]]),
                               Team[ColourTeam[teamon]].Score, 0, 0, GameTime, teamname, GameDuration, 0]));  // create a DM.StatsTable entry for the team
            end;

        for teamon := 1 to Team.Count - 1 do  // for each Team box,
            for playeron := 0 to Team[teamon].Items.Count - 1 do begin  // for each player in the Team box,
                if Team[teamon].Items[playeron].SubItems.Count > tbAccuracy then begin
                    with Team[teamon].Items[playeron] do begin
                        idon := StrToIntDef(SubItems[tbPackID], 59);
                        if translatefrom.IndexOf(IntToStr(idon)) <> -1 then  // if it's on the translate list,
                            idon := StrToInt(translateto[translatefrom.IndexOf(IntToStr(idon))]);  // translate it

                        stats := VarArrayOf([GameNum + 2000, idon, (idon mod 3) + 1, '', Caption, 0, 0, 100,//StrToIntDef(SubItems[tbHandicap], 100),
                                             GetHandiScore(SubItems), StrToIntDef(SubItems[tbRank], 0), StrToIntDef(SubItems[tbAccuracy], 0), GameTime, GameTitle, GameDuration, 1]);   // set up a DM.StatsTable line like 'Red 9'
                        if SubItems[tbMemberAlias] <> '' then begin  // set up a DM.StatsTable line like 'RONiN 441'
                            stats[3] := SubItems[tbMemberID];
                            stats[4] := SubItems[tbMemberAlias];
                        end;
                        VarArrayRedim(stats, 50);
                        for i := tbStatsStart to Min(39, SubItems.Count - 1) do  // copy in all the remaining guff for the DM.StatsTable line
                            stats[i + 8] := StrToIntDef(SubItems[i], 0);
                        if SubItems.Count > 42 then begin
                            stats[48] := StrToFloatDef(SubItems[40], 0.0);  // MemberAverageScore
                            stats[49] := StrToFloatDef(SubItems[41], 0.0);  // MemberAverageAccuracy
                            stats[50] :=   StrToIntDef(SubItems[42], 0);    // FormID
                        end;

                        if ViewGcc and not baseretaliation then
                            for i := 0 to 5 do
                                if DM.HitTable.Locate('Game;HitOnID;HitByID', VarArrayOf([GameNum, i + 104, StrToIntDef(SubItems[tbPackID], 59)]), []) then  // find out how many shots this player put into this base
                                    stats[i + tbRedBase1] := DM.HitTable.Fields[3].AsVariant;  // and put it in the matching field in the DM.StatsTable line

                        DM.WriteStatsLine(stats);
                        DM.WriteHitNil(GameNum + 2000, idon, idon);  // this is the player versus himself; nil hits 'cos obviously you can't hit yourself
                        newpackids.Add(IntToStr(idon));
                    end;  // with Team[teamon].Items[playeron]

                    for teamby := 1 to Team.Count - 1 do                // again for each Team box,
                        for playerby := 0 to Team[teamby].Items.Count - 1 do  // again for each player in the Team box,
                            if Team[teamby].Items[playerby].SubItems.Count > tbAccuracy then begin
                                if (teamon = teamby) and (playeron = playerby) then Continue;  // this is the player versus himself; so skip it

                                idby := StrToIntDef(Team[teamby].Items[playerby].SubItems[tbPackID], 59);
                                if translatefrom.IndexOf(IntToStr(idby)) <> -1 then  // ditto
                                    idby := StrToInt(translateto[translatefrom.IndexOf(IntToStr(idby))]);

                                if DM.HitTable.Locate('Game;HitOnID;HitByID', VarArrayOf([GameNum,
                                        StrToIntDef(Team[teamon].Items[playeron].SubItems[tbPackID], -1),
                                        StrToIntDef(Team[teamby].Items[playerby].SubItems[tbPackID], -1)]), []) then begin
                                    for zone := 1 to 6 do
                                        hit[zone] := DM.HitTable.Fields[zone + 2].AsInteger;  // copy the hit data fields out of DM.HitTable
                                    DM.WriteHitLine(GameNum + 2000, idon, idby, hit);
                                end else
                                    DM.WriteHitNil(GameNum + 2000, idon, idby);
                            end;  // if [by].SubItems.Count > tbAccuracy; for playerby; for teamby
                end else begin  // [on].SubItems.Count > tbAccuracy
                    idon := FindFreeID(TeamColour[teamon]);
                    DM.WriteStatsLine(VarArrayOf([GameNum + 2000, idon, (idon mod 3) + 1, '', Team[teamon].Items[playeron].SubItems[tbMemberAlias], 0, 0, 100,
                                                GetHandiScore(Team[teamon].Items[playeron].SubItems), 99, 0, GameTime, GameTitle, GameDuration, 1]));  // write a non-player line like 'Adjustment'

                    for teamby := 1 to Team.Count - 1 do  // for each Team box,
                        for playerby := 0 to Team[teamby].Items.Count - 1 do begin  // for each player in the Team box,
                            if Team[teamby].Items[playerby].SubItems.Count <= tbAccuracy then Continue;  // this is the fake "player" versus another fake "player" (or possibly versus itself); skip it

                            idby := StrToIntDef(Team[teamby].Items[playerby].SubItems[tbPackID], 59);
                            if translatefrom.IndexOf(IntToStr(idby)) <> -1 then  // ditto
                                idby := StrToInt(translateto[translatefrom.IndexOf(IntToStr(idby))]);

                            DM.WriteHitNil(GameNum + 2000, idby, idon);  // note that idon and idby are backwards here
                        end;  // for playerby; for teamby
                end;
            end;  // for playeron; for teamon

        if packid = '' then
            Result := newpackids.CommaText  // return the list of packs to be printed
        else if translatefrom.IndexOf(packid) > -1 then
            Result := translateto[translatefrom.IndexOf(packid)]  // find the old pack ID in translatefrom, and pick out its match in translateto.
        else
            Result := packid;              // just give up and return whatever was passed to us
    finally
        DM.HitTable.Close;
        DM.HitTable.ReadOnly := True;
        DM.StatsTable.Close;
        translatefrom.Free;
        translateto.Free;
        newpackids.Free;
    end;
end;

// Generate the translation lists from old pack IDs to new for n-team scorecards.  New IDs are always on the green and/or yellow "teams".
procedure BuildNTeamTranslate(Team: TTeamBoxList; translatefrom, translateto: TStringList);
var teamn, player: Integer;  // loop indeces
    playernum:  Integer;  // number from 0 to 9 that shows where this player goes on the scorecard
    teamnum:    Integer;  // number from 1 to 2 that shows where this player goes on the scorecard: 1: green, 2: yellow
    groupnum:   Integer;  // number from 0 to 3 that shows where this player goes on the scorecard: 0: 1-10,  1: 11-20
begin
    playernum := 0;
    teamnum   := 1;
    groupnum  := 0;
    for teamn := 1 to Team.Count - 1 do // for each Team box,
        for player := 0 to Team[teamn].Items.Count - 1 do  // for each player in the Team box,
            if Team[teamn].Items[player].SubItems.Count > tbAccuracy then begin
                translatefrom.Add(Team[teamn].Items[player].SubItems[tbPackID]);
                translateto.Add(IntToStr((playernum * 3) + teamnum + groupnum * 30));

                Inc(playernum);
                if playernum > 9 then begin
                    playernum := 0;
                    Inc(teamnum);
                    if teamnum > 2 then begin
                        teamnum := 1;
                        Inc(groupnum);
                    end;
                end;
            end;  // for player; for teamn
end;

// Print scorecards for each player in each teambox.  Works for many-team games of at most 40 players and 15 teams.
// Under "red" section, put one line per team; under "green" and "yellow" sections, put individual players.
// Return the new ID of the pack passed, so that PrintOneCard will know what card to print.
function BuildNTeamScorecards(Team: TTeamBoxList; LeagueList: TLeagueList; packid: string; GameNum: Integer; GameTime: TDateTime; GameDuration: Integer; GameTitle: string; DM: TParadoxDataModule): string;
var i, team1, player1, team2, player2, zone: Integer;  // loop indeces
    translatefrom, translateto: TStringList;  // list of pack IDs as listed in the team box, and as we're going to list them in our DM.StatsTable and DM.HitTable entries.
    id1, id2:   Integer;  // IDs that we're going to use for these pack(s)
    baseretaliation: Boolean;  // did base retaliation hit anybody during this game?
    stats:      Variant;  // variant array to hold entries that will be used to build a DM.StatsTable line
    hit:        THitArray;
begin
    DM.CleanStats(2000);

    // Create a DM.StatsTable entry for each each team box that has players in it.
    baseretaliation := False;
    for team1 := 1 to Team.Count - 1 do  // loop through each team box
        if Team[team1].Items.Count <> 0 then begin  // if there are players on this team,
            DM.WriteStatsLine(VarArrayOf([GameNum + 2000, team1 * 3 - 3, 1, '', LeagueList.BuildTeamName(Team[team1]), 0, 0, BuildTeamHandicap(Team[team1]),
                           Team[team1].Score, Team[team1].Rank, BuildTeamHandicap(Team[team1]) * 10, GameTime, '', GameDuration, 1]));  // create the entry.

            for player1 := 0 to Team[team1].Items.Count - 1 do  // for each player,
                if Team[team1].Items[player1].SubItems.Count > tbYelBase2 then
                    for i := tbRedBase1 to tbYelBase2 do
                        baseretaliation := baseretaliation or (StrToIntDef(Team[team1].Items[player1].SubItems[i - 8], 0) > 0);  // check to see if any of RedBase1HitYou .. YellowBase2HitYou are non-zero
        end;

    translatefrom := TStringList.Create;
    translateto   := TStringList.Create;
    BuildNTeamTranslate(Team, translatefrom, translateto);

    if translatefrom.IndexOf(packid) = -1 then
        Result := '0'
    else
        Result := translateto[translatefrom.IndexOf(packid)];  // find the old pack ID in translatefrom, and pick out its match in translateto.

    DM.WriteStatsLine(VarArrayOf([GameNum + 2000, 60, 4, '', '', 0, 0, 100, 0, 0, 0, GameTime, 'TEAMS', GameDuration, 0]));  // create a DM.StatsTable entry for the red "team"
    DM.WriteStatsLine(VarArrayOf([GameNum + 2000, 61, 4, '', '', 0, 0, 100, 0, 0, 0, GameTime, 'PLAYERS', GameDuration, 0]));  // create a DM.StatsTable entry for the green "team"
    if translatefrom.Count > 10 then  // we put player entries under the yellow "team",
        DM.WriteStatsLine(VarArrayOf([GameNum + 2000, 62, 4, '', '', 0, 0, 100, 0, 0, 0, GameTime, 'PLAYERS', GameDuration, 0]));  // so create a DM.StatsTable entry for the yellow "team"

    DM.HitTable.Close;
    DM.HitTable.ReadOnly := False;
    DM.HitTable.Open;
    try
        for team1 := 1 to Team.Count - 1 do // for each Team box,
            for player1 := 0 to Team[team1].Items.Count - 1 do begin  // for each player in the Team box,
                if Team[team1].Items[player1].SubItems.Count > tbAccuracy then begin
                    with Team[team1].Items[player1] do begin
                        id1 := StrToInt(translateto[translatefrom.IndexOf(SubItems[tbPackID])]);  // find the old pack ID in translatefrom, and pick out its match in translateto.

                        stats := VarArrayOf([GameNum + 2000, id1, (id1 mod 3) + 1, '', Caption, 0, 0, 100,//StrToIntDef(SubItems[tbHandicap], 100),
                                             GetHandiScore(SubItems), StrToIntDef(SubItems[tbRank], 0), StrToIntDef(SubItems[tbAccuracy], 0), GameTime, GameTitle, GameDuration, 1]);   // set up a DM.StatsTable line like 'Red 9'
                        if SubItems[tbMemberAlias] <> '' then begin  // set up a DM.StatsTable line like 'RONiN 441'
                            stats[3] := SubItems[tbMemberID];
                            stats[4] := SubItems[tbMemberAlias];
                        end;
                        VarArrayRedim(stats, 50);
                        for i := tbStatsStart to Min(39, SubItems.Count - 1) do  // copy in all the remaining guff for the DM.StatsTable line
                            stats[i + 8] := StrToIntDef(SubItems[i], 0);
                        if SubItems.Count > 42 then begin
                            stats[48] := StrToFloatDef(SubItems[40], 0.0);  // MemberAverageScore
                            stats[49] := StrToFloatDef(SubItems[41], 0.0);  // MemberAverageAccuracy
                            stats[50] :=   StrToIntDef(SubItems[42], 0);    // FormID
                        end;

                        if not baseretaliation then
                            for i := 0 to 5 do
                                if DM.HitTable.Locate('Game;HitOnID;HitByID', VarArrayOf([GameNum, i + 104, StrToIntDef(SubItems[tbPackID], -1)]), []) then  // find out how many shots this player put into this base
                                    stats[i + 28] := DM.HitTable.Fields[3].AsVariant;  // and put it in the matching field in the DM.StatsTable line

                        DM.WriteStatsLine(stats);  // write the DM.StatsTable line
                    end;  // with

                    for team2 := 1 to Team.Count - 1 do begin
                        // Write a record for hits on this player _by_ a team.  These appear in the red "team" section of the player's scorecard.  These records are necessary to force the scorecard to show the red "player" names and scores
                        DM.WriteHitNil(GameNum + 2000, id1, (team2 - 1) * 3);

                        // Find hits by this player on a single other player, and write a record for them.  These appear in the green and/or yellow "team" sections of the player's scorecard.
                        for player2 := 0 to Team[team2].Items.Count - 1 do  // for each player in the other Team box,
                            if Team[team2].Items[player2].SubItems.Count > tbAccuracy then begin
                                id2 := StrToInt(translateto[translatefrom.IndexOf(Team[team2].Items[player2].SubItems[tbPackID])]);  // find the old pack ID in translatefrom, and pick out its match in translateto.
                                if (team1 = team2) and (player1 = player2) then begin  // this is the player versus himself
                                    DM.WriteHitNil(GameNum + 2000, id1, id2);
                                end else begin
                                    if DM.HitTable.Locate('Game;HitOnID;HitByID', VarArrayOf([GameNum,
                                            StrToIntDef(Team[team1].Items[player1].SubItems[tbPackID], -1),
                                            StrToIntDef(Team[team2].Items[player2].SubItems[tbPackID], -1)]), []) then
                                        for zone := 1 to 6 do
                                            hit[zone] := DM.HitTable.Fields[zone + 2].AsInteger  // copy the hit data fields out of DM.HitTable
                                    else
                                        for zone := 1 to 6 do
                                            hit[zone] := 0;  // fill in blanks for the hit data fields

                                    DM.WriteHitLine(GameNum + 2000, id1, id2, hit);
                                end;  // if not the same player
                            end;  // if 2.SubItems.Count > tbAccuracy; for player2
                    end;  // for team2
                end;  // if 1.SubItems.Count > tbAccuracy;
            end;  // for player1; for team1
    finally
        DM.HitTable.Close;
        DM.HitTable.ReadOnly := True;
        DM.StatsTable.Close;
        translatefrom.Free;
        translateto.Free;
    end;
end;

// Print scorecards for each player in each teambox.
procedure PrintScorecards(Team: TTeamBoxList; LeagueList: TLeagueList; GameNum: Integer; GameTime: TDateTime; GameDuration: Integer; GameTitle: string; ViewGcc: Boolean; Blue2, Blue3: Boolean; NumPrinters: Integer; DM: TParadoxDataModule);
var ColourTeam: TColourTeamArray;
    packids:    TStringList;
    i:          Integer;
    s1, s2, s3: string;
begin
    if FindTopThree(Team, ColourTeam, Blue2, Blue3) then begin
        packids := TStringList.Create;
        packids.CommaText := Build3TeamScoreCards(Team, LeagueList, ColourTeam, '', GameNum, GameTime, GameDuration, GameTitle, ViewGcc, DM);

        case NumPrinters of
            1:  SendUDP(12004, 'PRINT,1,1,"Game=' + IntToStr(GameNum + 2000) + ' AND PackID IN (' + packids.CommaText + ')",""');
            2: begin
                s1 := '';
                s2 := '';
                for i := 0 to packids.Count - 1 do  // break scorecards into two equal-length lists
                    if i mod 2 = 0 then
                        s1 := s1 + packids[i] + ','
                    else
                        s2 := s2 + packids[i] + ',';

                if Length(s1) > 0 then s1 := Copy(s1, 1, Length(s1) - 1);  // drop the trailing comma
                if Length(s2) > 0 then s2 := Copy(s2, 1, Length(s2) - 1);  // drop the trailing comma

                SendUDP(12004, 'PRINT,1,1,"Game=' + IntToStr(GameNum + 2000) + ' AND PackID IN (' + s1 + ')",""');
                SendUDP(12004, 'PRINT,2,2,"Game=' + IntToStr(GameNum + 2000) + ' AND PackID IN (' + s2 + ')",""');
            end;
            3: begin
                s1 := '';
                s2 := '';
                s3 := '';
                for i := 0 to packids.Count - 1 do  // break scorecards into three teams
                    case StrToIntDef(packids[i], 0) mod 3 of
                      0: s1 := s1 + packids[i] + ',';
                      1: s2 := s2 + packids[i] + ',';
                      2: s3 := s3 + packids[i] + ',';
                    end;

                if Length(s1) > 0 then s1 := Copy(s1, 1, Length(s1) - 1);  // drop the trailing comma
                if Length(s2) > 0 then s2 := Copy(s2, 1, Length(s2) - 1);  // drop the trailing comma
                if Length(s3) > 0 then s3 := Copy(s3, 1, Length(s3) - 1);  // drop the trailing comma

                SendUDP(12004, 'PRINT,1,1,"Game=' + IntToStr(GameNum + 2000) + ' AND PackID IN (' + s1 + ')",""');
                SendUDP(12004, 'PRINT,2,2,"Game=' + IntToStr(GameNum + 2000) + ' AND PackID IN (' + s2 + ')",""');
                SendUDP(12004, 'PRINT,3,3,"Game=' + IntToStr(GameNum + 2000) + ' AND PackID IN (' + s3 + ')",""');
            end;
        end;
    end else begin
        BuildNTeamScoreCards(Team, LeagueList, '0', GameNum, GameTime, GameDuration, GameTitle, DM);

        case NumPrinters of
            1:  SendUDP(12004, 'PRINT,1,1,"Game=' + IntToStr(GameNum + 2000) + ' AND PackID IN (1,2,4,5,7,8,10,11,13,14,16,17,19,20,22,23,25,26,28,29,31,32,34,35,37,38,40,41,43,44,46,47,49,50,52,53,55,56,58,59)",""');
            2: begin
                SendUDP(12004, 'PRINT,1,1,"Game=' + IntToStr(GameNum + 2000) + ' AND PackID IN (1,2,7,8,13,14,19,20,25,26,31,32,37,38,43,44,49,50,55,56)",""');
                SendUDP(12004, 'PRINT,2,2,"Game=' + IntToStr(GameNum + 2000) + ' AND PackID IN (4,5,10,11,16,17,22,23,28,29,34,35,40,41,46,47,52,53,58,59)",""');
            end;
            3: begin
                SendUDP(12004, 'PRINT,1,1,"Game=' + IntToStr(GameNum + 2000) + ' AND PackID IN (1,2,10,11,19,20,28,29,37,38,46,47,55,56)",""');
                SendUDP(12004, 'PRINT,2,2,"Game=' + IntToStr(GameNum + 2000) + ' AND PackID IN (4,7,13,16,22,25,31,34,40,43,49,52,58)",""');
                SendUDP(12004, 'PRINT,3,3,"Game=' + IntToStr(GameNum + 2000) + ' AND PackID IN (5,8,14,17,23,26,32,35,41,44,50,53,59)",""');
            end;
        end;
    end;
end;

// Print a scorecard for the selected player.
procedure PrintOneScorecard(Team: TTeamBoxList; LeagueList: TLeagueList; GameNum: Integer; GameTime: TDateTime; GameDuration: Integer; GameTitle: string; ViewGcc: Boolean; Blue2, Blue3: Boolean; TeamBox: TTeamBox; DM: TParadoxDataModule);
var ColourTeam: TColourTeamArray;
    playerid: string;
begin
    playerid := TeamBox.Selected.SubItems[tbPackID];

    if FindTopThree(Team, ColourTeam, Blue2, Blue3) then
        playerid := Build3TeamScoreCards(Team, LeagueList, ColourTeam, playerid, GameNum, GameTime, GameDuration, GameTitle, ViewGcc, DM)
    else
        playerid := BuildNTeamScoreCards(Team, LeagueList, playerid, GameNum, GameTime, GameDuration, GameTitle, DM);

    SendUDP(12004, 'PRINT,1,1,"Game=' + IntToStr(GameNum + 2000) + ' AND PackID=' + playerid + '",""');
end;

// Print a single summary scorecard for the game.
procedure PrintSummaryScorecard(Team: TTeamBoxList; LeagueList: TLeagueList; GameNum: Integer; GameTime: TDateTime; GameDuration: Integer; GameTitle: string; ViewGcc: Boolean; Blue2, Blue3: Boolean; DM: TParadoxDataModule);
var ColourTeam: TColourTeamArray;

    procedure Build3TeamSummaryCard(ColourTeam: TColourTeamArray);
    var TeamColour: array [1..60] of Integer;   // array showing what colour each team is.  This is the inverse of ColourTeam[].
        i, teamn, player, zone: Integer;        // loop indeces
        baseretaliation: Boolean;               // did base retaliation hit anybody during this game?
        teamname:   string;
        stats:      Variant;                    // variant array to hold entries that will be used to build the DM.StatsTable line for Red 19
        hit:        THitArray;
        numplayers: Integer;                    // total number of players in the game
    begin
        for i := 1 to Team.Count - 1 do
            TeamColour[i] := (i - 1) mod 3 + 1; // initialise the TeamColour array: 1, 2, 3, 1, 2, 3, ...

        for i := 1 to 3 do                      // set up to three values in the teamcolour array,
            if ColourTeam[i] <> -1 then         // showing what colour has been assigned
                TeamColour[ColourTeam[i]] := i; // to each of the top three teams

        DM.CleanStats(2000);
        stats := VarArrayOf([GameNum + 2000, 57, 1, '', Format('%d: Summary: %s', [GameNum, GameTitle]), 0, 0, 100, 0, 99, 0, GameTime, GameTitle, GameDuration, 1]);  // set up the DM.StatsTable line for Red 19, the "pack" whose card we will print for the summary
        VarArrayRedim(stats, 50);
        numplayers := 0;

        DM.HitTable.Close;
        DM.HitTable.ReadOnly := False;
        DM.HitTable.Open;
        try
            for teamn := 1 to Team.Count - 1 do begin  // for each Team box,
                for player := 0 to Team[teamn].Items.Count - 1 do begin  // for each player in the Team box,
                    if Team[teamn].Items[player].SubItems.Count > tbAccuracy then begin  // this is an Adjustment line or some such
                        if ViewGcc then begin
                            // Collect all the hits _on_ this player...
                            for zone := 1 to 6 do
                                hit[zone] := 0;

                            DM.HitTable.Filter := 'Game = ' + IntToStr(GameNum) + ' AND HitByID < 60 AND HitOnID = ' + Team[teamn].Items[player].SubItems[tbPackID];
                            DM.HitTable.Filtered := True;
                            DM.HitTable.First;
                            while not DM.HitTable.EOF do begin
                                for zone := 1 to 6 do
                                    hit[zone] := hit[zone] + DM.HitTable.Fields[zone + 2].AsInteger;  // copy the hit data fields out of DM.HitTable

                                DM.HitTable.Next;
                            end;

                            // ... and write them as a single record
                            DM.HitTable.Filtered := False;
                            DM.WriteHitLine(GameNum + 2000, 57, player * 3 + TeamColour[teamn] - 1, hit);

                            // Collect all the hits _by_ this player...
                            for zone := 1 to 6 do
                                hit[zone] := 0;

                            DM.HitTable.Filter := 'Game = ' + IntToStr(GameNum) + ' AND HitOnID < 60 AND HitByID = ' + Team[teamn].Items[player].SubItems[tbPackID];
                            DM.HitTable.Filtered := True;
                            DM.HitTable.First;
                            while not DM.HitTable.EOF do begin
                                for zone := 1 to 6 do
                                    hit[zone] := hit[zone] + DM.HitTable.Fields[zone + 2].AsInteger;  // copy the hit data fields out of DM.HitTable

                                DM.HitTable.Next;
                            end;

                            // ... and write them as a single record
                            DM.HitTable.Filtered := False;
                            DM.WriteHitLine(GameNum + 2000, player * 3 + TeamColour[teamn] - 1, 57, hit);
                        end else begin
                            DM.WriteHitNil(GameNum + 2000, 57, player * 3 + TeamColour[teamn] - 1);
                            DM.WriteHitNil(GameNum + 2000, player * 3 + TeamColour[teamn] - 1, 57);
                        end;  // if not ViewGcc

                        with Team[teamn].Items[player] do begin
                            if SubItems[tbMemberAlias] = '' then
                                DM.WriteStatsLine(VarArrayOf([GameNum + 2000, player * 3 + TeamColour[teamn] - 1, TeamColour[teamn], '', Caption, 0, 0, 100,//StrToIntDef(SubItems[tbHandicap], 100),
                                               GetHandiScore(SubItems), StrToIntDef(SubItems[tbRank], 0), StrToIntDef(SubItems[tbAccuracy], 0), GameTime, GameTitle, GameDuration, 1]))   // write a DM.StatsTable line like 'Red 9'
                            else
                                DM.WriteStatsLine(VarArrayOf([GameNum + 2000, player * 3 + TeamColour[teamn] - 1, TeamColour[teamn], SubItems[tbMemberID], SubItems[tbMemberAlias], 0, 0, 100,//StrToIntDef(SubItems[tbHandicap], 100),
                                               GetHandiScore(SubItems), StrToIntDef(SubItems[tbRank], 0), StrToIntDef(SubItems[tbAccuracy], 0), GameTime, GameTitle, GameDuration, 1]));  // write a DM.StatsTable line like 'RONiN 441'

                            stats[8] := Integer(stats[8]) + GetHandiScore(SubItems);  // add this player's score to Red 19's DM.StatsTable line
                            stats[10] := Integer(stats[10]) + StrToIntDef(SubItems[tbAccuracy], 0);  // add this player's accuracy to Red 19's DM.StatsTable line
                            for i := 7 to Min(37, SubItems.Count - 1) do  // add in all the remaining guff for Red 19's DM.StatsTable line
                                stats[i + 8] := Integer(stats[i + 8]) + StrToIntDef(SubItems[i], 0);
                            Inc(numplayers);
                        end;  // with
                    end else begin  // if SubItems.Count > tbAccuracy
                        DM.WriteStatsLine(VarArrayOf([GameNum + 2000, player * 3 + TeamColour[teamn] - 1, TeamColour[teamn], '', Team[teamn].Items[player].SubItems[tbMemberAlias], 0, 0, 100,
                                                    GetHandiScore(Team[teamn].Items[player].SubItems), 99, 0, GameTime, GameTitle, GameDuration, 1]));  // write a non-player line like 'Adjustment'
                        DM.WriteHitNil(GameNum + 2000, 57, player * 3 + TeamColour[teamn] - 1);
                        DM.WriteHitNil(GameNum + 2000, player * 3 + TeamColour[teamn] - 1, 57);
                    end;  // if SubItems.Count > tbAccuracy
                end;  // for player
                if (teamn <= 3) and (ColourTeam[teamn] <> -1) then begin
                    teamname := LeagueList.BuildTeamName(Team[ColourTeam[teamn]]);
                    if (Team[ColourTeam[teamn]].Handicap <> '0') and (Team[ColourTeam[teamn]].Handicap <> '100%') then
                        teamname := teamname + ' (' + Team[ColourTeam[teamn]].Handicap + ')';

                    DM.WriteStatsLine(VarArrayOf([GameNum + 2000, teamn + 59, 4, '', '', 0, 0, BuildTeamHandicap(Team[ColourTeam[teamn]]),
                                   Team[ColourTeam[teamn]].Score, 0, 0, GameTime, teamname, GameDuration, 0]));  // create a DM.StatsTable entry for the team
                end;
            end;  // for teamn

            baseretaliation := False;
            for i := tbRedBase1 to tbYelBase2 do
                if VarType(stats[i]) and varTypeMask = varInteger then
                    baseretaliation := baseretaliation or (stats[i] > 0);  // check to see if any of RedBase1HitYou .. YellowBase2HitYou are non-zero

            if ViewGcc and not baseretaliation then
                for i := 0 to 5 do begin
                    DM.HitTable.Filter := 'Game = ' + IntToStr(GameNum) + ' AND HitByID < 60 AND HitOnID = ' + IntToStr(i + 104);
                    DM.HitTable.Filtered := True;
                    DM.HitTable.First;
                    stats[i + 28] := 0;
                    while not DM.HitTable.EOF do begin
                        stats[i + 28] := stats[i + 28] + DM.HitTable.Fields[3].AsInteger;  // and put it in the matching field in the DM.StatsTable line
                        DM.HitTable.Next;
                    end;
                end;
        finally
            DM.HitTable.Filtered := False;
            DM.HitTable.Close;
            DM.HitTable.ReadOnly := True;
        end;
        try
            stats[8] := stats[8] div numplayers;    // convert Score from total to average
            stats[9] := numplayers + 1;             // set a fake rank
            stats[10] := stats[10] div numplayers;  // convert Accuracy from total to average
            for i := 15 to 27 do  // convert some DM.StatsTable fields (notably ShotsFired, TimesReloaded, PlayersHitYou, PlayersYouHit, NumberOfBases) from totals to averages
                if VarType(stats[i]) and varTypeMask = varInteger then
                    stats[i] := stats[i] div numplayers;
        except on EDivByZero do ;
        end;
        DM.WriteStatsLine(stats);  // write the DM.StatsTable line for Red 19
        DM.StatsTable.Close;
    end;

    procedure BuildNTeamSummaryCard;
    var i, teamn, player, zone: Integer;        // loop indeces
        translatefrom, translateto: TStringList;  // list of pack IDs as listed in the team box, and as we're going to list them in our DM.StatsTable and DM.HitTable entries.
        id1:        Integer;                    // IDs that we're going to use for these pack(s)
        baseretaliation: Boolean;               // did base retaliation hit anybody during this game?
        stats:      Variant;                    // variant array to hold entries that will be used to build the DM.StatsTable line for Red 19
        hit:        THitArray;
        numplayers: Integer;                    // total number of players in the game
    begin
        translatefrom := TStringList.Create;
        translateto   := TStringList.Create;
        BuildNTeamTranslate(Team, translatefrom, translateto);

        DM.CleanStats(2000);
        DM.HitTable.Close;
        DM.HitTable.ReadOnly := False;
        DM.HitTable.Open;

        // Create a DM.StatsTable entry for each each team box that has players in it.
        for teamn := 1 to Team.Count - 1 do begin // loop through each team box
            if Team[teamn].Items.Count <> 0 then  // if there are players on this team,
                DM.WriteStatsLine(VarArrayOf([GameNum + 2000, teamn * 3 - 3, 1, '', LeagueList.BuildTeamName(Team[teamn]), 0, 0, BuildTeamHandicap(Team[teamn]),
                               Team[teamn].Score, Team[teamn].Rank, BuildTeamHandicap(Team[teamn]) * 10, GameTime, GameTitle, GameDuration, 1]));  // create the entry.

            DM.WriteHitNil(GameNum + 2000, 57, teamn * 3 - 3);  // Write a record for hits on the summary player _by_ a team.  These appear in the red "team" section of the player's scorecard.  These records are necessary to force the scorecard to show the red "player" names and scores
        end;

        DM.WriteStatsLine(VarArrayOf([GameNum + 2000, 60, 4, '', '', 0, 0, 100, 0, 0, 0, GameTime, 'TEAMS', GameDuration, 0]));  // create a DM.StatsTable entry for the red "team"
        DM.WriteStatsLine(VarArrayOf([GameNum + 2000, 61, 4, '', '', 0, 0, 100, 0, 0, 0, GameTime, 'PLAYERS', GameDuration, 0]));  // create a DM.StatsTable entry for the green "team"
        if translatefrom.Count > 10 then  // we put player entries under the yellow "team",
            DM.WriteStatsLine(VarArrayOf([GameNum + 2000, 62, 4, '', '', 0, 0, 100, 0, 0, 0, GameTime, 'PLAYERS', GameDuration, 0]));  // so create a DM.StatsTable entry for the yellow "team"

        stats := VarArrayOf([GameNum + 2000, 57, 1, '', Format('%d: Summary: %s', [GameNum, GameTitle]), 0, 0, 100, 0, 99, 0, GameTime, GameTitle, GameDuration, 1]);  // set up the DM.StatsTable line for Red 19, the "pack" whose card we will print for the summary
        VarArrayRedim(stats, 50);
        numplayers := 0;

        try
            for teamn := 1 to Team.Count - 1 do // for each Team box,
                for player := 0 to Team[teamn].Items.Count - 1 do begin  // for each player in the Team box,
                    if Team[teamn].Items[player].SubItems.Count <= tbAccuracy then continue;  // this is an Adjustment line or some such.  Skip it.

                    id1 := StrToInt(translateto[translatefrom.IndexOf(Team[teamn].Items[player].SubItems[tbPackID])]);  // find the old pack ID in translatefrom, and pick out its match in translateto.

                    if ViewGcc then begin
                        // Collect all the hits _on_ this player...
                        for zone := 1 to 6 do
                            hit[zone] := 0;

                        DM.HitTable.Filter := 'Game = ' + IntToStr(GameNum) + ' AND HitByID < 60 AND HitOnID = ' + Team[teamn].Items[player].SubItems[tbPackID];
                        DM.HitTable.Filtered := True;
                        DM.HitTable.First;
                        while not DM.HitTable.EOF do begin
                            for zone := 1 to 6 do
                                hit[zone] := hit[zone] + DM.HitTable.Fields[zone + 2].AsInteger;  // copy the hit data fields out of DM.HitTable

                            DM.HitTable.Next;
                        end;

                        // ... and write them as a single record
                        DM.HitTable.Filtered := False;
                        DM.WriteHitLine(GameNum + 2000, 57, id1, hit);

                        // Collect all the hits _by_ this player...
                        for zone := 1 to 6 do
                            hit[zone] := 0;

                        DM.HitTable.Filter := 'Game = ' + IntToStr(GameNum) + ' AND HitOnID < 60 AND HitByID = ' + Team[teamn].Items[player].SubItems[tbPackID];
                        DM.HitTable.Filtered := True;
                        DM.HitTable.First;
                        while not DM.HitTable.EOF do begin
                            for zone := 1 to 6 do
                                hit[zone] := hit[zone] + DM.HitTable.Fields[zone + 2].AsInteger;  // copy the hit data fields out of DM.HitTable

                            DM.HitTable.Next;
                        end;
                        // ... and write them as a single record
                        DM.HitTable.Filtered := False;
                        DM.WriteHitLine(GameNum + 2000, id1, 57, hit);
                    end else begin
                        DM.WriteHitNil(GameNum + 2000, 57, id1);
                        DM.WriteHitNil(GameNum + 2000, id1, 57);
                    end;  // if not ViewGcc

                    with Team[teamn].Items[player] do begin
                        if SubItems[tbMemberAlias] = '' then
                            DM.WriteStatsLine(VarArrayOf([GameNum + 2000, id1, (id1 mod 3) + 1, '', Caption, 0, 0, 100,//StrToIntDef(SubItems[tbHandicap], 100),
                                              GetHandiScore(SubItems), StrToIntDef(SubItems[tbRank], 0), StrToIntDef(SubItems[tbAccuracy], 0), GameTime, GameTitle, GameDuration, 1]))   // write a DM.StatsTable line like 'Red 9'
                        else
                            DM.WriteStatsLine(VarArrayOf([GameNum + 2000, id1, (id1 mod 3) + 1, SubItems[tbMemberID], SubItems[tbMemberAlias], 0, 0, 100,//StrToIntDef(SubItems[tbHandicap], 100),
                                              GetHandiScore(SubItems), StrToIntDef(SubItems[tbRank], 0), StrToIntDef(SubItems[tbAccuracy], 0), GameTime, GameTitle, GameDuration, 1]));  // write a DM.StatsTable line like 'RONiN 441'

                        stats[8] := Integer(stats[8]) + GetHandiScore(SubItems);  // add this player's score to Red 19's DM.StatsTable line
                        stats[10] := Integer(stats[10]) + StrToIntDef(SubItems[tbAccuracy], 0);  // add this player's accuracy to Red 19's DM.StatsTable line
                        for i := 7 to Min(37, SubItems.Count - 1) do  // add in all the remaining guff for Red 19's DM.StatsTable line
                            stats[i + 8] := Integer(stats[i + 8]) + StrToIntDef(SubItems[i], 0);
                        Inc(numplayers);
                    end;  // with
                end;  // for player, for teamn

            baseretaliation := False;
            for i := tbRedBase1 to tbYelBase2 do
                if VarType(stats[i]) and varTypeMask = varInteger then
                    baseretaliation := baseretaliation or (stats[i] > 0);  // check to see if any of RedBase1HitYou .. YellowBase2HitYou are non-zero

            if ViewGcc and not baseretaliation then
                for i := 0 to 5 do begin
                    DM.HitTable.Filter := 'Game = ' + IntToStr(GameNum) + ' AND HitByID < 60 AND HitOnID = ' + IntToStr(i + 104);
                    DM.HitTable.Filtered := True;
                    DM.HitTable.First;
                    stats[i + 28] := 0;
                    while not DM.HitTable.EOF do begin
                        stats[i + 28] := stats[i + 28] + DM.HitTable.Fields[3].AsInteger;  // and put it in the matching field in the DM.StatsTable line
                        DM.HitTable.Next;
                    end;
                end;
        finally
            DM.HitTable.Filtered := False;
            DM.HitTable.Close;
            DM.HitTable.ReadOnly := True;
            translatefrom.Free;
            translateto.Free;
        end;

        try
            stats[8] := stats[8] div numplayers;    // convert Score from total to average
            stats[9] := numplayers + 1;             // set a fake rank
            stats[10] := stats[10] div numplayers;  // convert Accuracy from total to average
            for i := 15 to 27 do  // convert some DM.StatsTable fields (notably ShotsFired, TimesReloaded, PlayersHitYou, PlayersYouHit, NumberOfBases) from totals to averages
                if VarType(stats[i]) and varTypeMask = varInteger then
                    stats[i] := stats[i] div numplayers;
        except on EDivByZero do ;
        end;

        DM.WriteStatsLine(stats);  // write the DM.StatsTable line for Red 19
        DM.StatsTable.Close;
    end;

begin  //---PrintSummaryClick---
    if FindTopThree(Team, ColourTeam, Blue2, Blue3) then
        Build3TeamSummaryCard(ColourTeam)
    else
        BuildNTeamSummaryCard;

    SendUDP(12004, 'PRINT,1,1,"Game=' + IntToStr(GameNum + 2000) + ' AND PackID=57",""');
end;


end.
