unit TornTeamBox1;

interface

uses Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, StdCtrls,
     ComCtrls;

const
// Team box subitems:
    tbMemberAlias = 0;
    tbScore       = 1;
    tbMemberID    = 2;
    tbRank        = 3;
    tbFriendly    = 4;
    tbAccuracy    = 5;
    tbPackID      = 6;
    tbStatsStart  = 7;
    tbHandicap    = 8  + 7;
    tbHitYou      = 26 + 7;
    tbYouHit      = 27 + 7;
    tbBaseHits    = 28 + 7;
    tbBaseDestroys= 29 + 7;
    tbStatsEnd    = 29 + 7;
    tbRedBase1    = 28;
    tbYelBase2    = 33;

type
  TTeamBox = class(TListView)
  private
    FRank:     Integer;
    FRankText: TStaticText;

    FScore:    Integer;
    FScoreText: TStaticText;

    FHandicap: TEdit;
    FHandiDefault: char;

    FVictoryPoints: Double;
    FVictoryHandicap: Double;

    FOnResize: TNotifyEvent;
    FOnHandicapChange: TNotifyEvent;

    procedure SetRank(Value: Integer);
    procedure SetScore(Value: Integer);
    procedure SetVictoryPoints(const Value: Double);
    function  GetTeamName: String;
    procedure SetTeamName(Value: String);
    function  GetHandicap: String;
    procedure SetHandicap(Value: String);
    function  GetHandicapVisible: Boolean;
    procedure SetHandicapVisible(Value: Boolean);
    procedure SetHandiDefault(Value: char);
    function  GetHandicapColor: TColor;
    procedure SetHandicapColor(Value: TColor);

    procedure HandicapChange(Sender: TObject);
    procedure HandicapExit(Sender: TObject);
    procedure HandicapKeyPress(Sender: TObject; var Key: Char);
    procedure RefreshRank;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure Resize; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure Clear;
    function CalcScore: Integer;

    property Rank:     Integer  read FRank        write SetRank;
    property Score:    Integer  read FScore       write SetScore;
    property VictoryPoints: Double read FVictoryPoints write SetVictoryPoints;
    property TeamName: String   read GetTeamName    write SetTeamName;
    property Handicap: String   read GetHandicap    write SetHandicap;
    property VictoryHandicap: Double read FVictoryHandicap write FVictoryHandicap;
    property HandicapVisible: Boolean read GetHandicapVisible write SetHandicapVisible;
    property HandiDefault: char read FHandiDefault  write SetHandiDefault;
    property HandicapColor: TColor read GetHandicapColor write SetHandicapColor;
  published
    property OnResize: TNotifyEvent read FOnResize write FOnResize;
    property OnHandicapChange: TNotifyEvent read FOnHandicapChange write FOnHandicapChange;
  end;

type TTeamBoxList = class(TList)
  protected
    procedure Put(Index: Integer; Item: TTeamBox);
    function  Get(Index: Integer): TTeamBox;

  public
    procedure Add(Item: TTeamBox);
    property Items[Index: Integer]: TTeamBox read Get write Put; default;
  end;

procedure Register;

implementation

procedure Register;
begin
    RegisterComponents('Samples', [TTeamBox]);
end;

{ TTeamBox ------------------------------------------------------------------}

constructor TTeamBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  ColumnClick := False;
  DragMode    := dmAutomatic;
  Font.Charset:= DEFAULT_CHARSET;
  Font.Color  := clWindowText;
  Font.Height := -11;
  Font.Name   := 'MS Sans Serif';
  Font.Style  := [];
  MultiSelect := True;
  ParentFont  := False;
  ReadOnly    := True;
  RowSelect   := True;
  SortType    := stNone;
  ViewStyle   := vsReport;
  Columns.Add;
  Columns[0].Width := 54;
  Columns.Add;
  Columns[1].Caption := 'Alias';
  Columns[1].Width := 74;
  Columns.Add;
  Columns[2].Alignment := taRightJustify;
  Columns[2].Width := 42;

  FRank := 0;
  FRankText := TStaticText.Create(Self);
  with FRankText do begin
    Parent      := Self;
    Left        := 1;
    Top         := 0;
    Width       := 19;
    Height      := 15;
    AutoSize    := False;
    Caption     := ' ';
    Color       := clBtnFace;
    Font.Charset := DEFAULT_CHARSET;
    Font.Color  := clWindowText;
    Font.Height := -13;
    Font.Name   := 'MS Sans Serif';
    Font.Style  := [fsBold];
    ParentFont  := False;
    PopupMenu   := Self.PopUpMenu;
  end;

  FScore := 0;
  FScoreText := TStaticText.Create(Self);
  with FScoreText do begin
    Parent      := Self;
    Left        := 116;
    Top         := 0;
    Width       := 50;
    Height      := 15;
    Alignment   := taRightJustify;
    AutoSize    := False;
    Caption     := '0';
    Color       := clBtnFace;
    Font.Charset := DEFAULT_CHARSET;
    Font.Color  := clWindowText;
    Font.Height := -13;
    Font.Name   := 'MS Sans Serif';
    Font.Style  := [fsBold];
    ParentFont  := False;
    PopupMenu   := Self.PopUpMenu;
  end;

  FHandicap := TEdit.Create(Self);
  with FHandicap do begin
    Parent      := Self;
    Left        := 20;
    Top         := -2;
    Width       := 41;
    Height      := 19;
    AutoSize    := False;
    HelpContext := 8;
    Hint        := 'Fill in a number preceded by + or - for a point handicap;'#13'or followed by % for a percentage handicap.';
    ParentShowHint := False;
    ShowHint    := True;
    Text        := '0';
    Visible     := False;
    PopupMenu   := Self.PopUpMenu;
    OnChange    := HandicapChange;
    OnKeyPress  := HandicapKeyPress;
    OnExit      := HandicapExit;
  end;

  FHandiDefault := '+';
end;

destructor TTeamBox.Destroy;
begin
  FRankText.Free;
  FScoreText.Free;
  FHandicap.Free;

  inherited Destroy;
end;

procedure TTeamBox.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);

  Params.Width := 190;
  Params.Height := 77;
end;

function TTeamBox.GetHandicap: String;
begin
  Result := FHandicap.Text;
end;

procedure TTeamBox.SetHandicap(Value: String);
begin
  FHandicap.Text := Value;
end;

function TTeamBox.GetHandicapVisible: Boolean;
begin
  Result := FHandicap.Visible;
end;

function TTeamBox.GetHandicapColor: TColor;
begin
  Result := FHandicap.Color;
end;

procedure TTeamBox.SetHandicapColor(Value: TColor);
begin
  FHandicap.Color := Value;
end;

procedure TTeamBox.SetHandicapVisible(Value: Boolean);
begin
  FHandicap.Visible := Value;
  if Value then
    FHandicap.SetFocus
  else
    FHandicap.Text := '0';
end;

procedure TTeamBox.SetHandiDefault(Value: char);
begin
  if Value in ['+', '-', '%'] then
    FHandiDefault := Value;
end;

procedure TTeamBox.SetRank(Value: Integer);
begin
  FRank := Value;
  RefreshRank;
end;

procedure TTeamBox.RefreshRank;
begin
  if FRank = 0 then             FRankText.Caption := ' '
  else if FRank mod 100 in [11..13] then  FRankText.Caption := IntToStr(FRank) + 'th'
  else if FRank mod 10 = 1 then FRankText.Caption := IntToStr(FRank) + 'st'
  else if FRank mod 10 = 2 then FRankText.Caption := IntToStr(FRank) + 'nd'
  else if FRank mod 10 = 3 then FRankText.Caption := IntToStr(FRank) + 'rd'
  else                          FRankText.Caption := IntToStr(FRank) + 'th';

  if FVictoryPoints <> 0 then
    FRankText.Caption := FRankText.Caption + ' (' + FloatToStr(FVictoryPoints) + ' pts)';

  FRankText.Width := Canvas.TextWidth(FRankText.Caption) * 3 div 2;  // The 3/2 is to fudge for boldface.
end;

procedure TTeamBox.SetScore(Value: Integer);
begin
  FScore := Value;
  Columns[2].Caption := IntToStr(Value);
  FScoreText.Caption := IntToStr(Value);
end;

function TTeamBox.GetTeamName: String;
begin
  Result := Columns[1].Caption;
end;

procedure TTeamBox.SetTeamName(Value: String);
begin
  Columns[1].Caption := Value;
  FScoreText.Invalidate;  // force repaint of team score, in case it got blatted when we painted the new team name
end;

procedure TTeamBox.SetVictoryPoints(const Value: Double);
begin
  FVictoryPoints := Value;
  RefreshRank;
end;

procedure TTeamBox.Clear;
begin
  Items.Clear;

  Rank  := 0;
  Score := 0;
  VictoryPoints := 0;
  FVictoryHandicap := 0;
  TeamName := 'Alias';
  HandicapVisible := False;  // this also sets the Handicap to 0 points
  Color := clWindow;
  HandicapColor := clWindow;
end;

procedure TTeamBox.Resize;
begin
  if Assigned(FOnResize) then FOnResize(Self);

  if Items.Count > VisibleRowCount then begin
    Columns[1].Width := Width - 116;
    FScoreText.Left := Width - 74;
  end else begin
    Columns[1].Width := Width - Columns[0].Width - Columns[2].Width - 4;
    FScoreText.Left := Width - 59;
  end;

  Scroll(0,0);
  FRankText.Invalidate;
  FScoreText.Invalidate;
  FHandicap.Invalidate;
end;

// Validate the number in a Handicap field.  It may begin with a '+' or '-',
// or end with a '%' (but not both); and must otherwise consist of digits.
procedure TTeamBox.HandicapChange(Sender: TObject);
var i: Integer;
    s: String;
    place: Integer;
    plusminus: Boolean;
begin
  with Sender as TEdit do
    if Modified and (Length(Text) > 0) then begin
      place := SelStart;  // SelStart is the cursor position; so this remembers our place.
      s := Text;
      while (Length(s) > 0) and not (s[1] in ['+', '-', '0'..'9']) do begin
        s := Copy(s, 2, 7);  // truncate the first character off s
        if place > 1 then Dec(place);
      end;

      plusminus := (Length(s) > 0) and (s[1] in ['+', '-']);

      i := 2;
      while (Length(s) > 1) and (i < Length(s)) do begin
        if not (s[i] in ['0'..'9']) then begin
          s := Copy(s, 1, i-1) + Copy(s, i+1, 7);  // cut the i'th character out of s
          if place > i then Dec(place);
        end;
        i := i + 1;
      end;

      if (Length(s) > 1) and not (s[Length(s)] in ['%', '0'..'9']) then
        s := Copy(s, 1, Length(s) - 1);  // truncate the last character off s

      if (Length(s) > 1) and plusminus and (s[Length(s)] = '%') then
        s := Copy(s, 1, Length(s) - 1);  // truncate the last character off s

      if (Length(s) > 5) and (s[Length(s)] = '%') then
        s := Copy(s, 1, 4) + '%'  // keep percent handicaps to a max of 9999%
      else if (Length(s) > 7) and plusminus then
        s := Copy(s, 1, 7)  // keep point handicaps to a max of +/- 99999
      else if (Length(s) > 6) and not plusminus then
        s := Copy(s, 1, 6);  // keep as-yet-unsigned point handicaps to a max of 99999

      Text := s;
      SelStart := place;  // Restore the cursor to wherever it was when we started.
    end;  // if Modified and Length(Text) > 0
end;

// Check as focus leaves a Handicap field.  Make sure there is a '+', '-' or '%',
// and remove leading zeroes.
procedure TTeamBox.HandicapExit(Sender: TObject);
var s: String;
    val:     Integer;
    plus:    Boolean;
    minus:   Boolean;
    percent: Boolean;
begin
  with Sender as TEdit do begin
    s := Text;
    plus    := (Length(s) > 0) and (s[1] = '+');
    minus   := (Length(s) > 0) and (s[1] = '-');
    percent := (Length(s) > 0) and (s[Length(s)] = '%');

    if plus or minus then s:= Copy(s, 2, 7);

    if Length(s) = 0 then val := 0
    else if percent then  val := StrToIntDef(Copy(s, 1, Length(s)-1), 100)
    else                  val := StrToIntDef(s, 0);

    if percent then                     Text := IntToStr(val) + '%'
    else if val = 0 then                Text := '0'
    else if minus then                  Text := '-' + IntToStr(val)
    else if plus then                   Text := '+' + IntToStr(val)
    else if FHandiDefault = '+' then    Text := '+' + IntToStr(val)
    else if FHandiDefault = '-' then    Text := '-' + IntToStr(val)
    else { must be '%' }                Text := IntToStr(val) + '%';
  end;  // with Sender

  CalcScore;
  if Assigned(FOnHandicapChange) then FOnHandicapChange(Self);
end;

// Cause a call to HandicapExit when the user presses Enter.
procedure TTeamBox.HandicapKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = #13 then Self.SetFocus;
end;

// Recalculate this team's score (including handicap),
// and make the handicap field invisible if necessary.
function TTeamBox.CalcScore: Integer;
var s:    String;
    i:    Integer;
    adjust: Integer;
    col0width, col2width: Integer;

  // get a player's score out of a TeamBox SubItem
  function GetScore(SubItems: TStrings): Integer;
  begin
    if Pos(' (', SubItems[1]) = 0 then
      Result := StrToIntDef(SubItems[1], 0)
    else
      Result := StrToIntDef(Copy(SubItems[1],
                                 Pos(' (', SubItems[1]) + 2,
                                 Pos(')',  SubItems[1]) - Pos(' (', SubItems[1]) - 2),  // yuck
                            0);
  end;

begin
  adjust := 0;  // recalculate the Team's score
  FScore := 0;
  for i := 0 to Items.Count - 1 do
    if Items[i].SubItems[0] = 'Adjustment' then
      adjust := adjust + StrToIntDef(Items[i].SubItems[1], 0)
    else
      FScore := FScore + GetScore(Items[i].SubItems);

  if Self.Items.Count = 0 then begin
    FHandicap.Text := '0';
    FHandicap.Visible := False;
  end;

  s := FHandicap.Text;
  if (Length(s) > 0) then begin
    if s[Length(s)] = '%' then  // it's a percent
      FScore := adjust + FScore * StrToIntDef(Copy(s, 1, Length(s)-1), 0) div 100  // adjustment scores are not multiplied by the handicap
    else  // it's a plus/minus
      FScore := adjust + FScore + StrToIntDef(s, 0);
  end;

  col0width := 42;
  col2width := 42;
  for i := 0 to Items.Count - 1 do begin
    if col0width < Canvas.TextWidth(Items[i].Caption) + 22 then
      col0width := Canvas.TextWidth(Items[i].Caption) + 22;
    if col2width < Canvas.TextWidth(Items[i].SubItems[1]) + 12 then
      col2width := Canvas.TextWidth(Items[i].SubItems[1]) + 12;
  end;

  Columns[0].Width := col0width;
  Columns[2].Width := col2width;
  Columns[1].Width := Width - col2width - col0width - 4;

  Columns[2].Caption := IntToStr(FScore);
  FScoreText.Caption := IntToStr(FScore);

  Result := FScore;
end;

{ TTeamBoxList --------------------------------------------------------------}

procedure TTeamBoxList.Add(Item: TTeamBox);
begin
  inherited Add(Item);
end;

procedure TTeamBoxList.Put(Index: Integer; Item: TTeamBox);
begin
  inherited Put(Index, Item);
end;

function TTeamBoxList.Get(Index: Integer): TTeamBox;
begin
  Result := inherited Get(Index);
end;

end.
