// Torn is a tournament scores manager for Acacia / Infusion, Nexus, Helios and Laserforce, written in Delphi 5.
// It is written by and copyright Doug Burbidge, 1998 - 2020.

{$DEFINE DEBUG}

unit TornMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  StdCtrls, ComCtrls, ExtCtrls, Menus,
  Registry, Dialogs, IniFiles, ImgList, Math, Contnrs, Printers,
  TornTeamBox1, ZoomReport, ReportDialog, LadderDialog, DramaDialog, PyramidDialog, FixtureDialog,
  Buttons, Spin, Utility, League, Games, Scoreboard, WinSock,
  Db, ZAbstractRODataset, ZDataset, ZConnection, Psock{, NMFtp1, NMFtp};

const
    ReadAcaciaMsg =  WM_USER + 99;  // Assign message number for port 12050 listening
    TBOARD_PORT = 21570;
    LASERFORCE_PORT = 12080;
    ADJUSTMENT_IMAGE = 48;
type
    TLadder = record    // used when sorting league ladder for display on scoreboard
        Name: string;
        Score:  Integer;
        Points: Real;
    end;
    PLadder = ^TLadder;

type
  TSystemType = (stAcacia, stNexus, stNexusWithZeon, stLaserforce, stDemo);

  TMainForm = class(TForm)
    TeamBoxImageList: TImageList;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    PrintDialog1: TPrintDialog;
    SaveReportsDialog: TSaveDialog;
    MainMenu1: TMainMenu;
        LeagueMenu: TMenuItem;
            NewLeague: TMenuItem;
            OpenLeague: TMenuItem;
            SaveLeague2: TMenuItem;
            ShowLeague: TMenuItem;
            CloseLeague: TMenuItem;
            EditGames: TMenuItem;
            ReportLeague: TMenuItem;
            Pyramid: TMenuItem;
            Fixtures: TMenuItem;
            N1: TMenuItem;
            RememberAllTeams: TMenuItem;
            N2: TMenuItem;
            MRU1: TMenuItem;
            MRU2: TMenuItem;
            MRU3: TMenuItem;
            MRU4: TMenuItem;
            MRU5: TMenuItem;
            MRU6: TMenuItem;
            MRU7: TMenuItem;
            MRU8: TMenuItem;
            MRU9: TMenuItem;
            N3: TMenuItem;
            Exit1: TMenuItem;
        HandicapMenu: TMenuItem;
            HandicapAll: TMenuItem;
            HandicapNone: TMenuItem;
        ViewMenu: TMenuItem;
            ViewGcc: TMenuItem;
            ViewHistorical: TMenuItem;
            N5: TMenuItem;
            Preferences: TMenuItem;
        ScoreboardMenu: TMenuItem;
            UpdateScoreboard: TMenuItem;
            HideTBoard: TMenuItem;
            SortTeams: TMenuItem;
            N6: TMenuItem;
            HandicapThisTeam: TMenuItem;
            RememberThisTeam: TMenuItem;
            UpdateThisTeam: TMenuItem;
            NameThisTeam: TMenuItem;
            IdentifyThisTeam: TMenuItem;
            IdentifyThisPlayer: TMenuItem;
            N7: TMenuItem;
            AddAnAdjustment: TMenuItem;
    AdjustVictoryPoints: TMenuItem;
        HelpMenu: TMenuItem;
            Help: TMenuItem;
            About: TMenuItem;
    AllPlayersPopupMenu: TPopupMenu;
        SortTeams2: TMenuItem;
    TeamboxPopupMenu: TPopupMenu;
        SortTeamsP: TMenuItem;
        N8: TMenuItem;
        HandicapThisTeamP: TMenuItem;
        RememberThisTeamP: TMenuItem;
        UpdateThisTeamP: TMenuItem;
        NameThisTeamP: TMenuItem;
        IdentifyThisTeamP: TMenuItem;
        IdentifyThisPlayerP: TMenuItem;
        N9: TMenuItem;
        AddAnAdjustmentP: TMenuItem;
    Bevel1: TBevel;
    GamesCombo: TComboBox;
    GamesComboSpin: TSpinButton;
    Latest: TBitBtn;
    UpdateSB: TBitBtn;
    Commit: TButton;
    AllPlayers: TListView;
    MenuImageList: TImageList;
    HandicapThisPlayer: TMenuItem;
    HandicapThisPlayerP: TMenuItem;
    PlayerDetailPanel: TPanel;
    PlayerDetailName: TLabel;
    PlayerDetailScore: TLabel;
    PlayerDetailHandicap: TLabel;
    PlayerDetailTeam: TLabel;
    ZConnection: TZConnection;
    NexusGameTimer: TTimer;
    AdjustVictoryPointsP: TMenuItem;
    N4: TMenuItem;
    MergePlayersP: TMenuItem;
    Match1: TMenuItem;
    RememberAllTeams2: TMenuItem;
    Match2: TMenuItem;
    TeamboxMenu: TMenuItem;
    N10: TMenuItem;
    MergePlayers1: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure LatestClick(Sender: TObject);
    procedure GamesComboChange(Sender: TObject);
    procedure GamesComboDrawItem(Control: TWinControl; Index: Integer; Rect: TRect; State: TOwnerDrawState);
    procedure PlayersDragOver(Sender, Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
    procedure PlayersDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure PlayersDblClick(Sender: TObject);
    procedure PlayersSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
    procedure PlayersCompare(Sender: TObject; Item1, Item2: TListItem; Data: Integer; var Compare: Integer);
    procedure AllPlayersCompare(Sender: TObject; Item1, Item2: TListItem; Data: Integer; var Compare: Integer);
    procedure HandicapChange(Sender: TObject);
    procedure SortTeamsClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure UpdateScoreboardClick(Sender: TObject);
    procedure AllPlayersClick(Sender: TObject);
    procedure TeamboxMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure TeamboxMenuPopup(Sender: TObject);
    procedure HandicapClick(Sender: TObject);
    procedure HandicapAllClick(Sender: TObject);
    procedure HandicapNoneClick(Sender: TObject);
    procedure RememberAllTeamsClick(Sender: TObject);
    procedure RememberClick(Sender: TObject);
    procedure UpdateClick(Sender: TObject);
    procedure ForgetClick(Sender: TObject);
    procedure AdjustClick(Sender: TObject);
    procedure RenameClick(Sender: TObject);
    procedure IdPlayerClick(Sender: TObject);
    procedure HelpClick(Sender: TObject);
    procedure AboutClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Exit1Click(Sender: TObject);
    procedure ShowLeagueClick(Sender: TObject);
    procedure NewLeagueClick(Sender: TObject);
    procedure SaveLeague2Click(Sender: TObject);
    procedure OpenLeagueClick(Sender: TObject);
    procedure CloseLeagueClick(Sender: TObject);
    procedure EditGamesClick(Sender: TObject);
    procedure IdentifyTeam(Sender: TObject);
    procedure CommitClick(Sender: TObject);
    procedure ViewGamesClick(Sender: TObject);
    procedure GamesComboKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure GamesComboSpinDownClick(Sender: TObject);
    procedure GamesComboSpinUpClick(Sender: TObject);
    procedure PreferencesClick(Sender: TObject);
    procedure ReportLeagueClick(Sender: TObject);
    procedure ExceptionHandler(Sender: TObject; E: Exception);
    procedure HandicapPlayerClick(Sender: TObject);
    procedure MRUClick(Sender: TObject);
    procedure HideTBoardClick(Sender: TObject);
    procedure PlayerDetailNameClick(Sender: TObject);
    procedure PyramidClick(Sender: TObject);
    procedure FixturesClick(Sender: TObject);
    procedure NexusGameTimerTimer(Sender: TObject);
    function NexusGameInProgress: Boolean;
    procedure PlayerDetailPanelClick(Sender: TObject);
    procedure AjustVictoryPointsClick(Sender: TObject);
    procedure MergePlayersClick(Sender: TObject);
    procedure MatchClick(Sender: TObject);
    procedure CommitMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
//    procedure NMFTP1Failure(var Handled: Boolean; Trans_Type: TCmdType);
//    procedure NMFTP1Success(Trans_Type: TCmdType);
  private
    Team:       TTeamBoxList;   // list of Team box controls currently displayed
    GameNum:    Integer;        // currently selected game
    GameTime:   TDateTime;      // currently selected game
    GameTitle:  string;         // title of the currently selected game
    GameDuration: Integer;      // duration of the currently selected game
    GameHits:   Integer;        // total number of hits scored in this game; used for calculating T2 Effectiveness
    DefHandi:   array [0..NUM_COLOURS-1] of Integer;  // default handicap for each colour RGYB
    TeamboxSelected: Integer;   // last team box that was clicked on, for TeamboxMenu options
    StaleColor: TColor;         // colour to set team boxes to when they're not up to date
    LeagueMode: Boolean;        // is there a League open?
    LeagueList: TLeagueList;    // list of League teams currently remembered
    GameList:   TGameList;      // list of games played in this league
    ServerGameList: TServerGameList;  // list of games from the system game server
    MRUList:    TStringList;    // Most Recently Used files list
    Blue2, Blue3: Boolean;      // used to remember if the second or third teams are Blue
    FFChest, FFBack, FFLeft, FFRight, FFLaser: Integer;  // used to remember Friendly Fire penalty rates
    DoCalc:     Boolean;        // used to speed processing of DoLeague by only calculating scores once
    ReportForm: TReportForm;    // used to ask user what reports they want
    LadderForm: TLadderForm;    // used to display reports on screen
    DramaForm:  TDramaForm;     // used for Drama Mode
    PyramidForm: TPyramidForm;  // used for creation of pyramid rounds (solos, doubles, triples rounds and repechages)
    FixtureForm: TFixtureForm;  // used for creation of league fixtures: set up who plays who ahead of time
    Exceptions: string;
    mySocket:   TSocket;        // handle to the Windows socket used to listen for port 12050/12100 broadcasts
    GameInProgress: Boolean;    // is there a game playing right now?
    ViewingInProgress: Boolean; // Is the game we are curently viewing the game in progress?
    TimeSinceStart: TDateTime;  // Infusion: if there's a game playing right now, when did it start?  Nexus: how much time is remaining?
    StatusDisplayCount: Integer; // used for managing visiblity of labels that conflict with Status Display at bottom left
    IsSaved:    Boolean;        // has this league file had any changes saved to it yet?  Used to manage backups.
    SekritsVisible: Boolean;    // will we show the Pack Report checkbox on TReportForm?
    FSystemType: TSystemType;
    FHeliosType: Integer;       // What DB Version is this? (Helios schema changed in update 47, so =>47 means new membership style.)
    TeamLadderReport: TZoomReport;  // Pointer to the team ladder, if there is one, so its ranking can be used by other reports.
    FShiftState: TShiftState;

    procedure ConnectDB;
    procedure ListenForAcaciaStatus();
    function  RegReadBoolDef(R: TRegistry; Name: String; Default: Boolean): Boolean;
    function  RegReadIntDef(R: TRegistry; Name: String; Default: Integer): Integer;
    function  RegReadStringDef(R: TRegistry; Name, Default: string): string;
    function  GetScore(SubItems: TStrings): Integer;
    procedure DoMRU(R: TRegistry; Name: string; MRUMenuItem: TMenuItem);
    procedure UpdateMRU(MRUMenuItem: TMenuItem; Number: Integer);
    procedure CreateTeamBox(Num: Integer);
    procedure LoadHistorical;
    procedure InitTeamBoxes;
    procedure AddAdjustment(TeamNum, valu: Integer);
    procedure DoColourTransfer;
    procedure DoLeagueTransfer;
    procedure RankTeams;
    function  MixColors(Colors: array of TColor; Weights: array of Integer): TColor;
    procedure CalcTeamScore(TeamNum: Integer; DoName: Boolean);
    procedure CalcAllScores(Recalc, DoName: Boolean);
    function  FindAdjust(TeamNum: Integer): Integer;
    procedure BuildIdentifyMenu;
    procedure LeagueEnable;
    procedure SaveLeague;
    procedure LoadLeague;
    procedure CommitPlayer(Player: TListItem; Team: Integer);
    procedure BuildGamesCombo;
    procedure GridSize(OldSize: Integer);
    procedure SetPlayerHandicap(v: string);
    procedure BuildDetailedGame(Reports: TZoomReports; From, Too: TDateTime; Description: Boolean);
    procedure DoReportLeague(LeagueTitle: string; Prefs: TPrefs; LeagueList: TLeagueList; Application: TApplication; Screen: TScreen; Printer: TPrinter; OwnerForm: TForm; (*ReportForm: TReportForm; LadderForm: TLadderForm;*) PrintDialog: TPrintDialog; SaveDialog: TSaveDialog);
    procedure SocketReadAcacia(var Msg : TMessage); message ReadAcaciaMsg;  // Handler for FD_Read message
    procedure Debug(s: string);
    procedure AdjustAllPlayersColumns;
    function SetThisTeamColour(thisgame, thisteam: Integer; Colour: string; Color: Integer): TNexusColour;
    function GetNumTeamColourIndex(gamenum, teamnum: Integer): TNexusColour;
//    procedure UploadToFtp(filename: string);

  public
    Prefs:      TPrefs;         // stores global preferences

    property SystemType: TSystemType read FSystemType;
    property HeliosType: Integer read FHeliosType write FHeliosType;
  end;

  function  TornForm: TMainForm;

var
    MainForm:   TMainForm;    // this form
    CompareColumn: Integer;   // Used by CompareTeams.
    GamesColumn: Integer;   // Used by CompareTeams.
    ColumnDirection : Integer;
    PyramidSet: set of Byte;  // Used by ComparePyramid.

implementation

uses Reports, Lists,
     AdjustDialog, AliasDialog, AliasDBDialog, AliasLaserforceDialog, FriendlyDialog,
     GamesDialog, LeagueDialog, PrefsDialog, RenameDialog, RestoreDialog, MatchDialog, TornSplash;
{$R *.DFM}

// Return the global MainForm variable, thus allowing any form to access this one.
function TornForm: TMainForm;
begin
    Result := MainForm;
end;

// Free the passed-in object, and suppress any exception.
// This is used because with latest Firebird, ZEOS' query object throws an access violation when freed.
procedure FreeSquish(Obj: TObject);
begin
    try
        Obj.Free;
    finally
        // Squish.
    end;
end;

// Create the main form:
// read preferences from the registry;
// create the Team boxes;
// size the window;
// build the games list;
// do some misc init.
procedure TMainForm.FormCreate(Sender: TObject);
var i: Integer;
    MyRegistry: TRegistry;
    SystemType: string;
begin
    Exceptions := '""';
    Application.OnException := ExceptionHandler;

    if not Assigned(MainForm) then
        MainForm := Self;

    MyRegistry := TRegistry.Create;  // open the Registry
    try
        MyRegistry.RootKey := HKEY_LOCAL_MACHINE;
        if not MyRegistry.KeyExists('Software\DougWare\Torn') then
            MyRegistry.CreateKey('Software\DougWare\Torn');  // registry key for Torn doesn't exist yet; create it.

        MyRegistry.OpenKey('Software\DougWare\Torn', True);
        Prefs.GridHigh    := RegReadIntDef(MyRegistry, 'GridHigh',     5);
        Prefs.GridWide    := RegReadIntDef(MyRegistry, 'GridWide',     2);
        Prefs.GridPlayers := RegReadIntDef(MyRegistry, 'GridPlayers',  4);

        Prefs.SortMode    := RegReadIntDef(MyRegistry, 'SortMode',     1);
        if not Prefs.SortMode in [0, 1, 2] then Prefs.SortMode := 1;  // legal values are 0, 1 and 2.

        with Prefs do begin
            HandicapStyle         := RegReadStringDef(MyRegistry, 'HandicapStyle',    '%')[1];  // the [1] at the end is to convert string to char.
            SortTeamsByRank       := RegReadBoolDef(MyRegistry,   'SortByRank',       False);
            NexusScores           := RegReadBoolDef(MyRegistry,   'NexusScores',      True);
            NexusOneShotDenyFriend:= RegReadIntDef(MyRegistry,    'NexusOneShotDenyFriend', 103);
            NexusOneShotDenyEnemy := RegReadIntDef(MyRegistry,    'NexusOneShotDenyEnemy', 101);
            NexusTwoShotDenyFriend:= RegReadIntDef(MyRegistry,    'NexusTwoShotDenyFriend', 104);
            NexusTwoShotDenyEnemy := RegReadIntDef(MyRegistry,    'NexusTwoShotDenyEnemy', 102);
            SoloHandicapsShowPercentage := RegReadBoolDef(MyRegistry, 'SoloHandicapsShowPercentage', True);
            AutoUpdateScoreboard  := RegReadBoolDef(MyRegistry,   'AutoUpdate',       True);
            AutoUpdateTeams       := RegReadBoolDef(MyRegistry,   'UpdateTeams',      True);
            ScoreboardStyle       := TScoreboardStyle(RegReadIntDef(MyRegistry,    'ScoreboardStyle', 0));
            AliasAutocomplete     := RegReadIntDef(MyRegistry,    'AliasAutocomplete', 3);
            GamesLimit            := RegReadIntDef(MyRegistry,    'GamesLimit', 1000);
            StatusServerIP        := RegReadStringDef(MyRegistry, 'StatusServerIP',  '127.0.0.1');
            DBServerIP            := RegReadStringDef(MyRegistry, 'DBServerIP',      '127.0.0.1');
            SystemType            := RegReadStringDef(MyRegistry, 'Type',            '');
            if      SystemType = 'Nexus'         then FSystemType := stNexus
            else if SystemType = 'NexusWithZeon' then FSystemType := stNexusWithZeon
            else if SystemType = 'Laserforce'    then FSystemType := stLaserforce
            else if SystemType = 'Acacia'        then FSystemType := stAcacia
            else                                      FSystemType := stDemo;
        end;

        MRUList := TStringList.Create;
        DoMRU(MyRegistry, 'MRU1', MRU1);
        DoMRU(MyRegistry, 'MRU2', MRU2);
        DoMRU(MyRegistry, 'MRU3', MRU3);
        DoMRU(MyRegistry, 'MRU4', MRU4);
        DoMRU(MyRegistry, 'MRU5', MRU5);
        DoMRU(MyRegistry, 'MRU6', MRU6);
        DoMRU(MyRegistry, 'MRU7', MRU7);
        DoMRU(MyRegistry, 'MRU8', MRU8);
        DoMRU(MyRegistry, 'MRU9', MRU9);

        MyRegistry.CloseKey;
    finally
        MyRegistry.Free;
    end;

    AllPlayers.Height := ClientHeight - 135;
    PlayerDetailPanel.Top := ClientHeight - 41;

    Team := TTeamBoxList.Create;
    Team.Add(TTeamBox.Create(Self));  // create a dummy box to fill '0' in the List.
    Team[0].Parent := Self;
    Team[0].Visible := False;

    for i := 1 to Prefs.GridHigh * Prefs.GridWide do  // dynamically create Teams, Scores and Ranks
        CreateTeamBox(i);

    { Note that we have to do all the TeamBox creation _before_ we Show the form,
      'cos as soon as we do MainForm.Show, it calls TMainForm.Resize,
      which then _refers_ to all the dynamically created objects. }

    with (Sender as TMainForm) do begin
        Top := 0;
        if Prefs.GridWide * 200 + 210 < Screen.Width then
            Left := Screen.Width - (Prefs.GridWide * 250 + 260)  // Place the form butted up to the right-hand side of the screen.
        else
            Left := 0;
        Show;
        ClientHeight := Prefs.GridHigh * (Prefs.GridPlayers * 14 + 26) - 2;
        ClientWidth  := Prefs.GridWide * 250 + 252;
    end;

    if GamesCombo.ItemHeight > 0 then
        GamesCombo.DropDownCount := (Max(ClientHeight, Screen.Height div 2) - 35) div GamesCombo.ItemHeight;

    FFChest := -300;  // set initial Friendly Fire penalty values
    FFBack  := -300;
    FFLeft  := -100;
    FFRight := -100;
    FFLaser := -100;
    DoCalc  := True;  // enable team scores calculation

    LeagueMode := False;
    LeagueList := TLeagueList.Create;
    GameList   := TGameList.Create;
    ServerGameList := TServerGameList.Create;
    ReportForm := nil;
    LadderForm := nil;
    DramaForm := nil;
    PyramidForm := nil;
    FixtureForm := nil;

    StaleColor := MixColors([ColorToRGB(clWindow), clGray], [4, 1]);

    ConnectDB;

    if FSystemType = stAcacia then
        ListenForAcaciaStatus;
end;

procedure TMainForm.ConnectDB;
var MyRegistry: TRegistry;
    DbPath: string;
    zquery: TZReadOnlyQuery;
begin
    ZConnection.Disconnect;
    case FSystemType of
        stNexus, stNexusWithZeon: begin
            ZConnection.HostName := Prefs.DBServerIP;
            ZConnection.Protocol := 'mysql';
            ZConnection.Database := 'ng_system';
            ZConnection.User := 'root';
            ZConnection.Password := 'password';
            Caption := 'TORN -- a tournament scores manager for Nexus';

            // Query ng_system.ng_registry Registry_ID = 0, field Int_Data_1 >= 47.
            zquery := TZReadOnlyQuery.Create(Self);
            try
                zquery.Connection := ZConnection;
                zquery.SQL.Add('SELECT Int_Data_1 FROM ng_registry WHERE Registry_ID = 0');
                zquery.Open;
                if not zquery.EOF then
                    FHeliosType := zquery.FieldValues['Int_Data_1'];
            finally
                FreeSquish(zquery);
            end;
        end;
        stAcacia: begin
            MyRegistry := TRegistry.Create;  // open the Registry
            try
                MyRegistry.RootKey := HKEY_LOCAL_MACHINE;
                // Try to pull a path to the database server out of the registry.
                if MyRegistry.KeyExists('Software\PCMicro\Acacia') then begin
                    MyRegistry.OpenKey('Software\PCMicro\Acacia', False);
                    ZConnection.HostName := RegReadStringDef(MyRegistry, 'DBServerIP', Prefs.DBServerIP);
                    DbPath := RegReadStringDef(MyRegistry, 'DBPath', 'C:\Program Files\P&CMicros\Database\ZONESYSTEMS.GDB');
                    MyRegistry.CloseKey;
                end;
            finally
                MyRegistry.Free;
            end;
            ZConnection.Protocol := 'interbase-5';
            ZConnection.Database := DbPath;
            ZConnection.User := 'sysdba';
            ZConnection.Password := 'masterkey';
            Caption := 'TORN -- a tournament scores manager for Acacia/Infusion';
        end;
        stLaserforce: begin
            // Laserforce data connection is not done through the ZeosDBO ZConnection; instead it is done through the proprietary Torn Connector.
            Caption := 'TORN -- a tournament scores manager for Laserforce';
        end;
    end;
    ZConnection.ReadOnly := True;
    BuildGamesCombo;
end;

// Open a socket to listen for port 12050 broadcasts.
procedure TMainForm.ListenForAcaciaStatus();
var localaddr: TSockAddrIn;  // internet socket address structure
    serveraddr: TSockAddrIn;  // internet socket address structure
    WSAData: tWSAData;  // structure to receive details of the Windows socket, from WSAStartup
begin
    WSAStartup(MakeWord(1, 1), WSAData);         // request version 1.1 of the Windows Socket system
    mySocket := socket(AF_INET, SOCK_STREAM, 0); // create a TCP socket
    localaddr.sin_family := AF_INET;
    localaddr.sin_port := htons(0);
    localaddr.sin_addr.s_addr := htonl(INADDR_ANY);  // if this machine has multiple IP addresses, use all of them

    if bind(mySocket, localaddr, sizeof(localaddr)) <> 0 then  // binds mySocket to UDP port FPort.
        ShowMessage('Socket error ' + IntToStr(WSAGetLastError) + ' occurred.');

    serveraddr.sin_family := AF_INET;
    serveraddr.sin_addr.s_addr := inet_addr(PChar(Prefs.StatusServerIP));
    serveraddr.sin_port := htons(12050);
    if connect(mySocket, serveraddr, sizeof(serveraddr)) < 0 then begin
        PlayerDetailPanel.Font.Size := 8;
        PlayerDetailPanel.Caption := 'Connect error ' + IntToStr(WSAGetLastError) + '.';
    end;

    WSAAsyncSelect(mySocket, Handle, ReadAcaciaMsg, FD_READ);  // posts a ReadMyMsg message on FD_READ event, which is handled by SocketRead
end;

// read a Boolean from the registry if it exists, otherwise return a default value
function TMainForm.RegReadBoolDef(R: TRegistry; Name: String; Default: Boolean): Boolean;
begin
    if R.ValueExists(Name) and (R.GetDataType(Name) = rdInteger) then
        Result := R.ReadBool(Name)
    else
        Result := Default;
end;

// read an integer from the registry if it exists, otherwise return a default value
function TMainForm.RegReadIntDef(R: TRegistry; Name: String; Default: Integer): Integer;
begin
    if R.ValueExists(Name) and (R.GetDataType(Name) = rdInteger) then
        Result := R.ReadInteger(Name)
    else
        Result := Default;
end;

// read a string from the registry if it exists, otherwise return a default value
function TMainForm.RegReadStringDef(R: TRegistry; Name, Default: string): string;
begin
    if R.ValueExists(Name) and (R.GetDataType(Name) = rdString) then
        Result := R.ReadString(Name)
    else
        Result := Default;
end;

// get a player's raw score out of a TeamBox SubItem
function TMainForm.GetScore(SubItems: TStrings): Integer;
begin
    if Pos(' (', SubItems[1]) = 0 then
        Result := StrToIntDef(SubItems[1], 0)
    else
        Result := StrToIntDef(Copy(SubItems[1], 1, Pos(' (', SubItems[1]) - 1), 0);
end;

// read the named Most Recently Used entry from the registry,
// add it to the MRUList,
// and put it on the menu, in the named menu item.
procedure TMainForm.DoMRU(R: TRegistry; Name: string; MRUMenuItem: TMenuItem);
var s: string;
begin
    s := RegReadStringDef(R, Name, '');
    if s <> '' then begin
        MRUList.AddObject(s, MRUMenuItem);
        MRUMenuItem.Caption := '&' + IntToStr(MRUList.Count) + ' ' + s;
        MRUMenuItem.Visible := True;
    end;
end;

// Put the numbered MRU item on the menu, in the named menu item.
procedure TMainForm.UpdateMRU(MRUMenuItem: TMenuItem; Number: Integer);
begin
    if MRUList.Count >= Number then begin
        MRUMenuItem.Caption := '&' + IntToStr(Number) + ' ' + MRUList[Number - 1];
        MRUMenuItem.Visible := True;
    end;
end;

// Do the Create for one TeamBox.
procedure TMainForm.CreateTeamBox(Num: Integer);
begin
    Team.Add(TTeamBox.Create(Self));
    with Team[Num] do begin
        Parent      := Self;
        Left        := ((Num - 1) div Prefs.GridHigh) * 200 + 208;
        Top         := ((Num - 1) mod Prefs.GridHigh) * 72;
        Width       := 190;
        Height      := Prefs.GridPlayers * 14 + 21;
        HelpContext := 6;
        OnCompare   := PlayersCompare;
        OnMouseDown := TeamboxMouseDown;
        OnDblClick  := PlayersDblClick;
        OnDragDrop  := PlayersDragDrop;
        OnDragOver  := PlayersDragOver;
        OnHandicapChange := HandicapChange;
        OnSelectItem := PlayersSelectItem;
        SmallImages := TeamBoxImageList;
        Tag         := Num;  // Tag will be used later to identify this Team box.
        PopUpMenu   := TeamboxPopupMenu;  //TPopUpMenu.Create(self);
    end;  // with Team[i]

    Team[Num].HandiDefault := Prefs.HandicapStyle;
end;

function TMainForm.NexusGameInProgress: Boolean;
var zquery: TZReadOnlyQuery;
    s: string;
begin
    Debug('NexusGameInProgress start');
    zquery := TZReadOnlyQuery.Create(Self);
    try
        zquery.Connection := ZConnection;
        zquery.SQL.Add('SELECT Event_Type, Time_Logged, CURRENT_TIMESTAMP ' +
                       'FROM ng_game_log ' +
                       'ORDER BY Time_Logged DESC');
        zquery.Open;
        Result := not zquery.EOF and (zquery.FieldValues['Event_Type'] = 0);  // 0 is 'Game Started'.
        TimeSinceStart := zquery.FieldValues['Time_Logged'];
        if Result then begin
            TimeSinceStart := zquery.FieldValues['CURRENT_TIMESTAMP'] - zquery.FieldValues['Time_Logged'];
            DateTimeToString(s, 'n:ss', TimeSinceStart);
            PlayerDetailPanel.Caption := '+' + s;
        end else begin
            TimeSinceStart := 0;
            PlayerDetailPanel.Caption := '';
            s := 'idle';
        end;

        if ViewingInProgress and (GameTime - TimeSinceStart < 8 / 84600) then  // 8 seconds after the end of the game.
            GamesComboChange(nil);

        Commit.Enabled := not ViewingInProgress;
        NexusGameTimer.Enabled := Result;
    finally
        FreeSquish(zquery);
    end;
    Debug('NexusGameInProgress ' + s);
end;

// Select the highest game number so far today, and cause the refresh.
procedure TMainForm.LatestClick(Sender: TObject);
begin
    Screen.Cursor := crAppStart;
    try
        if FSystemType = stAcacia then
            ZConnection.Disconnect;
        BuildGamesCombo;
        if ((FSystemType in [stNexus, stNexusWithZeon]) and NexusGameInProgress) or GameInProgress then
            GamesCombo.ItemIndex := 1
        else
            GamesCombo.ItemIndex := 0;
        GamesComboChange(GamesCombo);
    finally
        Screen.Cursor := crDefault;
    end;
end;

// The user has selected a new game to view, so load it.
procedure TMainForm.GamesComboChange(Sender: TObject);

    procedure LoadNexus;

        function FindPlayer(playerid: string): TListItem;
        var i: Integer;
        begin
            i := 0;
            while Assigned(AllPlayers.Items[i]) and (AllPlayers.Items[i].Subitems[tbPackID] <> playerid) do
                Inc(i);
            Result := AllPlayers.Items[i];
        end;

    var i: Integer;
        zquery: TZReadOnlyQuery;
        packscore: Int64;
    begin
        Debug('LoadNexus start');

        ZConnection.Connect;
        zquery := TZReadOnlyQuery.Create(Self);
        try
            zquery.Connection := ZConnection;
            zquery.SQL.Add('SELECT S.Start_Time, S.Duration, P.Game ' +
                           'FROM ng_game_stats S ' +
                           'JOIN ng_profiles P ON S.Profile_ID = P.Profile_ID ' +
                           'WHERE S.Game_ID = ' + IntToStr(GameNum));
            zquery.Open;
            if not zquery.EOF then begin
                GameTime     := zquery.FieldValues['Start_Time'];
                GameDuration := zquery.FieldValues['Duration'];
                GameTitle    := zquery.FieldValues['Game'];
            end else GameDuration := -1;
        finally
           FreeSquish(zquery);
        end;

        Debug('LoadNexus selected gamenum');

        if (GameDuration > 0) or (Prefs.SortMode = 2) then begin  // SortMode 2 = Lord of the Ring.
            zquery := TZReadOnlyQuery.Create(Self);
            try
                zquery.Connection := ZConnection;
                if FSystemType = stNexus then
                  zquery.SQL.Add('SELECT Player_ID, Player_Team_ID, SUM(Score) AS Score, Pack_Name, Button_ID, Alias ' +
                                  'FROM ng_player_event_log EL ' +
                                  'JOIN ng_player_stats S ON EL.Game_ID = S.Game_ID AND EL.Player_ID = S.Pack_ID ' +
                                  'LEFT JOIN members M ON S.Member_ID = M.Member_ID ' +
                                  'WHERE EL.Game_ID = ' + IntToStr(GameNum) +
                                  ' GROUP BY Player_ID ' +
                                  'ORDER BY Score DESC')
                else  // stNexusWithZeon
                    if FHeliosType < 47 then  // This is the old schema, with QRCode in demographics.customer table.
                  zquery.SQL.Add('SELECT Player_ID, Player_Team_ID, SUM(Score) AS Score, Pack_Name, COALESCE(OMemb_ID, Button_ID) AS Button_ID, M.Alias ' +
                                  'FROM ng_player_event_log EL ' +
                                  'LEFT JOIN ng_player_stats S ON EL.Game_ID = S.Game_ID AND EL.Player_ID = S.Pack_ID ' +
                                  'LEFT JOIN members M ON S.Member_ID = M.Member_ID ' +
                                  'WHERE EL.Game_ID = ' + IntToStr(GameNum) +
                                  ' GROUP BY Player_ID ' +
                                  'ORDER BY Score DESC')
                else  // This is the new schema, with QRCode in members table.
                  zquery.SQL.Add('SELECT Player_ID, Player_Team_ID, SUM(Score) AS Score, Pack_Name, QRCode AS Button_ID, M.Alias ' +
                                  'FROM ng_player_event_log EL ' +
                                  'LEFT JOIN ng_player_stats S ON EL.Game_ID = S.Game_ID AND EL.Player_ID = S.Pack_ID ' +
                                  'LEFT JOIN members M ON S.Member_ID = M.Member_ID ' +
                                  'WHERE EL.Game_ID = ' + IntToStr(GameNum) + ' AND (Event_Type <> 36 OR EL.Score <> 0)' +
                                  ' GROUP BY Player_ID ' +
                                  'ORDER BY Score DESC');

                                (*Accuracy, Handicap, Played, Team, Rank,*)
                zquery.Open;
                Debug('LoadNexus selected players');
                while not zquery.EOF do begin
                    with AllPlayers.Items.Add do begin

                        ImageIndex := zquery.FieldByName('Player_Team_ID').AsInteger;
                        Caption := zquery.FieldByName('Pack_Name').AsString;
//                        if zquery.FieldValues['Eliminated'] <> 0 then ImageIndex := ImageIndex + NUM_COLOURS;  // if this player was Eliminated, jump to an Eliminated icon

                        if (FSystemType = stNexusWithZeon) and (zquery.FieldByName('Pack_Name').AsString = zquery.FieldByName('Alias').AsString) then
                            SubItems.Add('')  // If the pack name and the alias are the same, assume this is a no-logon.  This assumption is wrong if the player's alias is genuinely the same as the pack name.
                        else
                            SubItems.Add(zquery.FieldByName('Alias').AsString);
//                        SubItems.Add(NonNull(zquery.FieldValues['Alias']));
                        try
                          packscore := zquery.FieldByName('Score').AsInteger;
//                          packscore := Integer(VarAsType(zquery.FieldValues['Score'], varInteger));
                        except
                          packscore := -1;
                        end;
                        SubItems.Add(IntToStr(packscore));

                        if (FSystemType = stNexusWithZeon) and (zquery.FieldByName('Pack_Name').AsString = zquery.FieldByName('Alias').AsString) then
                            SubItems.Add('')
                        else
                            SubItems.Add(zquery.FieldByName('Button_ID').AsString);

                        SubItems.Add(IntToStr(AllPlayers.Items.Count));  // tbRank;
                        SubItems.Add('0');              // tbFriendly
                        SubItems.Add('0');              // Accuracy
                        SubItems.Add(zquery.FieldByName('Player_ID').AsString);
                        for i := 15 to 50 do  // pad the rest of the line
                            SubItems.Add('');
                    end;  // with AllPlayers.Add
                    zquery.Next;
                end;
            finally
                FreeSquish(zquery);
            end;

            Debug('LoadNexus loaded players');

            zquery := TZReadOnlyQuery.Create(Self);
            try
                zquery.Connection := ZConnection;
                zquery.SQL.Add('SELECT DISTINCT Player_ID, Player_Team_ID ' +  // Get each player's team ID.
                               'FROM ng_player_event_log EL ' +
                               'WHERE EL.Game_ID = ' + IntToStr(GameNum) + ' AND Event_Type = 35');
                zquery.Open;
                while not zquery.EOF do begin
                    if Assigned(FindPlayer(zquery.FieldValues['Player_ID'])) then
                        FindPlayer(zquery.FieldValues['Player_ID']).ImageIndex := zquery.FieldValues['Player_Team_ID'];
                    zquery.Next;
                end;
            finally
                FreeSquish(zquery);
            end;

            Debug('LoadNexus loaded teams');

            zquery := TZReadOnlyQuery.Create(Self);
            try
                zquery.Connection := ZConnection;
                zquery.SQL.Add('SELECT DISTINCT Player_ID ' +  // Find out if each player has been eliminated.
                                'FROM ng_player_event_log EL ' +
                                'WHERE EL.Game_ID = ' + IntToStr(GameNum) + ' AND Event_Type = 32');
                zquery.Open;
                while not zquery.EOF do begin
                    with FindPlayer(zquery.FieldValues['Player_ID']) do
                        ImageIndex := ImageIndex + NUM_COLOURS;
                    zquery.Next;
                end;
            finally
                FreeSquish(zquery);
            end;

            Debug('LoadNexus loaded eliminations');
        end;
    end;

    procedure LoadAcacia;
    var i: Integer;
        deviceid: Integer;
        zquery: TZReadOnlyQuery;
    begin
        ZConnection.Connect;
        zquery := TZReadOnlyQuery.Create(Self);
        try
            zquery.Connection := ZConnection;
            zquery.SQL.Add('SELECT G.Game_ID, G.Profile_ID, G.Start_Time, G.Duration, ' +
                                  'P.Description, ' +
                                  'S.Device_ID, S.Eliminated, S.Member_ID, S.Calculated_Score, S.Accuracy, S.Playing ' +
                           'FROM T2GAMES G ' +
                           'JOIN T2PROFILES P ON G.Profile_ID = P.Profile_ID ' +
                           'JOIN T2GAMESTATS S ON G.Game_ID = S.Game_ID ' +
                           'WHERE G.Game_ID = ' + IntToStr(GameNum) +
                           ' AND S.Playing = 1' +
                           ' ORDER BY S.Device_ID');
                           (*Handicap, Played, Team, Terminated, Rank,*)

            zquery.Open;
            if not zquery.EOF then begin
                GameTime     := zquery.FieldValues['Start_Time'];
                GameDuration := zquery.FieldValues['Duration'];
                GameTitle    := zquery.FieldValues['Description'];
            end;

            while not zquery.EOF do begin
                with AllPlayers.Items.Add do begin

                    deviceid := zquery.FieldValues['Device_ID'];
                    ImageIndex := deviceid mod 3;
                    case ImageIndex of
                        0:  Caption := 'Red';
                        1:  Caption := 'Blu';
                        2:  Caption := 'Yel';
                       else Caption := '???';
                    end;  // case
                    if ImageIndex = 2 then ImageIndex := 3;  // if ImageIndex is supposed to be yellow, adjust it
                    if zquery.FieldValues['Eliminated'] <> 0 then ImageIndex := ImageIndex + NUM_COLOURS;  // if this player was Eliminated, jump to an Eliminated icon

                    if deviceid < 27 then
                        Caption := Caption + ' ';
                    Caption := Caption + IntToStr(deviceid div 3 + 1);

                    SubItems.Add(NonNull(zquery.FieldValues['Member_ID']));
                    SubItems.Add(zquery.FieldValues['Calculated_Score']);
                    SubItems.Add(NonNull(zquery.FieldValues['Member_ID']));
                    SubItems.Add('0');              // tbRank;
                    SubItems.Add('0');              // tbFriendly
                    SubItems.Add(zquery.FieldValues['Accuracy']);
                    SubItems.Add(zquery.FieldValues['Device_ID']);
                    for i := 15 to 50 do  // pad the rest of the line
                        SubItems.Add('');

                end;  // with AllPlayers.Add
                zquery.Next;
            end;
        finally
            FreeSquish(zquery);
        end;

        for i := 0 to AllPlayers.Items.Count - 1 do
            if AllPlayers.Items[i].SubItems[tbMemberAlias] = '' then begin  // look up pack name
                zquery := TZReadOnlyQuery.Create(Self);
                try
                    zquery.Connection := ZConnection;
                    zquery.SQL.Add('SELECT Pack_ID, Alias FROM T2PACKS ' +
                                                  'WHERE Pack_ID = "' + AllPlayers.Items[i].SubItems[tbPackID] + '"');
                    zquery.Open;
                    if not zquery.EOF then
                        AllPlayers.Items[i].SubItems[tbMemberAlias] := '(' + zquery.FieldValues['Alias'] + ')';
                finally
                    FreeSquish(zquery);
                end;
            end else begin  // treat alias as an ABC+123 member id and look it up
                zquery := TZReadOnlyQuery.Create(Self);
                try
                    zquery.Connection := ZConnection;
                    zquery.SQL.Add('SELECT User_ID, Player_Alias FROM MEMBERS ' +
                                                  'WHERE User_ID = "' + AllPlayers.Items[i].SubItems[tbMemberAlias] + '"');
                    zquery.Open;
                    if not zquery.EOF then begin
                        if Copy(zquery.FieldValues['Player_Alias'], 1, 6) = 'Adjust' then  // it can be mistaken for an Adjustment
                            AllPlayers.Items[i].SubItems[tbMemberAlias] := 'Doofus'
                        else
                            AllPlayers.Items[i].SubItems[tbMemberAlias] := zquery.FieldValues['Player_Alias'];
                    end;
                finally
                    FreeSquish(zquery);
                end;
            end;
    end;

    procedure LoadLaserforce;

        function ToNexusColour(colourname: string; colournumber: Integer): TNexusColour;
        var cn: string;
        begin
            cn := LowerCase(colourname);
                 if Pos('red', cn) > 0 then Result := 0
            else if Pos('blue', cn) > 0 then Result := 1
            else if Pos('green', cn) > 0 then Result := 2
            else if Pos('yellow', cn) > 0 then Result := 3
            else if Pos('purple', cn) > 0 then Result := 4
            else if Pos('pink', cn) > 0 then Result := 5
            else if Pos('aqua', cn) > 0 then Result := 6
            else if Pos('orange', cn) > 0 then Result := 7
            else if Pos('white', cn) > 0 then Result := 8
            else if Pos('black', cn) > 0 then Result := 9
            else if Pos('fire', cn) > 0 then Result := 10
            else if Pos('ice', cn) > 0 then Result := 11
            else if Pos('earth', cn) > 0 then Result := 12
            else if Pos('crystal', cn) > 0 then Result := 13
            else if Pos('rainbow', cn) > 0 then Result := 14
            else case colournumber of
                1: Result := 0;  // red
                2: Result := 2;  // green
                3: Result := 3;  // yellow
                4: Result := 1;  // blue
                5: Result := 6;  // aqua
                6: Result := 4;  // purple
                7: Result := 8;  // white
                8: Result := 7;  // orange
                9: Result := 5;  // pink
                10 .. NUM_COLOURS: Result := colournumber - 1;  // black, fire, ice, earth, crystal, rainbow, cops
              else Result := -1;
            end
        end;

    var
        query: TZoomReport;
        i, j: Integer;
    begin
        // Get game overview.
        query := GetTable(Prefs.DBServerIP, LASERFORCE_PORT,
                         'SELECT M.start, datediff(s, M.start, M.[end]) AS [Duration], COALESCE(MT.desc1, MT.desc0, MG.[desc]) AS [Description] ' +
                         'FROM Mission M ' +
                         'LEFT JOIN MissionGroup MG ON MG.ref = M.[group] ' +
                         'LEFT JOIN MissionType MT ON MT.ref = M.[type]' +
                         'WHERE M.ref = ' + IntToStr(GameNum));

        try
            if query.Rows.Count > 0 then begin
                GameTime     := StrToDateTime(query.RowCol(0, 'start').Text);
                GameDuration := query.RowCol(0, 'Duration').AsInteger;
                GameTitle    := query.RowCol(0, 'Description').Text;
            end;
        finally
           query.Free;
        end;

        if GameDuration > 0 then begin
            // Get details for each player in the game.
            query := GetTable(Prefs.DBServerIP, LASERFORCE_PORT,
                             'SELECT MAT.colourTeam AS [Colour],  MP.score, U.[desc] AS [Pack_Name], cast(C.region as varchar) + ''-'' + cast(C.site as varchar) + ''-'' + cast(Member.id as varchar) as [ID], ' +
                             '  member.codename AS [codename], MAT.[desc] AS [Team] ' +
                             'FROM MissionPlayer MP ' +
                             'LEFT JOIN Unit U ON U.ref = MP.unit ' +
                             'LEFT JOIN Member ON Member.ref = MP.member ' +
                             'LEFT JOIN Centre C ON C.ref = Member.centre ' +
                             'LEFT JOIN Mission ON Mission.ref = MP.mission ' +
                             'LEFT JOIN MissionType MT ON MT.ref = Mission.type ' +
                             'LEFT JOIN MissionAlignmentTeam MAT ON MAT.alignment = MT.alignment AND MAT.seq = MP.team ' +
                             'WHERE Mission.ref = ' + IntToStr(GameNum) + ' and unitType = 0 ' +
                             'ORDER BY score DESC');
            try
                for i := 0 to query.Rows.Count - 1 do
                    with AllPlayers.Items.Add do begin

                        ImageIndex := ToNexusColour(query.RowCol(i, 'Team').Text, query.RowCol(i, 'Colour').AsInteger);
                        Caption := query.RowCol(i, 'Pack_Name').Text;
//                        if Eliminated then ImageIndex := ImageIndex + NUM_COLOURS;  // if this player was Eliminated, jump to an Eliminated icon

                        SubItems.Add(query.RowCol(i, 'codename').Text);
                        SubItems.Add(query.RowCol(i, 'score').Text);
                        SubItems.Add(query.RowCol(i, 'ID').Text);
                        SubItems.Add(IntToStr(AllPlayers.Items.Count));  // tbRank;
                        SubItems.Add('0');              // tbFriendly
                        SubItems.Add('0');              // Accuracy
                        SubItems.Add(query.RowCol(i, 'ID').Text);
                        for j := 15 to 50 do  // pad the rest of the line
                            SubItems.Add('');
                    end;  // with AllPlayers.Add
            finally
                query.Free;
            end;
        end;
    end;

    procedure LoadDemo;
    const ADJECTIVE: array [0..6] of string = ('Laser ', 'That ', 'Super ', 'Zone ', 'Mega ', 'Ultra ', 'The ');
    const NOUN: array [0..6] of string = ('Guy ', 'Man ', 'Girl ', 'Player ', 'Ace ', 'Rogue ', 'Shark ');
    var
        i, j, numplayers: Integer;
    begin
        GameTime     := Now;
        GameDuration := 900;
        GameTitle    := 'Demo game';

        numplayers := Random(11) + 10;
        for i := 1 to numplayers do
            with AllPlayers.Items.Add do begin

                ImageIndex := Random(8);
                Caption := 'Pack ' + IntToStr(i);

                SubItems.Add(ADJECTIVE[Random(7)] + NOUN[Random(7)]);  // alias
                SubItems.Add(IntToStr(Random(100) * 10 + Random(3) + (numplayers - i) * 1000));  // score
                SubItems.Add('');  // ID
                SubItems.Add(IntToStr(AllPlayers.Items.Count));  // tbRank;
                SubItems.Add('0');              // tbFriendly
                SubItems.Add('0');              // Accuracy
                SubItems.Add('');               // ID
                for j := 15 to 50 do  // pad the rest of the line
                    SubItems.Add('');
            end;  // with AllPlayers.Add
    end;

var s: string;
    i: Integer;
    zquery: TZReadOnlyQuery;
begin  // GamesComboChange
    Debug('GamesComboChange start');

    InitTeamBoxes;

    if ViewGcc.Checked then begin
        s := GamesCombo.Items[GamesCombo.ItemIndex];
        GameNum := StrToIntDef(Copy(s, 1, Pos(', ', s) - 1), 0);

        // get a list of players in this game, and put them in AllPlayers
        AllPlayers.Items.Clear;
        AllPlayers.SortType := stNone;
        case FSystemType of
          stNexus, stNexusWithZeon: LoadNexus;
          stAcacia: LoadAcacia;
          stLaserforce: LoadLaserforce;
          stDemo: LoadDemo;
        end;
        AllPlayers.SortType := stBoth;

        for i := 0 to NUM_COLOURS - 1 do
            DefHandi[i] := 100;

        if LeagueMode then begin
            if GameList.Find(GameTime) > -1 then
                Commit.Caption := 'Re&commit'
            else
                Commit.Caption := '&Commit';
        end;
        Commit.Visible   := LeagueMode;
        UpdateSB.Visible := not LeagueMode;

        // Figure out whether the game selected is the game currently playing.  If it is, disable Commit button.
        if FSystemType in [stNexus, stNexusWithZeon] then
            GameInProgress := NexusGameInProgress;

        ViewingInProgress := GameInProgress and (GamesCombo.ItemIndex = 0);

        Commit.Enabled := not ViewingInProgress;

        if FSystemType in [stNexus, stNexusWithZeon] then begin
            zquery := TZReadOnlyQuery.Create(Self);
            try
                zquery.Connection := ZConnection;
                zquery.SQL.Add('SELECT IF (Finish_Time = "0000-00-00 00:00:00", 1, 0) AS ''In_Progress'' FROM ng_game_stats ORDER BY Game_ID DESC LIMIT 1');
                zquery.Open;
            finally
                FreeSquish(zquery);
            end;
        end;
        // The following query is allegedly useful from Nexus: SELECT IF (Finish_Time = "0000-00-00 00:00:00", 1, 0) FROM ng_game_stats ORDER BY Game_ID DESC LIMIT 1;
    end else begin
        GameNum := GamesCombo.Items.Count - GamesCombo.ItemIndex;
        LoadHistorical;
        Commit.Enabled := True;
    end;

    if Prefs.SortMode = 1 then DoColourTransfer else DoLeagueTransfer;

    TeamboxMenu.Enabled := False;

    Debug('GamesComboChange end');
end;

// get a list of players in this game, and put them in AllPlayers
procedure TMainForm.LoadHistorical;
var green, yellow, blue: Boolean;
    thisgame: Integer;
    s: string;
    i: Integer;
    packid: Integer;
begin
    if GamesCombo.ItemIndex <> -1 then begin
        GameTime := StrToDateTime(GamesCombo.Items[GamesCombo.ItemIndex]);
        AllPlayers.Items.Clear;
        AllPlayers.SortType := stNone;
        Commit.Caption := 'Re&commit';
        Commit.Visible   := True;
        UpdateSB.Visible := False;
        green := False;
        yellow := False;
        blue := False;
        thisgame := GameList.Find(GameTime);
        if thisgame > -1 then
            for i := 0 to GameList[thisgame].Players.Count - 1 do
                with AllPlayers.Items.Add do begin
                    Caption := GameList[thisgame].Players[i].Pack;

                    packid := 0;
                    if Length(Caption) > 0 then case Caption[1] of
                        'R': begin packid := 0;  ImageIndex := 0;  end;
                        'B': begin packid := 1;  ImageIndex := 1;  blue   := True; end;
                        'G': begin packid := 1;  ImageIndex := 2;  green  := True; end;
                        'Y': begin packid := 2;  ImageIndex := 3;  yellow := True; end;
                    end;

                    s := LeagueList.FindID(GameList[thisgame].Players[i].PlayerID);
                    if s = Caption then SubItems.Add('')
                                   else SubItems.Add(s);
                    SubItems.Add(IntToStr(GameList[thisgame].Players[i].Score));
                    s := GameList[thisgame].Players[i].PlayerID;
                    if s[4] in ['+', '-', '%'] then
                        SubItems.Add(s)
                    else
                        SubItems.Add('');
                    SubItems.Add('0');  // rank
                    SubItems.Add('0');  // friendly fire
                    SubItems.Add('0');  // accuracy
                    s := Caption;
                    while (Length(s) > 0) and not (s[1] in ['0' .. '9']) do
                      s := Copy(s, 2, 999);
                    packid := StrToIntDef(s, 19) * 3 - 3 + packid;
                    SubItems.Add(IntToStr(packid));  // pack id
                end; // with; for; if

        AllPlayers.OnCompare := PlayersCompare;  // sort on score
        AllPlayers.SortType := stBoth;

        for i := 0 to AllPlayers.Items.Count - 1 do  // set up ranks for all players
            AllPlayers.Items[i].SubItems[tbRank] := IntToStr(i + 1);

        AllPlayers.SortType := stNone;
        AllPlayers.OnCompare := AllPlayersCompare;  // sort on pack caption
        AllPlayers.SortType := stBoth;

        Blue2 := yellow and blue;   // if there's yellow and blue packs in this game, then blue must be the second team (i.e. it replaces green)
        Blue3 := green and blue;    // if there's green and blue packs in this game, then blue must be the third team (i.e. it replaces yellow)
    end;
end;

// For each team box, reset everything.
procedure TMainForm.InitTeamBoxes;
var i: Integer;
begin
    for i := 1 to Team.Count - 1 do
        Team[i].Clear;
end;

// Custom draw for the GamesCombo drop-down.
// Scale down font size until the text fits in the width available.
procedure TMainForm.GamesComboDrawItem(Control: TWinControl; Index: Integer;
  Rect: TRect; State: TOwnerDrawState);
var availablewidth: Integer;
    i: Integer;
begin
    with Control as TComboBox do begin // draw on control canvas, not on the form
        // Look for this game.
        if ServerGameList.Count > 0 then
            if not Assigned(ServerGameList[Index].Game) then begin
                i := GameList.Find(ServerGameList[Index].GameTime);
                if i > -1 then
                    ServerGameList[Index].Game := GameList[i];
            end;

        if (ServerGameList.Count > 0) and Assigned(ServerGameList[Index].Game) then begin  // If found, paint it green.
            if odSelected in State then
                Canvas.Brush.Color := $0200C000
            else
                Canvas.Brush.Color := $02C0FFC0;
        end;

        Canvas.FillRect(Rect);       // clear the rectangle

        if Items.Count <= DropDownCount then
            availablewidth := Width - 2
        else
            availablewidth := Width - 19;

        Canvas.Font.Height := ItemHeight;
        while (Canvas.Font.Height > ItemHeight div 3 + 1) and
              (Canvas.TextWidth(Items[Index]) > availablewidth) do
            Canvas.Font.Height := Canvas.Font.Height - 1;

        Canvas.TextOut(Rect.Left + 1, (Rect.Top + Rect.Bottom - Canvas.Font.Height - 1) div 2, Items[Index])  // display the text
    end;
end;

// Add a dummy "Adjustment" player to a team.
procedure TMainForm.AddAdjustment(TeamNum, valu: Integer);
begin
    Team[TeamNum].SortType := stNone;  // turn sorting off while we add to the list; otherwise it tries to sort on partially added items.

    with Team[TeamNum].Items.Add do begin
        Caption := '';
        ImageIndex := ADJUSTMENT_IMAGE;
        SubItems.Add('Adjustment');     // MemberAlias
        SubItems.Add(IntToStr(valu));   // Score
        SubItems.Add('');               // MemberID
        SubItems.Add('0');              // Rank
    end; // with

    Team[TeamNum].SortType := stBoth;  // Turn sorting back on
end;

// Transfers players to team boxes, based on what colour they are.
procedure TMainForm.DoColourTransfer;
var i, j, k:    Integer;
    NextTeamBox:Integer;
    match:      Boolean;
    thisgame,
    thisplayer,
    thisteam:   Integer;
    s:          string;
    playerindex:Integer;
begin
    DoCalc := False;
    NextTeamBox := 1;
    for j := 0 to NUM_COLOURS - 1 do begin
        match := False;
        i := AllPlayers.Items.Count - 1;
        while (i >= 0) and (AllPlayers.Items.Count > 0) do begin
            if ((AllPlayers.Items[i].ImageIndex mod NUM_COLOURS) = j) then begin  // if colour matched
                match := True;
                AllPlayers.Items[i].Selected := True;  // add this item to the selection
            end;
            Dec(i);
        end;  // while i

        if match then begin
            PlayersDragDrop(Team[NextTeamBox], AllPlayers, 1, 1);  // move all selected into a team box

            if LeagueMode then begin
                thisgame := GameList.Find(GameTime);
                i := LeagueList.FindAnyInLeague(Team[NextTeamBox]);

                for k := 0 to Team[NextTeamBox].Items.Count - 1 do begin
                    if (thisgame > -1) then begin
                        thisplayer := GameList[thisgame].Players.FindPack(Team[NextTeamBox].Items[k].Caption);
                        if (thisplayer > -1) and (GameList[thisgame].Players[thisplayer].TeamID = i) then begin // if packid matched in DM.LeagueTable  (note that this line does not work)
                            s := GameList[thisgame].Players[thisplayer].PlayerID;
                            Team[NextTeamBox].Items[k].SubItems[tbMemberAlias] := LeagueList.FindID(s);

                            if i = -1 then
                                playerindex := -1
                            else
                                playerindex := LeagueList[i].Players.FindButton(Team[NextTeamBox].Items[k].SubItems[tbMemberAlias]);
                            if playerindex = -1 then
                                Team[NextTeamBox].Items[k].SubItems[tbMemberID] := s
                            else
                                Team[NextTeamBox].Items[k].SubItems[tbMemberID] := LeagueList[i].Players[playerindex].Name;
                        end;  // if PackID matched
                    end;
                end;  // for k

                if (thisgame > -1) and (i > -1) then begin
                    thisteam := GameList[thisgame].Teams.Find(LeagueList[i].TeamID);
                    if (thisteam > -1) and (GameList[thisgame].Teams[thisteam].Adjustment <> 0) then   // found an Adjustment for this team in GameList
                        AddAdjustment(NextTeamBox, GameList[thisgame].Teams[thisteam].Adjustment);
//                    if (thisteam > -1) and (GameList[thisgame].Teams[thisteam].PointsAdjustment <> 0) then   // found an Adjustment for this team in GameList
//                        AddPointsAdjustment(NextTeamBox, GameList[thisgame].Teams[thisteam].PointsAdjustment);
                end;
            end;

            if (Prefs.SortMode = 1) and ViewGcc.Checked and (DefHandi[j] <> 100) then
                Team[NextTeamBox].Handicap := IntToStr(DefHandi[j]) + '%';

            Inc(NextTeamBox);
        end;
        if Team.Count <= NextTeamBox then Break;  // if there aren't any more Team boxes, stop.
    end;  // for j

    DoCalc := True;
    CalcAllScores(True, True);

    if Prefs.SortTeamsByRank then SortTeamsClick(AllPlayersPopupMenu);
end;

// Transfers players to team boxes, based on what team they're in.
procedure TMainForm.DoLeagueTransfer;
var i, j:       Integer;
    NextTeamBox:Integer;
    match:      Boolean;
    thisgame,
    thisplayer,
    thisteam:   Integer;
begin
    DoCalc := False;
    NextTeamBox := 1;
    if LeagueMode then begin
        thisgame := GameList.Find(GameTime);

        for j := 0 to LeagueList.Count - 1 do begin
            match := False;

            i := AllPlayers.Items.Count - 1;
            while (i >= 0) and (AllPlayers.Items.Count > 0) do begin
                if thisgame = -1 then
                    thisplayer := -1
                else
                    thisplayer := GameList[thisgame].Players.FindPack(AllPlayers.Items[i].Caption);

                if ((LeagueList[j].Players.FindName(AllPlayers.Items[i].SubItems[tbMemberAlias]) <> -1)) or // if alias matched in League list
                    ((thisgame > -1) and (thisplayer > -1) and (GameList[thisgame].Players[thisplayer].TeamID = j))  // or TeamID matched in GameList
                then begin
                    match := True;
                    AllPlayers.Items[i].Selected := True;  // add this item to the selection
                end;
                Dec(i);
            end;  // while i

            if match then begin
                PlayersDragDrop(Team[NextTeamBox], AllPlayers, 1, 1);  // move all selected into a team box

                if thisgame > -1 then begin
                    thisteam := GameList[thisgame].Teams.Find(LeagueList[j].TeamID);
                    if (thisteam > -1) and (GameList[thisgame].Teams[thisteam].Adjustment <> 0) then   // found an Adjustment for this team in GameList
                        AddAdjustment(NextTeamBox, GameList[thisgame].Teams[thisteam].Adjustment);
                end;

                Inc(NextTeamBox);
            end;
            if Team.Count <= NextTeamBox then Break;  // if there aren't any more Team boxes, stop.
        end;  // for j
    end;  // if LeagueMode

    DoCalc := True;
    CalcAllScores(True, True);

    SortTeamsClick(SortTeams);
end;

// Sets the Rank value of all Team boxes.  Called when you change a team score, handicap or adjustment.  Does not move TeamBoxes.
procedure TMainForm.RankTeams;

    // If all players on a team are the same colour, then return that 0..15 colour.  Else, return -1.
    function CalcColour(Team: TTeamBox): TNexusColour;
    var i: Integer;
    begin
        Result := -1;

        for i := 0 to Team.Items.Count - 1 do  // First, find a pack.
            if (Team.Items[i].ImageIndex in [0..23]) then
                Result := Team.Items[i].ImageIndex mod NUM_COLOURS;

        for i := 1 to Team.Items.Count - 1 do  // Second, check that colour against all packs.
            if (Team.Items[i].ImageIndex in [0..23]) and (Team.Items[i].ImageIndex mod NUM_COLOURS <> Result) then
                Result := -1;
    end;

type
  TSorty = record
    Index: Integer;  // Which TeamBox does this record point at?
    Score: Integer;  // What is that TeamBox's score?
    Colour: TNexusColour;  // What colour are the players in that TeamBox?
    Rank: Integer;   // What should that TeamBox's rank be?
  end;
var i, j: Integer;
    colour: TNexusColour;
    currentrank: Integer;
    ranks: array [1..60] of TSorty;
    temp: TSorty;
    previousindex: Integer;
begin
    for i := 1 to Team.Count - 1 do begin
        ranks[i].Index := i;
        if Team[i].Items.Count = 0 then  // there are no players on this team
            ranks[i].Score := -999999  // give the team a bogus negative score
        else
            ranks[i].Score := Team[i].Score;  // give them their actual score
        if Prefs.SortMode = 2 then
            ranks[i].Colour := CalcColour(Team[i]);
    end;

    // Sort the array of ranks by Score.
    for i := 1 to Team.Count - 1 do  // Bogosort.  So shoot me.
        for j := i to Team.Count - 1 do
            if ranks[i].Score < ranks[j].Score then begin // swap i with j
                temp := ranks[i];  ranks[i] := ranks[j];  ranks[j] := temp;
            end;

    // Set Ranks for teams.
    if Prefs.SortMode in [0, 1] then begin
        ranks[1].Rank := 1;
        for i := 2 to Team.Count - 1 do
            if ranks[i-1].Score = ranks[i].Score then
                ranks[i].Rank := ranks[i-1].Rank  // This is a tie; give a better rank.
            else
                ranks[i].Rank := i;
    end else begin  // Lord of the Ring mode:
        for i := 1 to Team.Count - 1 do
            ranks[i].Rank := 0;

        for colour := Low(TNexusColour) to High(TNexusColour) do begin  // For each colour,
            currentrank := 1;
            previousindex := -1;
            for i := 1 to Team.Count - 1 do
                if ranks[i].Colour = colour then begin
                    if (previousindex <> -1) and (ranks[previousindex].Score = ranks[i].Score) then
                        ranks[i].Rank := ranks[previousindex].Rank  // This is a tie; give a better rank.
                    else
                        ranks[i].Rank := currentrank;  // Assign the first team of that colour rank = 1,
                    Inc(currentrank);              // the second rank = 2, and so on.

                    previousindex := i;
                end;
        end;
    end;

    // Sort the array of ranks by Index; i.e. sort them back into the order we started with, so the one that refers to TeamBox #1 is first, etc.
    for i := 1 to Team.Count - 1 do  // Bogosort again.
        for j := i to Team.Count - 1 do
            if ranks[i].Index > ranks[j].Index then begin // swap i with j
                temp := ranks[i];  ranks[i] := ranks[j];  ranks[j] := temp;
            end;

    // Set each TeamBox's rank from its ranks[] entry.
    for i := 1 to Team.Count - 1 do
        if Team[i].Items.Count = 0 then  // there are no players on this team
            Team[i].Rank := 0
        else
            Team[i].Rank := ranks[i].Rank;
end;

procedure TMainForm.AdjustAllPlayersColumns;
var i: integer;
begin
    i := AllPlayers.Width - AllPlayers.Columns.Items[0].Width - AllPlayers.Columns.Items[2].Width;
    
    if AllPlayers.Items.Count > AllPlayers.VisibleRowCount then
        AllPlayers.Columns.Items[1].Width := i - 21
    else
        AllPlayers.Columns.Items[1].Width := i - 4;
end;

// Used by the internal guts of drag/drop.
procedure TMainForm.PlayersDragOver(Sender, Source: TObject; X, Y: Integer;
  State: TDragState; var Accept: Boolean);
begin
    Accept := ((Source = AllPlayers) or (Source is TTeamBox)) and
              (((Source as TListView).SelCount > 1) or ((Source as TListView).ItemFocused.Caption <> ''));
end;

// move the dragged line(s) from Source to Sender (i.e. target); and recalculate scores
procedure TMainForm.PlayersDragDrop(Sender, Source: TObject; X, Y: Integer);
var Player: TListItem;
    i: Integer;
begin
    if (Sender <> Source) and ((Source as TListView).SelCount > 0) then begin
        (Sender as TListView).SortType := stNone;  // turn sorting off while we add to the list; otherwise it tries to sort on partially added items.
        while (Source as TListView).SelCount > 0 do begin  // for each selected Player in the Source
            Player := (Source as TListView).Selected;
            if (DefHandi[Player.ImageIndex mod NUM_COLOURS] <> 100) and ((Sender as TListView).Items.Count = 0) then begin // this is the first player to be dragged into this box
                i := (Sender as TListView).Tag;
                if i <> 0 then begin
                    Team[i].Handicap := IntToStr(DefHandi[Player.ImageIndex mod NUM_COLOURS]) + '%';
                    Team[i].HandicapVisible := True;
                end;
            end;
            with (Sender as TListView).Items.Add do begin  // add an Item to the target ...
                Caption             := Player.Caption;     // ... and copy in settings from the Source's Player
                if Player.ImageIndex in [0..23] then
                    ImageIndex      := Player.ImageIndex;
                SubItems.CommaText  := Player.SubItems.CommaText;
                if SubItems.Count > tbFriendly then begin
                    if StrToIntDef(SubItems[tbFriendly], 0) <> 0 then
                        SubItems[tbScore] := IntToStr(GetScore(SubItems) - StrToIntDef(SubItems[tbFriendly], 0));
                    SubItems[tbFriendly]:= '0';
                end;
            end; // with

            (Source as TListView).Items.Delete(Player.Index);  // delete Player from Source
        end;
        (Sender as TListView).SortType := stBoth;  // Turn sorting back on for the target team

        if ((Source as TListView).Items.Count = 1) and (Copy((Source as TListView).Items[0].SubItems[tbMemberAlias], 1, 6) = 'Adjust') then  // the only thing left in this TeamBox is an Adjustment
            (Source as TListView).Items.Delete(0);  // delete the Adjustment

        CalcAllScores(DoCalc, True);
//        CalcTeamScore((Sender as TListView).Tag, True);
//        CalcTeamScore((Source as TListView).Tag, True);

        if (Sender = AllPlayers) or (Source = AllPlayers) then  // Resize Column1 of AllPlayers, to leave room for a scrollbar or not.
            AdjustAllPlayersColumns;

    end; // if Sender <> Source
end;  // procedure PlayersDragDrop

// Move all selected players to the first empty Team box, by calling PlayersDragDrop.
procedure TMainForm.PlayersDblClick(Sender: TObject);
var i: Integer;
begin
    if (Sender as TListView).ItemFocused <> nil then begin
        for i := 1 to Team.Count - 1 do
            if Team[i].Items.Count = 0 then break; // this Team box is empty

        if (i >= Prefs.GridHigh * Prefs.GridWide) and (Team[Prefs.GridHigh * Prefs.GridWide].Items.Count <> 0) then
            // there is no empty box to move players into; so do nothing.
        else
            PlayersDragDrop(Team[i], Sender, 1, 1);
    end;
end;

// Display details for this player or team at the bottom left of the window.
procedure TMainForm.PlayersSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
var thisplayer: TLeaguePlayer;
begin
    if (Sender as TListView).SelCount = 1 then begin
        if Item.SubItems.Count > tbHandicap then
        begin
            StatusDisplayCount := 0;
            PlayerDetailName.Visible := True;
            PlayerDetailTeam.Visible := True;
            PlayerDetailScore.Visible := True;
            PlayerDetailHandicap.Visible := True;

            if Item.SubItems[tbMemberAlias] = '' then begin
                PlayerDetailName.Caption := Item.Caption;
                PlayerDetailTeam.Caption := '';
            end else begin
                PlayerDetailName.Caption := Item.SubItems[tbMemberAlias] + '  ';
                PlayerDetailTeam.Caption := LeagueList.FindTeamByButton(Item.SubItems[tbMemberID]);
            end;
            PlayerDetailScore.Caption := 'Score: ' + Item.SubItems[tbScore] + '  ';
            thisplayer := LeagueList.FindPlayer(Item.SubItems[tbMemberID]);
            if Assigned(thisplayer) then
                PlayerDetailHandicap.Caption := thisplayer.Handicap + '   ' + thisplayer.Comment
            else
                PlayerDetailHandicap.Caption := '';
        end
    end else begin
        PlayerDetailName.Caption := '';
        PlayerDetailScore.Caption := '';
        PlayerDetailHandicap.Caption := '';
        if (Sender is TTeamBox) and (TTeamBox(Sender).TeamName <> 'Alias') then
            PlayerDetailTeam.Caption := TTeamBox(Sender).TeamName
        else
            PlayerDetailTeam.Caption := '';
    end;
end;

// Used by the TListView to sort players within Team boxes.
procedure TMainForm.PlayersCompare(Sender: TObject; Item1,
  Item2: TListItem; Data: Integer; var Compare: Integer);
begin
    Compare := GetScore(Item2.SubItems) - GetScore(Item1.SubItems);
end;

// Used to sort players within AllPlayers, by team: RGBY
// If Item1 < Item2, set Compare < 0. If Item1 > Item2, set Compare > 0.
procedure TMainForm.AllPlayersCompare(Sender: TObject; Item1,
  Item2: TListItem; Data: Integer; var Compare: Integer);
var s1, s2: string;
begin
    s1 := Copy(Item1.Caption, 1, 3);
    s2 := Copy(Item2.Caption, 1, 3);

    if s1 = s2 then begin
        if Item1.Caption < Item2.Caption then Compare := -1 else Compare := 1;
    end else if (s1 = 'Red') or (s2 = 'Yel') or ((s1 = 'Grn') and (s2 = 'Blu')) then Compare := -1
    else Compare := 1;
end;

procedure TMainForm.HandicapChange(Sender: TObject);
var i: Integer;
begin
    i := LeagueList.FindAnyInLeague(Team[(Sender as TTeamBox).Tag]);
    if ((Sender as TTeamBox).Items.Count = 0) or (i = -1) or (Team[(Sender as TTeamBox).Tag].Handicap <> LeagueList[i].Handicap) then
        (Sender as TTeamBox).HandicapColor := StaleColor
    else
        (Sender as TTeamBox).HandicapColor := clWindow;
    RankTeams;
end;

// Reorganise teams by Rank, so that the #1 ranked team occupies the #1 spot, etc.
procedure TMainForm.SortTeamsClick(Sender: TObject);

    function RankEmptyLast(Team: TTeamBox): Integer;
    begin
        Result := Team.Rank;
        if Team.Items.Count = 0 then Inc(Result, 1000);
    end;

var TempTeam:  TTeamBox;
    i, j:      Integer;
begin
    for i := 1 to Team.Count - 1 do  // Bogosort.  So shoot me.
        for j := i to Team.Count - 1 do
            if RankEmptyLast(Team[i]) > RankEmptyLast(Team[j]) then begin // swap i with j
                TempTeam := Team[i];  Team[i] := Team[j];  Team[j] := TempTeam;
            end;

    for i := 1 to Team.Count - 1 do  // maintain Tags, used for finding.
        Team[i].Tag := i;

    FormResize((Sender as TComponent).Owner as TMainForm);
end;

// Resize the form.  We don't care about the horizontal, but for the vertical,
// we must readjust the Tops and Heights of the Team boxes.
procedure TMainForm.FormResize(Sender: TObject);
var
    i: Integer;
    teamheight: Integer;
    teamwidth: Integer;
    perteam: Integer;
begin
    with Sender as TMainForm do begin
        if ClientHeight < Prefs.GridHigh * 54 - 2 then ClientHeight := Prefs.GridHigh * 54 - 2;

        { AllPlayers.Height is quantised: a Team box must contain
          an integer number of rows.  Each row is 14 pixels;
          the title and border of the box total 21 pixels.  }
//        AllPlayers.Height := ((ClientHeight - AllPlayers.Top - 24) div 14) * 14 + 23;

        { teamheight is similarly quantised. }
        if Prefs.GridHigh = 0 then
            perteam := 40
        else
            perteam := (ClientHeight + 2) div Prefs.GridHigh;
        teamheight := ((perteam - 26) div 14) * 14 + 23;  // The -26 is for the 23 pixels used by the team box's title line, plus 3 pixels for minimum spacing between team boxes.
        if teamheight < 51 then teamheight := 51;

        teamwidth := (ClientWidth - 2) div (Prefs.GridWide + 1);
        if teamwidth < 157 then teamwidth := 157;  // minimum team width ensures that there is room in title of each team for Rank, Handicap and Score.
    end;  // with

    AllPlayers.Width := teamwidth - 10;
    AdjustAllPlayersColumns;

    Bevel1.Width := teamwidth - 2;
    PlayerDetailPanel.Width := teamwidth - 10;

    if (Prefs.GridHigh <> 0) and Assigned(Team) then
        for i := 1 to Team.Count - 1 do begin
            Team[i].Top     := perteam * ((i-1) mod Prefs.GridHigh);
            Team[i].Height  := teamheight;

            Team[i].Left    := teamwidth * ((i-1) div Prefs.GridHigh + 1) + 8;
            Team[i].Width   := teamwidth - 10;
        end;

    GamesCombo.Width     := teamwidth - 25;
    GamesComboSpin.Left  := teamwidth - 18;
    Commit.Left         := teamwidth - Commit.Width - 2;
    UpdateSB.Left       := teamwidth - UpdateSB.Width - 2;

    if GamesCombo.ItemHeight > 0 then GamesCombo.DropDownCount := (Max(ClientHeight, Screen.Height div 2) - 35) div GamesCombo.ItemHeight;
end;

// Average together the given colors, according to the given weights.
function TMainForm.MixColors(Colors: array of TColor; Weights: array of Integer): TColor;
var i: Integer;
    red, green, blue: Integer;
    totalweight: Integer;
begin
    red := 0;  green := 0;  blue := 0;
    totalweight := 0;

    for i := 0 to High(Colors) do begin
        red   := red   + (Colors[i] and $FF) * Weights[i];
        green := green + (Colors[i] and $FF00) * Weights[i];
        blue  := blue  + (Colors[i] and $FF0000) * Weights[i];

        totalweight := totalweight + Weights[i];
    end;

    red   := red   div totalweight;
    green := green div totalweight;
    blue  := blue  div totalweight;
    if red   > $FF     then red   := $FF;
    if green > $FF00   then green := $FF00;
    if blue  > $FF0000 then blue  := $FF0000;

    Result := $02000000 + red + green and $FF00 + blue and $FF0000;
end;

// Send game data out to any TBoards that might be listening.
procedure TMainForm.UpdateScoreboardClick(Sender: TObject);

    // Return a color for a team, by mixing together the colours of its players
    function CalcColor(Team: TTeamBox): TColor;
    var colours: array[TNexusColour] of Integer;
        i: Integer;
    begin
        for i := Low(TNexusColour) to High(TNexusColour) do
            colours[i] := 0;

        for i := 0 to Team.Items.Count - 1 do
            if Team.Items[i].ImageIndex in [0..23] then
                Inc(colours[Team.Items[i].ImageIndex mod NUM_COLOURS]);
                
        Result := MixColors(NexusColors, colours);
    end;

var i, j:      Integer;
    numteams:  Integer;
    ScoreData: TStringList;
    teamcolor: TColor;
    s:         string;
begin  //---UpdateScoreboardClick---
    SortTeamsClick(Sender);
    ScoreData := TStringList.Create;
    with ScoreData do begin
        Add('DISPLAYREPORTS');  // command

        numteams := 0;
        for i := 1 to Team.Count - 1 do
            if Team[i].Items.Count > 0 then
                Inc(numteams);

        if numteams <= 6 then begin
            for i := 1 to Team.Count - 1 do  // for each Team box,
                if Team[i].Items.Count > 0 then begin
                    teamcolor := CalcColor(Team[i]);

                    Add('$02000000');                                               // TitleFontColor: black
                    Add(ColorToString(MixColors([teamcolor, $02000000], [3, 1])));  // TitleBackColor: team color darkened
                    Add(ColorToString(MixColors([teamcolor, $02FFFFFF], [2, 1])));  // BackgroundColor: team color lightened
                    Add(ColorToString(MixColors([teamcolor, $02FFFFFF], [1, 1])));  // OddColor: team color lightened some more

                    if (Team[i].TeamName = '') or (Team[i].TeamName = 'Alias') then
                        s := 'Team ' + IntToStr(i) + '  '
                    else
                        s := Team[i].TeamName + '  ';

                    if (Team[i].Handicap <> '') and (Team[i].Handicap <> '0') and (Team[i].Handicap <> '100%') then
                        s := s + Team[i].Handicap + '  ';

                    Add(s + IntToStr(Team[i].Score));

                    Add('Player,Score,Rank');
                    Add('left,right,right');

                    for j := 0 to Team[i].Items.Count - 1 do begin // for each player in the Team box,
                        if Team[i].Items[j].SubItems[tbMemberAlias] = '' then  // add a player name
                          Add(Team[i].Items[j].Caption)
                        else
                          Add(Team[i].Items[j].SubItems[tbMemberAlias]);
                        Add('clNone');
                        Add('1');

    //                    if Prefs.SoloHandicapsShowPercentage then
                            Add(Team[i].Items[j].SubItems[tbScore]);  // and a score,
    //                    else
    //                        Add(IntToStr(GetScore(Team[i].Items[j].SubItems)) + ' (' + Team[i].Items[j].SubItems[tbHandicap] + ')');
                        Add('clNone');
                        Add('1');

                        Add(Team[i].Items[j].SubItems[tbRank]);  // and a rank.
                        Add('clNone');
                        Add('1');

                        Add('EOREOR');
                    end;  // for j
                    Add('EOTEOT');
                end  // if Team[i] has players
        end else begin  // Show all the teams in one 'team' box, with one team per row.
            Add(ColorToString(clWhite));
            Add(ColorToString(clGray));
            Add(ColorToString(clWhite));
            Add(ColorToString($02FFFFE0));

            Add('Scores for ' + FormatDateTime(ShortTimeFormat, GameTime) + ' ' + FormatDateTime(ShortDateFormat, GameTime));
            Add('Team,Players,Score,Rank');
            Add('left,left,right,right');

            for i := 1 to Team.Count - 1 do  // for each Team box,
                if Team[i].Items.Count > 0 then begin
                    teamcolor := MixColors([CalcColor(Team[i]), $02FFFFFF], [2, 1]);

                    if (Team[i].TeamName = '') or (Team[i].TeamName = 'Alias') then
                        s := 'Team ' + IntToStr(i)
                    else
                        s := Team[i].TeamName;

                    Add(s);  // add a team name
                    Add(ColorToString(teamcolor));
                    Add('1');

                    s := '';
                    if not ((Team[i].Items.Count = 1) and (Team[i].TeamName = Team[i].Items[0].SubItems[tbMemberAlias])) then  // Skip the adding players bit if there's exactly one player and the team name exactly matches the player name.
                        for j := 0 to Team[i].Items.Count - 1 do  // for each player in the Team box,
                            s := s + Team[i].Items[j].SubItems[tbMemberAlias] + ' ' + Team[i].Items[j].SubItems[tbScore] + ', ';  // grab the player's alias

                    s := Copy(s, 1, Length(s) - 2);

                    Add(s);  // add the player list
                    Add(ColorToString(teamcolor));
                    Add('1');

                    Add(IntToStr(Team[i].Score));  // and a score,
                    Add(ColorToString(teamcolor));
                    Add('1');

                    Add(IntToStr(Team[i].Rank));  // and a rank.
                    Add(ColorToString(teamcolor));
                    Add('1');

                    Add('EOREOR');
                end;

            Add('EOTEOT');
        end;
        Add('EOSEOS');

      if Count > 1 then
          SendUDPs(TBOARD_PORT, CommaText);
      Free;
    end;  // with ScoreData
end;

// Recalculate friendly fire penalties (if enabled),
// recalculate the team's score (including handicap),
// optionally set the team name,
// make the handicap field invisible if necessary,
// manage enabledness of items on the popup menu,
// and call RankTeams.
procedure TMainForm.CalcTeamScore(TeamNum: Integer; DoName: Boolean);
var i, j, k: Integer;
    s: string;
//    handiscore: Integer;
begin
    if TeamNum < 1 then Exit;
    if not DoCalc then Exit;  // DoCalc is a global flag used by DoLeague, to prevent unnecessary recalculation of scores.

    Debug('CalcTeamScore ' + IntToStr(TeamNum) + ' start');

    with Team[TeamNum] do begin
        if DoName then begin
            i := LeagueList.FindAnyInLeague(Team[Tag]);
            if i = -1 then
                TeamName := 'Alias'
            else begin
                TeamName := LeagueList[i].BuildTeamName;
                Handicap := LeagueList[i].Handicap;
                HandicapVisible := (Handicap <> '100%') and (Handicap <> '0');
                if (Team[TeamNum].Items.Count = 0) or (i = -1) or (Team[TeamNum].Handicap <> LeagueList[i].Handicap) then
                    Team[TeamNum].HandicapColor := StaleColor
                else
                    Team[TeamNum].HandicapColor := clWindow;

                for j := 0 to Items.Count - 1 do begin
                    k := LeagueList[i].Players.FindButton(Items[j].SubItems[tbMemberID]);
                    if (k > -1) and (k < LeagueList[i].Players.Count) then begin
                        s := LeagueList[i].Players[k].Handicap;
//                        handiscore := GetScore(Items[j].SubItems);
//                        if Items[j].SubItems.Count > tbHandicap then
//                            Items[j].SubItems[tbHandicap] := '100';
//                             if Pos('+', s) > 0 then handiscore := handiscore + StrToIntDef(Copy(s, 2, 99), 0)
//                        else if Pos('-', s) > 0 then handiscore := handiscore - StrToIntDef(Copy(s, 2, 99), 0)
//                        else if Pos('%', s) > 0 then begin
//                            handiscore := handiscore * StrToIntDef(Copy(s, 1, Pos('%', s) - 1), 100) div 100;
//                            if Items[j].SubItems.Count > tbHandicap then
//                                Items[j].SubItems[tbHandicap] := Copy(s, 1, Pos('%', s) - 1);
//                        end;
                    end;
                end;
            end;
        end;  // if DoName

        SortType := stNone;  // Encourage a re-sort of players
        SortType := stBoth;  // based on their new scores
        CalcScore;

        if LeagueMode and (LeagueList.FindAllInLeague(Team[Tag]) = -1) then
            Color := StaleColor
        else
            Color := clWindow;
    end;  // with TeamBox

    RankTeams;

    Debug('CalcTeamScore ' + IntToStr(TeamNum) + ' start');
end;

procedure TMainForm.CalcAllScores(Recalc, DoName: Boolean);
type TPlayerData = record
        Team,     // what team is this player on?  -1 means did not play.
        Index,    // this player's index within his team
        Handicap,
        Rank,
        HitsBy,
        HitsOn,
        BaseHits,
        BaseDestroys: Integer;
        Score: Real;
    end;

    TAcaciaDeviceType = (dtFriend, dtEnemy, dtSentinel, dtMine, dtBase, dtTerminator);
    TOnBy = (obOn, obBy);


    // Convert a device ID number to a TAcaciaDeviceType.
    function AcaciaDeviceType(device: Integer): TAcaciaDeviceType;
    begin
        case device of
            (*4..6,*) 64..69: Result := dtBase;
            (*7,*) 70..75:    Result := dtSentinel;
            (*16,*) 76:       Result := dtTerminator;
            else              Result := dtEnemy;
        end
    end;

var
    Scores:     array [TAcaciaDeviceType, TOnBy, 1..6] of Integer;  // array to store all the important values from a scoring set

    procedure LoadAcaciaScores;

        // Grab a line of data from ScoresSetTable and stuff it into Scores[].
        procedure GetScoresSetLine(ScoringSet, DeviceNum: Integer; DeviceType: TAcaciaDeviceType);
        const SELECT_SCORES = 'SELECT HitOn_Location_1, HitOn_Location_2, HitOn_Location_3, HitOn_Location_4, HitOn_Location_5, HitOn_Location_6, ' +
                            '      ShotBy_Location_1, ShotBy_Location_2, ShotBy_Location_3, ShotBy_Location_4, ShotBy_Location_5, ShotBy_Location_6 ' +
                            ' FROM T2SCORING WHERE ScoreSet_ID = ';
        var i: Integer;
            zquery: TZReadOnlyQuery;
            bEOF: Boolean;
        begin
            // Grab scoring values for red team or any team shooting the DeviceNum team...
            zquery := TZReadOnlyQuery.Create(Self);
            try
                zquery.Connection := ZConnection;
                zquery.SQL.Add(SELECT_SCORES + IntToStr(ScoringSet) +
                               ' AND (ShotBy_Team = 1 OR ShotBy_Team = -1) AND (HitOn_Team = ' + IntToStr(DeviceNum) + ')');
                zquery.Open;
                bEOF := zquery.EOF;

                if not bEOF then
                  for i := 1 to 6 do
                      Scores[DeviceType, obBy, i] := zquery.Fields[i + 5].AsInteger;
            finally
                FreeSquish(zquery);
            end;

              // ... or if there's not a specific row, for shooting _any_ team.
            if bEOF then begin
                zquery := TZReadOnlyQuery.Create(Self);
                try
                    zquery.Connection := ZConnection;
                    zquery.SQL.Add(SELECT_SCORES + IntToStr(ScoringSet) +
                                   ' AND (ShotBy_Team = 1 OR ShotBy_Team = -1) AND (HitOn_Team = -1)');
                    zquery.Open;

                    if not zquery.EOF then
                        for i := 1 to 6 do
                            Scores[DeviceType, obBy, i] := zquery.Fields[i + 5].AsInteger;
                finally
                    FreeSquish(zquery);
                end;
            end;

            // Grab scoring values for the DeviceNum team shooting the read team or any team...
            zquery := TZReadOnlyQuery.Create(Self);
            try
                zquery.Connection := ZConnection;
                zquery.SQL.Add(SELECT_SCORES + IntToStr(ScoringSet) +
                               ' AND (HitOn_Team = 1 OR HitOn_Team = -1) AND (ShotBy_Team = ' + IntToStr(DeviceNum) + ')');
                zquery.Open;
                bEOF := zquery.EOF;

                if not bEOF then
                    for i := 1 to 6 do
                        Scores[DeviceType, obOn, i] := zquery.Fields[i - 1].AsInteger;
            finally
                FreeSquish(zquery);
            end;

            // ... or failing that, _any_ team shooting the red team or any team.
            if bEOF then begin
                zquery := TZReadOnlyQuery.Create(Self);
                zquery.Connection := ZConnection;
                try
                    zquery.SQL.Add(SELECT_SCORES + IntToStr(ScoringSet) +
                                   ' AND (HitOn_Team = 1 OR HitOn_Team = -1) AND (ShotBy_Team = -1)');
                    zquery.Open;

                    if not zquery.EOF then
                        for i := 1 to 6 do
                            Scores[DeviceType, obOn, i] := zquery.Fields[i - 1].AsInteger;
                finally
                    FreeSquish(zquery);
                end;
            end;
        end;

var scoresset:  Integer;
    zquery: TZReadOnlyQuery;

    begin  //---LoadAcaciaScores---
        zquery := TZReadOnlyQuery.Create(Self);
        try
            if not ZConnection.Connected then
                ZConnection.Connect;
            zquery.Connection := ZConnection;
            zquery.SQL.Add('SELECT C.ScoreSet_ID FROM T2GAMES G ' +
                                          'JOIN T2SCORESETCONFIG C ON G.ScoreSet = C.Description ' +
                                          'WHERE G.Game_ID = ' + IntToStr(GameNum));
            zquery.Open;
            if not zquery.EOF then begin
                scoresset := zquery.FieldValues['ScoreSet_ID'];

                GetScoresSetLine(scoresset, 1, dtFriend);
                GetScoresSetLine(scoresset, 2, dtEnemy);
                GetScoresSetLine(scoresset, 7, dtSentinel);
                GetScoresSetLine(scoresset, 7, dtMine);
                GetScoresSetLine(scoresset, 4, dtBase);
                GetScoresSetLine(scoresset, 16, dtTerminator);
            end;
        finally
            FreeSquish(zquery);
        end;
    end;

var
    Players:    array [0..59] of TPlayerData;

    procedure CalcAcaciaHits;
    var i:            Integer;
        hiton, hitby: Integer;
        thishit:      Integer;
        zquery: TZReadOnlyQuery;
    begin
        zquery := TZReadOnlyQuery.Create(Self);
        try
            zquery.Connection := ZConnection;
            zquery.SQL.Add('SELECT * FROM T2HITS ' +
                           'WHERE Game_ID = ' + IntToStr(GameNum));  // + ' AND Playing = 1');
            zquery.Open;
            while not zquery.Eof do begin
                hiton := zquery.FieldValues['HitOn_ID'];
                hitby := zquery.FieldValues['ShotBy_ID'];
                if ((hiton >= 60) or (Players[hiton].Team <> -1)) and
                   ((hitby >= 60) or (Players[hitby].Team <> -1)) then  // this hit includes a player that played
                    for i := 1 to 6 do begin
                        thishit := zquery.Fields[i + 4].AsInteger;
                        if thishit <> 0 then  // if thishit = 0, skip it, 'cos it won't change scores
                            if (hiton < 60) and (hitby < 60) and (Players[hiton].Team = Players[hitby].Team) then begin  // player vs player, friendly fire
                                Players[hiton].Score := Players[hiton].Score + Scores[dtFriend, obOn, i] *
                                                        thishit * Players[hitby].Handicap / 100;
                                Inc(Players[hiton].HitsOn, thishit);
                                Inc(GameHits, thishit);
                                Players[hitby].Score := Players[hitby].Score + Scores[dtFriend, obBy, i] *
                                                        thishit * Players[hitby].Handicap / 100;
                            end else begin  // hiton or hitby is >= 60; i.e. is a game unit
                                if hiton < 60 then begin
                                    if hitby < 60 then begin  // player vs player, enemy fire
                                        Players[hiton].Score := Players[hiton].Score + Scores[dtEnemy, obOn, i] *
                                                                thishit * Players[hitby].Handicap / 100;
                                        Inc(Players[hiton].HitsOn, thishit);
                                    end else  // player hit by game unit
                                        Players[hiton].Score := Players[hiton].Score + Scores[AcaciaDeviceType(hitby), obOn, i] * thishit;
                                end;
                                if hitby < 60 then begin  // Now add points to the player who did the hitting.
                                    Players[hitby].Score := Players[hitby].Score + Scores[AcaciaDeviceType(hiton), obBy, i] *
                                                            thishit * Players[hitby].Handicap / 100;
                                    if hiton < 60 then begin
                                        Inc(Players[hitby].HitsBy, thishit);
                                        Inc(GameHits, thishit);
                                    end;
                                end;
                            end;  // else >= 60; i.e. is a game unit
                    end;  // for i;  if real players
                zquery.Next;
            end;  // while not EOF
        finally
            FreeSquish(zquery);
        end;
    end;

var
    // NexusScores is an array to store all the important values from a Nexus profile / scoring set. ng_event_types:
    //  0..6:   tagged foe;
    //  7..13:  tagged ally;
    //  14..20: tagged by foe;
    //  21..27: tagged by ally;
    //  28: warning;
    //  29: termination;
    //  30: hit base;
    //  31: destroyed base;
    //  32: eliminated;
    //  33: hit by base;
    //  34: hit by mine;
    //  35: trigger pressed;
    //  36: game state (whatever that means);
    //  37..46: player tagged target (whatever that means);
    //  1402: score denial points.
    //  1404: lose points for being denied.
    // Used by LoadNexusScores and CalcNexusHits.
    NexusScores: array of Integer;

    procedure LoadNexusScores;
    var
        i, maxeventttype: Integer;
        zquery: TZReadOnlyQuery;
    begin
        zquery := TZReadOnlyQuery.Create(Self);
        try
            zquery.Connection := ZConnection;
            zquery.SQL.Add('SELECT MAX(Event_Type) as Max_Event_Type ' +
                           'FROM ng_game_stats GS ' +
                           'JOIN ng_profile_event_results PER ON GS.Profile_ID = PER.Profile_ID ' +
                           'WHERE Game_ID = ' + IntToStr(GameNum));
            zquery.Open;
            if VarIsNull(zquery.FieldValues['Max_Event_Type']) then
                maxeventttype := 0
            else
                maxeventttype := zquery.FieldValues['Max_Event_Type'];
        finally
            FreeSquish(zquery);
        end;

        SetLength(NexusScores, maxeventttype + 1);
        for i := 0 to maxeventttype do
            NexusScores[i] := 0;  // In Nexus, if a score is not given a value in ng_profile_results, it is assumed to be 0.

        zquery := TZReadOnlyQuery.Create(Self);
        try
            zquery.Connection := ZConnection;
            zquery.SQL.Add('SELECT Event_Type, Event_Score ' +
                           'FROM ng_game_stats GS ' +
                           'JOIN ng_profile_event_results PER ON GS.Profile_ID = PER.Profile_ID ' +
                           'WHERE Game_ID = ' + IntToStr(GameNum));
            zquery.Open;
            while not zquery.Eof do begin
                if (Low(NexusScores) <= zquery.FieldValues['Event_Type']) and (zquery.FieldValues['Event_Type'] <= High(NexusScores)) then
                    NexusScores[Integer(zquery.FieldValues['Event_Type'])] := zquery.FieldValues['Event_Score'];
                zquery.Next;
            end;  // while not EOF
        finally
            FreeSquish(zquery);
        end;
    end;

    procedure CalcNexusHits;
    var id, id2: Integer;
        eventtype: Integer;
        zquery: TZReadOnlyQuery;
    begin
        // ng_event_types: 0..6: tagged foe;  7..13: tagged ally;  14..20: tagged by foe;  21..27: tagged by ally; //
        //                 28: warning; 29: termination; 30: hit base; 31: destroyed base; 32: eliminated; 33: hit by base; 34: hit by mine;
        //                 35: trigger pressed; 36: game state (whatever that means); 37..46: player tagged target (whatever that means).
        //                 1402: score denial points.
        // TPlayerData: Team, Index, Handicap, Rank, HitsBy, HitsOn, BaseHits, BaseDestroys, Score
        //
        // Player_ID: player that scores the points.
        // Result_Data_1: player or device they scored the points off.
        // Result_Data_2: team of the player or device they scored the points off.
        // Result_Data_3: if hit is Event_Type = 1402 (base denier) or 1404 (base denied), shows points the player hit lost.
        // Result_Data_4: if hit is Event_Type = 1402 or 1404, number of shots denied.
        // Result_Data_5: if hit is Event_Type = 1402 or 1404, equals denier's Score (i.e. is redundant).
        zquery := TZReadOnlyQuery.Create(Self);
        try
            zquery.Connection := ZConnection;
            zquery.SQL.Add('SELECT Player_ID, Event_Type, Result_Data_1, Result_Data_3, Score ' +
                           'FROM ng_player_event_log ' +
                           'WHERE Game_ID = ' + IntToStr(GameNum));
            zquery.Open;
            while not zquery.EOF do begin
                id := zquery.FieldValues['Player_ID'];
                id2 := zquery.FieldValues['Result_Data_1'];

                eventtype := zquery.FieldValues['Event_Type'];
//                if eventtype = 47 then begin  // XXX special code for Nats 2011 -- remove before shipping XXX
//                    Players[id].Score := Players[id].Score + (250 * zquery.FieldValues['Result_Data_3']);
//                end else
                if ((eventtype = Prefs.NexusOneShotDenyFriend) or (eventtype = Prefs.NexusTwoShotDenyFriend) or
                    (eventtype = Prefs.NexusOneShotDenyEnemy) or (eventtype = Prefs.NexusTwoShotDenyEnemy)) and
                   (Players[id].Team = Players[id2].Team) then begin
                    // Do nothing: these are Vector's special events for Torn to gives points to base deniers.  But if the denier is on the same team, they should not score for this.
                end else
                if (Low(NexusScores) <= eventtype) and (eventtype <= High(NexusScores)) then begin
                    if ((eventtype in [0..6]) or (eventtype in [14..20])) and (Players[id].Team = Players[id2].Team) then  // event concerns foe, but it's an ally
                        Inc(eventtype, 7)
                    else if ((eventtype in [7..13]) or (eventtype in [21..27])) and (Players[id].Team <> Players[id2].Team) then  // event concerns ally, but it's a foe
                        Dec(eventtype, 7);

                    if eventtype = 1402 then  // Score denial points. This is marked as being worth 250, but is actually worth 250 per denial hit, so disregard NexusScores[] and use the score directly from ng_player_event_log.
                        Players[id].Score := Players[id].Score + zquery.FieldValues['Score']
                    else if Length(NexusScores) > eventtype then
                        Players[id].Score := Players[id].Score + NexusScores[eventtype];

                    if Integer(zquery.FieldValues['Event_Type']) in [0..13] then
                        Inc(Players[id].HitsBy)
                    else if Integer(zquery.FieldValues['Event_Type']) in [14..27] then
                        Inc(Players[id].HitsOn)
                    else if Integer(zquery.FieldValues['Event_Type']) in [30] then
                        Inc(Players[id].BaseHits)
                    else if Integer(zquery.FieldValues['Event_Type']) in [31] then
                        Inc(Players[id].BaseDestroys);
                end
                else if (eventtype = 1407) or (eventtype = 1408) or (eventtype = 1409) then
                    // do nothing -- 1407: player reloaded; 1408: player did not reload; 1409 player shot streak.
                else
                    Debug('Found unexpected Event_Type ' + IntToStr(eventtype));

                zquery.Next;
            end;  // while not EOF
        finally
            FreeSquish(zquery);
        end;
    end;

var i, j: Integer;
    thisplayer: TLeaguePlayer;
    thisgameplayer: TPlayerData;
    bestscore:  Real;
    bestindex, nextrank: Integer;
begin //---CalcAllScores---
    if Recalc and ((FSystemType = stAcacia) or ((FSystemType in [stNexus, stNexusWithZeon]) and not Prefs.NexusScores)) then begin  // If it's Acacia, or if it's Nexus and we're supposed to be calculating Nexus scores.  (It's impossible to recalculate scores for Laserforce.)
        Debug('CalcAllScores start');

        Screen.Cursor := crAppStart;
        try
            case FSystemType of
              stNexus, stNexusWithZeon: LoadNexusScores;
              stAcacia: LoadAcaciaScores;
            end;

            GameHits := 0;
            for i := 0 to 59 do
                with Players[i] do begin
                    Team  := -1;
                    Index := -1;
                    Handicap := 100;
                    Rank  := -1;
                    HitsBy := 0;
                    HitsOn := 0;
                    BaseHits := 0;
                    BaseDestroys := 0;
                    Score := 0;
                end;

            // For each player that is in a team box, load data into Players[].
            for i := 1 to Team.Count - 1 do  // for each Team box,
                for j := 0 to Team[i].Items.Count - 1 do  // for each player in the Team box,
                    if (Team[i].Items[j].SubItems.Count > tbPackID) then begin
                        thisgameplayer := Players[StrToInt(Team[i].Items[j].SubItems[tbPackID])];
                        thisgameplayer.Team  := i;
                        thisgameplayer.Index := j;

                        thisplayer := LeagueList.FindPlayer(Team[i].Items[j].SubItems[tbMemberID]);
                        if Assigned(thisplayer) and (Pos('%', thisplayer.Handicap) > 0) then
                            thisgameplayer.Handicap := StrToIntDef(Copy(thisplayer.Handicap, 1, Pos('%', thisplayer.Handicap) - 1), 100);
                        Players[StrToInt(Team[i].Items[j].SubItems[tbPackID])] := thisgameplayer;
                    end;  // if SubItems; for j; for i

            case FSystemType of
              stNexus, stNexusWithZeon: CalcNexusHits;
              stAcacia: CalcAcaciaHits;
            end;

            // Set up Players[].Ranks
            nextrank := 1;
            for i := 0 to 59 do begin
                bestscore := -1000000.0;
                bestindex := -1;

                for j := 0 to 59 do
                    if (bestscore < Players[j].Score) and (Players[j].Rank = -1) and (Players[j].Index <> -1) then begin
                        bestscore := Players[j].Score;
                        bestindex := j;
                    end;

                if bestindex > -1 then
                    Players[bestindex].Rank := nextrank;
                Inc(nextrank);
            end;  // for i

            // Put calculated scores back into Team boxes.
            for i := 0 to 59 do
                if Players[i].Team > -1 then
                    with Team[Players[i].Team].Items[Players[i].Index] do begin
                        SubItems[tbScore] := IntToStr(Round(Players[i].Score));
                        SubItems[tbRank]  := IntToStr(Players[i].Rank);
                        while SubItems.Count <= tbStatsEnd do
                            SubItems.Add('');
                        SubItems[tbHitYou] := IntToStr(Players[i].HitsOn);
                        SubItems[tbYouHit] := IntToStr(Players[i].HitsBy);
                        SubItems[tbBaseHits] := IntToStr(Players[i].BaseHits);
                        SubItems[tbBaseDestroys] := IntToStr(Players[i].BaseDestroys);
                    end;  // with; if; for
        finally
            Screen.Cursor := crDefault;
        end;
        Debug('CalcAllScores end');
    end;  // if Recalc etc.

    for i := 1 to Team.Count - 1 do     //         for any boxes
        if Team[i].Items.Count > 0 then // that have got players in them,
            CalcTeamScore(i, DoName);   //       recalculate scores
end;

// The mouse has been clicked in the AllPlayers box; disable the Teambox menu.
procedure TMainForm.AllPlayersClick(Sender: TObject);
begin
    TeamboxMenu.Enabled := False;
end;

// The mouse has been clicked in a team box; set TeamboxSelected to point to it.
procedure TMainForm.TeamboxMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
    TeamboxSelected := (Sender as TTeamBox).Tag;
    TeamboxMenu.Enabled := True;
end;

// Enable/disable menu items for the teambox popup menu, and also for the main menu's Teambox menu.
procedure TMainForm.TeamboxMenuPopup(Sender: TObject);
var i, index, playercount: Integer;
    oneplayerselected: Boolean;
    packname, teamname: string;
begin
    if Sender is TPopupMenu then
        TeamboxSelected := ((Sender as TPopupMenu).PopupComponent as TTeamBox).Tag;

    index := LeagueList.FindAnyInLeague(Team[TeamboxSelected]);
    playercount := Team[TeamboxSelected].Items.Count;
    oneplayerselected := (Team[TeamboxSelected].SelCount = 1) and (Team[TeamboxSelected].Selected.SubItems.Count > tbAccuracy);

    if oneplayerselected then begin
        packname := Team[TeamboxSelected].Selected.Caption;
        IdentifyThisPlayerP.Caption := 'Identify ' + packname + '...';
        HandicapThisPlayerP.Caption := 'Handicap ' + packname + '...';
    end else begin
        IdentifyThisPlayerP.Caption := 'Identify this player...';
        HandicapThisPlayerP.Caption := 'Handicap this player...';
    end;
    IdentifyThisPlayerP.Default := oneplayerselected;

    if (Team[TeamboxSelected].TeamName = '') or (Team[TeamboxSelected].TeamName = 'Alias') then
        teamname := 'this team'
    else begin
        teamname := Team[TeamboxSelected].TeamName;
        if Length(teamname) > 16 then teamname := Copy(teamname, 1, 12) + '...';
    end;

    HandicapThisTeamP.Caption := 'Handicap ' + teamname;
    RememberThisTeamP.Caption := 'Remember ' + teamname;
    UpdateThisTeamP.Caption   := 'Update ' + teamname;

    HandicapThisTeamP.Checked := Team[TeamboxSelected].HandicapVisible;

    HandicapThisTeamP.Enabled := (playercount > 0);
    RememberThisTeamP.Enabled := (playercount > 0) and LeagueMode;
    UpdateThisTeamP.Enabled   := (playercount > 0) and LeagueMode and
        ((index > -1) or (Team[TeamboxSelected].TeamName <> 'Alias')) and
        ((LeagueList.FindAllInLeague(Team[TeamboxSelected]) = -1) or (index > -1) and (Team[TeamboxSelected].Handicap <> LeagueList[index].Handicap));
    NameThisTeamP.Enabled     := (playercount > 0) and LeagueMode and (index > -1);
    IdentifyThisTeamP.Enabled := (playercount > 0) and LeagueMode and (LeagueList.Count > 0);
    IdentifyThisPlayerP.Enabled := oneplayerselected;
    HandicapThisPlayerP.Enabled := oneplayerselected;
    MergePlayersP.Enabled := Team[TeamboxSelected].SelCount > 1;
    AddAnAdjustmentP.Visible  := (playercount > 0);
    AdjustVictoryPointsP.Visible := (playercount > 0) and LeagueMode and IsPoints(Prefs);

    // Now copy the values we've just set up for the popup menu into the main menu.
    HandicapThisTeam.Checked  := HandicapThisTeamP.Checked;
    for i := 2 to TeamboxMenu.Count - 1 do begin
       TeamboxMenu.Items[i].Caption := TeamboxPopupMenu.Items[i].Caption;
       TeamboxMenu.Items[i].Enabled := TeamboxPopupMenu.Items[i].Enabled;
       TeamboxMenu.Items[i].Visible := TeamboxPopupMenu.Items[i].Visible;
    end;
end;

// Set a TeamBox's Handicap field visible/invisible, and recalculate the team's score.
procedure TMainForm.HandicapClick(Sender: TObject);
begin
    Team[TeamboxSelected].HandicapVisible := not Team[TeamboxSelected].HandicapVisible;
    Team[TeamboxSelected].CalcScore;
end;

// Set all TeamBox's handicap fields to visible.
procedure TMainForm.HandicapAllClick(Sender: TObject);
var i: Integer;
begin
    for i := 1 to Team.Count - 1 do
        if (Team[i].Items.Count > 0) and not Team[i].HandicapVisible then begin
            Team[i].HandicapVisible := True;
        end;
end;

// Set all TeamBox's handicap fields to invisible.
procedure TMainForm.HandicapNoneClick(Sender: TObject);
var i: Integer;
begin
    for i := 1 to Team.Count - 1 do begin
        Team[i].HandicapVisible := False;
        Team[i].CalcScore;
    end;
    RankTeams;
end;

// for each team box, if it's not already in the League, add it.
procedure TMainForm.RememberAllTeamsClick(Sender: TObject);
var i: Integer;
begin
    Screen.Cursor := crAppStart;
    try
        for i := 1 to Team.Count - 1 do
            if (Team[i].Items.Count > 0) and (LeagueList.FindTeamByName(Team[i]) = -1) then
                LeagueList.RememberTeam(Team[i]);
    finally
        Screen.Cursor := crDefault;
    end;
end;

// Remember all the players on this team in the League.
procedure TMainForm.RememberClick(Sender: TObject);
var i: Integer;
    s: string;
begin
    if (LeagueList.FindAnyInLeague(Team[TeamboxSelected]) > -1) and
       (MessageDlg('Hey!  There''s already a team like this!'#13#10'Do you really want to remember this set of players as a new team?', mtWarning, [mbYes, mbNo], 0) = mrNo) then
        Exit;

    s := '';
    for i := 0 to Team[TeamboxSelected].Items.Count - 1 do begin  // for each player in the Team box,
        s := Team[TeamboxSelected].Items[i].SubItems[tbMemberID];  // grab the player's button id
        if s <> '' then Break;
    end;  // for i
    if s = '' then begin
        ShowMessage('This team has no identified players, and so cannot be remembered.'#10'Please Identify one or more players, and try again.');
        Exit;
    end;

    LeagueList.RememberTeam(Team[TeamboxSelected]);
    RenameClick(Sender);
end;

// Remember any players that aren't already on this team in the League.
procedure TMainForm.UpdateClick(Sender: TObject);
var i: Integer;
begin
    i := LeagueList.FindTeamByName(Team[TeamboxSelected]);
    if i = -1 then Exit;

    LeagueList.UpdateTeam(Team[TeamboxSelected], LeagueList[i]);
    SaveLeague;
end;

// Remove this team from the League.
procedure TMainForm.ForgetClick(Sender: TObject);
var i: Integer;
begin
    i := LeagueList.FindTeamByName(Team[TeamboxSelected]);
    if i = -1 then Exit;

    LeagueList[i].Free;
    LeagueList.Delete(i);

    Team[TeamboxSelected].TeamName := 'Alias';
    Team[TeamboxSelected].Color := StaleColor;
    SaveLeague;
    BuildIdentifyMenu;
end;

// Add/edit a dummy "Adjustment" player to a team.
procedure TMainForm.AdjustClick(Sender: TObject);

    // Let the user edit the Adjustment value for a team, and return the new value in valu.
    function EditAdjust(var valu: Integer): TModalResult;
    var AdjustForm: TAdjustForm;
    begin
        AdjustForm := TAdjustForm.Create(Application);
        try
            AdjustForm.Handicap := valu;
            Result := AdjustForm.ShowModal;
            if Result = mrOK then
                valu := AdjustForm.Handicap;
        finally
            AdjustForm.Release;
        end;
    end;

var i, valu: Integer;
begin
    i := FindAdjust(TeamboxSelected);
    if i = -1 then begin
        valu := -1000;
        if EditAdjust(valu) = mrOK then begin
            Team[TeamboxSelected].SortType := stNone;  // turn sorting off while we add to the list; otherwise it tries to sort on partially added items.

            AddAdjustment(TeamboxSelected, valu);
        end;
    end else begin
        valu := GetScore(Team[TeamboxSelected].Items[i].SubItems);
        if EditAdjust(valu) = mrOK then begin
            if valu = 0 then
                Team[TeamboxSelected].Items.Delete(i)
            else
                Team[TeamboxSelected].Items[i].SubItems[tbScore] := IntToStr(valu);
        end;
    end;
    Team[TeamboxSelected].CalcScore;
    RankTeams;
end;

// Let the user edit the number of victory points the team will receive.
procedure TMainForm.AjustVictoryPointsClick(Sender: TObject);
var AdjustForm: TAdjustForm;
    thisteam : TTeamBox;
begin
    AdjustForm := TAdjustForm.Create(Application);
    try
        thisteam := Team[TeamboxSelected];
        AdjustForm.Handicap := Round(thisteam.VictoryHandicap);
        AdjustForm.Step := 1;
        if AdjustForm.ShowModal = mrOK then begin
            thisteam.VictoryHandicap := AdjustForm.Handicap;
            if thisteam.VictoryPoints <> 0 then
                thisteam.VictoryPoints := thisteam.VictoryPoints + thisteam.VictoryHandicap;
        end;
    finally
        AdjustForm.Release;
    end;
    RankTeams;
end;

// Set a name for this team, or change the existing name.
procedure TMainForm.RenameClick(Sender: TObject);
var i: Integer;
    s: string;
begin
    if Team[TeamboxSelected].TeamName = 'Alias' then
        s := ''
    else
        s := Team[TeamboxSelected].TeamName;

    if InputQuery('Rename', 'Set a new name for this team:', s) then begin
        Team[TeamboxSelected].TeamName := s;
        i := LeagueList.FindTeamByName(Team[TeamboxSelected]);
        LeagueList[i].TeamName := s;

        SaveLeague;
        BuildIdentifyMenu;
    end;
end;

procedure TMainForm.SetPlayerHandicap(v: string);
//var handiscore: Integer;
begin
//    handiscore := GetScore(Team[TeamboxSelected].Selected.SubItems);
//    if Items[j].SubItems.Count > tbHandicap then
//        Items[j].SubItems[tbHandicap] := '100';
//         if Pos('+', v) > 0 then handiscore := handiscore + StrToIntDef(Copy(v, 2, 99), 0)
//    else if Pos('-', v) > 0 then handiscore := handiscore - StrToIntDef(Copy(v, 2, 99), 0)
//    else if Pos('%', v) > 0 then handiscore := handiscore * StrToIntDef(Copy(v, 1, Pos('%', v) - 1), 100) div 100
//    else if Prefs.HandicapStyle = '+' then handiscore := handiscore + StrToIntDef(v, 0)
//    else if Prefs.HandicapStyle = '-' then handiscore := handiscore - StrToIntDef(v, 0)
//    else if Prefs.HandicapStyle = '%' then handiscore := handiscore * StrToIntDef(v, 100) div 100;
//    if Items[j].SubItems.Count > tbHandicap then
//        Items[j].SubItems[tbHandicap] := Copy(s, 1, Pos('%', s) - 1);
end;

// Pick an alias for this player.
procedure TMainForm.IdPlayerClick(Sender: TObject);
var Alias: TAlias;
    thisplayer: TLeaguePlayer;
begin
    if TornForm.SystemType = stLaserforce then
        Alias := TAliasLaserforce.Create(Application)
    else
        Alias := TAliasDB.Create(Application);
    try
        Alias.PlayerAlias.Text := Team[TeamboxSelected].Selected.SubItems[tbMemberAlias];
        if Alias.ShowModal = mrOK then begin
            Team[TeamboxSelected].Selected.SubItems[tbMemberAlias] := Alias.PlayerAlias.Text;
            Team[TeamboxSelected].Selected.SubItems[tbMemberID]    := Alias.ButtonID.Text;
        end else begin
            Team[TeamboxSelected].Selected.SubItems[tbMemberAlias] := '';
            Team[TeamboxSelected].Selected.SubItems[tbMemberID]    := '';
        end;
    finally
        Alias.Release;
    end;

    thisplayer := LeagueList.FindPlayer(Team[TeamboxSelected].Selected.SubItems[tbMemberID]);
    if Assigned(thisplayer) then
        SetPlayerHandicap(thisplayer.Handicap);

    if LeagueMode and (LeagueList.FindAllInLeague(Team[TeamboxSelected]) = -1) then
        Team[TeamboxSelected].Color := StaleColor
    else
        Team[TeamboxSelected].Color := clWindow;

    CalcTeamScore(TeamBoxSelected, False);
end;

procedure TMainForm.HandicapPlayerClick(Sender: TObject);
var s, v: string;
    thisplayer: TLeaguePlayer;
begin
    case Prefs.HandicapStyle of
      '+': v := '+0';
      '-': v := '-0';
      '%': v := '100%';
    end;

    s := 'Enter a handicap for ';
    if Team[TeamboxSelected].Selected.SubItems[tbMemberAlias] = '' then
        s := s + 'this player'
    else begin
        s := s + Team[TeamboxSelected].Selected.SubItems[tbMemberAlias];

        thisplayer := LeagueList.FindPlayer(Team[TeamboxSelected].Selected.SubItems[tbMemberID]);
        if Assigned(thisplayer) then
            v := thisplayer.Handicap;
    end;

    if InputQuery(s, s + ':', v) then
        SetPlayerHandicap(v);
    Team[TeamBoxSelected].CalcScore;
end;

// Merge two (or more) player scores for a single game into one player score which is the sum of those two.
procedure TMainForm.MergePlayersClick(Sender: TObject);
var selected: array of Integer;
    listindex, selectedindex: Integer;
    score: Integer;
    alias: string;
    buttonid: string;
    s: string;
begin
    SetLength(selected, Team[TeamboxSelected].SelCount);

    listindex := 0;
    selectedindex := 0;
    score := 0;
    alias := '';

    while listindex < Team[TeamboxSelected].Items.Count do begin
        if Team[TeamboxSelected].Items[listindex].Selected then begin
            selected[selectedindex] := listindex;

            if alias = '' then begin
                alias := Team[TeamboxSelected].Items[listindex].SubItems[tbMemberAlias];
                buttonid := Team[TeamboxSelected].Items[listindex].SubItems[tbMemberID];
            end;

            Inc(selectedindex);
            Inc(score, StrToInt(Team[TeamboxSelected].Items[listindex].SubItems[tbScore]));
        end;
        Inc(listindex);
    end;

    s := 'Merge together these ' + IntToStr(Length(selected)) + ' scores for ' + alias + '?';
    if Application.MessageBox(PChar(s),
      PChar('Merge ' + alias + '?'), MB_YESNO) = IDYES then begin
        // Do the merge.
            Team[TeamboxSelected].Items[selected[0]].SubItems[tbMemberAlias] := alias;
            Team[TeamboxSelected].Items[selected[0]].SubItems[tbMemberID]    := buttonid;
            Team[TeamboxSelected].Items[selected[0]].SubItems[tbScore]       := IntToStr(score);
            for selectedindex := 1 to High(selected) do begin
                Team[TeamboxSelected].Items[selected[selectedindex]].SubItems[tbMemberAlias] := '';
                Team[TeamboxSelected].Items[selected[selectedindex]].SubItems[tbMemberID]    := '';
                Team[TeamboxSelected].Items[selected[selectedindex]].SubItems[tbScore]       := '0';
        end;
    end;

    //CalcTeamScore(TeamBoxSelected, False);  // Merging players shouldn't affect scores.
end;

// Find an 'Adjustment' player on this team.
function TMainForm.FindAdjust(TeamNum: Integer): Integer;
var i: Integer;
begin
    Result := -1;

    for i := 0 to Team[TeamNum].Items.Count - 1 do  // for each player in the Team box,
        if Copy(Team[TeamNum].Items[i].SubItems[tbMemberAlias], 1, 6) = 'Adjust' then
            Result := i
end;

procedure TMainForm.HelpClick(Sender: TObject);
begin
    Application.HelpCommand(HELP_CONTENTS, 0);
end;

// Show the About box
procedure TMainForm.AboutClick(Sender: TObject);
begin
    Splash := TSplash.Create(Application);
    Splash.Memo2.Lines.CommaText := Exceptions;
    Splash.ShowModal;
    Splash.Release;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
var i: Integer;
begin
    try  // This whole sub is wrapped in a 'try' so that if an error occurs, the program will still exit.  Maybe.
        if LeagueMode then
            SaveLeague;

        LeagueList.Free;
        GameList.Flush;  // Free games, and free those games' teams and players.
        GameList.Free;
        ServerGameList.Free;

        If Assigned(Prefs.LeaguePoints) then FreeAndNil(Prefs.LeaguePoints);
        MRUList.Free;

        for i := Team.Count - 1 downto 0 do
            Team[i].Free;

        Team.Free;
        Team := nil;

        closesocket(mySocket);
        WSACleanup;

        if ReportForm  <> nil then ReportForm.Release;
        if LadderForm  <> nil then LadderForm.Release;
        if DramaForm   <> nil then DramaForm.Release;
        if PyramidForm <> nil then PyramidForm.Release;
        if FixtureForm <> nil then FixtureForm.Release;

//        if FIsNexus then
//            ZConnection.Disconnect;
    finally
        ;
    end;
end;

procedure TMainForm.Exit1Click(Sender: TObject);
begin
    ((Sender as TMenuItem).Owner as TMainForm).Close;  // This will call TForm1.FormClose on the way out.
end;

// Show the Leagues form.
// Note that we are calling a custom Create, to which we can pass league info.
procedure TMainForm.ShowLeagueClick(Sender: TObject);
var LeagueForm: TLeagueForm;
    i, j: Integer;
begin
    LeagueForm := TLeagueForm.CreateLeague(LeagueList, GameList, Prefs, SaveDialog1.FileName, Prefs.HandicapStyle, Application);
    try
        if LeagueForm.ShowModal = mrOK then begin
            SaveLeague;
            BuildIdentifyMenu;

            for i := 1 to Team.Count - 1 do begin
                if (Team[i].Items.Count = 0) or (LeagueList.FindAllInLeague(Team[i]) <> -1) then
                    Team[i].Color := clWindow
                else
                    Team[i].Color := StaleColor;

                j := LeagueList.FindTeamByName(Team[i]);
                if (Team[i].Items.Count = 0) or (j = -1) or (Team[i].Handicap = LeagueList[j].Handicap) then
                    Team[i].HandicapColor := clWindow
                else
                    Team[i].HandicapColor := StaleColor;
            end;
        end;
    finally
        LeagueForm.Release;
    end;
    CalcAllScores(True, True);
end;

// Delete all existing teams from the League.
// Choose a name for this new league, and enable all the League Mode options.
procedure TMainForm.NewLeagueClick(Sender: TObject);
var i: Integer;
begin
//  ShowMessage('begin');
    if not SaveDialog1.Execute then Exit;

    LeagueList.Clear;
    GameList.Clear;
//  ShowMessage('cleared');

    LeagueMode := True;
    LeagueEnable;
    Caption := 'TORN -- ' + ChangeFileExt(ExtractFileName(SaveDialog1.FileName), '');
//  ShowMessage('enabled');

    if Assigned(Prefs.LeaguePoints) then
        Prefs.LeaguePoints.Clear
    else
        Prefs.LeaguePoints := TDoubleList.Create;

    for i := 1 to 3 do          // set initial values for point scores gained for first place, second place, etc.
        Prefs.LeaguePoints.Add(0);

    Prefs.LeaguePointsHigh := 0;

    IsSaved := True;  // prevent backup code from running and trying to backup a file that does not yet exist
//  ShowMessage('saving');
    SaveLeague;
//  ShowMessage('end');
end;

procedure TMainForm.MRUClick(Sender: TObject);
begin
    SaveDialog1.FileName := Copy((Sender As TMenuItem).Caption, 4, MaxInt);
    OpenDialog1.FileName := SaveDialog1.FileName;
    LoadLeague;
end;

// Delete all existing teams from the League list.
// Enable/disable all the League Mode options.
// Create the data table for this league, if necessary.
// Update the Most Recently Used list.
procedure TMainForm.LeagueEnable;
var i: integer;
    MyRegistry: TRegistry;
begin
    ShowLeague.Enabled        := LeagueMode;
    SaveLeague2.Enabled       := LeagueMode;
    CloseLeague.Enabled       := LeagueMode;
    EditGames.Enabled         := LeagueMode;
    ReportLeague.Enabled      := LeagueMode;
    Pyramid.Enabled           := LeagueMode;
    Fixtures.Enabled          := LeagueMode;
    RememberAllTeams.Enabled  := LeagueMode;
    RememberAllTeams2.Enabled := LeagueMode;
    Match1.Enabled            := LeagueMode;
    Match2.Enabled            := LeagueMode;
    ViewGcc.Enabled           := LeagueMode;
    ViewHistorical.Enabled    := LeagueMode;
    ViewGcc.Checked           := True;

    Commit.Visible   := LeagueMode;
    UpdateSB.Visible := not LeagueMode;

//    LeagueList.Clear;
//    GameList.Clear;

    i := MRUList.IndexOf(SaveDialog1.FileName);
    if i <> -1 then MRUList.Delete(i);
    MRUList.Insert(0, SaveDialog1.FileName);    // update the Most Recently Used list

    UpdateMRU(MRU1, 1);                         // and update the menu
    UpdateMRU(MRU2, 2);
    UpdateMRU(MRU3, 3);
    UpdateMRU(MRU4, 4);
    UpdateMRU(MRU5, 5);
    UpdateMRU(MRU6, 6);
    UpdateMRU(MRU7, 7);
    UpdateMRU(MRU8, 8);
    UpdateMRU(MRU9, 9);

    MyRegistry := TRegistry.Create;  // open the Registry
    try
        MyRegistry.RootKey := HKEY_LOCAL_MACHINE;
        MyRegistry.OpenKey('SOFTWARE\DougWare\Torn', True);

        for i := 0 to Min(3, MRUList.Count - 1) do      // and write the MRU list out to the Registry
            MyRegistry.WriteString('MRU' + IntToStr(i+1), MRUList[i]);

        MyRegistry.CloseKey;
    finally
        MyRegistry.Free;
    end;
end;

procedure TMainForm.SaveLeague;
begin
    try
        if not IsSaved then
            MakeBackup(ChangeFileExt(SaveDialog1.FileName, '.Torn'));
    finally
        SaveLeagueAsXml(ChangeFileExt(SaveDialog1.FileName, '.Torn'), Prefs, LeagueList, GameList);
        IsSaved := True;
    end;
end;

procedure TMainForm.SaveLeague2Click(Sender: TObject);
begin
    SaveLeague;
end;

procedure TMainForm.LoadLeague;
var OldSize: Integer;  // Used for resizing the grid of TeamBoxes.
begin
    OldSize := Prefs.GridHigh * Prefs.GridWide;
    LoadLeagueAsXml(OpenDialog1.FileName, Prefs, LeagueList, GameList);

    LeagueMode := True;
    LeagueEnable;
    IsSaved := False;
    Caption := 'TORN -- ' + ChangeFileExt(ExtractFileName(SaveDialog1.FileName), '');
    GridSize(OldSize);
    BuildIdentifyMenu;
end;

procedure TMainForm.OpenLeagueClick(Sender: TObject);
begin
    if not OpenDialog1.Execute then Exit;
    SaveDialog1.FileName := OpenDialog1.FileName;
    LoadLeague;
end;

procedure TMainForm.CloseLeagueClick(Sender: TObject);
begin
    if Application.MessageBox('Save changes to league file?'#13#10'(If you answer No, any changes will be lost.)',
      'Save changes?', MB_YESNO) = IDYES then
        SaveLeague;

    LeagueMode := False;
    LeagueEnable;

    LeagueList.Clear;
    GameList.Clear;
    If Assigned(Prefs.LeaguePoints) then FreeAndNil(Prefs.LeaguePoints);
end;

procedure TMainForm.EditGamesClick(Sender: TObject);
var GamesForm: TGamesForm;
    i: Integer;
begin
    GamesForm := TGamesForm.Create(Application);
    GamesForm.Top := 0;
    GamesForm.Left := (Screen.Width - GamesForm.Width) div 2;

    GamesForm.LeagueList := LeagueList;
    GamesForm.GameList := GameList;
    for i := 0 to LeagueList.Count - 1 do
        GamesForm.TeamCombo.Items.Add(Format('%4.4d: ', [LeagueList[i].TeamID]) + LeagueList[i].BuildTeamName);

    GamesForm.ShowModal;
    GamesForm.Release;
end;

// For each Team box, build the Identify submenu, which has one entry for each team in the League list.
// Called by routines that affect this list: Forget, Rename, LoadLeague
procedure TMainForm.BuildIdentifyMenu;

    procedure BuildOneMenu(ThisMenu: TMenuItem);
    var i: integer;
    begin
        ThisMenu.Clear;
        for i := 0 to LeagueList.Count - 1 do
            with ThisMenu do begin
                Add(TMenuItem.Create(Self));

                Items[i].Caption := LeagueList[i].BuildTeamName;
                Items[i].OnClick := IdentifyTeam;
                Items[i].Enabled := True;
                Items[i].HelpContext := 9;
            end;
    end;

begin
    BuildOneMenu(IdentifyThisTeam);
    BuildOneMenu(IdentifyThisTeamP);
end;

// Change the listed name for this team.
procedure TMainForm.IdentifyTeam(Sender: TObject);
var i: Integer;
begin
    Team[TeamboxSelected].TeamName := (Sender as TMenuItem).Caption;

    i := LeagueList.FindTeamByName(Team[TeamboxSelected]);
    if i > -1 then
        Team[TeamboxSelected].Handicap := LeagueList[i].Handicap;

    CalcTeamScore(TeamboxSelected, False);
end;

// Write a record for this player in this game to the GameList.
procedure TMainForm.CommitPlayer(Player: TListItem; Team: Integer);
var thisgame,
    thisteam: Integer;
    c: Integer;
begin
    thisgame := GameList.Find(GameTime);
    if thisgame = -1 then Exit;

    if Copy(Player.SubItems[tbMemberAlias], 1, 6) = 'Adjust' then begin  // it's an Adjustment
        thisteam := GameList[thisgame].Teams.Find(Team);
        if thisteam > -1 then
            GameList[thisgame].Teams[thisteam].Adjustment := GetScore(Player.SubItems);
        Exit;
    end;

    with GameList[thisgame].Players.Add do begin
        if (Player.SubItems[tbMemberID] = '') or
                    (Player.SubItems[tbMemberID] = 'DUP') or
                    (Player.SubItems[tbMemberID] = 'INVALID') then
            PlayerID := Player.Caption
        else
            PlayerID := Player.SubItems[tbMemberID];

        TeamID := Team;

        Pack   := Player.Caption;
        Score  := GetScore(Player.SubItems);
        Rank   := StrToIntDef(Player.SubItems[tbRank], 0);
        try
          c := Player.ImageIndex;
          if (c >= -1) and (c < NUM_COLOURS) then
            Colour := c
          else if (c >= NUM_COLOURS) and (c < 2 * NUM_COLOURS) then
            Colour := c - NUM_COLOURS
          else
            Colour := -1;
        except
          Colour := -1;
        end;
        if Player.SubItems.Count > tbStatsEnd then begin
            HitsOn := StrToIntDef(Player.SubItems[tbHitYou], 0);
            HitsBy := StrToIntDef(Player.SubItems[tbYouHit], 0);
            BaseHits := StrToIntDef(Player.SubItems[tbBaseHits], 0);
            BaseDestroys := StrToIntDef(Player.SubItems[tbBaseDestroys], 0);
        end;
    end;
end;

function TMainForm.SetThisTeamColour(thisgame, thisteam: Integer; Colour: string; Color: Integer): TNexusColour;
begin
    GameList[thisgame].Teams[thisteam].Colour := Colour;
    GameList[thisgame].Teams[thisteam].NexusColour := Color;
    Result := Color;
end;

// Return a colour for a team in a given game.  teamnum is the index in teams within this game, not the list of all games.
function TMainForm.GetNumTeamColourIndex(gamenum, teamnum: Integer): TNexusColour;
var colour: string;                     // Infusion only
    colourcounts: array [TNexusColour] of Integer;
    i: Integer;
    max: Integer;
begin
    Result := GameList[gamenum].Teams[teamnum].NexusColour;
    if Result > -1 then Exit;

    for i := Low(TNexusColour) to High(TNexusColour) do
        colourcounts[i] := 0;

    colour := GameList[gamenum].Teams[teamnum].Colour;
    if (Length(colour) > 0) then begin
      Result := StringToNexusColour(colour);
      Exit;
    end else begin
      for i := 0 to GameList[gamenum].Players.Count - 1 do
          if GameList[gamenum].Players[i].TeamID = GameList[gamenum].Teams[teamnum].TeamID then begin
              case FSystemType of
                stAcacia: begin
                  colour := GameList[gamenum].Players[i].Pack;
                  if Length(colour) > 0 then
                      case UpperCase(colour)[1] of
                          'R': Inc(colourcounts[0]);
                          'B': Inc(colourcounts[1]);
                          'G': Inc(colourcounts[2]);
                          'Y': Inc(colourcounts[3]);
                      end;
                end;
                else
                    Inc(colourcounts[GameList[gamenum].Players[i].Colour]);
              end;
          end;
    end;

    case FSystemType of
      stAcacia: begin
        if      (colourcounts[0] > colourcounts[2])  and (colourcounts[0] > colourcounts[1])  and (colourcounts[0] > colourcounts[3]) then
            Result := SetThisTeamColour(gamenum, teamnum, 'Red', 0)  // red
        else if (colourcounts[2] > colourcounts[0])  and (colourcounts[2] > colourcounts[1])  and (colourcounts[2] > colourcounts[3]) then
            Result := SetThisTeamColour(gamenum, teamnum, 'Grn', 2)  // green
        else if (colourcounts[1] > colourcounts[0])  and (colourcounts[1] > colourcounts[2])  and (colourcounts[1] > colourcounts[3]) then
            Result := SetThisTeamColour(gamenum, teamnum, 'Blu', 1)  // blue
        else if (colourcounts[3] > colourcounts[0])  and (colourcounts[3] > colourcounts[2])  and (colourcounts[3] > colourcounts[1]) then
            Result := SetThisTeamColour(gamenum, teamnum, 'Yel', 3)  // yellow
        else if (colourcounts[0] = colourcounts[1])  and (colourcounts[0] > colourcounts[2])  and (colourcounts[0] > colourcounts[3]) then Result := 5  // red + blue
        else if (colourcounts[0] = colourcounts[3])  and (colourcounts[0] > colourcounts[2])  and (colourcounts[0] > colourcounts[1]) then Result := 7  // red + yellow
        else if (colourcounts[2] = colourcounts[1])  and (colourcounts[2] > colourcounts[0])  and (colourcounts[2] > colourcounts[3]) then Result := 6  // green + blue
        else if (colourcounts[2] = colourcounts[3])  and (colourcounts[2] > colourcounts[0])  and (colourcounts[2] > colourcounts[1]) then Result := 3  // green + yellow
        else if (colourcounts[1] = colourcounts[3])  and (colourcounts[1] > colourcounts[0])  and (colourcounts[1] > colourcounts[2]) then Result := 2  // blue + yellow
        else Result := -1;
        // note no case for red + green, as there's no good colour to choose for that case
      end;
      else begin
        if GameList[gamenum].Players.Count = 0 then Result := -1
        else begin
          max := 0;
          Result := -1;
          for i := 0 to high(colourcounts) do
            if max < colourcounts[i] then begin
              max := colourcounts[i];
              Result := i;
            end;
        end;

        if Result <> -1 then
            GameList[gamenum].Teams[teamnum].NexusColour := Result;
      end;
    end;  // case
end;

procedure TMainForm.CommitMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
    FShiftState := Shift;
end;

// Write fresh records for this game to the GameList, and optionally refresh the scoreboard.
procedure TMainForm.CommitClick(Sender: TObject);

    procedure UpdateScoreboardDetailed;
    var reports: TZoomReports;
    begin
        reports := TZoomReports.Create;
        try
            BuildDetailedGame(reports, GameTime - 1/86400, GameTime + 1/86400, False);
            ShowOnScoreboard(reports[0]);
        finally
            reports.Free;
        end;
    end;

var i, j, k: Integer;
    thisgame,
    thisteam: Integer;
    totalscore: Integer;
begin
    Screen.Cursor := crAppStart;
    try
        // Find this game and clear it.
        thisgame := GameList.Force(GameTime);
        GameList[thisgame].Hits := GameHits;  // record the total number of hits scored in this game, for calculating T2 Effectiveness
        GameList[thisgame].Teams.Clear;
        GameList[thisgame].Players.Clear;

        totalscore := 0;
        for i := 1 to Team.Count - 1 do
            Inc(totalscore, Team[i].Score);

        for i := 1 to Team.Count - 1 do begin
            j := LeagueList.FindTeamByName(Team[i]);

            if Prefs.AutoUpdateTeams and (Team[i].Items.Count > 0) then begin
                if j = -1 then  // this is a new team, so remember it
                    j := LeagueList.RememberTeam(Team[i])
                else begin  // this is an existing team, so update it
                    LeagueList[j].Handicap := Team[i].Handicap;
                    LeagueList.UpdateTeam(Team[i], LeagueList[j]);
                end;
            end;  // if AutoUpdateTeams

            if Team[i].Items.Count = 0 then
                // do nothing
            else if j = -1 then begin  // this team was not found in the league list; commit players as if they're not on a team
                for k := 0 to Team[i].Items.Count - 1 do  // commit each player in this team box to the database
                    CommitPlayer(Team[i].Items[k], 0);
            end else begin  // this team was found; commit players for league team j
                thisteam := GameList[thisgame].Teams.Find(LeagueList[j].TeamID);
                if thisteam > -1 then begin // this team has already been listed in this game
                    Team[i].TeamName := 'Alias';
                    Team[i].Color := StaleColor;
                end else begin  // this team is not already listed in this game, so add it
                    thisteam := GameList[thisgame].Teams.Force(LeagueList[j].TeamID);
                    with GameList[thisgame].Teams[thisteam] do begin
                        Score := Team[i].Score;

                        if Team[i].Rank <= Prefs.LeaguePoints.Count then  // calculate the points for the team's rank in the game
                            Points := Prefs.LeaguePoints[Team[i].Rank - 1] + Team[i].VictoryHandicap
                        else
                            Points := 0 + Team[i].VictoryHandicap;

                        Team[i].VictoryPoints := Points;

                        for k := 0 to Team[i].Items.Count - 1 do
                            if StrToIntDef(Team[i].Items[k].SubItems[tbRank], 0) = 1 then
                                Points := Points + Prefs.LeaguePointsHigh;  // add in the high-score points

                        if totalscore <> 0 then
                            Points := Points + Round(Prefs.LeaguePointsProportional * Team[i].Score * 1.0 / totalscore);
                    end;

                    for k := 0 to Team[i].Items.Count - 1 do  // commit each player in this team box to the database
                        CommitPlayer(Team[i].Items[k], LeagueList[j].TeamID);

                    GameList[thisgame].Teams[thisteam].NexusColour := -1;
                    GetNumTeamColourIndex(thisgame, thisteam);
                end;  // if not Locate
            end;  // if j <> -1
        end;  // for i to Team.Count

        for k := 0 to AllPlayers.Items.Count - 1 do  // now commit any players left in AllPlayers, as if they're not on a team
            CommitPlayer(AllPlayers.Items[k], -1);

        SaveLeague;
        if Prefs.AutoUpdateScoreboard then begin
            if (Prefs.ScoreboardStyle = ssDetailed) then
                UpdateScoreboardDetailed
            else
                UpdateScoreboardClick(Sender);
        end;
        Commit.Caption := 'Re&commit';
    finally
        Screen.Cursor := crDefault;
    end;

    if (ssAlt in FShiftState) or (ssCtrl in FShiftState) or (ssShift in FShiftState) then
        MatchClick(Self);

    FShiftState := [];
end;

// "View GCC games" or "View historical games" has been clicked.
// Rebuild the contents of GamesCombo.
procedure TMainForm.ViewGamesClick(Sender: TObject);
begin
    (Sender as TMenuItem).Checked := True;

    Latest.Enabled := Sender = ViewGcc;

    BuildGamesCombo;
    GamesCombo.ItemIndex := 0;
    GamesComboChange(GamesCombo);
end;

function BuildGameTime(GameTime: TDateTime): string;
begin
    if Trunc(GameTime) = Date then  // if this game is today,
        Result := FormatDateTime(ShortTimeFormat, GameTime)  // just show the short time
    else if Trunc(GameTime) = Date - 1 then  // if this game is yesterday,
        Result := FormatDateTime(ShortTimeFormat, GameTime) + ' yesterday'  // show the short time plus ' yesterday'
    else
        Result := FormatDateTime(ShortTimeFormat, GameTime) + ' ' + FormatDateTime(ShortDateFormat, GameTime);  // show the short time and short date
end;

// Rebuild the contents of GamesCombo.
procedure TMainForm.BuildGamesCombo;
    procedure BuildGamesFromDB;
    var zquery: TZReadOnlyQuery;
        query: TZoomReport;
        i: Integer;
        servergame: TServerGame;
    begin
        ServerGameList.Clear;

        if FSystemType = stLaserforce then begin
            query := GetTable(Prefs.DBServerIP, LASERFORCE_PORT,
                              'SELECT TOP ' + IntToStr(Prefs.GamesLimit)+ ' M.ref AS [Game_ID], M.start AS [Start_Time], COALESCE(MT.desc1, MT.desc0, MG.[desc]) AS [Description] ' +
                              'FROM Mission M ' +
                              'LEFT JOIN MissionGroup MG ON MG.ref = M.[group] ' +
                              'LEFT JOIN MissionType MT ON MT.ref = M.[type] ' +
                              'ORDER BY M.start DESC');
            try
                for i := 0 to query.Rows.Count - 1 do
                begin
                    servergame := ServerGameList.Add;
                    servergame.Game_ID := query.RowCol(i, 'Game_ID').AsInteger;
                    servergame.Description := query.RowCol(i, 'Description').Text;
                    servergame.GameTime := StrToDateTime(query.RowCol(i, 'Start_Time').Text);
                end;
            finally
               query.Free;
            end;
        end else begin
            zquery := TZReadOnlyQuery.Create(Self);
            try
                zquery.Connection := ZConnection;
                if FSystemType <> stDemo then
                    ZConnection.Connect;  // This takes 5 seconds under MySQL / Nexus.  Why?

                case FSystemType of
                  stNexus, stNexusWithZeon: zquery.SQL.Add('SELECT S.Game_ID, S.Start_Time, P.Profile_Description AS Description ' +
                                          'FROM ng_game_stats S ' +
                                          'JOIN ng_profiles P ON S.Profile_ID = P.Profile_ID ' +
                                          'ORDER BY Start_Time');
                  stAcacia: zquery.SQL.Add('SELECT Game_ID, Start_Time, Description ' +
                                          'FROM T2GAMES G ' +
                                          'JOIN T2PROFILES P ON G.Profile_ID = P.Profile_ID ' +
                                          'ORDER BY Start_Time');
                end;

                try
                    if FSystemType <> stDemo then
                        zquery.Open;
                except
                    ZConnection.Disconnect;
                    ZConnection.Connect;
                    zquery.Open;
                end;

                if FSystemType = stDemo then
                    GamesCombo.Items.Insert(0, 'Demo game')
                else
                    while not zquery.EOF do begin
                        servergame := ServerGameList.Add;
                        servergame.Game_ID := zquery.FieldValues['Game_ID'];
                        servergame.Description := NonNull(zquery.FieldValues['Description']);
                        servergame.GameTime := zquery.FieldValues['Start_Time'];

                    zquery.Next;
                end;
            finally
                FreeSquish(zquery);
            end;
        end;

        ServerGameList.Sort;
        for i := 0 to ServerGameList.Count - 1 do
            GamesCombo.Items.Add(IntToStr(ServerGameList[i].Game_ID) + ', ' +
                NonNull(ServerGameList[i].Description) + ' ' +
                BuildGameTime(ServerGameList[i].GameTime));
    end;

var i: Integer;
begin  //---BuildGamesCombo---
    GamesCombo.Items.Clear;

    if ViewGcc.Checked then begin
        BuildGamesFromDB
    end else
        for i := 0 to GameList.Count - 1 do
            GamesCombo.Items.Add(DateTimeToStr(GameList[i].GameTime));
end;

procedure TMainForm.GamesComboKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
    if ViewGcc.Checked and (Key = VK_UP) and (GamesCombo.ItemIndex = 0) then begin
        BuildGamesCombo;
        GamesCombo.ItemIndex := 0;  // select the most recent game
    end;
end;

procedure TMainForm.GamesComboSpinDownClick(Sender: TObject);
begin
    if GamesCombo.ItemIndex < GamesCombo.Items.Count then
        GamesCombo.ItemIndex := GamesCombo.ItemIndex + 1;  // select the prior game
    GamesComboChange(Sender);
end;

procedure TMainForm.GamesComboSpinUpClick(Sender: TObject);
begin
    if GamesCombo.ItemIndex = 0 then begin
        if ViewGcc.Checked then BuildGamesCombo;
        GamesCombo.ItemIndex := 0;  // select the most recent game
    end else
        GamesCombo.ItemIndex := GamesCombo.ItemIndex - 1;  // select the next game

    GamesComboChange(Sender);
end;

// Change the number of Team boxes shown, by creating/destroying Teams as required.
procedure TMainForm.GridSize(OldSize: Integer);
var i: Integer;
    jamright: Boolean;
begin
    if Prefs.GridHigh * Prefs.GridWide > 60 then Prefs.GridWide := 60 div Prefs.GridHigh;  // make sure we don't blow past 60 teams

    for i := Team.Count to Prefs.GridHigh * Prefs.GridWide do  // Create extra team boxes (if any).
        CreateTeamBox(i);

    for i := Team.Count - 1 downto Prefs.GridHigh * Prefs.GridWide + 1 do begin  // Destroy un-needed team boxes (if any).
        AllPlayers.SortType := stNone;  // turn sorting off while we add to AllPlayers; otherwise it tries to sort on partially added items.
        while Team[i].Items.Count > 0 do begin  // for each selected Player in the Source
            with AllPlayers.Items.Add do begin  // add an Item to the target ...
                Caption := Team[i].Items.Item[0].Caption;                 // ... and copy in settings from the Source's Player
                ImageIndex := Team[i].Items.Item[0].ImageIndex;
                SubItems := Team[i].Items.Item[0].SubItems;
            end; // with

            Team[i].Items.Delete(0);  // delete Item[0] from Team[i]
        end;

        Team[i].Free;
        Team.Delete(i);
    end;  // for i
    AllPlayers.SortType := stBoth;  // Turn sorting back on for AllPlayers

    jamright := Left > Screen.Width - Width - Left;  // if the space at the right is smaller than the space at the left, then after the resize,
                                                     // we're going to place the form butted up to the right-hand side of the screen.
    ClientHeight := Prefs.GridHigh * (Prefs.GridPlayers * 14 + 26) - 2;
    ClientWidth  := Prefs.GridWide * 250 + 252;

    if jamright and (Width < Screen.Width) then
        Left := Screen.Width - Width  // Place the form butted up to the right-hand side of the screen.
    else
        Left := 0;

    if (Top + Height > Screen.Height) and (Height < Screen.Height) then
        Top := Screen.Height - Height;
end;

// Set up the Prefs form, display it, then save the selections.
procedure TMainForm.PreferencesClick(Sender: TObject);
var PrefsForm: TPrefsForm;
    Grids: TStringList;
    OldSize: Integer;
    MyRegistry: TRegistry;
    i: Integer;
    changed: Boolean;
begin
    Grids := TStringList.Create;
    Grids.CommaText := IntToStr(Prefs.GridHigh) + ',' +
                       IntToStr(Prefs.GridWide) + ',' +
                       IntToStr(Prefs.GridPlayers);

    OldSize := Prefs.GridHigh * Prefs.GridWide;

    PrefsForm := TPrefsForm.CreatePrefsForm(Grids, Application);
    try  // fill in values on the PrefsForm...
        case Prefs.HandicapStyle of
            '+': PrefsForm.DefaultHandicapStyle.ItemIndex := 0;
            '-': PrefsForm.DefaultHandicapStyle.ItemIndex := 1;
            '%': PrefsForm.DefaultHandicapStyle.ItemIndex := 2;
        end;

        PrefsForm.SystemType.ItemIndex := Integer(FSystemType);

        if ViewGcc.Checked then PrefsForm.GamesToView.ItemIndex := 0
                           else PrefsForm.GamesToView.ItemIndex := 1;

        if Prefs.NexusScores then PrefsForm.NexusScores.ItemIndex := 0
                             else PrefsForm.NexusScores.ItemIndex := 1;
        PrefsForm.NexusScores.Enabled := FSystemType in [stNexus, stNexusWithZeon];

        PrefsForm.GroupPlayersBy.ItemIndex := Prefs.SortMode;

        if Prefs.SortTeamsByRank then PrefsForm.SortTeamsBy.ItemIndex := 0
                                 else PrefsForm.SortTeamsBy.ItemIndex := 1;

        PrefsForm.AutoUpdateScoreboard.Checked  := Prefs.AutoUpdateScoreboard;
        PrefsForm.AutoUpdateTeams.Checked       := Prefs.AutoUpdateTeams;

        if Prefs.ScoreboardStyle = ssColumns then PrefsForm.ScoreboardStyle.ItemIndex := 0
                                             else PrefsForm.ScoreboardStyle.ItemIndex := 1;

        PrefsForm.AliasAutocomplete.Value       := Prefs.AliasAutocomplete;
        PrefsForm.GamesLimit.Value              := Prefs.GamesLimit;
        PrefsForm.StatusServerIP.Text           := Prefs.StatusServerIP;
        PrefsForm.DBServerIP.Text               := Prefs.DBServerIP;

        if Prefs.SoloHandicapsShowPercentage then PrefsForm.SoloHandicaps.ItemIndex := 0
                                             else PrefsForm.SoloHandicaps.ItemIndex := 1;

        if PrefsForm.ShowModal = mrOK then begin  // read values back from the PrefsForm...
            case PrefsForm.DefaultHandicapStyle.ItemIndex of
                0: Prefs.HandicapStyle := '+';
                1: Prefs.HandicapStyle := '-';
                2: Prefs.HandicapStyle := '%';
            end;

            changed := not (Integer(FSystemType) = PrefsForm.SystemType.ItemIndex) or
               not (Prefs.DBServerIP = PrefsForm.DBServerIP.Text);
            if changed then
            begin
                FSystemType             := TSystemType(PrefsForm.SystemType.ItemIndex);
                Prefs.DBServerIP        := PrefsForm.DBServerIP.Text;
            end;
            
            ViewGcc.Checked             := PrefsForm.GamesToView.ItemIndex = 0;
            ViewHistorical.Checked      := PrefsForm.GamesToView.ItemIndex = 1;

            Prefs.GridHigh              := StrToInt(Grids[0]);
            Prefs.GridWide              := StrToInt(Grids[1]);
            Prefs.GridPlayers           := StrToInt(Grids[2]);

            Prefs.NexusScores           := PrefsForm.NexusScores.ItemIndex = 0;
            Prefs.SortMode              := PrefsForm.GroupPlayersBy.ItemIndex;

            Prefs.SortTeamsByRank       := PrefsForm.SortTeamsBy.ItemIndex = 0;
            Prefs.SoloHandicapsShowPercentage := PrefsForm.SoloHandicaps.ItemIndex = 0;
            Prefs.AutoUpdateScoreboard  := PrefsForm.AutoUpdateScoreboard.Checked;
            Prefs.AutoUpdateTeams       := PrefsForm.AutoUpdateTeams.Checked;

            case PrefsForm.ScoreboardStyle.ItemIndex of
                0: Prefs.ScoreboardStyle := ssColumns;
                1: Prefs.ScoreboardStyle := ssDetailed;
            end;

            Prefs.AliasAutocomplete     := PrefsForm.AliasAutocomplete.Value;
            Prefs.GamesLimit            := PrefsForm.GamesLimit.Value;
            Prefs.StatusServerIP        := PrefsForm.StatusServerIP.Text;

            if PrefsForm.SekritsVisible then SekritsVisible := True;

            MyRegistry := TRegistry.Create;  // ... and write 'em to the registry
            MyRegistry.RootKey := HKEY_LOCAL_MACHINE;
            if MyRegistry.OpenKey('SOFTWARE\DougWare\Torn', True) then begin
                case FSystemType of
                  stNexus:         MyRegistry.WriteString('Type', 'Nexus');
                  stNexusWithZeon: MyRegistry.WriteString('Type', 'NexusWithZeon');
                  stAcacia:        MyRegistry.WriteString('Type', 'Acacia');
                  stLaserforce:    MyRegistry.WriteString('Type', 'Laserforce');
                  stDemo:          MyRegistry.WriteString('Type', 'Demo');
                end;

                MyRegistry.WriteString('HandicapStyle', Prefs.HandicapStyle);

                MyRegistry.WriteInteger('GridHigh',   Prefs.GridHigh);
                MyRegistry.WriteInteger('GridWide',   Prefs.GridWide);
                MyRegistry.WriteInteger('GridPlayers',Prefs.GridPlayers);
                MyRegistry.WriteInteger('SortMode',   Prefs.SortMode);

                MyRegistry.WriteBool('SortByRank',      Prefs.SortTeamsByRank);
                MyRegistry.WriteBool('NexusScores',     Prefs.NexusScores);
                MyRegistry.WriteBool('SoloHandicapsShowPercentage', Prefs.SoloHandicapsShowPercentage);
                MyRegistry.WriteBool('AutoUpdate',      Prefs.AutoUpdateScoreboard);
                MyRegistry.WriteBool('UpdateTeams',     Prefs.AutoUpdateTeams);
                MyRegistry.WriteInteger('ScoreboardStyle', Integer(Prefs.ScoreboardStyle));
                MyRegistry.WriteInteger('AliasAutocomplete', Prefs.AliasAutocomplete);
                MyRegistry.WriteInteger('GamesLimit',   Prefs.GamesLimit);
                MyRegistry.WriteString('StatusServerIP',Prefs.StatusServerIP);
                MyRegistry.WriteString('DBServerIP',    Prefs.DBServerIP);

                MyRegistry.CloseKey;
            end;
            MyRegistry.Free;

            if LeagueMode then SaveLeague;  // if there's a League open, write 'em to that, too

            if changed then
                ConnectDB;

            for i := 1 to Team.Count - 1 do
                Team[i].HandiDefault := Prefs.HandicapStyle;

            GridSize(OldSize);
        end;  // if ShowModal = mrOK
    finally
        PrefsForm.Release;
        Grids.Free;
    end;
end;

// Show various reports, on various output devices.
procedure TMainForm.ReportLeagueClick(Sender: TObject);
var LeagueTitle: string;
begin
    LeagueTitle := StringReplace(ChangeFileExt(ExtractFileName(SaveDialog1.FileName), ''), '_', ' ', [rfReplaceAll]);
    if LeagueTitle[1] in ['a'..'z'] then
        LeagueTitle[1] := Chr(Ord(LeagueTitle[1]) - Ord('a') + Ord('A'));

    DoReportLeague(LeagueTitle, Prefs, LeagueList, Application, Screen, Printer, Self, (*ReportForm, LadderForm,*) PrintDialog1, SaveReportsDialog);
end;

procedure TMainForm.ExceptionHandler(Sender: TObject; E: Exception);
begin
    Exceptions := Exceptions + ',"' + DateTimeToStr(Now) + ': ' +
                  E.Message + ' at address ' + Format('%p', [ExceptAddr]) + '."';
    Beep;
    MessageDlg(E.Message, mtError, [mbOK], E.HelpContext);
end;

procedure TMainForm.Debug(s: string);
var d: string;
begin
    { $IFDEF DEBUG }
    DateTimeToString(d, 'HH:mm:ss.zzz', Now);
    Exceptions := Exceptions + ',"' + d + ': debug: ' + s + '."';
    { $ENDIF }
end;

procedure TMainForm.HideTBoardClick(Sender: TObject);
begin
    SendUDP(TBOARD_PORT, 'MINIMIZE');
end;

procedure TMainForm.PlayerDetailNameClick(Sender: TObject);
begin
  SekritsVisible := True;
end;

// Called by BuildTeamLadder
function CompareTeams(Item1, Item2: Pointer): Integer;
var i: Integer;
    numgames: Integer;
    val1, val2: Double;  // Average points or average scores for each team.
begin
    if GamesColumn > -1 then begin
        numgames := StrToIntDef(TZRow(Item1)[GamesColumn].Text, 0);
        if numgames > 0 then
            val1 := StrToIntDef(TZRow(Item1)[CompareColumn].Text, 0) / numgames
        else
            val1 := 0;

        numgames := StrToIntDef(TZRow(Item2)[GamesColumn].Text, 0);
        if numgames > 0 then
            val2 := StrToIntDef(TZRow(Item2)[CompareColumn].Text, 0) / numgames
        else
            val2 := 1;

        Result := Sign(val2 - val1);
    end else
        Result := 0;

    i := CompareColumn;
    while (Result = 0) and (i > 0) and (i < TZRow(Item2).Count) do begin
        Result := Sign(StrToPercentDef(TZRow(Item2)[i].Text, 0) - StrToPercentDef(TZRow(Item1)[i].Text, 0));
        i := i + ColumnDirection;
    end;
end;

// Called by BuildTeamsColours
function CompareTeamsColours(Item1, Item2: Pointer): Integer;
begin
    Result := StrToIntDef(TZRow(Item2)[11].Text, 0) - StrToIntDef(TZRow(Item1)[11].Text, 0);
//    if (LeaguePoints.Count < 1) and (StrToIntDef(LeaguePoints[0], 0) < StrToIntDef(LeaguePoints[1], 0)) then Result := -Result;
    if Result = 0 then
        Result := StrToIntDef(TZRow(Item2)[12].Text, 0) - StrToIntDef(TZRow(Item1)[12].Text, 0);
end;

// Called by BuildSoloLadder
function CompareSoloScores(Item1, Item2: Pointer): Integer;
begin
    Result := StrToIntDef(TZRow(Item2)[3].Text, 0) - StrToIntDef(TZRow(Item1)[3].Text, 0);
    if Result = 0 then
        Result := Round(100 * StrToFloatDef(TZRow(Item2)[4].Text, 0) - StrToFloatDef(TZRow(Item1)[4].Text, 0));
end;

// Called by BuildSoloLadder
function CompareSoloRanks(Item1, Item2: Pointer): Integer;
begin
    Result := Round(1000*(StrToFloatDef(TZRow(Item1)[4].Text, 0) - StrToFloatDef(TZRow(Item2)[4].Text, 0)));
    if Result = 0 then
        Result := CompareSoloScores(Item1, Item2);
end;

// Called by BuildSoloLadder
function CompareSoloTagRatios(Item1, Item2: Pointer): Integer;
begin
    Result := StrToPercentDef(TZRow(Item2)[5].Text, 0) - StrToPercentDef(TZRow(Item1)[5].Text, 0);
    if Result = 0 then
        Result := CompareSoloRanks(Item1, Item2);
end;

// Called by BuildSoloLadder
function CompareSoloScoreRatios(Item1, Item2: Pointer): Integer;
begin
    Result := StrToPercentDef(TZRow(Item2)[6].Text, 0) - StrToPercentDef(TZRow(Item1)[6].Text, 0);
    if Result = 0 then
        Result := CompareSoloTagRatios(Item1, Item2);
end;

// Called by BuildPackReport
function ComparePacks(Item1, Item2: Pointer): Integer;
begin
    Result := StrToPercentDef(TZRow(Item2)[5].Text, 0) - StrToPercentDef(TZRow(Item1)[5].Text, 0);  // Tag ratio
    if Result = 0 then
        Result := StrToPercentDef(TZRow(Item2)[6].Text, 0) - StrToPercentDef(TZRow(Item1)[6].Text, 0);  // Score ratio
    if Result = 0 then
        Result := StrToPercentDef(TZRow(Item1)[4].Text, 0) - StrToPercentDef(TZRow(Item2)[4].Text, 0);  // Average rank
end;

// Called by BuildGamesCombo
function CompareGames(Item1, Item2: Pointer): Integer;
begin
    Result := StrToIntDef(TStringList(Item2)[2], 0) - StrToIntDef(TStringList(Item1)[2], 0);
    if Result = 0 then
        Result := StrToIntDef(TStringList(Item2)[1], 0) - StrToIntDef(TStringList(Item1)[1], 0);
end;

// Called by BuildGameGrid, BuildKombi.
function CompareLastColumn(Item1, Item2: Pointer): Integer;
begin
    Result := StrToIntDef(TZRow(Item2)[TZRow(Item2).Count - 1].Text, 0) -
              StrToIntDef(TZRow(Item1)[TZRow(Item1).Count - 1].Text, 0);
end;

// Called by BuildGameGrid
//function CompareGridWithDrops(Item1, Item2: Pointer): Integer;
//begin
//    Result := StrToIntDef(TZRow(Item2)[TZRow(Item2).Count - 2].Text, 0) -
//              StrToIntDef(TZRow(Item1)[TZRow(Item1).Count - 2].Text, 0);
//end;

function LastGame(Row: TZRow; Columns: TZColumns = nil): Integer;
begin
  Result := Row.Count - 1;

  if Assigned(Columns) then
    while (Result > 0) and ((Columns[Result].Heading = 'Dropped') or (Columns[Result].Heading = 'Average score') or (Columns[Result].Heading = 'Pts')) do
      Dec(Result);

  while (Result > 0) and (Row[Result].Text = '') do
    Dec(Result);
end;

// Called by BuildGameGrid
function CompareAscension(Item1, Item2: Pointer): Integer;
begin
    Result := LastGame(Item2) - LastGame(Item1);  // Rank higher the team with the more recent game.
    If Result = 0 then
        Result := StrToIntDef(TZRow(Item2)[LastGame(Item2)].Text, 0) - StrToIntDef(TZRow(Item1)[LastGame(Item1)].Text, 0);  // Compare points in each team's most recent game.
    If Result = 0 then
        Result := StrToIntDef(TZRow(Item2)[LastGame(Item2) - 1].Text, 0) - StrToIntDef(TZRow(Item1)[LastGame(Item1) - 1].Text, 0);  // Compare scores in each team's most recent game.
end;

// Called by BuildGameGrid
function ComparePyramid(Item1, Item2: Pointer): Integer;
var last1, last2: Integer;
begin
    last1 := LastGame(Item1);
    last2 := LastGame(Item2);

    if (last1 > 255) or (last2 > 255) then
        Result := last2 - last1  // There are too many games for set operators to work.
    else begin
        if (last1 < last2) and (PyramidSet >= [last1..last2-1]) or
           (last1 > last2) and (PyramidSet >= [last2..last1-1]) then  // These teams' last games are in the same round.
            Result := 0
        else
            Result := last2 - last1;  // Whoever survived longest ranks highest.
    end;

    if Result = 0 then  // Compare their last scores.
        Result := StrToIntDef(TZRow(Item2)[LastGame(Item2)].Text, 0) - StrToIntDef(TZRow(Item1)[LastGame(Item1)].Text, 0);
end;

// Called by BuildTeamLadder when 'scale up teams with less games' is checked.
procedure TeamLadderScaledCalcBar(report: TZoomReport; row, col: Integer; max: Real; var fill: Real);
begin
    if (report.Columns[report.Columns.Count - 1].Heading = 'Scaled points') and
    (report.Rows[row][report.Rows[row].Count - 1].Text <> '') and
    (max > 0) then
        fill := StrToFloatDef(report.Rows[row][report.Rows[row].Count - 1].Text, 0) / max;  // Scale to the 'Scaled' column.
end;

// Called by BuildTeamLadder when 'show colours' is checked.
procedure TeamLadderColourCalcBar(report: TZoomReport; row, col: Integer; max: Real; var fill: Real);
begin
    if col in [2, 5, 8, 11, 14, 17, 20, 23] then
        max := StrToFloatDef(report.Rows[row][col + 1].Text, MaxInt) + StrToFloatDef(report.Rows[row][col + 2].Text, MaxInt) + StrToFloatDef(report.Rows[row][col].Text, MaxInt);
    if col in [2, 5, 8, 11, 14, 17, 20, 23, 9] then begin
        if max = 0 then
            fill := 0
        else
            fill := StrToFloatDef(report.Rows[row][col].Text, 0) / max;
    end;
end;

// Called by BuildTeamsVersus
//procedure TeamsVersusCalcBar(report: TZoomReport; row, col: Integer; max: Real; var fill: Real);
//var s: String;
//begin
//    s := report.Rows[row][col].Text;
//    fill := StrToFloatDef(Copy(s, 1, Pos('/', s) - 1), 0) / StrToIntDef(Trim(Copy(s, Pos('/', s) + 1, 3)), 1);
//end;

// Called by BuildTeamsVersus
procedure TeamsVersusPaintBar(report: TZoomReport; row, col: Integer; canvas: TCanvas; r: TRect; color, barcolor: TColor);
var s: String;
    fill: Real;
    cell: TZCell;
    points: array [0..2] of TPoint;
begin
    if col = row + 1 then begin // we are on the diagonal
        canvas.Brush.Color := barcolor;
        canvas.Pen.Color := barcolor;
        canvas.Pen.Width := 0;

        // Draw bottom left triangle.
        // [0]: bottom left         [1]: top left               [2]: bottom right
        points[0].y := r.Bottom;    points[1].y := r.Top + 3;   points[2].y := r.Bottom;
        points[0].x := r.Left + 1;  points[1].x := r.Left + 1;  points[2].x := r.Right - 2;
        canvas.Polygon(points);

        // Draw top right triangle.
        // [0]: top right          [1]: top left               [2]: bottom right
        points[0].y := r.Top + 1;  points[1].y := r.Top + 1;   points[2].y := r.Bottom - 2;
        points[0].x := r.Right;    points[1].x := r.Left + 3;  points[2].x := r.Right;
        canvas.Polygon(points);
    end else if col = 0 then
        // no shading for cells in the left column, because that's team names
    else begin
        // paint dark bar
        canvas.Pen.Color := barcolor;
        canvas.Pen.Width := 0;
        canvas.Brush.Color := barcolor;

        cell := report.Rows[row][col];
        s := cell.Text;
        if (Pos(' ', s) > 0) and (cell.Custom is TDouble) and (TDouble(cell.Custom).Value <> 0) then  // it's got both a win ratio like '3/5' and an average victory points like '3.00'
            fill := StrToFloatDef(Copy(s, Pos(' ', s) + 1, MaxInt), 0) / TDouble(cell.Custom).Value
        else
            fill := StrToFloatDef(Copy(s, 1, Pos('/', s) - 1), 0) / StrToIntDef(Trim(Copy(s, Pos('/', s) + 1, 3)), 1);

        if fill > 0 then
            canvas.Rectangle(r.Left + 1, r.Top + 1, r.Left + Round((r.Right - r.Left) * fill), r.Bottom);
    end;
end;

//procedure TeamsVersusPaintBar(report: TZoomReport; row, col: Integer; canvas: TCanvas; r: TRect; color, barcolor: TColor);
//var s: String;
//    margin, middlex, slopex, halfbar: Real;
//    count: Integer;
//    cell: TZCell;
//    points: array [0..3] of TPoint;
//begin
//    if col = row + 1 then begin // we are on the diagonal
//        canvas.Brush.Color := barcolor;
//        canvas.Pen.Color := barcolor;
//        canvas.Pen.Width := 0;
//
//        // Draw bottom left triangle.
//        points[1].y := r.Top + 3;   points[2].y := r.Top + 3;  // [1]: top left     [2]: same as [1]
//        points[0].y := r.Bottom;    points[3].y := r.Bottom;   // [0]: bottom left  [3]: bottom right
//        points[0].x := r.Left + 1;  points[1].x := r.Left + 1;
//        points[2].x := r.Left + 1;  points[3].x := r.Right - 2;
//        canvas.Polygon(points);
//
//        // Draw top right triangle.
//        points[1].y := r.Top + 1;   points[2].y := r.Top + 1;     // [1]: top left     [2]: top right
//        points[0].y := r.Bottom - 2;  points[3].y := r.Bottom - 2;  // [0]: same as [3]  [3]: bottom right
//        points[0].x := r.Right;     points[1].x := r.Left + 3;
//        points[2].x := r.Right;     points[3].x := r.Right;
//        canvas.Polygon(points);
//    end else begin
//        count := report.Rows.Count;
//        cell := report.Rows[row][col];
//
//        s := cell.Text;
//        middlex := StrToFloatDef(Copy(s, 1, Pos('/', s) - 1), 0) / StrToFloatDef(Copy(s, Pos('/', s) + 1, MaxInt), 1);
//        if middlex > 0 then begin
//            if cell.Custom is TDouble then begin
//                margin := TDouble(cell.Custom).Value;
//
//                halfbar := Min(middlex, 1 - middlex);  // This is the horizontal space from middlex to the nearest end of the barcell drawing area.
//
//                if margin = 2 then
//                    slopex := 0  // 2 is approximately the median margin, and we will draw a simple vertical end to our bar.
//                else if margin < 2 then begin
//                    slopex := -(margin - (count / (count - 1))) / (2 - (count / (count - 1))) * halfbar  // Negative slope: the top of our sloped bar will be further left than its bottom.
//                end else  // margin > 2
//                    slopex := (margin - 2) / (count - 2) * halfbar;  // Positive slope: the top of our sloped bar will be further right than its bottom.
//            end else
//                slopex := 0;
//
//            // paint light background
//            canvas.Brush.Color := color;
//            canvas.FillRect(r);
//
//            // paint dark bar
//            canvas.Pen.Color := barcolor;
//            canvas.Pen.Width := 0;
//            canvas.Brush.Color := barcolor;
//            points[1].y := r.Top + 1;   points[2].y := r.Top + 1;     // [1]: top left     [2]: top right
//            points[0].y := r.Bottom;    points[3].y := r.Bottom;  // [0]: bottom left  [3]: bottom right
//            points[0].x := r.Left + 1;  points[1].x := r.Left + 1;
//            points[2].x := r.Left + Round((r.Right - r.Left) * (middlex + slopex));
//            points[3].x := r.Left + Round((r.Right - r.Left) * (middlex - slopex));
//            canvas.Polygon(points);
//        end;
//    end;
//end;

// Called by BuildTeamsVersus
procedure TeamsVersusCellDestroy(cell: TZCell);
begin
    if cell.Custom is TObject then
        TObject(cell.Custom).Free;
end;
(*
procedure TMainForm.UploadToFtp(filename: string);
var
  remotefile: string;
  F: File of Byte;
  FSize: Integer;
begin
    NMFTP1.Host := 'dougburbidge.com';
    NMFTP1.UserID := 'dougburb';
    NMFTP1.Password := 'Net1234.';
    NMFTP1.Connect;
    try
        NMFTP1.Mode(MODE_BYTE);

    AssignFile(F, filename);
    Reset(F);
    FSize := FileSize(F);
    CloseFile(F);
    NMFTP1.Allocate(FSize);
//    NMFTP1.ChangeDir('incoming');
    remotefile := 'test.html';//ExtractFileName(filename);
        NMFTP1.UploadUnique(filename);//, remotefile);
//    except
//        on E: Exception do ShowMessage(E.Message);
    finally
        if NMFTP1.Connected then
            NMFTP1.Disconnect;
    end;
end;

const FTP_COMMANDS: array[TCmdType] of string = ('ChangeDir', 'MakeDir', 'Delete', 'RemoveDir', 'List', 'Rename', 'UpRestore', 'DownRestore',
'Download', 'Upload', 'Append', 'ReInit', 'Allocate', 'NList', 'DoCommand', 'CurrentDir');

procedure TMainForm.NMFTP1Failure(var Handled: Boolean; Trans_Type: TCmdType);
begin
  ShowMessage(FTP_COMMANDS[Trans_Type] + ' failure');
end;

procedure TMainForm.NMFTP1Success(Trans_Type: TCmdType);
begin
  PlayerDetailPanel.Caption := PlayerDetailPanel.Caption + FTP_COMMANDS[Trans_Type] + ' success ';
end;
*)

procedure TMainForm.BuildDetailedGame(Reports: TZoomReports; From, Too: TDateTime; Description: Boolean);

    function BlankZero(i: Integer): string;
    begin
        if i = 0 then
            Result := ''
        else
            Result := IntToStr(i);
    end;

    function RoundTwo(d: Double): string;
    begin
        Result := FloatToStrF(d, ffNumber, 15, 2);
    end;

var game, team: Integer;
    thisgame: TGame;
    thisteam: TTeam;
    player: Integer;
    report: TZoomReport;
    row: TZRow;
    leagueteam: TLeagueTeam;
    color: TColor;
    teamhitsby, teamhitson, teambasehits, teambasedestroys,
    totalhitsby, totalhitson, totalbasehits, totalbasedestroys,
    totalscore: Integer;
begin
    game := 0;
    GameList.Sort;
    while (game < GameList.Count) and (GameList[game].GameTime < From) do
        Inc(game);

    while (game < GameList.Count) and (GameList[game].GameTime < Too) do begin  // loop through each game, and create an entry for it
        thisgame := GameList[game];
        report := Reports.Add(thisgame.Title + ' Game ' + FormatDateTime(ShortDateFormat + ' ' + ShortTimeFormat, thisgame.GameTime),
                              'Rank,Name,Score,"Tags +","Tags -","Tag Ratio","Score Ratio","TR x SR","Base Hits","Base Destroys"',
                              'center,left,right,right,right,right,right,right,right,right,right');
        report.MaxBarByColumn := True;

        totalhitsby := 0;
        totalhitson := 0;
        totalbasehits := 0;
        totalbasedestroys := 0;
        totalscore := thisgame.TotalScore;
        if totalscore = 0 then totalscore := 1;  // Prevent division by 0.
        thisgame.Teams.Sort;

        for team := 0 to thisgame.Count - 1 do begin
            teamhitsby := 0;
            teamhitson := 0;
            teambasehits := 0;
            teambasedestroys := 0;
            thisteam := thisgame.Teams[team];

            for player := 0 to thisgame.Players.Count - 1 do
                // Add a row for each player on the team.
                if thisgame.Players[player].TeamID = thisteam.TeamID then begin
                    row := report.Rows.AddRow;
                    color := NexusColors[thisteam.NexusColour];
                    row.Add(IntToStr(thisgame.Players[player].Rank), color, 2);
                    row.Add(LeagueList.FindID(thisgame.Players[player].PlayerID), color, 2);
                    row.Add(IntToStr(thisgame.Players[player].Score), color, 2);
                    row.Add(IntToStr(thisgame.Players[player].HitsBy), color, 2);
                    row.Add(IntToStr(thisgame.Players[player].HitsOn), color, 2);

                    if thisgame.Players[player].HitsOn = 0 then  // Tag ratio
                        row.Add('infinite', color, 2)
                    else
                        row.Add(RoundTwo(thisgame.Players[player].HitsBy / thisgame.Players[player].HitsOn), color, 2);

                    row.Add(RoundTwo(thisgame.Players[player].Score / totalScore * thisgame.Players.Count), color, 2);  // Score ratio

                    if thisgame.Players[player].HitsOn = 0 then  // TR x SR
                        row.Add('infinite', color, 2)
                    else
                        row.Add(RoundTwo(thisgame.Players[player].HitsBy / thisgame.Players[player].HitsOn *
                                           thisgame.Players[player].Score / totalScore * thisgame.Players.Count), color, 2);

                    row.Add(BlankZero(thisgame.Players[player].BaseHits), color, 2);
                    row.Add(BlankZero(thisgame.Players[player].BaseDestroys), color, 2);

                    Inc(teamhitsby, thisgame.Players[player].HitsBy);
                    Inc(teamhitson, thisgame.Players[player].HitsOn);
                    Inc(teambasehits, thisgame.Players[player].BaseHits);
                    Inc(teambasedestroys, thisgame.Players[player].BaseDestroys);

                    Inc(totalhitsby, thisgame.Players[player].HitsBy);
                    Inc(totalhitson, thisgame.Players[player].HitsOn);
                    Inc(totalbasehits, thisgame.Players[player].BaseHits);
                    Inc(totalbasedestroys, thisgame.Players[player].BaseDestroys);
                end;

            // Add a row for the team.
            row := report.Rows.AddRow;
            color := MixColors([NexusColors[thisteam.NexusColour], $02000000], [3, 1]);
            leagueteam := LeagueList.FindTeamID(thisteam.TeamID);
            row.Add('');  // Rank.
            if Assigned(leagueteam) then
                row.Add(leagueteam.BuildTeamName, color, 2)
            else
                row.Add('Team ' + IntToStr(thisteam.TeamID), color, 2);
            row.Add(IntToStr(thisteam.Score), color, 2);
            row.Add(IntToStr(teamhitsby), color, 2);
            row.Add(IntToStr(teamhitson), color, 2);

            if teamhitson = 0 then  // Tag ratio
                row.Add('infinite', color, 2)
            else
                row.Add(RoundTwo(teamhitsby / teamhitson), color, 2);

            row.Add(RoundTwo(thisteam.Score / totalScore * thisgame.Count), color, 2);  // Score ratio

            if teamhitson = 0 then  // TR x SR
                row.Add('infinite', color, 2)
            else
                row.Add(RoundTwo(teamhitsby / teamhitson * thisteam.Score / totalScore * thisgame.Count), color, 2);

            row.Add(BlankZero(teambasehits), color, 2);
            row.Add(BlankZero(teambasedestroys), color, 2);
        end;

        // Add an overall game average row.
        row := report.Rows.AddRow;
        row.Add('');  // Rank.
        row.Add('Team Average');  // Name.
        row.Add(IntToStr(Round(thisgame.TotalScore / thisgame.Count)));
        row.Add(RoundTwo(totalhitsby / thisgame.Count));
        row.Add(RoundTwo(totalhitson / thisgame.Count));

        row.Add('1.00');  // Tag ratio
        row.Add('1.00');  // Score ratio
        row.Add('1.00');  // TR x SR

        row.Add(RoundTwo(totalbasehits / thisgame.Count));
        row.Add(RoundTwo(totalbasedestroys / thisgame.Count));

        if (totalhitsby = 0) and (totalhitson = 0) and (totalbasehits = 0) and (totalbasedestroys = 0) then begin // There is no hit data, probably because we are Calculate By Nexus/Helios.
            report.DeleteColumn(9);  // base destroys
            report.DeleteColumn(8);  // base hits
            report.DeleteColumn(7);  // tag ratio x score ratio
            report.DeleteColumn(5);  // tag ratio
            report.DeleteColumn(4);  // hits on
            report.DeleteColumn(3);  // hits by
        end;

        Inc(game);
    end;  // while in date/time range From .. Too

end;

// Show various reports, on various output devices.
procedure TMainForm.DoReportLeague(LeagueTitle: string; Prefs: TPrefs; LeagueList: TLeagueList; Application: TApplication; Screen: TScreen; Printer: TPrinter; OwnerForm: TForm; (*ReportForm: TReportForm; LadderForm: TLadderForm;*) PrintDialog: TPrintDialog; SaveDialog: TSaveDialog);

type TGridType = (gtGameGrid, gtAscension, gtPyramid);

var reports: TZoomReports;

    function FirstGameDate: TDateTime;
    var i: Integer;
    begin
        if GameList.Count = 0 then
            Result := Date
        else begin
            Result := 1000000.0;

            for i := 0 to GameList.Count - 1 do
                if Result > GameList[i].GameTime then
                    Result := GameList[i].GameTime;
        end;

        Result := Int(Result);
    end;

    function LastGameDate: TDateTime;
    var i: Integer;
    begin
        if GameList.Count = 0 then
            Result := Date
        else begin
            Result := 0.0;

            for i := 0 to GameList.Count - 1 do
                if Result < GameList[i].GameTime then
                    Result := GameList[i].GameTime;
        end;

        Result := Int(Result);
    end;

    procedure SetRanks(rows: TZRows);
    var i: Integer;
    begin
        for i := 0 to rows.Count - 1 do  // loop through each row
            rows[i][0].Text := IntToStr(i + 1);  // set Ranks
    end;

    procedure GetDropLowHigh(DropWorst, DropBest: Integer; PercentGames: Boolean; ListCount: Integer; out Low, High: Integer);
    begin
        if PercentGames then begin
            Low  := DropWorst * ListCount div 100;
            High := ListCount - (DropBest * ListCount div 100) - 1;
        end else begin
            Low  := DropWorst;
            High := ListCount - DropBest - 1;
        end;
    end;

    function DropWorstAndBest(DropWorst, DropBest: Integer; PercentGames: Boolean; List: TIntList; var Count: Integer; CalcDeviation: Boolean = False): Double;
    var low, high:  Integer;
        total: Integer;  // used to sum up a team's total score, total league points, and number of games played
        i : Integer;
    begin
        List.Sort;

        GetDropLowHigh(DropWorst, DropBest, PercentGames, List.Count, low, high);
        total := 0;
        Count := 0;
        if (List.Count > 0) and (low > high) then begin  // there are scores in the list, but we've been told to drop them
            if DropWorst = 0 then begin  // pick the worst score from the list
                Count := 1;
                Result := List[0];
            end else if DropBest = 0 then begin  // pick the best score from the list
                Count := 1;
                Result := List[List.Count - 1];
            end else if List.Count mod 2 = 0 then begin  // there are an even number of scores: just grab the middle two and average them
                Count := 2;
                Result := (List[List.Count div 2 - 1] + List[List.Count div 2]) / 2;
            end else begin  // there are an odd number of scores: grab the middle one
                Count := 1;
                Result := List[List.Count div 2];
            end;

//            if CalcDeviation then
//                stddev := -1;
        end else begin // there are no scores in the list, or there are enough scores to calculate an average, even after we've dropped games as requested
            for i := low to high do begin
                Inc(total, List[i]);
                Inc(Count);
            end;

            if Count = 0 then
                Result := total
            else
                Result := total * 1.0 / Count;

//            if CalcDeviation then begin
//                if Count > 1 then begin
//                    stddev := 0;
//                    for i := low to high do begin
//                        stddev := stddev + Sqr(List[i] - Result);
//                        Inc(Count);
//                    end;
//
//                    stddev := SqRt(stddev / (Count-1))
//                end else
//                    stddev := -1;
//            end;
        end;
    end;

    function DropWorstAndBestRatio(DropWorst, DropBest: Integer; PercentGames: Boolean; List: TDoubleList; CalcDeviation: Boolean = False): Double;
    var low, high:  Integer;
        total: Double;  // used to sum the list
        count: Integer;
        i : Integer;
    begin
        List.Sort;

        GetDropLowHigh(DropWorst, DropBest, PercentGames, List.Count, low, high);
        if (List.Count > 0) and (low > high) then begin  // there are ratios in the list, but we've been told to drop them
            if DropWorst = 0 then  // pick the worst ratio from the list
                Result := List[0]
            else if DropBest = 0 then  // pick the best ratio from the list
                Result := List[List.Count - 1]
            else if List.Count mod 2 = 0 then  // there are an even number of ratios: just grab the middle two and average them
                Result := List[List.Count div 2 - 1] + List[List.Count div 2]
            else  // there are an odd number of ratios: grab the middle one
                Result := List[List.Count div 2];

//            if CalcDeviation then
//                stddev := -1;
        end else begin  // there are no ratios in the list, or there are enough ratios to calculate an average, even after we've dropped games as requested
            total := 0;
            count := 0;

            for i := low to high do begin
                total := total + List[i];
                Inc(count);
            end;

            if count = 0 then
                Result := total
            else
                Result := total / Count;

//            if CalcDeviation then begin
//                if Count > 1 then begin
//                    stddev := 0;
//                    for i := low to high do begin
//                        stddev := stddev + Sqr(List[i] - Result);
//                        Inc(Count);
//                    end;
//
//                    stddev := SqRt(stddev / (Count-1))
//                end else
//                    stddev := -1;
//            end;
        end;
    end;

    procedure CalculateOnePlayer(DropWorst, DropBest: Integer; PercentGames, ScaleGames, WithinDateRange: Boolean; From, Too: TDateTime;
        button: string; var averagescore, averagerank, tagratio, scoreratio: Double; var gamesplayed, gamesdropped: Integer);
    var j, k: Integer;
        scoreslist, rankslist: TIntList;  // lists of stats for an individual player
        countscores, countranks, hitsby, hitson, scoretotal, gamescoretotal, gameplayerstotal: Integer;
        thisplayer: Integer;
    begin
        hitsby := 0;
        hitson := 0;
        scoretotal := 0;
        gamescoretotal := 0;
        gameplayerstotal := 0;
        scoreslist := TIntList.Create;
        rankslist  := TIntList.Create;
        try
            for j := 0 to GameList.Count - 1 do begin  // Roll through all the games...
                thisplayer := GameList[j].Players.Find(button);
                if (thisplayer > -1) and               // ...looking for ones featuring this player.
                   (not WithinDateRange or ((GameList[j].GameTime >= From) and
                                            (GameList[j].GameTime <= Too))) then begin
                    scoreslist.Add(GameList[j].Players[thisplayer].Score);
                    rankslist.Add( GameList[j].Players[thisplayer].Rank);

                    Inc(hitsby, GameList[j].Players[thisplayer].HitsBy);
                    Inc(hitson, GameList[j].Players[thisplayer].HitsOn);

                    Inc(scoretotal, GameList[j].Players[thisplayer].Score);
                    for k := 0 to GameList[j].Players.Count - 1 do begin
                        Inc(gamescoretotal, GameList[j].Players[k].Score);
                        Inc(gameplayerstotal);
                    end;
                end;
            end;

            averagescore := Round(DropWorstAndBest(DropWorst, DropBest, PercentGames, scoreslist, countscores));
            averagerank := DropWorstAndBest(DropBest, DropWorst, PercentGames, rankslist, countranks);

            gamesplayed := scoreslist.Count;
        finally
            scoreslist.Free;
            rankslist.Free;
        end;

        if countranks = 0 then averagerank := 0;

        if hitson = 0 then
            tagratio := 1000000.0
        else
            tagratio := hitsby / hitson;

        if (gamescoretotal = 0) or (gamesplayed = 0) then
            scoreratio := 0
        else
            scoreratio := scoretotal / gamescoretotal * gameplayerstotal / gamesplayed;

        gamesdropped := gamesplayed - countscores;
    end;

    //function SetTeamColour(Game: TDateTime; ID: Integer; Colour: string; Color: TColor): TColor;
    //var thisgame,
    //    thisteam: Integer;
    //begin
    //    thisgame := GameList.Find(Game);
    //    if thisgame > -1 then begin
    //        thisteam := GameList[thisgame].Teams.Find(ID);
    //        if thisteam > -1 then
    //            GameList[thisgame].Teams[thisteam].Colour := Colour;
    //    end;
    //    Result := Color;
    //end;

    function StringToColor(s: string): TColor;
    begin
        if (Length(s) > 0) then
            case UpperCase(s)[1] of
                'R': Result := $02B0B0FF;  // red
                'G': Result := $02A0FFA0;  // green
                'B': Result := $02FFD0A0;  // blue
                'Y': Result := $0290FFFF;  // yellow
                else Result := clNone;
            end
        else Result := clNone;
    end;

    // Return a colour for a team in a given game.  teamnum is an index within the list of all games, not this game.
    function GetTeamColourIndex(Game: TDateTime; teamnum: Integer): TNexusColour;
    var thisgame,
        thisteam: Integer;
    begin
        Result := -1;

        thisgame := GameList.Find(Game);
        if thisgame = -1 then Exit;

        thisteam := GameList[thisgame].Teams.Find(teamnum);
        if thisteam = -1 then Exit;

        Result := GetNumTeamColourIndex(thisgame, thisteam);
    end;

    function GetTeamColor(Game: TDateTime; teamnum: Integer): TColor;
    var i: Integer;
    begin
        i := GetTeamColourIndex(Game, teamnum);
        if i in [0..NUM_COLOURS] then
            Result := NexusColors[i]
        else
            Result := clNone;
    end;

    // True if teams [index] and [index + 1] should have the same rank.
    function CompareRanks(game: TGame; var index: Integer): Boolean;
    begin
        if Prefs.SortMode in [0, 1] then
            Result := (index < game.Teams.Count - 2) and
              (game.Teams[index].Points = game.Teams[index + 1].Points) and
              (game.Teams[index].Score = game.Teams[index + 1].Score)
        else  // For Lord of the Ring mode, compare only on points, not on score.
            Result := (index < game.Teams.Count - 2) and
              (game.Teams[index].Points = game.Teams[index + 1].Points);
    end;

    // Display on the scoreboard the current overall league ladder, one line per team,
    // showing how many points each team has total.
    procedure BuildTeamLadder(DropWorst, DropBest: Integer; PercentGames, ScaleGames, WithinDateRange: Boolean; From, Too: TDateTime; OrderBy: Integer; ShowColours, Description: Boolean);
    var i, j, k: Integer;
        headings:   string;
        barcol:     Integer;            // column on which we are going to draw the horizontal bar chart
        report:     TZoomReport;
        scoreslist: TIntList;
        ratioslist: TDoubleList;
        thisteam:   Integer;
        count, score: Integer;          // used to sum up a team's number of games played, total score,
        points:     Double;             // and total league points
        ratio:      Double;             // team's score ratio
        countlist:  TIntList;           // number of games each team has played, for scaling
        mode:       Integer;            // used in calculating mode; i.e. number of games which most teams have played
        countval, modeval: Integer;     // used in calculating mode
        colour:     string;
        nexuscolour: TNexusColour;
        colourcounts: array [TNexusColour, 0..2] of Integer;  // number of times this team has ranked 1st, 2nd or 3rd on each colour
        row:        TZRow;
        coloffset:  Integer;
        datelimited, more, less, zeroes: Boolean;
        lastrank:   array [0..2] of Integer;  // lastrank[0] is the last index (in the ordered Teams[] for this game) that placed first.  If there is just one first place, this is 0; if there's a tie, it may be higher.  lastrank[1] and [2] are for second and third.
        totals:     array of Integer;   // Used for building totals row at bottom of report.
        colours:    Integer;            // Number of different colours shown in the report.
    begin
        headings := 'Rank,Team,';

        if ShowColours then begin
            headings := headings + '"Red 1st","Red 2nd","Red 3rd","Blue 1st","Blue 2nd","Blue 3rd","Grn 1st","Grn 2nd","Grn 3rd","Yel 1st","Yel 2nd","Yel 3rd",' +
                                   '"Pur 1st","Pur 2nd","Pur 3rd","Pink 1st","Pink 2nd","Pink 3rd","Cyan 1st","Cyan 2nd","Cyan 3rd","Ora 1st","Ora 2nd","Ora 3rd",' +
                                   '"Whi 1st","Whi 2nd","Whi 3rd","Blk 1st","Blk 2nd","Blk 3rd","Fire 1st","Fire 2nd","Fire 3rd","Ice 1st","Ice 2nd","Ice 3rd",' +
                                   '"Earth 1st","Earth 2nd","Earth 3rd","Crys 1st","Crys 2nd","Crys 3rd","Rnbw 1st","Rnbw 2nd","Rnbw 3rd","Cops 1st","Cops 2nd","Cops 3rd",';
            coloffset := 48;
        end else
            coloffset := 0;

        barcol := coloffset + 2; // This will point at the next column added: either "Points", "Average Score" or "Score Ratio".

        if IsPoints(Prefs) then
            headings := headings + 'Points,';

        if OrderBy = 0 then
            headings := headings + '"Average score",Games'
        else
            headings := headings + '"Score Ratio",Games';

        if WithinDateRange then
            report := reports.Add(LeagueTitle + ' Team Ladder from ' + DateToStr(From) + ' to ' + DateToStr(Too), headings, 'center,left,right,right,right')
        else
            report := reports.Add(LeagueTitle + ' Team Ladder to ' + DateToStr(LastGameDate), headings, 'center,left,right,right,right');

        if (DropWorst <> 0) or (DropBest <> 0) then
            report.Columns.Add('Dropped', 'right');

        report.MultiColumnOK := True;

        countlist := TIntList.Create;
        scoreslist := TIntList.Create;
        ratioslist := TDoubleList.Create;

        if ShowColours then
            for j := 0 to GameList.Count - 1 do  // Roll through all the games...
                GameList[j].Teams.Sort;  // ... and sort them so that the winning team is [0], second is [1], etc.

        datelimited := False;
        try
            for i := 0 to LeagueList.Count - 1 do begin  // Create a row for each League team.
                report.Rows.Add(TZRow.Create);

                report.Rows[i].Add('0');  // put in a temporary Rank

                report.Rows[i].Add(LeagueList[i].BuildTeamName);  // set up team name

                points := 0;
                scoreslist.Clear;
                ratioslist.Clear;
                for j := Low(TNexusColour) to High(TNexusColour) do
                    for k := 0 to 2 do
                        colourcounts[j][k] := 0;

                for j := 0 to GameList.Count - 1 do begin  // Roll through all the games...
                    thisteam := GameList[j].Teams.Find(LeagueList[i].TeamID);
                    if (thisteam > -1) then begin                 // ...looking for ones featuring this team.
                        if (not WithinDateRange or ((GameList[j].GameTime >= From) and
                                                    (GameList[j].GameTime <= Too))) then begin
                            scoreslist.Add(GameList[j].Teams[thisteam].Score);
                            if GameList[j].TotalScore <> 0 then
                                ratioslist.Add(GameList[j].Teams[thisteam].Score / GameList[j].TotalScore * GameList[j].Count);
                            points := points + GameList[j].Teams[thisteam].Points;

                            if ShowColours then begin
                                lastrank[0] := 0;
                                while (lastrank[0] < GameList[j].Teams.Count - 2) and CompareRanks(GameList[j], lastrank[0]) do
                                    Inc(lastrank[0]);

                                lastrank[1] := lastrank [0] + 1;
                                while (lastrank[1] < GameList[j].Teams.Count - 2) and CompareRanks(GameList[j], lastrank[1]) do
                                    Inc(lastrank[1]);

                                lastrank[2] := lastrank [1] + 1;
                                while (lastrank[2] < GameList[j].Teams.Count - 2) and CompareRanks(GameList[j], lastrank[2]) do
                                    Inc(lastrank[2]);

                                if thisteam <= lastrank[2] then begin
                                    nexuscolour := GetTeamColourIndex(GameList[j].GameTime, LeagueList[i].TeamID);  // Ensure that this team has a Colour set.

                                    if thisteam <= lastrank[0] then
                                        Inc(colourcounts[nexuscolour][0])
                                    else if thisteam <= lastrank[1] then
                                        Inc(colourcounts[nexuscolour][1])
                                    else if thisteam <= lastrank[2] then
                                        Inc(colourcounts[nexuscolour][2]);
                                end;
                            end;
                        end else
                            datelimited := True;
                    end;
                end;  // for j

                if ShowColours then
                    for j := 0 to High(TNexusColour) do
                        for k := 0 to 2 do
                            report.Rows[i].Add(IntToStr(colourcounts[j][k]), NexusColors[j], j * 3 + 2);

                score := Round(DropWorstAndBest(DropWorst, DropBest, PercentGames, scoreslist, count));
                ratio := DropWorstAndBestRatio(DropWorst, DropBest, PercentGames, ratioslist);

                if IsPoints(Prefs) then
                    report.Rows[i].Add(FloatToStr(points));  // league points scored

                if count = 0 then
                    report.Rows[i].Add('-')               // totalled game scores
                else if OrderBy = 0 then
                    report.Rows[i].Add(IntToStr(score))   // totalled game scores
                else
                    report.Rows[i].Add(Format('%.1f%%', [ratio * 100]));  // totalled game scores

                report.Rows[i].Add(IntToStr(scoreslist.Count));  // total games played

                if (DropWorst <> 0) or (DropBest <> 0) then
                    report.Rows[i].Add(IntToStr(scoreslist.Count - count));  // games dropped

                countlist.Add(count);
                if count > 0 then
                    report.Rows[i].HasData := True;

                if ShowColours then
                    for j := barcol to report.Rows[i].Count - 1 do
                        report.Rows[i][j].BarCell := barcol
                else
                    for j := 0 to report.Rows[i].Count - 1 do
                        report.Rows[i][j].BarCell := barcol;
            end;  // for each team in LeagueList

            countlist.Sort;
            modeval := 1;
            if ScaleGames and (countlist.Count > 0) and (countlist[0] <> countlist[countlist.Count - 1]) then begin  // Find the mode, and add scaled columns.
                count    := 0;
                mode     := 0;
                countval := 0;
                for i := 0 to countlist.Count - 1 do begin
                    if countlist[i] = 0 then Continue;
                    if countlist[i] = countval then begin
                        Inc(count);
                        if mode <= count then begin  // Note that the "<=" instead of just "<" ensures that if there are several values with the same count, the highest of these values will be chosen.
                            mode := count;
                            modeval := countval;
                        end;
                    end else begin
                        count := 1;
                        countval := countlist[i];
                    end;
                end;
                if IsPoints(Prefs) then
                    report.Columns.Add('Scaled points', 'right');

                for i := 0 to report.Rows.Count - 1 do begin
                    if coloffset + 4 < report.Rows[i].Count then begin
                        if ((DropWorst = 0) and (DropBest = 0)) then
                            count := StrToIntDef(Trim(report.Rows[i][coloffset + 4].Text), 0)
                        else
                            count := StrToIntDef(Trim(report.Rows[i][coloffset + 4].Text), 0) - StrToIntDef(Trim(report.Rows[i][coloffset + 5].Text), 0);
                    end else
                        count := 1;

                    if IsPoints(Prefs) then begin
                        if (modeval <> count) and (count > 0) then begin
                            report.Rows[i].Add(Format('%.1f', [StrToIntDef(report.Rows[i][coloffset + 2].Text, 0) * modeval / count]));  // scaled points
                            if modeval < count then
                                more := True
                            else
                                less := True;
                        end else
                            report.Rows[i].Add('');  // scaled points
                    end;
                end;
                report.OnCalcBar := TeamLadderScaledCalcBar;
            end;  // if scalegames

            CompareColumn := barcol;
            if ScaleGames and IsPoints(Prefs) then  // Only scale if we're using victory points: score is already averaged, so we don't want to "average" a second time.
                GamesColumn := report.Columns.Find('Games')
            else
                GamesColumn := -1;
            ColumnDirection := 1;
            BubbleSort(report.Rows, CompareTeams);
            SetRanks(report.Rows);
        finally
            countlist.Free;
            scoreslist.Free;
        end;

        if ShowColours then begin  // Add a Totals row at the bottom of the report.
            row := TZRow.Create;
            report.Rows.Add(row);
            row.Add('', clGray);  // Blank Rank.
            row.Add('Totals', clGray);  // set up team name
            row.HasData := True;

            SetLength(totals, report.Columns.Count);
            for k := 2 to report.Columns.Count - 1 do
                totals[k] := 0;

            for j := 0 to report.Rows.Count - 2 do  // One more time for the Totals row.
                for k := 2 to High(totals) do
                    totals[k] := totals[k] + report.Rows[j][k].AsInteger;

            for k := 2 to High(totals) do
                row.Add(IntToStr(totals[k]), MixColors([report.Rows[0][k].Color, clGray], [1, 1]), ((k - 2) div 3) * 3 + 2);

            if IsPoints(Prefs) then
                row[report.Columns.Count - 3].Text := '';  // points

            if OrderBy = 0 then
                row[report.Columns.Count - 2].Text := FloatToStr(row[report.Columns.Count - 2].AsInteger / report.Rows.Count)  // average score
            else
                row[report.Columns.Count - 2].Text := '';  // score ratio
//            row[report.Columns.Count - 1].Text := FloatToStr(row[report.Columns.Count - 1].AsInteger / report.Rows.Count);  // games

            // Clear groups of three columns that contain only '0's.
            colours := 8;
            for j := 0 to High(TNexusColour) do begin
                zeroes := True;
                for k := 0 to 2 do
                    for i := 0 to report.Rows.Count - 1 do
                        zeroes := zeroes and (report.Rows[i][j * 3 + k + 2].Text = '0');

                if zeroes then begin
                    for k := 0 to 2 do
                        for i := 0 to report.Rows.Count - 1 do
                            report.Rows[i][j * 3 + k + 2].Text := '';
                    Dec(colours);
                end;
            end;

            row[1].Text := Format('Totals (average = %d)', [GameList.Count]);

            report.OnCalcBar := TeamLadderColourCalcBar;
        end; // if ShowColours

        report.Rows.PurgeBlankRows;

        if Description then begin
            report.Description := 'This report ranks teams.  ';
            if (DropWorst > 0) and (DropBest > 0) then
                if PercentGames then
                    report.Description := report.Description + 'The top ' + IntToStr(DropBest) + '% and bottom ' + IntToStr(DropWorst) + '% of scores for each team have been dropped.  '
                else
                    report.Description := report.Description + 'The top ' + IntToStr(DropBest) + ' and bottom ' + IntToStr(DropWorst) + ' scores for each team have been dropped.  '
            else if DropWorst > 0 then
                if PercentGames then
                    report.Description := report.Description + 'The bottom ' + IntToStr(DropWorst) + '% of scores for each team have been dropped.  '
                else
                    report.Description := report.Description + 'The bottom ' + IntToStr(DropWorst) + ' scores for each team have been dropped.  '
            else if DropWorst > 0 then
                if PercentGames then
                    report.Description := report.Description + 'The top ' + IntToStr(DropBest) + '% of scores for each team have been dropped.  '
                else
                    report.Description := report.Description + 'The top ' + IntToStr(DropBest) + ' scores for each team have been dropped.  ';

            if more and less then
                report.Description := report.Description + 'Teams with more or less than ' + IntToStr(modeval) + ' games have been scaled down or up.  This is shown in the Scaled column.  '
            else if more then
                report.Description := report.Description + 'Teams with more than ' + IntToStr(modeval) + ' games have been scaled down.  This is shown in the Scaled column.  '
            else if less then
                report.Description := report.Description + 'Teams with less than ' + IntToStr(modeval) + ' games have been scaled up.  This is shown in the Scaled column.  ';

            if datelimited then
                report.Description := report.Description + 'The report has been limited to games from ' + DateToStr(From) + ' to ' + DateToStr(Too) + '.  ';

            if ShowColours then
                report.Description := report.Description + 'For each team, the report shows the number of times they place first, second and third on each colour.  The Totals row shows the total number of firsts, seconds and thirds that were scored by each colour.';

            if report.Description = 'This report ranks teams.  ' then  // No interesting boxes are checked,
                report.Description := '';  // so this description is too boring to show.
        end;

        TeamLadderReport := report;
    end;

    // Lists games where possible problems are detected: too few players on a team, too many players on a team, players not logged in.
    procedure BuildSanityCheckReport(Description: Boolean);
    var game, team, player, row: Integer;
        players, teams: Integer;
//        logins: Integer;
        report: TZoomReport;
        averageplayers: Integer;
//        averagelogins: Double;
        leagueteam: TLeagueTeam;
    begin
        players := 0;
        teams := 0;
//        logins := 0;
        for game := 0 to GameList.Count - 1 do begin
            Inc(players, GameList[game].Players.Count);
            Inc(teams, GameList[game].Count);
//            for player := 0 to GameList[game].Players.Count - 1 do
//                if GameList[game].Players[player].PlayerID <> '' then
//                    Inc(logins);
        end;

        report := reports.Add('Sanity Check Report', 'Game,Team,Issue', 'left,left,left');

        if Abs(players / teams - Round(players / teams)) < 0.2 then
            averageplayers := Round(players / teams)
        else
            averageplayers := -1;

//        averagelogins := logins / teams;

        for game := 0 to GameList.Count - 1 do begin
            for team := 0 to GameList[game].Teams.Count - 1 do begin
                if averageplayers <> -1 then begin
                    players := 0;
                    for player := 0 to GameList[game].Players.Count - 1 do
                        if GameList[game].Players[player].TeamID = GameList[game].Teams[team].TeamID then
                            Inc(players);

                    if players < averageplayers then begin  // Team has too few players in this game.
                        row := report.Rows.Add(TZRow.Create);
                        leagueteam := LeagueList.FindTeamID(GameList[game].Teams[team].TeamID);
                        if Assigned(leagueteam) then begin
                            report.Rows[row].Add(FormatDateTime(ShortDateFormat + ' ' + ShortTimeFormat, GameList[game].GameTime));
                            report.Rows[row].Add(leagueteam.TeamName, NexusColors[GetNumTeamColourIndex(game, team)]);
                            report.Rows[row].Add('Team has only ' + IntToStr(players) + ' players.');
                        end;
                    end
                    else if players > averageplayers then begin  // Team has too many players in this game.
                        row := report.Rows.Add(TZRow.Create);
                        leagueteam := LeagueList.FindTeamID(GameList[game].Teams[team].TeamID);
                        if Assigned(leagueteam) then
                            report.Rows[row].Add(FormatDateTime(ShortDateFormat + ' ' + ShortTimeFormat, GameList[game].GameTime));
                            report.Rows[row].Add(leagueteam.TeamName, NexusColors[GetNumTeamColourIndex(game, team)]);
                            report.Rows[row].Add('Team has ' + IntToStr(players) + ' players.  (A player might have changed packs.)');
                    end;
                end;

                for player := 0 to GameList[game].Players.Count - 1 do
                    if GameList[game].Players[player].PlayerID = '' then begin  // Not-logged-in player.
                        row := report.Rows.Add(TZRow.Create);
                        leagueteam := LeagueList.FindTeamID(GameList[game].Players[player].TeamID);
                        if Assigned(leagueteam) then
                            report.Rows[row].Add(FormatDateTime(ShortDateFormat + ' ' + ShortTimeFormat, GameList[game].GameTime));
                            report.Rows[row].Add(leagueteam.TeamName, NexusColors[GetNumTeamColourIndex(game, team)]);
                            report.Rows[row].Add('Pack ' + GameList[game].Players[player].Pack + ' did not log on.  Score: ' + IntToStr(GameList[game].Players[player].Score));
                    end;
            end;  // for team
        end;  // for game

        if Description then
            report.Description := 'This report lists possible issues with the way games are committed.  ' +
                'If a team seems to have too few or too many players, or if a player is not logged on in a tournament ' +
                'where most players are logged on, a line will be added to the report to let you know.  ' +
                'Take remedial action (e.g. by fixing the problem and recommitting the game) where appropriate.';
    end;

    // List teams and their players' performance in each game.  One column per game.
    procedure BuildKombiReport(From, Too: TDateTime; Description: Boolean);
    var i, playerindex, game, team, row, col, score, count: Integer;
        player: TPlayer;
        lastgamedate: TDateTime;
        report: TZoomReport;
    begin
        report := nil;
        for i := 0 to LeagueList.Count - 1 do begin  // Create a report for each League team.
            report := reports.Add('', '.', 'left');  // No title for this report.  No caption for the first column (which contains team name in the first row, and player aliases in subsequent rows).
            report.Rows.Add(TZRow.Create);
            report.Rows[0].Add(LeagueList[i].BuildTeamName);  // Set up team name in row 0.

            // Add a row for each player on this team.
            for playerindex := 0 to LeagueList[i].Players.Count - 1 do begin
                report.Rows.Add(TZRow.Create);
                report.Rows[playerindex+1].Add(LeagueList[i].Players[playerindex].ButtonID);  // Add the player button ID.  later we'll replace this with their alias.  Name);  // Set up player name.
            end;

            // Add a column for each of this team's games. Fill in values for team score and player scores.
            lastgamedate := 0;
            for game := 0 to GameList.Count - 1 do
                if (GameList[game].GameTime >= From) and (GameList[game].GameTime <= Too) then begin
                    team := GameList[game].Teams.Find(LeagueList[i].TeamID);
                    if team > -1 then begin  // This team is in this game.
                        if Trunc(lastgamedate) = Trunc(GameList[game].GameTime) then  // If it's on the same day as the last game,
                            report.Columns.Add(FormatDateTime(ShortTimeFormat, GameList[game].GameTime), 'right')  // show just the game time
                        else
                            report.Columns.Add(FormatDateTime(ShortDateFormat + ' ' + ShortTimeFormat, GameList[game].GameTime), 'right');  // show the full date and time.

                        if IsPoints(Prefs) then  // There are victory points for this league
                            report.Columns.Add('', 'right');  // so add a column to display the victory points for this team in this game.
                        lastgamedate := GameList[game].GameTime;

                        // Add the team score to the team row.
                        report.Rows[0].Add(IntToStr(GameList[game].Teams[team].Score),
                                           NexusColors[GetNumTeamColourIndex(game, team)],
                                           report.Rows[0].Count);
                        if IsPoints(Prefs) then  // There are victory points for this league
                            report.Rows[0].Add(FloatToStr(GameList[game].Teams[team].Points),
                                               NexusColors[GetNumTeamColourIndex(game, team)],
                                               report.Rows[0].Count - 1);

                        // Add player scores to player rows.
                        for row := 1 to report.Rows.Count - 1 do begin
                            playerindex := GameList[game].Players.Find(report.Rows[row][0].Text);
                            if playerindex > -1 then begin  // This player is in this game.
                                player := GameList[game].Players[playerindex];
                                report.Rows[row].Add(IntToStr(player.Score),
                                                     NexusColors[player.Colour],
                                                     report.Rows[row].Count);
                            end else
                                report.Rows[row].Add('');  // This player did not play in this game -- add a blank.
                            if IsPoints(Prefs) then  // There are victory points for this league
                                report.Rows[row].Add('');  // so add a blank for the victory points column.  (Later I might show player rank here?)
                        end;
                    end;
                end;  // if From..Too; for GameList

            for playerindex := 0 to LeagueList[i].Players.Count - 1 do begin
                report.Rows[playerindex+1][0].Text := LeagueList[i].Players[playerindex].Name;  // Replace button ID with player name.
            end;

            // Calculate averages.
            for row := 0 to report.Rows.Count - 1 do begin
                score := 0;
                count := 0;
                for col := 1 to report.Columns.Count - 1 do begin
                    if (report.Columns[col].Heading <> '') and (report.Rows[row][col].Text <> '') then begin
                        Inc(score, StrToInt(report.Rows[row][col].Text));
                        Inc(count);
                    end;
                end;
                if count > 0 then
                    if row = 0 then
                        report.Rows[row].Add(IntToStr(score div count))  // No bar for the team average -- it's not useful because you can't compare from team to team.
                    else
                        report.Rows[row].Add(IntToStr(score div count), clNone, report.Rows[row].Count);  // Player average, including bar.
            end;
            report.Columns.Add('Average', 'right');
            BubbleSort(report.Rows, CompareLastColumn); // Sort by average.
            report.MaxBarByRow := True;
//              report.OnCalcBar := KombiReportCalcBar; TeamLadderCalcBar;
            report.NumberStyle := nsBrief;
        end;

        if Assigned(report) then
            report.Description := 'This report shows teams and their players.  Each column is one game for that team.';
    end;

    procedure BuildSoloLadder(DropWorst, DropBest, TopN, AtLeast: Integer; PercentGames, WithinDateRange, ShowComment: Boolean; From, Too: TDateTime; OrderBy: Integer; Scattergram, Description: Boolean);

        procedure Scatter(list: TList; cell: TZCell);
        var i: Integer;
        begin
            if Scattergram then begin
                cell.Scatter := TIntList.Create;
                for i := 0 to list.Count - 1 do
                    if list is TIntList then
                        cell.Scatter.Add(TIntList(list)[i])
                    else if list is TDoubleList then
                        cell.Scatter.Add(Round(TDoubleList(list)[i] * 100));
            end;
        end;

    var i, j, k: Integer;
        report: TZoomReport;
        buttonlist: TStringList;  // list of all players who will appear in the report
        scoreslist, rankslist: TIntList;  // lists of stats for an individual player
        tagratioslist, scoreratioslist: TDoubleList;
        tagratio, scoreratio: Double;
        s: string;
        score, countscores, countranks, gamescore: Integer;
        averagescore, averagerank: Double;
        gamesplayed, gamesdropped: Integer;
        thisplayer: Integer;
        player: TLeaguePlayer;
        row: TZRow;
    begin
        report := reports.Add('title',
                              'Rank,Player,Team,"Average Score","Average Rank","Tag Ratio","Score Ratio","TR x SR",Games',
                              'center,left,left,right,right,right,right,right,right,right');

        if WithinDateRange then
            report.Title := LeagueTitle + ' Solo Ladder from ' + DateToStr(From) + ' to ' + DateToStr(Too)
        else
            report.Title := LeagueTitle + ' Solo Ladder to ' + DateToStr(LastGameDate);

        if (DropWorst <> 0) or (DropBest <> 0) then
            report.Columns.Add('Dropped', 'right');

        if ShowComment then
            report.Columns.Add('Comment', 'center');

        report.MaxBarByColumn := True;
        report.MultiColumnOK := True;

        buttonlist := TStringList.Create;
        scoreslist := TIntList.Create;
        rankslist  := TIntList.Create;
        tagratioslist := TDoubleList.Create;
        scoreratioslist := TDoubleList.Create;
        try
            for i := 0 to GameList.Count - 1 do
                for j := 0 to GameList[i].Players.Count - 1 do begin
                    s := GameList[i].Players[j].PlayerID;
                    if //((FSystemType = stLaserforce) or (Length(s) = 7) and (s[4] in ['+', '-', '%'])) and
                       (buttonlist.IndexOf(s) = -1) then  // if it's a button id, and it's not already in the list,
                        buttonlist.Add(s);  // add it to the list of button id's
                end;

            for i := 0 to buttonlist.Count - 1 do begin  // for each unique button id
                s := LeagueList.FindID(buttonlist[i]);
                if s = '' then Continue;  // button is not in the league list, so skip it

                row := report.Rows[report.Rows.Add(TZRow.Create)];
                row.Add('0');  // put in a temporary Rank
                row.Add(s);  // set up player name
                row.Add(LeagueList.FindTeamByButton(buttonlist[i]));  // set up team name

//                CalculateOnePlayer(DropWorst, DropBest, PercentGames, ScaleGames, WithinDateRange, From, Too,
//                  buttonlist[i], averagescore, averagerank, tagratio, scoreratio, gamesplayed, gamesdropped);
                scoreslist.Clear;
                rankslist.Clear;
                tagratioslist.Clear;
                scoreratioslist.Clear;
                for j := 0 to GameList.Count - 1 do begin  // Roll through all the games...
                    thisplayer := GameList[j].Players.Find(buttonlist[i]);
                    if (thisplayer > -1) and               // ...looking for ones featuring this player.
                       (not WithinDateRange or ((GameList[j].GameTime >= From) and
                                                (GameList[j].GameTime <= Too))) then begin
                        scoreslist.Add(GameList[j].Players[thisplayer].Score);
                        rankslist.Add(GameList[j].Players[thisplayer].Rank);
                        if GameList[j].Players[thisplayer].HitsOn <> 0 then
                            tagratioslist.Add(GameList[j].Players[thisplayer].HitsBy / GameList[j].Players[thisplayer].HitsOn);

                        gamescore := 0;
                        for k := 0 to GameList[j].Players.Count - 1 do
                            Inc(gamescore, GameList[j].Players[k].Score);
                        if gamescore <> 0 then
                            scoreratioslist.Add(GameList[j].Players[thisplayer].Score / gamescore * GameList[j].Players.Count);
                    end;
                end;  // for j

                score := Round(DropWorstAndBest(DropWorst, DropBest, PercentGames, scoreslist, countscores));
                averagerank := DropWorstAndBest(DropBest, DropWorst, PercentGames, rankslist, countranks);
                tagratio := DropWorstAndBestRatio(DropWorst, DropBest, PercentGames, tagratioslist);
                scoreratio := DropWorstAndBestRatio(DropWorst, DropBest, PercentGames, scoreratioslist);

                if countscores = 0 then begin
                    row.Add('0');                              // average score
                    row.Add('0.00');                           // average rank
                    row.Add('0');                              // tag ratio (grand mean)
                    row.Add('0');                              // score ratio
                    row.Add('0');                              // TR x SR
                    row.Add('0');                              // total games played
                    if (DropWorst > 0) or (DropBest > 0) then
                        row.Add('0');                          // games dropped
                end else begin
                    if countscores > 0 then                     // average score
                        row.Add(IntToStr(score))
                    else
                        row.Add('-');                          // no valid ranks were found

                    Scatter(scoreslist, row[row.Count - 1]);

                    if countranks > 0 then                     // average rank
                        row.Add(Format('%.1f', [averagerank]))
                    else
                        row.Add('-');                          // no valid ranks were found
                    Scatter(rankslist, row[row.Count - 1]);

                    if tagratioslist.Count = 0 then            // tag ratio
                        row.Add('')
                    else
                        row.Add(Format('%.0f%%', [100.0 * tagratio]));
                    Scatter(tagratioslist, row[row.Count - 1]);

                    if scoreratioslist.Count = 0 then          // score ratio
                        row.Add('-')
                    else
                        row.Add(Format('%.0f%%', [100.0 * scoreratio]));
                    Scatter(scoreratioslist, row[row.Count - 1]);

                    if (tagratioslist.Count = 0) or (scoreratioslist.Count = 0) then  // TR x SR
                        row.Add('')
                    else
                        row.Add(Format('%.0f%%', [100.0 * tagratio * scoreratio]));

                    row.Add(IntToStr(scoreslist.Count));       // total games played

                    if (DropWorst > 0) or (DropBest > 0) then  // games dropped
                        row.Add(IntToStr(scoreslist.Count - countscores));

                    row.HasData := True;
                end;

                if ShowComment then begin                      // comment
                    player := LeagueList.FindPlayer(buttonlist[i]);
                    if Assigned(player) then
                        row.Add(player.Comment);
                end;

                for j := 0 to 3 do
                    row[j].BarCell := 3;
                for j := 4 to 7 do
                    row[j].BarCell := j;
            end;  // for i

            case OrderBy of
              0: BubbleSort(report.Rows, CompareSoloScores);
              1: BubbleSort(report.Rows, CompareSoloRanks);
              2: BubbleSort(report.Rows, CompareSoloTagRatios);
              else BubbleSort(report.Rows, CompareSoloScoreRatios);
            end;

            for i := report.Rows.Count - 1 downto 0 do
                if StrToIntDef(report.Rows[i][8].Text, 0) < AtLeast then  // remove rows with too few games
                    report.Rows.Delete(i);

            if (TopN > 0) and (report.Rows.Count > TopN) then
                for i := report.Rows.Count - 1 downto TopN do  // remove rows near the bottom of the table, that aren't in the top N
                    report.Rows.Delete(i);

            SetRanks(report.Rows);
        finally
            buttonlist.Free;
            scoreslist.Free;
            rankslist.Free;
            tagratioslist.Free;
            scoreratioslist.Free;
        end;

        report.Rows.PurgeBlankRows;

        if Description then begin
            report.Description := 'This report ranks individual players.  Tag Ratio is the number of people you shot divided by the number of people that shot you.  ' +
                                  'Score ratio is your score in each game divided by the average score for that game.  TR x SR is tag ratio multiplied by score ratio.  ';

            if (DropWorst > 0) and (DropBest > 0) then
                if PercentGames then
                    report.Description := report.Description + 'The top ' + IntToStr(DropWorst) + '% and bottom ' + IntToStr(DropBest) + '% of scores for each player have been dropped.  '
                else
                    report.Description := report.Description + 'The top ' + IntToStr(DropWorst) + ' and bottom ' + IntToStr(DropBest) + ' scores for each player have been dropped.  '
            else if DropWorst > 0 then
                if PercentGames then
                    report.Description := report.Description + 'The bottom ' + IntToStr(DropBest) + '% of scores for each player have been dropped.  '
                else
                    report.Description := report.Description + 'The bottom ' + IntToStr(DropBest) + ' scores for each player have been dropped.  '
            else if DropWorst > 0 then
                if PercentGames then
                    report.Description := report.Description + 'The top ' + IntToStr(DropBest) + '% of scores for each player have been dropped.  '
                else
                    report.Description := report.Description + 'The top ' + IntToStr(DropBest) + ' scores for each player have been dropped.  ';

            if WithinDateRange then
                report.Description := report.Description + 'The report has been limited to games from ' + DateToStr(From) + ' to ' + DateToStr(Too) + '.  ';

            report.Description := report.Description + 'Players are ranked on their average ';
            case OrderBy of
              0: report.Description := report.Description + 'score.  ';
              1: report.Description := report.Description + 'rank.  ';
              2: report.Description := report.Description + 'tag ratio.  ';
              else report.Description := report.Description + 'score ratio.  ';
            end;

            if AtLeast > 1 then
                report.Description := report.Description + 'Only players with at least ' + IntToStr(AtLeast) + ' games played are shown.  ';

            if TopN > 0 then
                report.Description := report.Description + 'Only the top ' + IntToStr(TopN) + ' players are shown.  ';

            if Scattergram then
                report.Description := report.Description + 'Player''s scores are shown as a quartile plot: the bottom of the thin bar is their lowest score; the top of the thin bar is their highest score.  ' +
                                                           'The bottom of the thick bar is their 25% score; the top of the thick bar is their 75% score.  The pale mark is their median score.  The dark mark is their mean score.';
        end;
    end;

    procedure BuildScoreList(From, Too: TDateTime; Description: Boolean);
    var i, j: Integer;
        report: TZoomReport;
        s: string;
        row: TZRow;
        color: TColor;
    begin
        report := reports.Add('title','Game','left');

        report.Title := LeagueTitle + ' Score List from ' + DateToStr(From) + ' to ' + DateToStr(Too);

        for i := 0 to GameList.Count - 1 do
            if (GameList[i].GameTime >= From) and (GameList[i].GameTime <= Too) then begin
                row := report.Rows[report.Rows.Add(TZRow.Create)];
                row.Add(FormatDateTime(ShortDateFormat + ' ' + ShortTimeFormat, GameList[i].GameTime));
                for j := 0 to GameList[i].Players.Count - 1 do begin
                    if report.Columns.Count < j * 2 + 3 then begin
                        report.Columns.Add('Player', 'left');
                        report.Columns.Add('Score', 'right');
                        report.Columns.Add('Hit By', 'right');
                        report.Columns.Add('Hit On', 'right');
                    end;
                    s := GameList[i].Players[j].PlayerID;
                    if (Length(s) = 7) and (s[4] in ['+', '-', '%']) then  // if it's a button id,
                        s := LeagueList.FindID(s)
                    else
                        s := GameList[i].Players[j].Pack;
                    color := StringToColor(GameList[i].Players[j].Pack);
                    row.Add(s, color);
                    row.Add(IntToStr(GameList[i].Players[j].Score), color);
                    row.Add(IntToStr(GameList[i].Players[j].HitsBy), color);
                    row.Add(IntToStr(GameList[i].Players[j].HitsOn), color);
                end;
            end;

        if Description then
            report.Description := 'This report shows the scores of all players in games within the specified date/time range.  Its intent is that you extract the data from it and process it in a third-party tool such as Excel.';
    end;

    // Build an n x n grid showing how each team has scored against each other team.
    procedure BuildTeamsVersus(ShowPoints, WithinDateRange: Boolean; From, Too: TDateTime; Description: Boolean);
    type TTeamOffset = record
        Offset: Integer;
        Points: Double;
        Score: Integer;
    end;

    var i, j, k: Integer;
        report:     TZoomReport;
        teamoffsets: array of TTeamOffset;  // This is effectively a team ladder, to determine what order teams go into the report in.
        temp: TTeamOffset;
        team1, team2: Integer;
        count: Integer;  // How many times did we play a particular team?
        wins: Integer;  // How many times did we beat them?
        points: Double;  // How many points did we score doing it?
        margin: Real;  // How convincingly did we beat them?
    begin
        SetLength(teamoffsets, LeagueList.Count);

        if Assigned(TeamLadderReport) then  // Use the team ladder to rank teams for this report.
            for i := 0 to Length(teamoffsets) - 1 do begin
                teamoffsets[i].Offset := i;
                for j := 0 to TeamLadderReport.Rows.Count - 1 do
                    if TeamLadderReport.Rows[j][1].Text = LeagueList[i].TeamName then
                        teamoffsets[i].Points := -TeamLadderReport.Rows[j][0].AsInteger;
            end
        else
            for i := 0 to Length(teamoffsets) - 1 do begin  // Calculate a total score for each league team.
                teamoffsets[i].Offset := i;

                teamoffsets[i].Score := 0;
                for j := 0 to GameList.Count - 1 do begin  // Roll through all the games...
                    team1 := GameList[j].Teams.Find(LeagueList[i].TeamID);
                    if (team1 > -1) and                 // ...looking for ones featuring this team.
                       (not WithinDateRange or ((GameList[j].GameTime >= From) and
                                                (GameList[j].GameTime <= Too))) then begin
                        teamoffsets[i].Points := teamoffsets[i].Points + GameList[j].Teams[team1].Points;
                        Inc(teamoffsets[i].Score, GameList[j].Teams[team1].Score);
                    end;
                end;
            end;  // for each team in LeagueList

        for i := 0 to Length(teamoffsets) - 1 do  // Bubblesort teamoffsets.
            for j := i to Length(teamoffsets) - 1 do
                if (teamoffsets[i].Points < teamoffsets[j].Points) or (teamoffsets[i].Points = teamoffsets[j].Points) and (teamoffsets[i].Score < teamoffsets[j].Score) then begin  // Swap the two entries.
                    temp.Offset := teamoffsets[j].Offset;
                    temp.Points := teamoffsets[j].Points;
                    temp.Score := teamoffsets[j].Score;
                    teamoffsets[j].Offset := teamoffsets[i].Offset;
                    teamoffsets[j].Points := teamoffsets[i].Points;
                    teamoffsets[j].Score := teamoffsets[i].Score;
                    teamoffsets[i].Offset := temp.Offset;
                    teamoffsets[i].Points := temp.Points;
                    teamoffsets[i].Score := temp.Score;
                end;

        if WithinDateRange then
            report := reports.Add(LeagueTitle + ' Teams vs Teams from ' + DateToStr(From) + ' to ' + DateToStr(Too), 'Team', 'left')
        else
            report := reports.Add(LeagueTitle + ' Teams vs Teams to ' + DateToStr(LastGameDate), 'Team', 'left');

        for i := 0 to Length(teamoffsets) - 1 do  // Create a column for each League team.
            report.Columns.Add(LeagueList[teamoffsets[i].Offset].BuildTeamName, 'center');

        for i := 0 to Length(teamoffsets) - 1 do begin // Create a row for each League team.
            report.Rows.Add(TZRow.Create);

            report.Rows[i].Add(LeagueList[teamoffsets[i].Offset].BuildTeamName);  // set up team name

            for j := 0 to LeagueList.Count - 1 do
                if i = j then begin
                    report.Rows[i].Add('', clGray);
                    report.Rows[i][report.Rows[i].Count - 1].BarCell := report.Rows[i].Count - 1;  // Gotta set a BarCell so that the custom paint will get called.
                end else begin
                    count := 0;
                    wins := 0;
                    points := 0;
                    margin := 0;

                    for k := 0 to GameList.Count - 1 do begin  // Roll through all the games...
                        GameList[k].Teams.Sort;  // ... and sort them so that the winning team is [0], second is [1], etc.
                        team1 := GameList[k].Teams.Find(LeagueList[teamoffsets[i].Offset].TeamID);
                        team2 := GameList[k].Teams.Find(LeagueList[teamoffsets[j].Offset].TeamID);
                        if (team1 > -1) and (team2 > -1) and   // ...looking for ones featuring these teams.
                           (not WithinDateRange or ((GameList[k].GameTime >= From) and
                                                    (GameList[k].GameTime <= Too))) then begin
                            if (GameList[k].TotalScore <> 0) or (GameList[k].Teams[team1].Points <> 0) or (GameList[k].Teams[team2].Points <> 0) then begin  // If this game has all zero scores, ignore it.
                                if team1 < team2 then begin
                                    Inc(wins);
                                    margin := margin + (team2 + 1) / (team1 + 1);  // Each margin is a number from n/(n-1) to n/1, where n is the number of teams in the tournament.  Low means you beat them by a little bit; high means you beat them by a lot.
                                end;
                                points := points + GameList[k].Teams[team1].Points;
                                Inc(count);
                            end;
                        end;
                    end;  // for k

                    if count = 0 then
                        report.Rows[i].Add('-')
                    else if ShowPoints then
                        report.Rows[i].Add(IntToStr(wins) + '/' + IntToStr(count) + '  ' + FloatToStrF(points / count, ffNumber, 4, 2), clNone, j+1)
                    else
                        report.Rows[i].Add(IntToStr(wins) + '/' + IntToStr(count), clNone, j+1);

//                    report.Rows[i][report.Rows[i].Count - 1].Custom := TDouble.Create(margin);  // This line is only needed for SlopeBars.
                    if ShowPoints and (Prefs.LeaguePoints.Count > 0) then
                        report.Rows[i][report.Rows[i].Count - 1].Custom := TDouble.Create((Max(Prefs.LeaguePoints[0], Prefs.LeaguePoints[Prefs.LeaguePoints.Count - 1]) + Prefs.LeaguePointsHigh));
                end;  // if i<>j
        end;  // for each team in LeagueList

        if Description then begin
            report.Description := 'Teams vs Teams shows how each team performs against each other team.  For example, to see how ' +
                report.Rows[0][0].Text + ' did against ' + report.Rows[1][0].Text + ', read across the first row to the second column.';
            if ShowPoints then
                report.Description := report.Description + ' The first number, ' +
                    Copy(report.Rows[0][2].Text, 1, Pos(' ', report.Rows[0][2].Text) - 1) + ', is the number of times ' +
                    report.Rows[0][0].Text + ' scored higher than ' + report.Rows[1][0].Text +
                    ', compared to the total number of times they played each other.  The second number, ' +
                    Trim(Copy(report.Rows[0][2].Text, Pos(' ', report.Rows[0][2].Text), MaxInt)) + ', is the average number of victory points ' + report.Rows[0][0].Text + ' scored against ' + report.Rows[1][0].Text + '.'
            else
                report.Description := report.Description + ' This score, ' +
                    report.Rows[0][2].Text + ', is the number of times ' + report.Rows[0][0].Text + ' scored higher than ' + report.Rows[1][0].Text +
                    ', compared to the total number of times they played each other.';
        end;

//        report.OnCalcBar := TeamsVersusCalcBar;
//        if SlopeBars then
        report.OnPaintBar := TeamsVersusPaintBar;
        report.OnCellDestroy := TeamsVersusCellDestroy;
    end;

    procedure BuildPackReport(DropWorst, DropBest: Integer; PercentGames, WithinDateRange: Boolean; From, Too: TDateTime; Description: Boolean);
    var i, j, k: Integer;
        report: TZoomReport;
        buttonlist,  // list of all players who will appear in the report
        ratioslist,  // lists of tag ratios for all players
        packlist: TStringList;  // list of all packs which will appear in the report
        scoreslist, rankslist: TIntList;  // lists of stats for an individual pack
        s: string;
        score, countscores, countranks, gamepacks: Integer;
        scoreratio, scoreratiototal, scoreratiosquaretotal, hitsby, hitson, gameaverage: Double;
        stddev: double;
        rank: Double;
        thisplayer: Integer;
        thispack: Integer;
        packid: Integer;  // pack ID as a number: 0 = red 1, 1 = blu 1, etc.
        scale: Double;  // this is the ratio by which we're going to scale each value before it goes into our summary lists, reflecting the skill of the player who was wearing that pack.
        zquery: TZReadOnlyQuery;
    begin
        // First build a solo ladder, showing tag ratios for each player, so we can calibrate the pack report.
        buttonlist := TStringList.Create;
        ratioslist := TStringList.Create;
        packlist := TStringList.Create;
        scoreslist := TIntList.Create;
        rankslist  := TIntList.Create;
        try
            for i := 0 to GameList.Count - 1 do
                for j := 0 to GameList[i].Players.Count - 1 do begin
                    s := GameList[i].Players[j].PlayerID;
                    if (*(Length(s) = 7) and (s[4] in ['+', '-', '%']) and*) (buttonlist.IndexOf(s) = -1) then  // if it's a button id, and it's not already in the list,
                        buttonlist.Add(s);  // add it to the list of button id's
                end;

            for i := 0 to buttonlist.Count - 1 do begin  // for each unique button id,
                hitsby := 0;
                hitson := 0;
                score := 0;
                gameaverage := 0;
                for j := 0 to GameList.Count - 1 do begin  // Roll through all the games...
                    thisplayer := GameList[j].Players.Find(buttonlist[i]);
                    if (thisplayer > -1) and               // ...looking for ones featuring this player.
                       (not WithinDateRange or ((GameList[j].GameTime >= From) and
                                                (GameList[j].GameTime <= Too))) then begin
                        hitsby := hitsby + GameList[j].Players[thisplayer].HitsBy;
                        hitson := hitson + GameList[j].Players[thisplayer].HitsOn;
                        score := score + GameList[j].Players[thisplayer].Score;
                        gameaverage := gameaverage + GameList[j].TotalScore / GameList[j].Players.Count;
                    end;
                end;  // for j

                if (hitson = 0) and (hitsby = 0) then begin
                    if gameaverage = 0 then
                        ratioslist.Add('-')
                    else
                        ratioslist.Add(Format('%.0f%%', [score / gameaverage]))
                end else if hitson = 0 then
                    ratioslist.Add('1000000.0')
                else
                    ratioslist.Add(Format('%.0f%%', [100.0 * hitsby / hitson]));
            end;  // for i

            // Now build the pack report.
            report := reports.Add('title',
                                  'Rank,"Pack Number","Pack Name","Average Score","Average Rank","Tag Ratio","Score Ratio","Std Dev","Deviations from 100%",Games',
                                  'center,left,left,right,right,right,right,right,right,right,right');

            if WithinDateRange then
                report.Title := LeagueTitle + ' Pack Report from ' + DateToStr(From) + ' to ' + DateToStr(Too)
            else
                report.Title := LeagueTitle + ' Pack Report to ' + DateToStr(LastGameDate);

            if (DropWorst <> 0) or (DropBest <> 0) then
                report.Columns.Add('Dropped', 'right');

            report.MaxBarByColumn := True;

            for i := 0 to GameList.Count - 1 do
                for j := 0 to GameList[i].Players.Count - 1 do begin
                    s := GameList[i].Players[j].Pack;
                    if packlist.IndexOf(s) = -1 then  // if it's not already in the list,
                        packlist.Add(s);  // add it to the list of packs
                end;

            for i := 0 to packlist.Count - 1 do begin  // for each unique pack,
                report.Rows.Add(TZRow.Create);

                report.Rows[i].Add('0');  // put in a temporary Rank
                report.Rows[i].Add(packlist[i]);  // set up pack name

                s := packlist[i];
                if FSystemType = stAcacia then begin
                    s := packlist[i];
                    case UpperCase(s)[1] of
                        'R': packid := 0;  // red
                        'B': packid := 1;  // blue
                        else packid := 2;  // yellow
                    end;

                    Report.Rows[i][0].Color := StringToColor(s);

                    while (Length(s) > 0) and ((s[1] < '0') or (s[1] > '9')) do  // drop the colour; pull out just the pack number
                        s := Copy(s, 2, MaxInt);
                    Inc(packid, (StrToInt(s) - 1) * 3);

                    zquery := TZReadOnlyQuery.Create(Self);
                    try
                        zquery.Connection := ZConnection;
                        zquery.SQL.Add('SELECT Alias FROM T2PACKS ' +
                                       'WHERE Pack_ID = "' + IntToStr(packid) + '"');
                        zquery.Open;
                        if zquery.EOF then
                            s := ''
                        else
                            s := Trim(zquery.FieldValues['Alias']);
                    finally
                        FreeSquish(zquery);
                    end;
                end;

                report.Rows[i].Add(s);  // pack name

                scoreslist.Clear;
                rankslist.Clear;
                hitsby := 0;
                hitson := 0;
                scoreratiototal := 0;
                scoreratiosquaretotal := 0;
                for j := 0 to GameList.Count - 1 do begin  // Roll through all the games...
                    thispack := GameList[j].Players.FindPack(packlist[i]);
                    if (thispack > -1) and               // ...looking for ones featuring this pack.
                       (not WithinDateRange or ((GameList[j].GameTime >= From) and
                                                (GameList[j].GameTime <= Too))) then begin
                        thisplayer := buttonlist.IndexOf(GameList[j].Players[thispack].PlayerID);
                        if thisplayer <> -1 then begin
                            scale := 100 / StrToFloatDef(ratioslist[thisplayer], 100);

                            scoreslist.Add(Round(GameList[j].Players[thispack].Score * scale));
                            rankslist.Add( Round(GameList[j].Players[thispack].Rank * scale));

                            hitsby := hitsby + GameList[j].Players[thispack].HitsBy * scale;
                            hitson := hitson + GameList[j].Players[thispack].HitsOn * scale;

                            gamepacks := GameList[j].Players.Count;

                            scoreratio := GameList[j].Players[thispack].Score * scale / GameList[j].TotalScore * gamepacks;
                            scoreratiototal := scoreratiototal + scoreratio;
                            scoreratiosquaretotal := scoreratiosquaretotal + Sqr(scoreratio);
                        end;
                    end;
                end;  // for j

                score := Round(DropWorstAndBest(DropWorst, DropBest, PercentGames, scoreslist, countscores));
                rank := DropWorstAndBest(DropBest, DropWorst, PercentGames, rankslist, countranks, True);

                if countscores = 0 then begin
                    report.Rows[i].Add('0');                              // average score
                    report.Rows[i].Add('0.00');                           // average rank
                    report.Rows[i].Add('0');                              // tag ratio (grand mean)
                    report.Rows[i].Add('0');                              // score ratio
                    report.Rows[i].Add('-');                              // std dev
                    report.Rows[i].Add('0');                              // total games played
                    if (DropWorst > 0) or (DropBest > 0) then
                        report.Rows[i].Add('0');                          // games dropped
                end else begin
                    report.Rows[i].Add(IntToStr(score));                  // average score

                    if countranks > 0 then                                // average rank
                        report.Rows[i].Add(Format('%.1f', [rank]))
                    else
                        report.Rows[i].Add('-');                          // no valid ranks were found

                    if (hitson = 0) and (hitsby = 0) then                 // tag ratio
                        report.Rows[i].Add('-')
                    else if hitson = 0 then
                        report.Rows[i].Add('flawless')
                    else
                        report.Rows[i].Add(Format('%.0f%%', [100.0 * hitsby / hitson]));

                    if scoreslist.Count = 0 then                          // score ratio
                        report.Rows[i].Add('-')
                    else
                        report.Rows[i].Add(Format('%.1f%%',
//                            [100.0 * (scoretotal / scoreslist.Count) / (gamescoretotal / gamepackstotal)]));
                            [100.0 * scoreratiototal / scoreslist.Count]));

                    if scoreslist.Count <= 1 then begin
                        report.Rows[i].Add('-');                          // std dev
                        report.Rows[i].Add('-');                          // deviations from mean
                    end else begin
                        stddev := Power(1.0 / (scoreslist.Count - 1) * (scoreratiosquaretotal - (scoreratiototal * scoreratiototal / scoreslist.Count)), 0.5);
                        report.Rows[i].Add(Format('%.1f%%', [100.0 * stddev]));
                        report.Rows[i].Add(Format('%.2f', [(scoreratiototal / scoreslist.Count - 1) / stddev]));
                    end;

                    report.Rows[i].Add(IntToStr(scoreslist.Count));       // total games played

                    if (DropWorst > 0) or (DropBest > 0) then             // games dropped
                        report.Rows[i].Add(IntToStr(scoreslist.Count - countscores));
                end;

                for j := 0 to 6 do
                    if report.Rows[i][5].Text = '-' then
                        report.Rows[i][j].BarCell := 6
                    else
                        report.Rows[i][j].BarCell := 5;

                report.Rows[i][6].BarCell := 6;
                report.Rows[i][8].BarCell := 8;
            end;  // for i to packlist.Count

            BubbleSort(report.Rows, ComparePacks);
            SetRanks(report.Rows);
        finally
            buttonlist.Free;
            ratioslist.Free;
            packlist.Free;
            scoreslist.Free;
            rankslist.Free;
        end;

        if Description then begin
            report.Description := 'This report shows the performance of each pack, each time it is used by a logged-on player.  ' +
                'Each score the pack gains is scaled by the player''s ratio, effectively ''handicapping''.  ' +
                'You should ignore results from a pack where most of its games are played by a particular player.  ' +
                'You should ignore results from a pack with less than 10 or so games.  You should drop the top 10% and bottom 10% of games.  ' +
                'Once you know which packs are at the ends of the curve, you should remove any unusually good or bad packs.  ' +
                'In a team event, you should try to balance the teams, by removing the best pack from this team, the worst from this team, etc.';

            if not Prefs.NexusScores then
                report.Description := report.Description + '  Note that on Nexus or Helios, only games committed with "Calculate scores by Torn" selected in Preferences will show tag ratios.';

            if WithinDateRange then
                report.Description := report.Description + '  The report has been limited to games from ' + DateToStr(From) + ' to ' + DateToStr(Too) + '.';
        end;
    end;  // BuildPackReport

    // Build a list of games over a specified date/time range, showing all the teams in that game on a single row.
    procedure BuildGamesList(From, Too: TDateTime; Description: Boolean);
    var i, j, teamnum: Integer;
        teamname: string;
        index: Integer;
        thisgametime, lastgametime: TDateTime;
        thisgame, thisplayer, thisteam:   Integer;
        teamslist: TList;  // tracks all the team entries for a single game (i.e. a single row)
        teamentry: TStringList;
        color: TColor;
        maxteamcount, colcount: Integer;
        highid: string;
        highscore, highteamnum: Integer;
        report: TZoomReport;
        NewFrom: TDateTime;
    begin
        report := reports.Add('dummytitle', 'Game', 'right');
        report.MultiColumnOK := True;
        maxteamcount := 0;
        lastgametime := -1;
        thisgametime := 0;
        thisgame     := 0;
        GameList.Sort;

        teamslist := TList.Create;
        try
            NewFrom := From;
            while (thisgame < GameList.Count) and (GameList[thisgame].GameTime < From) do
                Inc(thisgame);
            if (thisgame < GameList.Count) then
                NewFrom := GameList[thisgame].GameTime;

            while (thisgame < GameList.Count) and (GameList[thisgame].GameTime < Too) do begin  // loop through each game, and create a row for it
                thisgametime := GameList[thisgame].GameTime;
                if Trunc(lastgametime) < Trunc(thisgametime) then begin  // we've crossed a date boundary
                    index := report.Rows.Add(TZRow.Create);  // write a row containing the new date
                    report.Rows[index].Add(FormatDateTime(ShortDateFormat, thisgametime));
                end;  // if lastgametime < thisgametime
                lastgametime := thisgametime;

                index := report.Rows.Add(TZRow.Create);

                report.Rows[index].Add(FormatDateTime(ShortTimeFormat, thisgametime));

                highscore   := -1000000;
                highteamnum := 0;
                highid      := '';
                for thisplayer := 0 to GameList[thisgame].Players.Count - 1 do
                    if highscore < GameList[thisgame].Players[thisplayer].Score then begin   // find the high scoring player
                        highscore   := GameList[thisgame].Players[thisplayer].Score;
                        highid      := GameList[thisgame].Players[thisplayer].PlayerID;
                        highteamnum := GameList[thisgame].Players[thisplayer].TeamID;
                    end;

                for thisteam := 0 to GameList[thisgame].Teams.Count - 1 do begin
                    teamnum := GameList[thisgame].Teams[thisteam].TeamID;
                    teamname := 'Team ' + IntToStr(teamnum);
                    for i := 0 to LeagueList.Count - 1 do
                        if LeagueList[i].TeamID = teamnum then
                            teamname := LeagueList[i].BuildTeamName;

                    teamentry := TStringList.Create;
                    teamentry.Add(teamname);
                    teamentry.Add(IntToStr(GameList[thisgame].Teams[thisteam].Score));
                    if IsPoints(Prefs) then  // there are victory points for this league
                        teamentry.Add(FloatToStr(GameList[thisgame].Teams[thisteam].Points))
                    else
                        teamentry.Add('');
                    teamentry.Add(Format('$%.8x', [GetTeamColourIndex(thisgametime, teamnum)]));
                    teamslist.Add(teamentry);
                end;  // for team

                if maxteamcount < teamslist.Count then
                    maxteamcount := teamslist.Count;

                BubbleSort(teamslist, CompareGames);

                for i := 0 to teamslist.Count - 1 do begin
                    report.Rows[index].Add('');

                    j := StrToIntDef(TStringList(teamslist[i])[3], -1);
                    if j in [0..NUM_COLOURS] then
                        color := NexusColors[j]
                    else
                        color := clNone;

                    report.Rows[index].Add(TStringList(teamslist[i])[0], color);
                    report.Rows[index].Add(TStringList(teamslist[i])[1], color);
                    report.Rows[index].Add(TStringList(teamslist[i])[2], color);

                    report.Rows[index][report.Rows[index].Count - 3].BarCell := report.Rows[index].Count - 2;
                    report.Rows[index][report.Rows[index].Count - 2].BarCell := report.Rows[index].Count - 2;
                    report.Rows[index][report.Rows[index].Count - 1].BarCell := report.Rows[index].Count - 2;

                    TStringList(teamslist[i]).Free;
                end;
                teamslist.Clear;

                if (Prefs.LeaguePointsHigh <> 0) and (highscore <> -1000000) then begin // show the highscoring player
                    report.Rows[index].Add('');
                    if LeagueList.FindID(highid) = '' then
                        report.Rows[index].Add(highid, GetTeamColor(thisgametime, highteamnum))
                    else
                        report.Rows[index].Add(LeagueList.FindID(highid), GetTeamColor(thisgametime, highteamnum));
                    report.Rows[index].Add(IntToStr(highscore), GetTeamColor(thisgametime, highteamnum));
                end;
                Inc(thisgame);
            end;  // while date/time <= Too
            if DateToStr(NewFrom) = DateToStr(thisgametime) then begin  // this report is for games on a single day
                report.Title := LeagueTitle + ' Games for ' + DateToStr(NewFrom);
                report.Rows.Delete(0);     // the first entry in rows is a date line; since there's only one date for the whole report, we can delete it.
            end else  // this report is for games over multiple days
                report.Title := LeagueTitle + ' Games from ' + DateToStr(NewFrom) + ' to ' + DateToStr(thisgametime);

            colcount := maxteamcount * 4;

            if Prefs.LeaguePointsHigh <> 0 then begin  // there is a highscore entry at the end of each row
                for i := 0 to report.Rows.Count - 1 do begin
                    while (report.Rows[i].Count > 1) and (report.Rows[i].Count < colcount + 1) do begin  // this game has less teams than the max; so pad with blanks
                        for j := 1 to 4 do
                            report.Rows[i].Insert(report.Rows[i].Count - 3, TZCell.Create);
                    end;
                end;
            end;
            for i := 1 to maxteamcount do begin  // set up the Headings text, to cater for however many columns the report has turned out to be
                report.Columns.Add('', 'left');
                report.Columns.Add('Team', 'left');
                report.Columns.Add('Score', 'right');
                if IsPoints(Prefs) then  // there are victory points for this league
                    report.Columns.Add('Pts', 'right')
                else
                    report.Columns.Add('', 'right');
            end;

            if Prefs.LeaguePointsHigh <> 0 then begin // set up Headings text to show the highscoring player
                report.Columns.Add('', 'left');
                report.Columns.Add('Best player', 'left');
                report.Columns.Add('Score', 'right');
            end;
        finally
            teamslist.Free;
        end;

        if Description then begin
            report.Description := 'This is a list of games.  Each row in the table is one game.  Across each row, you see the teams that were in that game (with the team that placed first listed first, and so on), and the score for each team.';

            if (Prefs.LeaguePointsHigh <> 0) then
                report.Description := report.Description + '  At the end of each row, you see the high-scoring player for that game, and their score.';

//            if WithinDateRange then
//                report.Description := report.Description + '  The report has been limited to games from ' + DateToStr(From) + ' to ' + DateToStr(Too) + '.';
        end;
    end;

    // Build a list of games over a specified date/time range, showing each team on a single row, and each game in a single column.
    procedure BuildGameGrid(DropWorst, DropBest: Integer; PercentGames: Boolean; From, Too: TDateTime; Description: Boolean; GridType: TGridType);

    var i, j: Integer;
        thisgametime: TDateTime;
        thisgame, thisteam:   Integer;
        totalpoints: Double;
        scoreslist: TIntList;
        count : Integer;  // used to sum up a team's number of games played
        report: TZoomReport;
        NewFrom: TDateTime;
        gamecount: Integer;  // number of columns containing games
        extragame : Boolean;

        function AddGameColumns(thisgame : Integer): TDateTime;
        var column: TZColumn;
        begin
            Result := GameList[thisgame].GameTime;
            column := report.Columns.Add(DateTimeToStr(Result), 'right');  // add a column for this game
            column.Heading2 := GameList[thisgame].Title;  // Set the Title (if any).
            if IsPoints(Prefs) then  // add a column for victory points for this game
                report.Columns.Add('', 'right');
            report.Columns.Add('', 'left');  // add a spacer column
        end;

        // An element in PyramidSet is set if teams in this game do _not_ play teams in the next game; i.e. this game and the next are likely part of the same round.
        procedure BuildPyramidSet;
        var step: Integer;
            i, j: Integer;
            found: Boolean;
        begin
            PyramidSet := [];

            if IsPoints(Prefs) then
                step := 3
            else
                step := 2;

            j := 2;
            if report.Rows.Count > 1 then
                while j < report.Rows[1].Count - step - 1 do begin
                    found := False;
                    for i := 0 to report.Rows.Count - 1 do
                        if (report.Rows[i][j].Text <> '') and (report.Rows[i][j + step].Text <> '') then begin
                            found := True;
                            break;
                        end;
                    if not found then
                        for i := 0 to step - 1 do
                            Include(PyramidSet, j + i);
                    Inc(j, step);
                end;  // while j
        end;

        procedure ColorPyramid(rows: TZRows);
        var step: Integer;
            i, j: Integer;
            last: Integer;
            even: Boolean;
        begin
            if IsPoints(Prefs) then
                step := 2
            else
                step := 1;

            for i := 0 to rows.Count - 1 do begin
                last := LastGame(rows[i]);
                while  (last in PyramidSet) and (last < rows[i].Count - 1) do
                    Inc(last);  // Move last out to the end of this round (or repechage).
                even := False;
                j := 2;
                while j <= last do begin
                    if even and (rows[i][j].Color = clNone) then
                        rows[i][j].Color := $02FFF0F0;
                    if not (j in PyramidSet) then begin
                        even := not even;
                        Inc(j, step);  // Move j out to the end of this round (or repechage).
                    end;
                    Inc(j);
                end;
            end;
        end;

    begin
        report := reports.Add('dummytitle', 'Rank,Team', 'center,left');

        thisgame := 0;
        NewFrom := From;
        GameList.Sort;
        while (thisgame < GameList.Count) and (GameList[thisgame].GameTime < From) do
            Inc(thisgame);
        if (thisgame < GameList.Count) then
            NewFrom := GameList[thisgame].GameTime;
        thisgametime := NewFrom;

        while (thisgame < GameList.Count) and (GameList[thisgame].GameTime < Too) do begin  // loop through each game, and create an entry for it
            thisgametime := AddGameColumns(thisgame);

            Inc(thisgame);
        end;  // while in date/time range From .. Too

        extragame := (GridType in [gtAscension, gtPyramid]) and (thisgame < GameList.Count);
        if extragame then  // There exists another game after our datetime range: include it temporarily on the basis that it's probably the first game of grand finals and thus tells us which teams should be on top.
            AddGameColumns(thisgame);

        if GridType = gtGameGrid then begin
            report.Columns.Add('Average score', 'right');  // add one last column for the team average score
            if IsPoints(Prefs) then  // add a column for victory points for this game
                report.Columns.Add('Pts', 'right');  // add one laster column for the team total points

            gamecount := report.Columns.Count - 4;
        end
        else
            gamecount := report.Columns.Count - 3;

        if (DropWorst <> 0) or (DropBest <> 0) then
            report.Columns.Add('Dropped', 'right');

        scoreslist := TIntList.Create;
        try
            for i := 0 to LeagueList.Count - 1 do begin  // for each League team, create a row
                report.Rows.Add(TZRow.Create);
                scoreslist.Clear;
                totalpoints := 0;

                report.Rows[i].Add('');  // rank
                report.Rows[i].Add(LeagueList[i].BuildTeamName);  // set up team name

                for j := 2 to gamecount + 1 do  // for each game, create a cell
                    if Length(report.Columns[j].Heading) > 1 then begin
                        thisgame := GameList.Find(StrToDateTime(report.Columns[j].Heading));
                        if thisgame > -1 then
                            thisteam := GameList[thisgame].Teams.Find(LeagueList[i].TeamID)
                        else
                            thisteam := -1;
                        if thisteam > -1 then begin  // this team played in this game, so show their score
                            report.Rows[i].Add(IntToStr(GameList[thisgame].Teams[thisteam].Score), GetTeamColor(StrToDateTime(report.Columns[j].Heading), LeagueList[i].TeamID));
                            report.Rows[i][report.Rows[i].Count - 1].BarCell := report.Rows[i].Count - 1;  // make this cell a data bar
                            if IsPoints(Prefs) then begin  // add a column for victory points for this game
                                report.Rows[i].Add(FloatToStr(GameList[thisgame].Teams[thisteam].Points), GetTeamColor(StrToDateTime(report.Columns[j].Heading), LeagueList[i].TeamID));
                                report.Rows[i][report.Rows[i].Count - 1].BarCell := report.Rows[i].Count - 2;
                            end;
                            report.Rows[i].HasData := True;
                            scoreslist.Add(GameList[thisgame].Teams[thisteam].Score);  // note that this is left-padded with 0's to ensure that it sorts correctly
                            totalpoints := totalpoints + GameList[thisgame].Teams[thisteam].Points;
                        end else begin  // this team did not play in this game, so show empty spacers
                            report.Rows[i].Add('');
                            if IsPoints(Prefs) then  // add a column for victory points for this game
                                report.Rows[i].Add('');
                        end;
                        report.Rows[i].Add('');  // spacer cell
                    end;

                if GridType = gtGameGrid then begin
                    report.Rows[i].Add(IntToStr(Round(DropWorstAndBest(DropWorst, DropBest, PercentGames, scoreslist, count))));  // add a column for their average score
                    report.Rows[i][report.Rows[i].Count - 1].BarCell := report.Rows[i].Count - 1;  // make this cell a data bar
                    if IsPoints(Prefs) then   // add a column for victory points for this game
                        report.Rows[i].Add(FloatToStr(totalpoints));

                    if (DropWorst <> 0) or (DropBest <> 0) then
                        report.Rows[i].Add(IntToStr(scoreslist.Count - count));  // games dropped
                end;
            end;  // for i
        finally
            scoreslist.Free;
        end;

        if DateToStr(NewFrom) = DateToStr(thisgametime) then begin  // this report is for games on a single day
            report.Title := LeagueTitle + ' Games for ' + DateToStr(NewFrom);
            for j := 2 to gamecount + 1 do
                if Length(report.Columns[j].Heading) > 1 then
                    report.Columns[j].Heading := FormatDateTime(ShortTimeFormat, StrToDateTime(report.Columns[j].Heading));  // convert date/time headings to just short time
        end else begin // this report is for games over multiple days
            report.Title := LeagueTitle + ' Games from ' + DateToStr(NewFrom) + ' to ' + DateToStr(thisgametime);
            for j := 2 to gamecount + 1 do
                if Length(report.Columns[j].Heading) > 1 then
                    report.Columns[j].Heading := FormatDateTime(ShortDateFormat + ' ' + ShortTimeFormat, StrToDateTime(report.Columns[j].Heading));  // convert date/time headings to just short time
        end;

        case GridType of
            gtGameGrid: begin
                if IsPoints(Prefs) then
                    CompareColumn := report.Columns.Find('Pts')
                else
                    CompareColumn := report.Columns.Find('Average score');

                GamesColumn := -1;
                ColumnDirection := -1;
                BubbleSort(report.Rows, CompareTeams);
            end;
            gtAscension: BubbleSort(report.Rows, CompareAscension);
            gtPyramid: begin
                BuildPyramidSet;
                BubbleSort(report.Rows, ComparePyramid);
                ColorPyramid(report.Rows);
            end;
        end;

        if extragame then  // Delete the columns we added for the extra game.
        begin
            report.Columns.Delete(report.Columns.Count - 1);
            if IsPoints(Prefs) then
                report.Columns.Delete(report.Columns.Count - 1);
            report.Columns.Delete(report.Columns.Count - 1);

            for i := 0 to report.Rows.Count - 1 do
                while report.Rows[i].Count > report.Columns.Count do
                    report.Rows[i].Delete(report.Rows[i].Count - 1);
        end;

        report.Rows.PurgeBlankRows;

        for i := report.Rows.Count - 1 downto 0 do  // for each team row
            report.Rows[i][0].Text := IntToStr(i+1);  // set the rank

        if Description then
            case GridType of
                gtGameGrid:
                    if IsPoints(Prefs) then
                        report.Description := 'The Game Grid shows one game in each column.  Each row is a team, and teams are sorted based on their victory points, with ties broken by average score.'
                     else
                        report.Description := 'The Game Grid shows one game in each column.  Each row is a team, and teams are sorted based on their average score.';
                gtAscension:
                    report.Description := 'The Ascension shows one game in each column.  Teams are shown in the ' +
                        'order they were eliminated, with the team(s) that survived the longest on top.';
                gtPyramid:
                    report.Description := 'The Pyramid shows one game in each column.  Teams are ordered by the ' +
                        'round they were eliminated in, with the team(s) that survived the longest on top.';

//                if WithinDateRange then
//                    report.Description := report.Description + '  The report has been limited to games from ' + DateToStr(From) + ' to ' + DateToStr(Too) + '.';
            end;
    end;

var From, Too: TDateTime;
    dropworst, dropbest, topn, atleast, i: Integer;

begin  //--- DoReportLeague ---
    if ReportForm = nil then
        ReportForm := TReportForm.Create(Application);

    if ReportForm.DateFrom.DateTime < 1 then
        ReportForm.DateFrom.DateTime := FirstGameDate;
    if ReportForm.DateTo.DateTime > 49999 then
        ReportForm.DateTo.DateTime   := LastGameDate;

    ReportForm.GameList.Clear;
//    ReportForm.GamePicker.ClearGames;
    for i := 0 to GameList.Count - 1 do begin
      ReportForm.GameList.Add(BuildGameTime(GameList[i].GameTime) + ' ' + GameList[i].Title);
//      ReportForm.GamePicker.AddGame(GameList[i].GameTime);
    end;

    ReportForm.SekritsVisible := SekritsVisible;

    reports := TZoomReports.Create;
    try
        if ReportForm.ShowModal = mrOK then begin
            Screen.Cursor := crAppStart;
            try
                reports.BarColor := ReportForm.BarColorPanel.Color;
                reports.OddColor := ReportForm.OddColorPanel.Color;
                From := ReportForm.DateFrom.Date + ReportForm.TimeFrom.Time;
                Too  := ReportForm.DateTo.Date + ReportForm.TimeTo.Time;

                TeamLadderReport := nil;
                if ReportForm.TeamLadder.Checked then
                    if ReportForm.TeamDropGames.Checked then
                        BuildTeamLadder(ReportForm.DropWorst.Value, ReportForm.DropBest.Value, ReportForm.GamesPercent.Checked, ReportForm.ScaleGames.Checked, ReportForm.TeamWithinRange.Checked, From, Too, ReportForm.TeamOrderBy.ItemIndex, ReportForm.ShowColours.Checked, ReportForm.Descriptions.Checked)
                    else
                        BuildTeamLadder(0, 0, False, ReportForm.ScaleGames.Checked, ReportForm.TeamWithinRange.Checked, From, Too, ReportForm.TeamOrderBy.ItemIndex, ReportForm.ShowColours.Checked, ReportForm.Descriptions.Checked);

                if ReportForm.TeamsVersus.Checked then
                    BuildTeamsVersus(ReportForm.ShowPoints.Checked, ReportForm.TeamWithinRange.Checked, From, Too, ReportForm.Descriptions.Checked);

                if ReportForm.SoloLadder.Checked then begin
                    if ReportForm.SoloDropGames.Checked then begin
                        dropworst := ReportForm.DropWorst.Value;
                        dropbest := ReportForm.DropBest.Value;
                    end else begin
                        dropworst := 0;
                        dropbest := 0;
                    end;
                    if ReportForm.SoloTopN.Checked then
                        topn := ReportForm.TopNSpin.Value
                    else
                        topn := 0;
                    if ReportForm.SoloAtLeast.Checked then
                        atleast := ReportForm.AtLeastSpin.Value
                    else
                        atleast := 1;

                    BuildSoloLadder(dropworst, dropbest, topn, atleast, ReportForm.GamesPercent.Checked, ReportForm.SoloWithinRange.Checked, ReportForm.ShowComments.Checked, From, Too, ReportForm.SoloOrderBy.ItemIndex, ReportForm.Scattergram.Checked, ReportForm.Descriptions.Checked);
                end;

                if ReportForm.Games.Checked      then BuildGamesList(From, Too, ReportForm.Descriptions.Checked);

                if ReportForm.GameGrid.Checked   then
                    if ReportForm.GridDropGames.Checked then
                        BuildGameGrid(ReportForm.DropWorst.Value, ReportForm.DropBest.Value, ReportForm.GamesPercent.Checked, From, Too, ReportForm.Descriptions.Checked, gtGameGrid)
                    else
                        BuildGameGrid(0, 0, False, From, Too, ReportForm.Descriptions.Checked, gtGameGrid);

                if ReportForm.Ascension.Checked  then BuildGameGrid(0, 0, False, From, Too, ReportForm.Descriptions.Checked, gtAscension);
                if ReportForm.Pyramid.Checked    then BuildGameGrid(0, 0, False, From, Too, ReportForm.Descriptions.Checked, gtPyramid);

                if ReportForm.SanityCheckReport.Checked then
                    BuildSanityCheckReport(ReportForm.Descriptions.Checked);

                if ReportForm.TeamAndPlayerReport.Checked then
                    BuildKombiReport(From, Too, ReportForm.Descriptions.Checked);

                if ReportForm.DetailedGames.Checked then
                    BuildDetailedGame(reports, From, Too, ReportForm.Descriptions.Checked);

                if ReportForm.ScoreList.Checked then
                    BuildScoreList(From, Too, ReportForm.Descriptions.Checked);

                if ReportForm.PackReport.Checked then
                    if ReportForm.PackDropGames.Checked then
                        BuildPackReport(ReportForm.DropWorst.Value, ReportForm.DropBest.Value, ReportForm.GamesPercent.Checked, ReportForm.PackWithinRange.Checked, From, Too, ReportForm.Descriptions.Checked)
                    else
                        BuildPackReport(0, 0, False, ReportForm.PackWithinRange.Checked, From, Too, ReportForm.Descriptions.Checked);

                if ReportForm.DramaMode.Checked and (reports.Count > 0) then
                    DramaMode(reports[0], DramaForm, OwnerForm, LadderForm)
                else begin
                    if ReportForm.ToScreen.Checked then ShowOnScreen(reports, OwnerForm, LadderForm);
                    if ReportForm.ToScoreboard.Checked and (reports.Count > 0) then ShowOnScoreboard(reports[0]);
                end;
            finally
                Screen.Cursor := crDefault;
            end;
            if ReportForm.ToHtml.Checked then begin
                SaveAsHtml(reports, SaveDialog, ReportForm.HtmlBars.Checked);
//                if ReportForm.HtmlFtp.Checked then UploadToFtp(SaveDialog.FileName);
            end;
            if ReportForm.ToPrinter.Checked      then ShowOnPrinter(reports, PrintDialog, ReportForm.Portrait.Checked);
            if ReportForm.ToTsv.Checked          then SaveAsTsv(reports, SaveDialog);
        end;
    finally
        reports.Free;
    end;
end;

// Read incoming UDP packet and put it in FReceived.
procedure TMainForm.SocketReadAcacia(var Msg : TMessage);
var receivebuf: array[0..1000] of char;
    i: Integer;
    s: string;
begin
    receivebuf[0] := #0;
    if LOWORD(msg.lParam) <> FD_READ then Exit;

    recv(mySocket, receivebuf, 999, 0);  // receive the datagram and store it in receivebuf.
    if receivebuf[0] <> #0 then begin
        s := StrPas(receivebuf);
        Inc(StatusDisplayCount);
        if StatusDisplayCount = 10 then begin
            PlayerDetailName.Visible := False;
            PlayerDetailTeam.Visible := False;
            PlayerDetailScore.Visible := False;
            PlayerDetailHandicap.Visible := False;
        end;
        if Pos('T2_GAME_SERVER_MODE', s) > 0 then begin
            GameInProgress := Pos('GAME_IN_PROGRESS', s) > 0;

            if GameInProgress then begin
                Latest.Caption := '&Last'#10'Game';

                s := Copy(s, Pos('GAME_IN_PROGRESS', s) + 17, MaxInt);
                i := StrToIntDef(Copy(s, 1, Pos(',', s) - 1), -1);
                if i > -1 then begin
                    PlayerDetailPanel.Caption := IntToStr(i div 60) + ':' + Format('%.2d', [i mod 60]);
                    TimeSinceStart := Now - i / 86400;
                end;
            end else begin
                Latest.Caption := '&Latest'#10'Game';
                PlayerDetailPanel.Caption := '';
            end;
        end;
    end;
    receivebuf[0] := #0;
end;

procedure TMainForm.PyramidClick(Sender: TObject);
begin
    if not Assigned(PyramidForm) then begin
        PyramidForm := TPyramidForm.Create(Application);

        PyramidForm.GameList := GameList;
        PyramidForm.LeagueList := LeagueList;
    end;

    if (PyramidForm.ShowModal = mrOK) and Assigned(PyramidForm.Reports) then begin
        if PyramidForm.ToScreen.Checked     then ShowOnScreen(PyramidForm.Reports, Self, LadderForm);
        if PyramidForm.ToScoreboard.Checked then ShowOnScoreboard(PyramidForm.Reports[0]);
        if PyramidForm.ToHtml.Checked       then SaveAsHtml(PyramidForm.Reports, SaveReportsDialog, False);
        if PyramidForm.ToPrinter.Checked    then ShowOnPrinter(PyramidForm.Reports, PrintDialog1, PyramidForm.Portrait.Checked);
        if PyramidForm.ToTsv.Checked        then SaveAsTsv(PyramidForm.Reports, SaveReportsDialog);
    end;
    SaveLeague;
end;

procedure TMainForm.FixturesClick(Sender: TObject);
begin
    if not Assigned(FixtureForm) then begin
        FixtureForm := TFixtureForm.Create(Application);

        FixtureForm.GameList := GameList;
        FixtureForm.LeagueList := LeagueList;
    end;

    FixtureForm.Initialise;
    if FixtureForm.ShowModal = mrOK then begin
        if FixtureForm.ToScreen.Checked     then ShowOnScreen(FixtureForm.Reports, Self, LadderForm);
        if FixtureForm.ToScoreboard.Checked then ShowOnScoreboard(FixtureForm.Reports[0]);
        if FixtureForm.ToHtml.Checked       then SaveAsHtml(FixtureForm.Reports, SaveReportsDialog, False);
        if FixtureForm.ToPrinter.Checked    then ShowOnPrinter(FixtureForm.Reports, PrintDialog1, FixtureForm.Portrait.Checked);
        if FixtureForm.ToTsv.Checked        then SaveAsTsv(FixtureForm.Reports, SaveReportsDialog);
    end;
end;

procedure TMainForm.NexusGameTimerTimer(Sender: TObject);
var s: string;
begin
    TimeSinceStart := TimeSinceStart + 1 / 86400;

    if Round(TimeSinceStart * 86400) mod 60 = 1 then // Once per minute, check if a game is running.
        GameInProgress := NexusGameInProgress;

    if GameInProgress then begin
        DateTimeToString(s, 'n:ss', TimeSinceStart);
        PlayerDetailPanel.Caption := '+' + s;
    end else begin
        TimeSinceStart := 0;
        PlayerDetailPanel.Caption := '';
    end;
end;

procedure TMainForm.PlayerDetailPanelClick(Sender: TObject);
begin
    if FSystemType in [stNexus, stNexusWithZeon] then
        GameInProgress := NexusGameInProgress;  // This will restart NexusGameTimer if appropriate.
end;

procedure TMainForm.MatchClick(Sender: TObject);

    function gameid(const s: string): Integer;
    begin
        gameid := StrToIntDef(Copy(s, 1, Pos(',', s) - 1), -1);
    end;

var MatchForm: TMatchForm;
    i: Integer;
    j: Integer;
    servergame: TServerGame;
    matchgames: TGameList;
    lastgame: TGame;
    teamid: Integer;
begin
    MatchForm := TMatchForm.Create(Application);
    try
        GameList.Sort;
        for i := 0 to GameList.Count - 1 do begin
            servergame := ServerGameList.Find(GameList[i].GameTime);
            if Assigned(servergame) then
                MatchForm.Games.Items.Add(IntToStr(servergame.Game_ID) + ', ' +
                    NonNull(servergame.Description) + ' ' +
                    BuildGameTime(servergame.GameTime));

            if Assigned(servergame) and (gameid(GamesCombo.Text) = servergame.Game_ID) then
                MatchForm.Games.Checked[MatchForm.Games.Items.Count - 1] := True;
        end;

        if MatchForm.ShowModal = mrOK then begin
            matchgames := TGameList.Create;
            try
                // Build a TGameList of selected games, linking to the actual TGame's (not to copies).
                for i := 0 to MatchForm.Games.Items.Count - 1 do
                    if MatchForm.Games.Checked[i] then
                        matchgames.Add(GameList[GameList.Find(ServerGameList.Find(gameid(MatchForm.Games.Items[i])).GameTime)]);

                lastgame := matchgames[matchgames.Count - 1];
                // For each game except the last, sum team scores onto the last game.
                // For each game except the last, zero out victory points.
                for i := 0 to matchgames.Count - 2 do begin
                    for j := 0 to lastgame.Teams.Count - 1 do begin
                        teamid := matchgames[i].Teams.Find(lastgame.Teams[j].TeamID);
                        if teamid > -1 then begin
                            lastgame.Teams[j].Adjustment := lastgame.Teams[j].Adjustment + matchgames[i].Teams[teamid].Score;
                            lastgame.Teams[j].Score := lastgame.Teams[j].Score + matchgames[i].Teams[teamid].Score;
                            matchgames[i].Teams[teamid].Adjustment := -matchgames[i].Teams[teamid].Score - matchgames[i].Teams[teamid].Adjustment;
                            matchgames[i].Teams[teamid].PointsAdjustment := -matchgames[i].Teams[teamid].Points - matchgames[i].Teams[teamid].PointsAdjustment;
                            matchgames[i].Teams[teamid].Score := 0;
                            matchgames[i].Teams[teamid].Points := 0;
                        end;
                    end;
                end;
                // Set victory points for the last game.
                  if lastgame.Teams.Count = 2 then begin
                      if lastgame.Teams[0].Score > lastgame.Teams[1].Score then begin
                          lastgame.Teams[0].Points := Prefs.LeaguePoints[0];
                          if Prefs.LeaguePoints.Count > 1 then
                              lastgame.Teams[1].Points := Prefs.LeaguePoints[1]
                          else
                              lastgame.Teams[1].Points := 0;
                      end else begin
                          if Prefs.LeaguePoints.Count > 1 then
                              lastgame.Teams[0].Points := Prefs.LeaguePoints[1]
                          else
                              lastgame.Teams[0].Points := 0;
                          lastgame.Teams[1].Points := Prefs.LeaguePoints[0];
                      end;
                  end;
            finally
                // Free TGameList, but be sure to not free its contents.
                matchgames.Free;
            end;
        end;
    finally
        MatchForm.Free;
    end;
end;

end.
